/*
 * Copyright (C) 2004 State of California, Department of
 * Water Resources.
 *
 * See the file FilterGUI.java in the package dwr.des.digitalfilter
 * for licensing information.
 */

package gov.ca.dwr.des.digitalfilter;

import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

import java.awt.GridBagLayout;
import javax.swing.JPanel;
import java.awt.GridBagConstraints;
import java.awt.GridLayout;
import java.awt.Insets;
import javax.swing.JButton;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

/**
 * This class defines the parameters of the digital filter.
 *
 * @author Aaron Miller <br>
 * last modified by $Author: miller $
 * @version $Revision: 1.1 $ <br>
 * $Date: 2004/09/13 20:33:27 $
 */
public class DigitalFilterDialog extends JDialog {

    private ButtonGroup buttonGroup_1 = new ButtonGroup();
    private ButtonGroup buttonGroup = new ButtonGroup();
    private JTextField textField_highstop;
    private JTextField textField_highpass;
    private JTextField textField_lowpass;
    private JTextField textField_lowstop;
    private JTextField[] textField_order = new JTextField[4];
    private JTextField textField_psd;
    private JTextField textField_taper;
    private JTextField textField_data;
    private JFrame frame;
    private String lastDir;
    private JPanel panel_1;
    private JCheckBox checkbox_taper;
    private JCheckBox checkbox_psd;
    private JRadioButton radioBut_zero;
    private JRadioButton radioBut_mirror;
    private JRadioButton radioBut_lowpass;
    private JRadioButton radioBut_highpass;
    private JRadioButton radioBut_bandpass;
    private JRadioButton radioBut_bandstop;
    private DigitalFilterIO input = null;
    private DigitalFilterParams params;
    private DigitalFilter filter;
    private int numTextFields;
    private boolean filterStatusComplete;

    /**
     * Constructor for GR type interleaved data.
     * @param frame the parent frame or dialog
     * @param iArray the GR interleaved data array
     * @param xDim the location of the x component
     * @param yDim the location of the y component
     * @param timestep the timestep in hours
     */
    public DigitalFilterDialog(JFrame frame, double iArray[], int numDims, int xDim,
            int yDim, double timestep){
        super(frame,"Digital Filter Parameters",true);
        input = new DigitalFilterIO(iArray,numDims,xDim,yDim,timestep);
        constructBox();
        setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);

    }
    /**
     * Constructs the dialog box.
     *
     */
    private void constructBox(){
        panel_1 = new JPanel();
        getContentPane().setLayout(new GridBagLayout());
        {
            final JPanel panel = new JPanel();
            panel.setBorder(new TitledBorder(new EtchedBorder(), "Filter Parameters",
                    TitledBorder.DEFAULT_JUSTIFICATION, TitledBorder.DEFAULT_POSITION, null, null));
            panel.setLayout(new GridBagLayout());
            final GridBagConstraints gridBagConstraints = new GridBagConstraints();
            gridBagConstraints.weighty = 1.0;
            gridBagConstraints.weightx = 1.0;
            gridBagConstraints.fill = GridBagConstraints.BOTH;
            getContentPane().add(panel, gridBagConstraints);

            {
                checkbox_taper = new JCheckBox();
                final GridBagConstraints gridBagConstraints_1 = new GridBagConstraints();
                gridBagConstraints_1.anchor = GridBagConstraints.WEST;
                gridBagConstraints_1.insets = new Insets(2, 2, 2, 2);
                panel.add(checkbox_taper, gridBagConstraints_1);
                checkbox_taper.setText("Taper Data");
            }
            {
                final JLabel label = new JLabel();
                final GridBagConstraints gridBagConstraints_1 = new GridBagConstraints();
                gridBagConstraints_1.gridwidth = 2;
                gridBagConstraints_1.gridx = 1;
                gridBagConstraints_1.insets = new Insets(2, 2, 2, 2);
                gridBagConstraints_1.anchor = GridBagConstraints.EAST;
                panel.add(label, gridBagConstraints_1);
                label.setText("Num of taper pts. Normally (2*taper/total)=5-10%");
            }
            {
                textField_taper = new JTextField();
                textField_taper.setColumns(5);
                final GridBagConstraints gridBagConstraints_1 = new GridBagConstraints();
                gridBagConstraints_1.anchor = GridBagConstraints.WEST;
                gridBagConstraints_1.insets = new Insets(2, 2, 2, 2);
                gridBagConstraints_1.gridx = 3;
                panel.add(textField_taper, gridBagConstraints_1);
            }
            {
                final JLabel label = new JLabel();
                final GridBagConstraints gridBagConstraints_1 = new GridBagConstraints();
                gridBagConstraints_1.gridwidth = 2;
                gridBagConstraints_1.gridx = 1;
                gridBagConstraints_1.anchor = GridBagConstraints.EAST;
                gridBagConstraints_1.insets = new Insets(2, 2, 2, 2);
                gridBagConstraints_1.gridy = 1;
                panel.add(label, gridBagConstraints_1);
                label.setText("PSD moving average - num pts from center");
            }
            {
                textField_psd = new JTextField();
                textField_psd.setColumns(5);
                final GridBagConstraints gridBagConstraints_1 = new GridBagConstraints();
                gridBagConstraints_1.anchor = GridBagConstraints.WEST;
                gridBagConstraints_1.insets = new Insets(2, 2, 2, 2);
                gridBagConstraints_1.gridy = 1;
                gridBagConstraints_1.gridx = 3;
                panel.add(textField_psd, gridBagConstraints_1);
            }
            {
                radioBut_lowpass = new JRadioButton();
                radioBut_lowpass.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        addLowPassPanel(panel_1);
                    }
                });
                buttonGroup.add(radioBut_lowpass);
                final GridBagConstraints gridBagConstraints_1 = new GridBagConstraints();
                gridBagConstraints_1.weightx = 1;
                gridBagConstraints_1.fill = GridBagConstraints.HORIZONTAL;
                gridBagConstraints_1.insets = new Insets(2, 2, 2, 2);
                gridBagConstraints_1.gridy = 3;
                panel.add(radioBut_lowpass, gridBagConstraints_1);
                radioBut_lowpass.setText("Low Pass");
            }
            {
                radioBut_highpass = new JRadioButton();
                radioBut_highpass.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        addHighPassPanel(panel_1);
                    }
                });
                buttonGroup.add(radioBut_highpass);
                final GridBagConstraints gridBagConstraints_1 = new GridBagConstraints();
                gridBagConstraints_1.weightx = 1;
                gridBagConstraints_1.fill = GridBagConstraints.HORIZONTAL;
                gridBagConstraints_1.insets = new Insets(2, 2, 2, 2);
                gridBagConstraints_1.gridy = 3;
                gridBagConstraints_1.gridx = 1;
                panel.add(radioBut_highpass, gridBagConstraints_1);
                radioBut_highpass.setText("High Pass");
            }
            {
                radioBut_bandpass = new JRadioButton();
                radioBut_bandpass.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        addBandPassPanel(panel_1);
                    }
                });
                buttonGroup.add(radioBut_bandpass);
                final GridBagConstraints gridBagConstraints_1 = new GridBagConstraints();
                gridBagConstraints_1.weightx = 1;
                gridBagConstraints_1.fill = GridBagConstraints.HORIZONTAL;
                gridBagConstraints_1.insets = new Insets(2, 2, 2, 2);
                gridBagConstraints_1.gridy = 3;
                gridBagConstraints_1.gridx = 2;
                panel.add(radioBut_bandpass, gridBagConstraints_1);
                radioBut_bandpass.setText("Band Pass");
            }
            {
                radioBut_bandstop = new JRadioButton();
                radioBut_bandstop.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        addBandStopPanel(panel_1);
                    }
                });
                buttonGroup.add(radioBut_bandstop);
                final GridBagConstraints gridBagConstraints_1 = new GridBagConstraints();
                gridBagConstraints_1.weightx = 1;
                gridBagConstraints_1.fill = GridBagConstraints.HORIZONTAL;
                gridBagConstraints_1.insets = new Insets(2, 2, 2, 2);
                gridBagConstraints_1.gridy = 3;
                gridBagConstraints_1.gridx = 3;
                panel.add(radioBut_bandstop, gridBagConstraints_1);
                radioBut_bandstop.setText("Band Stop");
            }
            {
                panel_1.setLayout(new GridLayout());
                final GridBagConstraints gridBagConstraints_1 = new GridBagConstraints();
                gridBagConstraints_1.fill = GridBagConstraints.BOTH;
                gridBagConstraints_1.gridwidth = 4;
                gridBagConstraints_1.gridy = 4;
                panel.add(panel_1, gridBagConstraints_1);

            }
            {
                checkbox_psd = new JCheckBox();
                final GridBagConstraints gridBagConstraints_1 = new GridBagConstraints();
                gridBagConstraints_1.anchor = GridBagConstraints.WEST;
                gridBagConstraints_1.insets = new Insets(2, 2, 2, 2);
                gridBagConstraints_1.gridy = 1;
                gridBagConstraints_1.gridx = 0;
                panel.add(checkbox_psd, gridBagConstraints_1);
                checkbox_psd.setText("PSD Output");
            }
            {
                radioBut_zero = new JRadioButton();
                buttonGroup_1.add(radioBut_zero);
                final GridBagConstraints gridBagConstraints_1 = new GridBagConstraints();
                gridBagConstraints_1.anchor = GridBagConstraints.WEST;
                gridBagConstraints_1.insets = new Insets(2, 2, 2, 2);
                gridBagConstraints_1.gridy = 2;
                panel.add(radioBut_zero, gridBagConstraints_1);
                radioBut_zero.setText("Zero Padding");
            }
            {
                radioBut_mirror = new JRadioButton();
                buttonGroup_1.add(radioBut_mirror);
                final GridBagConstraints gridBagConstraints_1 = new GridBagConstraints();
                gridBagConstraints_1.anchor = GridBagConstraints.WEST;
                gridBagConstraints_1.insets = new Insets(2, 2, 2, 2);
                gridBagConstraints_1.gridy = 2;
                gridBagConstraints_1.gridx = 1;
                panel.add(radioBut_mirror, gridBagConstraints_1);
                radioBut_mirror.setText("Mirror Padding");

            }
            {
                final GridBagConstraints gridBagConstraints_1 = new GridBagConstraints();
                gridBagConstraints_1.gridheight = 0;
                gridBagConstraints_1.gridy = 3;
            }

        }
        {
            final JPanel panel = new JPanel();
            panel.setLayout(new GridBagLayout());
            final GridBagConstraints gridBagConstraints = new GridBagConstraints();
            gridBagConstraints.fill = GridBagConstraints.BOTH;
            gridBagConstraints.gridy = 1;
            getContentPane().add(panel, gridBagConstraints);
            {
                final JButton button = new JButton();
                final GridBagConstraints gridBagConstraints_1 = new GridBagConstraints();
                gridBagConstraints_1.fill = GridBagConstraints.BOTH;
                panel.add(button, gridBagConstraints_1);
                button.setText("OK");
                button.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        filterStatusComplete = runFilter();
                        if(filterStatusComplete)
                            dispose();
                    }
                });

            }
            {
                final JButton button = new JButton();
                button.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        dispose();
                    }
                });
                final GridBagConstraints gridBagConstraints_1 = new GridBagConstraints();
                gridBagConstraints_1.gridx = 1;
                panel.add(button, gridBagConstraints_1);
                button.setText("CANCEL");
            }
        }
        // initial state
        radioBut_lowpass.setSelected(true);
        radioBut_zero.setSelected(true);
        addLowPassPanel(panel_1);
    }

    /**
     * Adds the band pass options to the provided panel.
     * @param panel_1 panel to add components to
     */
    private void addBandPassPanel(JPanel panel_1) {
        panel_1.removeAll();
        panel_1.repaint();
        {
            final JLabel label = new JLabel();
            label.setHorizontalAlignment(SwingConstants.TRAILING);
            panel_1.add(label);
            label.setText("low stop");
        }
        {
            textField_lowstop = new JTextField();
            textField_lowstop.setColumns(5);
            textField_lowstop.setName("low stop");
            panel_1.add(textField_lowstop);
        }
        {
            final JLabel label = new JLabel();
            label.setHorizontalAlignment(SwingConstants.TRAILING);
            panel_1.add(label);
            label.setText("low pass");
        }
        {
            textField_lowpass = new JTextField();
            textField_lowpass.setColumns(5);
            textField_lowpass.setName("low pass");
            panel_1.add(textField_lowpass);
        }
        {
            final JLabel label = new JLabel();
            label.setHorizontalAlignment(SwingConstants.TRAILING);
            panel_1.add(label);
            label.setText("high pass");
        }
        {
            textField_highpass = new JTextField();
            textField_highpass.setColumns(5);
            textField_highpass.setName("high pass");
            panel_1.add(textField_highpass);
        }
        {
            final JLabel label = new JLabel();
            label.setHorizontalAlignment(SwingConstants.TRAILING);
            panel_1.add(label);
            label.setText("high stop");
        }
        {
            textField_highstop = new JTextField();
            textField_highstop.setColumns(5);
            textField_highstop.setName("high stop");
            panel_1.add(textField_highstop);
        }
        textField_order[0] = textField_lowstop;
        textField_order[1] = textField_lowpass;
        textField_order[2] = textField_highpass;
        textField_order[3] = textField_highstop;
        numTextFields = 4;
    }
    /**
     * Adds the band stop options to the provided panel.
     * @param panel_1 panel to add components to
     */
    private void addBandStopPanel(JPanel panel_1) {
        panel_1.removeAll();
        panel_1.repaint();
        {
            final JLabel label = new JLabel();
            label.setHorizontalAlignment(SwingConstants.TRAILING);
            panel_1.add(label);
            label.setText("high pass");
        }
        {
            textField_highpass = new JTextField();
            textField_highpass.setColumns(5);
            textField_highpass.setName("high pass");
            panel_1.add(textField_highpass);
        }
        {
            final JLabel label = new JLabel();
            label.setHorizontalAlignment(SwingConstants.TRAILING);
            panel_1.add(label);
            label.setText("high stop");
        }
        {
            textField_highstop = new JTextField();
            textField_highstop.setColumns(5);
            textField_highstop.setName("high stop");
            panel_1.add(textField_highstop);
        }
        {
            final JLabel label = new JLabel();
            label.setHorizontalAlignment(SwingConstants.TRAILING);
            panel_1.add(label);
            label.setText("low stop");
        }
        {
            textField_lowstop = new JTextField();
            textField_lowstop.setColumns(5);
            textField_lowstop.setName("low stop");
            panel_1.add(textField_lowstop);
        }
        {
            final JLabel label = new JLabel();
            label.setHorizontalAlignment(SwingConstants.TRAILING);
            panel_1.add(label);
            label.setText("low pass");
        }
        {
            textField_lowpass = new JTextField();
            textField_lowpass.setColumns(5);
            textField_lowpass.setName("low pass");
            panel_1.add(textField_lowpass);
        }
        textField_order[0] = textField_highpass;
        textField_order[1] = textField_highstop;
        textField_order[2] = textField_lowstop;
        textField_order[3] = textField_lowpass;
        numTextFields = 4;

    }

    /**
     * Adds the low pass options to the provided panel.
     * @param panel_1 panel to add components to
     */
    private void addLowPassPanel(JPanel panel_1) {
        panel_1.removeAll();
        panel_1.repaint();
        {
            final JLabel label = new JLabel();
            label.setHorizontalAlignment(SwingConstants.TRAILING);
            panel_1.add(label);
            label.setText("low stop");
        }
        {
            textField_lowstop = new JTextField();
            textField_lowstop.setColumns(5);
            textField_lowstop.setName("low stop");
            panel_1.add(textField_lowstop);
        }
        {
            final JLabel label = new JLabel();
            label.setHorizontalAlignment(SwingConstants.TRAILING);
            panel_1.add(label);
            label.setText("low pass");
        }
        {
            textField_lowpass = new JTextField();
            textField_lowpass.setColumns(5);
            textField_lowpass.setName("low pass");
            panel_1.add(textField_lowpass);
        }
        {
            final JLabel label = new JLabel();
            label.setHorizontalAlignment(SwingConstants.TRAILING);
            label.setVisible(false);
            panel_1.add(label);
            label.setText("high pass");
        }
        {
            textField_highpass = new JTextField();
            textField_highpass.setColumns(5);
            textField_highpass.setName("high pass");
            textField_highpass.setVisible(false);
            panel_1.add(textField_highpass);
        }
        {
            final JLabel label = new JLabel();
            label.setHorizontalAlignment(SwingConstants.TRAILING);
            label.setVisible(false);
            panel_1.add(label);
            label.setText("high stop");
        }
        {
            textField_highstop = new JTextField();
            textField_highstop.setColumns(5);
            textField_highstop.setName("high stop");
            textField_highstop.setVisible(false);
            panel_1.add(textField_highstop);
        }
        textField_order[0] = textField_lowstop;
        textField_order[1] = textField_lowpass;
        textField_order[2] = textField_highpass;
        textField_order[3] = textField_highstop;
        numTextFields = 2;

    }

    /**
     * Adds the high pass options to the provided panel.
     * @param panel_1 panel to add components to
     */
    private void addHighPassPanel(JPanel panel_1) {
        panel_1.removeAll();
        panel_1.repaint();
        {
            final JLabel label = new JLabel();
            label.setHorizontalAlignment(SwingConstants.TRAILING);
            panel_1.add(label);
            label.setText("high pass");
        }
        {
            textField_highpass = new JTextField();
            textField_highpass.setColumns(5);
            textField_highpass.setName("high pass");
            panel_1.add(textField_highpass);
        }
        {
            final JLabel label = new JLabel();
            label.setHorizontalAlignment(SwingConstants.TRAILING);
            panel_1.add(label);
            label.setText("high stop");
        }
        {
            textField_highstop = new JTextField();
            textField_highstop.setColumns(5);
            textField_highstop.setName("high stop");
            panel_1.add(textField_highstop);
        }
        {
            final JLabel label = new JLabel();
            label.setHorizontalAlignment(SwingConstants.TRAILING);
            label.setVisible(false);
            panel_1.add(label);
            label.setText("low stop");
        }
        {
            textField_lowstop = new JTextField();
            textField_lowstop.setColumns(5);
            textField_lowstop.setName("low stop");
            textField_lowstop.setVisible(false);
            panel_1.add(textField_lowstop);
        }
        {
            final JLabel label = new JLabel();
            label.setHorizontalAlignment(SwingConstants.TRAILING);
            label.setVisible(false);
            panel_1.add(label);
            label.setText("low pass");
        }
        {
            textField_lowpass = new JTextField();
            textField_lowpass.setColumns(5);
            textField_lowpass.setName("low pass");
            textField_lowpass.setVisible(false);
            panel_1.add(textField_lowpass);
        }
        textField_order[0] = textField_highpass;
        textField_order[1] = textField_highstop;
        textField_order[2] = textField_lowstop;
        textField_order[3] = textField_lowpass;
        numTextFields = 2;

    }

    /**
     * Checks panel parameters for errors, sets up parameters for filter, and runs
     * filter.
     * @return true if filter was run successfully
     */
    private boolean runFilter() {
        if (checkParameters()) {
            if(radioBut_zero.isSelected())
                params = new DigitalFilterParams(input, DigitalFilterParams.PAD_ZERO);
            else
                params = new DigitalFilterParams(input, DigitalFilterParams.PAD_MIRROR);
            if(checkbox_taper.isSelected()){
                params.setTaper(DigitalFilterParams.TAPER_COS,
                        Integer.parseInt(textField_taper.getText()));
            }
            if(checkbox_psd.isSelected()){
                params.setPSDMovingAveLen(Integer.parseInt(textField_psd.getText()));
            }
            if(radioBut_lowpass.isSelected())
                params.setLowPass(Double.parseDouble(textField_lowstop.getText()),
                        Double.parseDouble(textField_lowpass.getText()));
            else if(radioBut_highpass.isSelected())
                params.setHighPass(Double.parseDouble(textField_highpass.getText()),
                        Double.parseDouble(textField_highstop.getText()));
            else if(radioBut_bandpass.isSelected())
                params.setBandPass(Double.parseDouble(textField_lowstop.getText()),
                        Double.parseDouble(textField_lowpass.getText()),
                        Double.parseDouble(textField_highpass.getText()),
                        Double.parseDouble(textField_highstop.getText()));
            else if(radioBut_bandstop.isSelected())
                params.setBandStop(Double.parseDouble(textField_highpass.getText()),
                        Double.parseDouble(textField_highstop.getText()),
                        Double.parseDouble(textField_lowstop.getText()),
                        Double.parseDouble(textField_lowpass.getText()));
            filter = new DigitalFilter(params);
            return true;
        }
        return false;
    }

    /**
     * Returns whether this filter has successfully completed it's computations.
     * @return true if filter was successfully run
     */
    public boolean filterComplete(){
        return filterStatusComplete;
    }

    /**
     * Returns the DigitalFilterIO object associated with this filter.
     * @return DigitalFilterIO object
     */
    public DigitalFilterIO getDigitalFilterIO(){
        return input;
    }

    /**
     * Checks the parameters in the dialog.
     * @return true if all parameters in dialog are acceptable
     */
    private boolean checkParameters() {
        //check file
        if (input == null) {
            JOptionPane.showMessageDialog(frame, "Select an input file", "ERROR!!", JOptionPane.ERROR_MESSAGE);
            return false;
        }
        //check taper
        if (checkbox_taper.isSelected()) {
            //check text
            if (textField_taper.getText() != null) {
                try {
                    Integer.parseInt(textField_taper.getText());
                } catch (NumberFormatException exc) {
                    JOptionPane.showMessageDialog(frame, "Taper length (\"" + textField_taper.getText()
                            + "\") is NOT AN INTEGER", "ERROR!!", JOptionPane.ERROR_MESSAGE);
                    return false;
                }
            }
        }
        //check psd
        if (checkbox_psd.isSelected()) {
            //check text
            if (textField_psd.getText() != null) {
                try {
                    Integer.parseInt(textField_psd.getText());
                } catch (NumberFormatException exc) {
                    JOptionPane.showMessageDialog(frame, "PSD moving ave length (\"" + textField_psd.getText()
                            + "\") is NOT AN INTEGER", "ERROR!!", JOptionPane.ERROR_MESSAGE);
                    return false;
                }
            }
        }
        //check padding
        //check filter type and values
        double tmpVal = 0;
        double prevVal = 0;
        for (int i = 0; i < numTextFields; i++) {
            try {
                tmpVal = Double.parseDouble(textField_order[i].getText());
            } catch (NumberFormatException exc) {
                JOptionPane.showMessageDialog(frame, "Value for " + textField_order[i].getName() + " (\""
                        + textField_order[i].getText() + "\") is NOT A NUMBER", "ERROR!!", JOptionPane.ERROR_MESSAGE);
                return false;
            }
            if (tmpVal < 0) {
                JOptionPane
                .showMessageDialog(frame, "Value for " + textField_order[i].getName() + " (\""
                        + textField_order[i].getText() + "\") should be POSITIVE", "ERROR!!",
                        JOptionPane.ERROR_MESSAGE);
                return false;
            }
            if (tmpVal < prevVal) {
                JOptionPane.showMessageDialog(frame, "Value for " + textField_order[i].getName() + " (\""
                        + textField_order[i].getText() + "\") should be LARGER than "
                        + textField_order[i - 1].getText(), "ERROR!!", JOptionPane.ERROR_MESSAGE);
                return false;
            }
            prevVal = tmpVal;
        }

        return true;
    }

}
