/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2007-07-19
 * Date started:    2007-07-19
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.crrl.data;

import gov.usgs.gr.*;
import gov.usgs.gr.data.*;
import gov.usgs.gr.visual.PageOfGraphs;
import gov.usgs.gr.visual.TextMarkup;
import gov.usgs.gr.visual.gl.GlParticleAnimation;
import java.awt.Component;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;

/**
 * 
 */
public class ParticleOptionsEditor extends gov.usgs.gr.GrObject
    implements gov.usgs.gr.visual.gui.GuiPanelProvider {

    /** An instance of the controller associated with this object. */
    GrController m_grCon = null;
    /** An instance of the page associated with this object. */
    PageOfGraphs m_page = null;
    /** Whether to show the tag number next to the particle. */
    protected boolean m_bShowParticleTags = false;
    /** The panel created by this class for displaying on a toolbar. */
    protected JPanel m_panel = new JPanel();

//------------------------------------------------------------------------------
    /**
     * Create an empty object.
     */
    public ParticleOptionsEditor() {

        // Create the panel once so it can be passed out later as needed.
        createPanel();
    }
  
//------------------------------------------------------------------------------
/**
 * Accessor method.
 * @return The editor panel created by this class at initialization time.
 */
    public void setShowParticleTags(boolean val) {
        
        m_bShowParticleTags = val;  
    }
    
//------------------------------------------------------------------------------
/**
 * Accessor method.
 * @return The editor panel created by this class at initialization time.
 */
    public JPanel getPanel() {

        return m_panel;
    }

//------------------------------------------------------------------------------
/**
 * Create the editor panel that will be displayed on the tool bar.
 */
    public void createPanel() {

        // Set up the panel's border and layout.
        m_panel.setBorder(BorderFactory.createTitledBorder("Particles"));
        m_panel.setLayout(new GridLayout(2, 1));

        // Define a method to handle the Add button being clicked.
        java.awt.event.ActionListener filterAL =
            new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                JDialog dialog = new JDialog(new JFrame(), "Options");
                dialog.setModal(true);
                ParticleDataFileOptionsPanel panel =
                    new ParticleDataFileOptionsPanel();
                panel.setShowTags(m_bShowParticleTags);
                dialog.add(panel);
                panel.setDialog(dialog);
                dialog.pack();
                dialog.setVisible(true);
                boolean bShowParticleTags = panel.getShowTags();
                if (m_bShowParticleTags != bShowParticleTags) {
                    m_bShowParticleTags = bShowParticleTags;
                    if (m_page != null) {
                        List<GlParticleAnimation> parts =
                            (List<GlParticleAnimation>)m_page.
                            findChildInstancesOf(
                            "gov.usgs.gr.visual.gl.GlParticleAnimation", true);
                        for (GlParticleAnimation part : parts) {
                            TextMarkup textMarkup = (TextMarkup)
                                part.firstChildInstanceOf(
                                "gov.usgs.gr.visual.TextMarkup", false);
                            if (textMarkup != null)
                                textMarkup.setProperty("show",
                                    "" + m_bShowParticleTags);
                        } 
                        
                    }
                }
            }
        };

        // Add a button to the main panel below the other components.
        JButton filterButton = createButton("Options", filterAL,
            "filter", "Filter first series in bottom graph");
        byte[] byteArray = {0};
        filterButton.setIcon(new ImageIcon(byteArray));
        filterButton.setMargin(new Insets(0, 10, 0, 10));
        filterButton.setAlignmentX(Component.CENTER_ALIGNMENT);
        m_panel.add(filterButton);

        m_panel.setMaximumSize(new java.awt.Dimension(400, 400));
    }

//------------------------------------------------------------------------------
/**
 *
 */
    protected JButton createButton(String text, ActionListener al,
        String actionText, String toolText) {

        JButton button = new JButton(text);
        button.addActionListener(al);
        button.setActionCommand(actionText);
        button.setToolTipText(toolText);
        byte[] byteArray = {0};
        button.setIcon(new ImageIcon(byteArray));
        button.setBorderPainted(true);
        button.setMargin(new Insets(0, 0, 0, 0));

        return button;
    }

//------------------------------------------------------------------------------
/**
 * Tells this interface what the current controller and page are.
 * @param grCon   The GrController to use.
 * @param page    The PageOfGraphs to use.
 */
    public void setControllerAndPage(GrController grCon, PageOfGraphs page) {

        m_grCon = grCon;
        m_page = page;
    }
}
