/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2003-06-30
 * Date started:    2002-01-09
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr;

import java.io.*;
import java.util.*;
import javax.xml.parsers.*;
import org.xml.sax.*;

//==============================================================================
/**
 * Reads an XML file containing default properties for GrObject classes and
 * stores them so they can be retrieved later whenever those classes are
 * instantiated.
 */
public class ConfigSAXHandler extends GrSAXHandler {

	/** The list of class names and associated properties. */
    GrHashList m_classProps = new GrHashList();

//------------------------------------------------------------------------------
/**
 * Construct an empty object.
 */
    public ConfigSAXHandler () {

    }

//------------------------------------------------------------------------------
/**
 * SAX DocumentHandler method.  Handles a new element being read from the
 * XML file.
 * @param name  The name of the element.
 * @param attrs The element's list of attributes.
 * @throws  SAXException
 */
    public void startElement (String uri, String localName, String qName,
        Attributes attrs) throws SAXException {

        try {
            // Declare local variables.
            String className = null;
            String idString = null;

            // Look for the class name in the list of attributes.
            if (attrs != null) {
                className = attrs.getValue("class");
            }

            if (className == null) {
                // Try to determine the class that should be instantiated.
                if (qName.equals("dataseries")) {
                    int numDimensions = 0;
                    String s = attrs.getValue("numDimensions");
                    if (s != null)
                        numDimensions = Integer.parseInt(s);

                    className = "gov.usgs.gr.data.DataSeries";
                }

                // Try to guess the class from the element name.  If it can't be
                // determined, return from this method.
                if (className == null) {
                    if (qName.equals("page"))
                        className = "gov.usgs.gr.visual.gl.PageOfGlGraphs";
                    if (qName.equals("dim"))
                        className = "gov.usgs.gr.data.DataDimension";
                    if (className == null)
                        return;
                }
            }

            // Look for the class name in the class hashmap.
            GrHashList hashMap = (GrHashList)m_classProps.get(className);
            if (hashMap == null) {
                // If there is no existing entry for the named class, create a
                // new hashmap for storing properties and add it to the
                // class hashmap with the key being the class name.
                hashMap = new GrHashList();
                m_classProps.put(className, hashMap);
            }

            // Add the properties to the hashMap.
            for (int i = 0; i < attrs.getLength(); i++) {
                String elementName = attrs.getQName(i);
                if (!elementName.equals("class"))
                    hashMap.put(elementName, attrs.getValue(i));
            }

        }
        catch (Throwable t) {
            t.printStackTrace ();
        }
    }

//------------------------------------------------------------------------------
/**
 * Accessor method.
 * @return The hashmap of classes and their default properties.
 */
    public GrHashList getClassProps () {

        return m_classProps;
    }
}




