/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2007-06-19
 * Date started:    2000-07-06
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr;

import java.awt.*;
import java.io.*;
import java.util.*;
import javax.swing.*;

//==============================================================================
/**
 *
 */
public abstract class GrFile extends GrObject implements
	java.awt.event.ActionListener {

    /** The title of the file. */
    protected String m_title = "";
    /** The name of the data set's file. */
    protected File m_file = new File("/");
    /** The tree icon to use for this class. */
	public static ArrayList TREE_ICON_HOLDER = new ArrayList(1);
    /** The location of the tree icon. */
    protected String m_iconFile = "GrObject/GrFile.gif";

//------------------------------------------------------------------------------
/**
 * Creates a GrFile.
 */
    public GrFile () {
    }

//------------------------------------------------------------------------------
/**
 * Provide the list of properties with the latest values.
 */
    public GrHashList getProperties() {

    	GrHashList props = super.getProperties();

		props.put("title", new GrProperty(m_title, "Title"));


        // Give the user a file name button...
        GrProperty p = new GrProperty(m_file.getAbsolutePath(), "File");
        final javax.swing.JButton fileButton = new
        	javax.swing.JButton(m_file.getAbsolutePath());

		fileButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent e) {

                int returnVal = showFileDialog(fileButton);
                if (returnVal == JFileChooser.APPROVE_OPTION) {
                    fileButton.setText(m_file.getAbsolutePath());
                    JOptionPane.showMessageDialog(GrController.getParentFrame(
                        fileButton),
                        "The file name has been set.  The file will not be "
                        + "saved until the Save command is given.",
                        "Just so you know...",
                        JOptionPane.INFORMATION_MESSAGE);
                }
			}
		});

        p.component = fileButton;
		props.put("file", p);

        return props;
    }

//------------------------------------------------------------------------------
/**
 * Provide the list of properties with the latest values.
 */
    public void setProperties(HashMap props) {

		super.setProperties(props);

    	GrProperty p;

        Iterator iter = props.keySet().iterator();
        while (iter.hasNext()) {
        	String key = (String)iter.next();
            Object obj = props.get(key);
            if (obj instanceof GrProperty) {
				p = (GrProperty)obj;

            	if (key.equals("title"))
            		m_title = (String)p.value;
            	if (key.equals("file")) {
                    if (p.component instanceof javax.swing.JButton) {
                        m_file = new File(((javax.swing.JButton)
                            p.component).getText());
                    }
                    else {
                        m_file = new File((String)p.value);
                    }
                }
            }
            if (obj instanceof String) {
                String value = (String)obj;
                if (key.equals("title"))
                    m_title = value;
                if (key.equals("file"))
                    m_file = new File(value);
            }
        }
    }

//------------------------------------------------------------------------------
/**
 * Set a single property.
 */
    public void setProperty(String key, String value) {

    	super.setProperty(key, value);

    	if (key.equals("title"))
        	m_title = value;
        if (key.equals("file"))
        	m_file = new File(value);
    }

//------------------------------------------------------------------------------
/**
 * Shows a file dialog and returns the file selected.
 */
    public int showFileDialog(java.awt.Container container) {

    	if (m_file == null) m_file = new File("/");

        java.awt.Frame frame = GrController.getParentFrame(container);

        // Show the open-file dialog and let the user choose the file.
        JFileChooser fc = new JFileChooser();
        fc.setDialogTitle("Choose File Name");
        fc.setCurrentDirectory(m_file);
//        fc.setApproveButtonText("Choose");
        int returnVal = fc.showDialog(frame, "Select");
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File chosenFile = fc.getSelectedFile();
            boolean bSetName = true;
            // If the file exists, check if the user wants to overwrite.
            if (chosenFile.exists()) {
                int choice = JOptionPane.showConfirmDialog(frame,
                    "Replace the existing file?", "Confirm Overwrite",
                    JOptionPane.YES_NO_OPTION,JOptionPane.QUESTION_MESSAGE);

                if (choice == JOptionPane.NO_OPTION) {
                    bSetName = false;
                    returnVal = JFileChooser.CANCEL_OPTION;
                }
            }
            if (bSetName) {
                m_file = chosenFile;
            }
        }

        return returnVal;
    }

//------------------------------------------------------------------------------
/**
 * Shows a dialog for configuring the file.
 */
    public boolean hasOptionsDialog() {

        return false;
    }

//------------------------------------------------------------------------------
/**
 * Shows a dialog for configuring the file.
 */
    public void showOptionsDialog(Frame frame) {

    }
    
//------------------------------------------------------------------------------
/**
 * 
 */
    public static boolean matchesFormat(String[] lines) {

        return false;
    }

//------------------------------------------------------------------------------
/**
 * Provide the list of menu items that execute commands on this object.
 * @return  A menu item that can be placed on a menu and send an ActionEvent
 *          back to this object when it is chosen.
 */
    public javax.swing.JMenuItem getMenuCommands() {

        javax.swing.JMenu subMenu = new javax.swing.JMenu("Gr File");
        javax.swing.JMenuItem saveItem = new javax.swing.JMenuItem("Save");
        saveItem.addActionListener(this);
        subMenu.add(saveItem);
        javax.swing.JMenuItem saveAsItem = new javax.swing.JMenuItem("Save As");
        saveAsItem.addActionListener(this);
        subMenu.add(saveAsItem);
    	return subMenu;
    }

//------------------------------------------------------------------------------
/**
 * Accessor method.
 * @return An ImageIcon to use when representing this object in a tree.
 */
    public javax.swing.ImageIcon getTreeIcon() {

        if (TREE_ICON_HOLDER.size() <= 0)
            setTreeIcon();

    	return (javax.swing.ImageIcon)TREE_ICON_HOLDER.get(0);
    }

//------------------------------------------------------------------------------
/**
 * Loads the tree icon from a file and stores it in the TREE_ICON_HOLDER.
 */
    protected javax.swing.ImageIcon setTreeIcon() {

        javax.swing.ImageIcon icon = loadTreeIcon(m_iconFile);

        if (icon == null)
            icon = super.setTreeIcon();

        TREE_ICON_HOLDER.clear();
        TREE_ICON_HOLDER.add(icon);

        return icon;
    }

//------------------------------------------------------------------------------
/**
 * Reads the file from disk.
 * <p><b> Pre Conditions </b><br>
 *      The filename is not null.
 * @param filename  The name of the file to load.
 */
    public void read () throws IOException {

    }

//------------------------------------------------------------------------------
/**
 * Saves the data to a file.
 * <p><b> Pre Conditions </b><br>
 *      The filename is not null.
 * @param filename  The name of the file to save to.
 */
    public void write() throws IOException {
    }

//------------------------------------------------------------------------------
    public void actionPerformed(java.awt.event.ActionEvent e) {

        javax.swing.JMenuItem source =
        	(javax.swing.JMenuItem)(e.getSource());
        String srcText = source.getText();
        if (srcText.equals("Save As"))
        	save(true);
        if (srcText.equals("Save"))
            save(false);
        }

//------------------------------------------------------------------------------
    public void save(boolean bShowDialog) {

        if (bShowDialog) {
            int returnVal = showFileDialog(null);
            if (returnVal == JFileChooser.CANCEL_OPTION)
                return;
        }
        try {
            write();
        }
        catch (IOException ioe) {
            ioe.printStackTrace();
            javax.swing.JOptionPane.showMessageDialog(null,
                "File could not be saved.",
                "IO Exception", javax.swing.JOptionPane.ERROR_MESSAGE);
        }
    }

//------------------------------------------------------------------------------
/**
 * Create a string representation of this object.
 * @return  The string that describes this object.
 */
    public String toString() {

        String fullClassName = getClass().getName();
        String name =
        	fullClassName.substring(fullClassName.lastIndexOf('.') + 1);

        String fileName = "";
        if (m_file != null) fileName += m_file.getName();

        return (name + ": " + fileName);
    }
}
