/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2004-03-03
 * Date started:    2000-06-26
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr;

import java.util.*;

//==============================================================================
/**
 * An ordered list with all the functions of a HashMap.
*/
public class GrHashList extends java.util.HashMap {

    private LinkedList m_list = new LinkedList();


//------------------------------------------------------------------------------
/**
 * Main constructor for a properties.
 */
    public GrHashList () {

    }

//------------------------------------------------------------------------------
/**
 * Wrapper for the interal list's listIterator method.
 * @param key	The key to remove.
 */
    public ListIterator listIterator() {

    	return m_list.listIterator();
    }

//------------------------------------------------------------------------------
/**
 * Wrapper for the interal list's add method.
 * @param key	The key to add.
 */
    public void add(int index, Object key, Object value) {

    	if (super.containsKey(key)) {
            int oldKeyIndex = m_list.indexOf(key);
            m_list.remove(key);
            if (oldKeyIndex < index)
                index --;
    	}
        m_list.add(index, key);

    	super.put(key, value);
    }

//------------------------------------------------------------------------------
/**
 * Wrapper for the interal list's get method.
 * @param key	The key to remove.
 */
    public Object getKey(int index) {

    	return m_list.get(index);
    }

//------------------------------------------------------------------------------
/**
 * Wrapper for the interal list's get method.
 * @param key	The key to remove.
 */
    public Object getValue(int index) {

    	return get(m_list.get(index));
    }

//------------------------------------------------------------------------------
/**
 * Overrides the clear method so that the internal ordered list is also
 * included in the operation.
 * @param key	The key to remove.
 */
    public void clear() {

    	m_list.clear();
    	super.clear();
	}

//------------------------------------------------------------------------------
/**
 * Overrides the put method so that the internal ordered list is also
 * included in the operation.
 * @param key	The key to remove.
 */
    public Object put(Object key, Object value) {

    	if (!super.containsKey(key)) m_list.add(key);

    	return super.put(key, value);
    }

//------------------------------------------------------------------------------
/**
 * Overrides the remove method so that the internal ordered list is also
 * included in the operation.
 * @param key	The key to remove.
 */
    public Object remove(Object key) {

    	m_list.remove(key);
    	return super.remove(key);
    }
}











