/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2006-04-06
 * Date started:    2001-08-08
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr;

import java.io.*;
import java.util.*;

//==============================================================================
/**
 *
 */
public class GrObjectContainer extends GrObject {

    // The name to report as a description of this object.
    String m_name = "Object Container";
    /** The list of properties that this object has. */
    GrHashList m_props = new GrHashList();
    /** The tree icon to use for this class. */
	public static ArrayList TREE_ICON_HOLDER = new ArrayList(1);
    /** The location of the tree icon. */
    protected String m_iconFile = "GrObject/GrObjectContainer.gif";

//------------------------------------------------------------------------------
/**
 * Creates a GrObjectContainer.
 */
    public GrObjectContainer () {
    }

//------------------------------------------------------------------------------
/**
 * Provide the list of properties with the latest values.
 */
    public GrHashList getProperties() {

    	GrHashList props = super.getProperties();

        // Add this objects properties to the list that is returned.
        Iterator iter = m_props.listIterator();
        while (iter.hasNext()) {
        	String key = (String)iter.next();
            GrProperty prop = (GrProperty)m_props.get(key);
		    props.put(key, prop);
        }

        return props;
    }

//------------------------------------------------------------------------------
/**
 * Provide the list of properties with the latest values.
 */
    public void setProperties(HashMap props) {

		super.setProperties(props);

        Iterator iter = props.keySet().iterator();
        if (props instanceof GrHashList)
            iter = ((GrHashList)props).listIterator();
        while (iter.hasNext()) {
        	String key = (String)iter.next();

            if (key.equals("class") || key.equals("id")) continue;

            Object obj = props.get(key);
            if (obj instanceof GrProperty)
		        m_props.put(key, obj);
            else
                m_props.put(key, new GrProperty(obj, key, false));
        }
    }

//------------------------------------------------------------------------------
/**
 * Set a single property.
 */
    public void setProperty(String key, String value) {

    	super.setProperty(key, value);

        if (key.equals("class") || key.equals("id")) return;

        m_props.put(key, new GrProperty(value, key, false));
    }

//------------------------------------------------------------------------------
/**
 * Accessor method.
 * @return An ImageIcon to use when representing this object in a tree.
 */
    public javax.swing.ImageIcon getTreeIcon() {

        if (TREE_ICON_HOLDER.size() <= 0)
            setTreeIcon();

    	return (javax.swing.ImageIcon)TREE_ICON_HOLDER.get(0);
    }

//------------------------------------------------------------------------------
/**
 * Loads the tree icon from a file and stores it in the TREE_ICON_HOLDER.
 */
    protected javax.swing.ImageIcon setTreeIcon() {

        javax.swing.ImageIcon icon = loadTreeIcon(m_iconFile);

        if (icon == null)
            icon = super.setTreeIcon();

        TREE_ICON_HOLDER.clear();
        TREE_ICON_HOLDER.add(icon);

        return icon;
    }

//------------------------------------------------------------------------------
/**
 * Return the GrObjectContainer within the specified parent.
 */
    public static GrObjectContainer getChildContainer(GrObject parent,
        String containerTitle) {

        // Look through the children of the parent for an ObjectContainer
        // with a matching title.
        Enumeration enumeration = parent.children();
        while (enumeration.hasMoreElements()) {
            GrObject gro = (GrObject)enumeration.nextElement();
            if (gro instanceof GrObjectContainer) {
                GrObjectContainer oc = (GrObjectContainer)gro;
                GrHashList props = oc.getProperties();
                GrProperty p = (GrProperty)props.get("title");
                if (p.value.equals(containerTitle)) {
                    return oc;
                }
            }
        }

        // Could not find a container with the given title.
        return null;
    }

//------------------------------------------------------------------------------
/**
 * Create a string representation of this object.
 * @return  The string that describes this object.
 */
    public void setName (String name) {

        m_name = name;
    }

//------------------------------------------------------------------------------
/**
 * Create a string representation of this object.
 * @return  The string that describes this object.
 */
    public String toString() {

        String title = "";
        GrProperty prop = (GrProperty)m_props.get("title");
        if (prop != null) {
            title += prop.value;
        }

        return (m_name + " " + title);
    }

}
