/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2001-02-01
 * Date started:    2000-06-26
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr;

//import com.sun.xml.tree.*;

//==============================================================================
/**
 * An object property stored suitable for storing in a list or hash table with
 * other properties.  It also includes a String description and a
 * read-only indicator.
*/
public class GrProperty implements Cloneable {

    public Object value;
    public String description;
    public boolean edit = true;
    public boolean save = true;
    public boolean display = true;
    public javax.swing.JComponent component = null;

//------------------------------------------------------------------------------
/**
 * Empty constructor for a property.
 */
    public GrProperty () {

    	this(null, "", true, true, true);
    }

//------------------------------------------------------------------------------
/**
 * Main constructor for a property.
 * @param key	The string that identifies the property.
 */
    public GrProperty (Object aValue) {

    	this(aValue, "", true, true, true);
    }

//------------------------------------------------------------------------------
/**
 * Main constructor for a property.
 * @param key	The string that identifies the property.
 */
    public GrProperty (Object aValue, String aDescription) {

    	this(aValue, aDescription, true, true, true);
    }

//------------------------------------------------------------------------------
/**
 * Main constructor for a property.
 * @param key	The string that identifies the property.
 */
    public GrProperty (Object aValue, String aDescription,
    	boolean editable) {

    	this(aValue, aDescription, editable, true, true);
    }

//------------------------------------------------------------------------------
/**
 * Main constructor for a property.
 * @param key	The string that identifies the property.
 */
    public GrProperty (Object aValue, String aDescription,
    	boolean editable, boolean saveable) {

    	this(aValue, aDescription, editable, saveable, true);
    }

//------------------------------------------------------------------------------
/**
 * Main constructor for a property.
 * @param key	The string that identifies the property.
 */
    public GrProperty (Object aValue, String aDescription,
    	boolean editable, boolean saveable, boolean displayable) {

        value = aValue;
        description = aDescription;
        edit = editable;
        save = saveable;
        display = displayable;
    }

//------------------------------------------------------------------------------
/**
 * Overrides the comparison method so that only the exact same object will be
 * considered equal.
 */
    public boolean equals(Object o) {

    	if (o instanceof GrProperty) {
        	return (o == this);
        }
        else {
        	return false;
        }
    }
}











