/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2007-06-19
 * Date started:    2000-08-28
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr;

import java.io.*;
import java.util.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;

//==============================================================================
/**
 * Handles XML input using the SAX parser.
 */
public class GrSAXHandler extends org.xml.sax.helpers.DefaultHandler {
    
    List m_topElementList = new ArrayList();
    List m_groList = new ArrayList();
    List m_nonIDObjects = new ArrayList();
    Map m_idObjects = new HashMap();
    protected boolean m_bFreshObject = true;
    
//------------------------------------------------------------------------------
    /**
     * Construct an empty graphical element.
     *
     */
    public GrSAXHandler() {
	
    }
    
//------------------------------------------------------------------------------
    /**
     * Create a copy of this object.
     */
    public List read(String filename) {
	
	// Use the default (non-validating) parser
	SAXParserFactory factory = SAXParserFactory.newInstance();
	try {
	    // Parse the input
	    SAXParser saxParser = factory.newSAXParser();
	    // Must use Reader instead of File so that files on the
	    // network don't give ftp errors.
	    saxParser.parse(new InputSource(new FileReader(filename)), this);
	} catch (org.xml.sax.SAXParseException e) {
	    System.out.println("Column # = " + e.getColumnNumber());
	    System.out.println("Line   # = " + e.getLineNumber());
	    System.out.println("Pub ID # = " + e.getPublicId());
	    System.out.println("Sys ID # = " + e.getSystemId());
	    e.printStackTrace();
	} catch (Throwable t) {
	    t.printStackTrace();
	}
	
	return m_topElementList;
    }
    
    //===========================================================
    // SAX DocumentHandler methods
    //===========================================================
    
//------------------------------------------------------------------------------
    /**
     * SAX DocumentHandler method.
     */
    public void startDocument() throws SAXException {
	
	try {
	    
	    m_groList.clear();
	    m_nonIDObjects.clear();
	    m_idObjects.clear();
	    
	} catch (Throwable t) {
	    t.printStackTrace();
	}
    }
    
//------------------------------------------------------------------------------
    /**
     * SAX DocumentHandler method.
     */
    public void endDocument() throws SAXException {
	
	try {
	    
	    // Finish processing for all objects.
	    Iterator iter = m_nonIDObjects.iterator();
	    while (iter.hasNext()) {
		GrObject gro = (GrObject)iter.next();
		gro.finishSAXInput(m_idObjects);
	    }
	    
	    List list = new ArrayList(m_idObjects.keySet());
	    iter = list.iterator();
	    while (iter.hasNext()) {
		GrObject gro = (GrObject)m_idObjects.get(iter.next());
		gro.finishSAXInput(m_idObjects);
	    }
	    
	} catch (Throwable t) {
	    t.printStackTrace();
	}
    }
    
//------------------------------------------------------------------------------
    /**
     * SAX DocumentHandler method.
     */
    public void startElement(String uri, String localName, String qName,
	Attributes attrs) throws SAXException {
	
	try {
	    
	    GrObject elObject = null;
	    String className = null;
	    String idString = null;
	    
	    // Assign attributes to the object.
	    if (attrs != null) {
		// Store this object in a list according to whether it has an ID.
		idString = attrs.getValue("id");
		className = attrs.getValue("class");
		
		if (idString != null) {
		    //Look for object in idObjects and use it instead, if found.
		    GrObject existingGro = (GrObject)m_idObjects.get(idString);
		    if (existingGro != null) {
			elObject = existingGro;
			m_bFreshObject = false;
		    }
		}
	    }
	    
	    if (elObject == null) {
		// Try to determine the class that should be instantiated.
		
		if (qName.equals("dataseries") || qName.equals("doubledataseries")){
		    int numDimensions = 0;
		    String s = attrs.getValue("numDimensions");
		    if (s != null)
			numDimensions = Integer.parseInt(s);
                    
                    // Use DataSeries as the class name if the class isn't
                    // derived from RafDataSeries or TimeSeries.
                    try {
                        if (Class.forName("gov.usgs.gr.data.RafDataSeries").
                            isAssignableFrom(Class.forName(className))) {

                            className = "gov.usgs.gr.data.RafDataSeries";
                            elObject = new gov.usgs.gr.data.RafDataSeries(
                                numDimensions);
                        } else {
                            if (Class.forName("gov.usgs.gr.data.TimeSeries").
                                isAssignableFrom(Class.forName(className))) {
 
                                className = "gov.usgs.gr.data.TimeSeries";
                                elObject = new gov.usgs.gr.data.TimeSeries(
                                    numDimensions);
                            }
                            else {
                                className = "gov.usgs.gr.data.DoubleDataSeries";
                                elObject = new gov.usgs.gr.data.
                                    DoubleDataSeries(numDimensions);
                            }
                        }
                    } catch (Exception e) {
                        className = "gov.usgs.gr.data.DoubleDataSeries";
                        elObject = new gov.usgs.gr.data.DoubleDataSeries(
                            numDimensions);
                    }
		}
		
		// Handle IntDataSeries because it requires an argument to its
		// constructor.
		if (qName.equals("intdataseries")) {
		    int numDimensions = 0;
		    String s = attrs.getValue("numDimensions");
		    if (s != null)
			numDimensions = Integer.parseInt(s);
		    if (Class.forName("gov.usgs.gr.data.IntDataSeries").
			isAssignableFrom(Class.forName(className))) {
			
			className = "gov.usgs.gr.data.IntDataSeries";
			elObject = new gov.usgs.gr.data.IntDataSeries(
			    numDimensions);
		    }
		}
		// Handle DoubleDataSeries because it requires an argument to its
		// constructor.
		if (qName.equals("floatdataseries")) {
		    int numDimensions = 0;
		    String s = attrs.getValue("numDimensions");
		    if (s != null)
			numDimensions = Integer.parseInt(s);
		    if (Class.forName("gov.usgs.gr.data.FloatDataSeries").
			isAssignableFrom(Class.forName(className))) {
			
			className = "gov.usgs.gr.data.FloatDataSeries";
			elObject = new gov.usgs.gr.data.FloatDataSeries(
			    numDimensions);
		    }
		}
		
		// Try to guess the class from the element name.  If it can't be
		// determined, return from this method.
		if (className == null) {
		    if (qName.equals("page"))
			className = "gov.usgs.gr.visual.gl.PageOfGlGraphs";
		    if (qName.equals("dim"))
			className = "gov.usgs.gr.data.DataDimension";
		    if (className == null)
			return;
		}
                
                // Backward compatability for classes that have moved...
		if (className.equals(
                    "gov.usgs.sfhydro.si3d.BarrierEditorProvider")) {
                    
		    className =
                        "gov.usgs.sfhydro.si3d.gui.BarrierEditorProvider";
		}
	    }
	    
	    // Create an instance of the class.
	    if (elObject == null)
		elObject = (GrObject)getObjectInstance(className);
	    
	    if (idString == null) {
		m_nonIDObjects.add(elObject);
	    } else {
		// Add it to the list, since it is the first instance
		// of an object with this id.
		m_idObjects.put(idString, elObject);
	    }
	    
	    // Apply the properties if needed.
	    if (m_bFreshObject) {
		GrHashList hashMap = new GrHashList();
		for (int i = 0; i < attrs.getLength(); i++) {
		    hashMap.put(attrs.getQName(i), attrs.getValue(i));
		}
		
		elObject.setProperties(hashMap);
	    }
	    
	    // Make the new object the current parent.
	    m_groList.add(elObject);
	    
	} catch (Throwable t) {
	    t.printStackTrace();
	}
    }
    
//------------------------------------------------------------------------------
    /**
     * SAX DocumentHandler method.
     */
    public void endElement(String uri, String localName, String qName)
    throws SAXException {
	
	try {
	    
	    if (m_groList.size() == 0) return;
	    
	    GrObject gro = (GrObject)m_groList.remove(m_groList.size() - 1);
	    if (m_bFreshObject) gro.endSAXElement();
	    
	    if (m_groList.size() == 0) {
		m_topElementList.add(gro);
	    } else {
		// Add the object to the current parent.
		GrObject parent =
		    (GrObject)m_groList.get(m_groList.size() - 1);
		parent.add(gro);
	    }
	    
	    m_bFreshObject = true;
	    
	} catch (Throwable t) {
	    t.printStackTrace();
	}
    }
    
//------------------------------------------------------------------------------
    /**
     * SAX DocumentHandler method.
     */
    public void characters(char buf [], int offset, int len)
    throws SAXException {
	
	if (m_groList.size() == 0 || len == 0) return;
	if (!m_bFreshObject) return;
	GrObject currentGro = (GrObject)m_groList.get(m_groList.size() - 1);
	if (currentGro != null) {
	    currentGro.receiveCharsFromSAX(buf, offset, len);
	}
    }
    
    //===========================================================
    // END ------------- SAX DocumentHandler methods
    //===========================================================
    
//------------------------------------------------------------------------------
    /**
     * Read an XML version of this element and its children.
     */
    public Object getObjectInstance(String className) {
	
	Class newClass = null;
	Object o = null;
	
	// Try making an instance of the class that reads the data format.
	// Show error dialog boxes if there was a problem...
	try {
	    newClass = Class.forName(className);
	} catch (ClassNotFoundException e2) {
	    e2.printStackTrace();
	    return null;
	} catch (ExceptionInInitializerError er1) {
	    int hey = 0;
	}
	
	try {
	    o = newClass.newInstance();
	} catch (InstantiationException e) {
	    e.printStackTrace();
	    return null;
	} catch (IllegalAccessException e3) {
	    e3.printStackTrace();
	    return null;
	}
	
	return o;
    }
    
//------------------------------------------------------------------------------
    /**
     * Returns an XML version of this element and its children.
     */
    public String getXmlElementName() {
	return "obj";
    }
    
//------------------------------------------------------------------------------
    /**
     * Converts newline characters to a backslash followed by an 'n'.
     */
    public String convertNewlineToBackslashN(String s) {
	
	String newString = "";
	int index1 = 0;
	
	while (index1 < s.length()) {
	    int index2 = s.indexOf("" + '\n', index1);
	    if (index2 < 0) index2 = s.length();
	    
	    newString += s.substring(index1, index2);
	    index1 = index2 + 1;
	    if (index1 < s.length()) newString += ("\\" + "n");
	}
	
	return newString;
    }
    
//------------------------------------------------------------------------------
    /**
     * Converts a backslash followed by an 'n' to newline characters.
     */
    public String convertBackslashNToNewline(String s) {
	
	String newString = "";
	int index1 = 0;
	
	while (index1 < s.length()) {
	    int index2 = s.indexOf("\\n", index1);
	    if (index2 < 0) index2 = s.length();
	    
	    newString += s.substring(index1, index2);
	    index1 = index2 + 2;
	    if (index1 < s.length()) newString += '\n';
	}
	
	return newString;
    }
}




