/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2007-07-15
 * Date started:    2007-01-04
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr;

import gov.usgs.gr.visual.Axis;
import gov.usgs.gr.visual.Graph;
import gov.usgs.gr.visual.PageOfGraphs;
import gov.usgs.gr.visual.gl.GlTexture;
import gov.usgs.gr.visual.gui.ImageOptionsPanel;
import java.io.*;
import java.util.ArrayList;
import java.util.List;
import javax.swing.JDialog;
import javax.swing.JFrame;

//==============================================================================
/**
 *
 */
public class ImageFile extends GrFile {
    
    /** The location of the tree icon. */
    protected String m_iconFile = "GrObject/GrFile.gif";
    
//------------------------------------------------------------------------------
    /**
     * Creates a GrFile.
     */
    public ImageFile() {
    }
    
//------------------------------------------------------------------------------
    /**
     * Accessor method.
     * @return An ImageIcon to use when representing this object in a tree.
     */
    public javax.swing.ImageIcon getTreeIcon() {
        
        if (TREE_ICON_HOLDER.size() <= 0)
            setTreeIcon();
        
        return (javax.swing.ImageIcon)TREE_ICON_HOLDER.get(0);
    }
    
//------------------------------------------------------------------------------
    /**
     * Loads the tree icon from a file and stores it in the TREE_ICON_HOLDER.
     */
    protected javax.swing.ImageIcon setTreeIcon() {
        
        javax.swing.ImageIcon icon = loadTreeIcon(m_iconFile);
        
        if (icon == null)
            icon = super.setTreeIcon();
        
        TREE_ICON_HOLDER.clear();
        TREE_ICON_HOLDER.add(icon);
        
        return icon;
    }
    
//------------------------------------------------------------------------------
    /**
     * Reads the file from disk.
     * <p><b> Pre Conditions </b><br>
     *      The filename is not null.
     * @param filename  The name of the file to load.
     */
    public void read() throws IOException {
        
        GlTexture glTexture = new GlTexture();
        glTexture.setProperty("file", m_file.getAbsolutePath());
        glTexture.preLoadImage();
        
        double[] corners = glTexture.getCorners();
        JDialog dialog = new JDialog(new JFrame(), "Coordinates");
        dialog.setModal(true);
        ImageOptionsPanel panel = new ImageOptionsPanel();
        panel.setValues(corners);
        dialog.add(panel);
        panel.setDialog(dialog);
        dialog.pack();
        dialog.setVisible(true);
        corners = panel.getValues();
        glTexture.setCorners(corners);
        
        boolean bUsedCurrentPage = false;
        
        // Add the texture to the current page if a 1:1 graph is present.
        GrController grCon = Gr.getController();
        if (grCon != null && grCon.getPage() != null) {
            PageOfGraphs page = grCon.getPage();
            List<Graph> list = page.getGraphList();
            
            for (Graph graph : list) {
                GrHashList props = graph.getProperties();
                GrProperty p = (GrProperty) props.get("aspect1to1");
                boolean bAspect1to1 = (Boolean)p.value;
                
                if (bAspect1to1) {
                    // Look for the right place to insert the texture.
                    List<GrObject> children = graph.getChildList();
                    List<GrObject> higherUps = new ArrayList<GrObject>();
                    for (int i = (children.size() - 1); i >= 0; i--) {
                        GrObject child = children.get(i);
                        if (child instanceof Axis ||
                            child instanceof GlTexture) {
                            break;
                        }
                        higherUps.add(child);
                    }
                    
                    // Insert the texture below the other children.
                    for (GrObject child : higherUps)
                        graph.remove(child);
                    graph.add(glTexture);
                    for (GrObject child : higherUps)
                        graph.add(child);
                    
                    bUsedCurrentPage = true;
                    break;
                }
            }
        } 
        
        if (!bUsedCurrentPage) {
            // Create a new page and add it as a child of this file.
            gov.usgs.gr.visual.gl.GlGraph graph =
                new gov.usgs.gr.visual.gl.GlGraph();
            graph.add(glTexture);
            graph.setTitle(m_file.getName());
            graph.setProperty("aspect1to1", "true");
            gov.usgs.gr.visual.gl.PageOfGlGraphs page =
                new gov.usgs.gr.visual.gl.PageOfGlGraphs();
            page.add(graph);
            page.updateZoomContents();
            page.showMoreGraphs();
            page.setProperty("formatter",
                "gov.usgs.sfhydro.gr.formats.VariableSizeFormatter");
            gov.usgs.gr.visual.Axis xAxis = graph.getXAxis();
            gov.usgs.gr.visual.Axis yAxis = graph.getYAxis();
            xAxis.setProperty("showMajorTicks", "false");
            yAxis.setProperty("showMajorTicks", "false");
            xAxis.setProperty("showMinorTicks", "false");
            yAxis.setProperty("showMinorTicks", "false");
            xAxis.setProperty("showLabels", "false");
            yAxis.setProperty("showLabels", "false");
            xAxis.setProperty("showGridLines", "false");
            yAxis.setProperty("showGridLines", "false");
            
            xAxis.getZoom().setOuterLimits(corners[0], corners[2]);
            yAxis.getZoom().setOuterLimits(corners[1], corners[5]);
            xAxis.getZoom().zoomAll();
            yAxis.getZoom().zoomAll();
            xAxis.getZoom().setContentLimits(corners[0], corners[2]);
            yAxis.getZoom().setContentLimits(corners[1], corners[5]);
            add(page);
        }
    }
}
