/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2006-04-06
 * Date started:    2000-07-06
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr;

import java.util.*;

//==============================================================================
/**
 *
 */
public  class ObjectEffigy extends GrObject {
    /** The object that this effigy depicts. */
    protected GrObject m_subject;
    /** The properties that represent those of the subject. */
    protected GrHashList m_gottenProps = new GrHashList();
    /** The properties that are stored to be applied to the subject later. */
    protected GrHashList m_setProps = new GrHashList();
    /** Whether the subject's properties should be depicted. */
    public boolean depictProperties = false;
    /** Whether this object has read the properties from its subject yet. */
    protected boolean m_bPropsRead = false;
    /** The tree icon to use for this class. */
	public static ArrayList TREE_ICON_HOLDER = new ArrayList(1);
    /** The location of the tree icon. */
    protected String m_iconFile = "GrObject/ObjectEffigy.gif";

//------------------------------------------------------------------------------
/**
 * Creates a blank effigy.
 */
    public ObjectEffigy () {
    }

//------------------------------------------------------------------------------
/**
 * Creates an effigy of the subject.
 */
    public ObjectEffigy (GrObject subject) {

		this(subject, false);
    }

//------------------------------------------------------------------------------
/**
 * Creates an effigy of the subject.
 */
    public ObjectEffigy (GrObject subject, boolean bDepictProperties) {

    	m_subject = subject;
        depictProperties = bDepictProperties;

        redepict();
    }

//------------------------------------------------------------------------------
/**
 * Returns the subject of this effigy.
 */
    public void redepict() {

    	if (m_subject == null) return;

        clear();
        Enumeration enumeration = m_subject.children();
        while (enumeration.hasMoreElements()) {
        	GrObject child = (GrObject)enumeration.nextElement();

            super.add(new ObjectEffigy(child, depictProperties));
        }

        m_setProps.clear();
//        if (depictProperties)
//			m_gottenProps = m_subject.getProperties();
//        else
//        	m_gottenProps = super.getProperties();
    }

//------------------------------------------------------------------------------
/**
 * The opposite of redepict().  Updates the subject to look like this effigy.
 */
    public void updateSubject() {

    	if (m_subject == null) return;

        m_subject.clear();
        Enumeration enumeration = children();
        while (enumeration.hasMoreElements()) {
        	ObjectEffigy child = (ObjectEffigy)enumeration.nextElement();
            m_subject.add(child.getSubject());
        }

        if (depictProperties)
			m_subject.setProperties(m_setProps);
    }

//------------------------------------------------------------------------------
/**
 * Add an effigy of gro.
 */
    public boolean add(GrObject gro) {

		// Wade through any effigies to the underlying object.
        GrObject subject = getSubject();
        while (subject instanceof ObjectEffigy)
        	subject = ((ObjectEffigy)subject).getSubject();

        // Find out if the underlying selected object would have
        // added the new object.
//        GrObject testGro = subject.createCopy(false);
        GrObject testGro = null;
        try {
        	testGro = (GrObject)subject.getClass().newInstance();
        }
        catch (IllegalAccessException iae) {return false;}
        catch (InstantiationException ie) {return false;}

        if (testGro.add(gro)) {
            gro.deleteObserver(testGro);
        	if (subject instanceof GrFile)
            	return super.add(new ObjectEffigy(gro, false));
        	else
            	return super.add(new ObjectEffigy(gro, depictProperties));
        }
        else {
        	return false;
        }
    }

//------------------------------------------------------------------------------
/**
 * Returns the subject of this effigy.
 */
    public GrObject getSubject() {

    	return m_subject;
    }

//------------------------------------------------------------------------------
/**
 * Sets the subject of this effigy.
 */
    public void setSubject(GrObject subject) {

    	m_subject = subject;
    }

//------------------------------------------------------------------------------
/**
 * Provide the list of properties with the latest values.
 */
    public GrHashList getProperties() {

        if (!m_bPropsRead) {
            if (depictProperties)
                m_gottenProps = m_subject.getProperties();
            else
                m_gottenProps = super.getProperties();
            m_bPropsRead = true;
        }

        if (depictProperties)
        	return m_gottenProps;
        else
        	return super.getProperties();
    }

//------------------------------------------------------------------------------
/**
 * Accessor method.
 * @return An ImageIcon to use when representing this object in a tree.
 */
    public javax.swing.ImageIcon getTreeIcon() {

    	if (depictProperties) {
            return m_subject.getTreeIcon();
        }
        else {
            if (TREE_ICON_HOLDER.size() <= 0)
                setTreeIcon();

            return (javax.swing.ImageIcon) TREE_ICON_HOLDER.get(0);
        }
    }

//------------------------------------------------------------------------------
/**
 * Loads the tree icon from a file and stores it in the TREE_ICON_HOLDER.
 */
    protected javax.swing.ImageIcon setTreeIcon() {

        javax.swing.ImageIcon icon = loadTreeIcon(m_iconFile);

        if (icon == null)
            icon = super.setTreeIcon();

        TREE_ICON_HOLDER.clear();
        TREE_ICON_HOLDER.add(icon);

        return icon;
    }

//------------------------------------------------------------------------------
/**
 * Set the properties for this object or for the subject, depending on the
 * value of depictProperties.
 */
    public void setProperties(HashMap props) {

    	if (depictProperties) {
        	// Transfer entries from props to m_props.
			Iterator iter = props.keySet().iterator();
            while (iter.hasNext()) {
            	String key = (String)iter.next();
                m_gottenProps.put(key, props.get(key));
                m_setProps.put(key, props.get(key));

        		m_subject.setProperties(props);
            }
        }
        else {
        	super.setProperties(props);
        }
    }

//------------------------------------------------------------------------------
/**
 * Set the property for this object or for the subject, depending on the
 * value of depictProperties.
 */
    public void setProperty(String key, String value) {

    	if (depictProperties) {
    		GrProperty prop = new GrProperty(value);
	        m_gottenProps.put(key, prop);
	        m_setProps.put(key, prop);
        }
        else {
        	super.setProperty(key, value);
        }
    }

//------------------------------------------------------------------------------
/**
 * Provide the list of menu items that execute commands on this object.
 * @return  A menu item that can be placed on a menu and send an ActionEvent
 *          back to this object when it is chosen.
 */
    public javax.swing.JMenuItem getMenuCommands() {

    	if (m_subject == null) return null;
        else return m_subject.getMenuCommands();
    }

//------------------------------------------------------------------------------
/**
 * Returns an XML version of this element and its children.
 */
    public org.w3c.dom.Element getDOMRepresentation(org.w3c.dom.Document doc,
    	boolean bIncludeChildren) {

        org.w3c.dom.Element el = doc.createElement(this.getXmlElementName());
        if (m_subject == null) return el;

        // Defer to the subject's xml method if it is a DataSeries or another
        // ObjectEfifgy.
        if (m_subject instanceof gov.usgs.gr.data.DataSeries ||
            m_subject instanceof gov.usgs.gr.data.IntDataSeries ||
            m_subject instanceof gov.usgs.gr.data.DoubleDataSeries ||
        	m_subject instanceof gov.usgs.gr.ObjectEffigy)
        	return m_subject.getDOMRepresentation(doc, false);

        // Write the properties...
        GrHashList props = m_subject.getProperties();

        // Just write the id and return if this object has been written out
        // to the file before.
        if (m_subject.m_bSaveIdOnlyToXml) {
        	GrProperty p = (GrProperty)props.get("id");
            el.setAttribute("id", p.value.toString());
        	p = (GrProperty)props.get("class");
            el.setAttribute("class", p.value.toString());

            return el;
        }

        // Add all of the object's properties to the output element.
        ListIterator iter = props.listIterator();
        while (iter.hasNext()) {
        	String key = (String)iter.next();
            GrProperty p = (GrProperty)props.get(key);

            if (p.save == true)
            	el.setAttribute(key, p.value.toString());
        }

    	if (bIncludeChildren) {
        	// Write the children...
        	ListIterator iterator = m_children.listIterator(0);
        	while (iterator.hasNext()) {
        		GrObject child = (GrObject)iterator.next();
            	el.appendChild(child.getDOMRepresentation(doc, true));
        	}
        }

        // If the id was saved then make it so the next time this object
        // is written, only the id will be written.
        if (m_subject.m_bSaveIdToXml)
        	m_subject.m_bSaveIdOnlyToXml = true;

        return el;
    }

//------------------------------------------------------------------------------
/**
 * Create a string representation of this object.
 * @return  The string that describes this object.
 */
    public String toString() {

        String subjectString = "null";

        if (m_subject != null) subjectString = m_subject.toString();

///        String s = "OE";
        String s = "";
        if (depictProperties) {
///        	s += "->";
        	s += subjectString;
        }
        else {
        	s += " ( ";
        	s += subjectString;
        	s += " )";
        }

        return s;


//        if (m_subject == null) return "";
//        return m_subject.toString();
    }

}
