/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2006-05-25
 * Date started:    2000-10-19
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr;

import gov.usgs.gr.*;
import gov.usgs.gr.data.*;
import gov.usgs.gr.visual.*;
import gov.usgs.gr.visual.gui.*;
import java.awt.Frame;
import java.io.*;
import java.text.*;
import java.util.*;
import javax.swing.*;

//==============================================================================
/**
 * An array of data in one or more dimensions.
 * @see gov.usgs.data.XyData.
 */
public class ScriptFile99 extends gov.usgs.gr.GrFile {

    /** The title of the data set. */
    protected String m_title = "";
    /** The name of the data set's file. */
    protected String m_filename = "";
    /** The controller to call. */
    public GrController m_grCon = null;
    /** The page to modify. */
    protected PageOfGraphs m_page = null;
    /** The path to the current working directory. */
    public String m_currentPath = "";
    /** The parent frame to use for modal dialogs. */
    public Frame m_parentFrame = null;

//------------------------------------------------------------------------------
/**
 * Creates a data file.
 */
    public ScriptFile99 () {

    	super();
        m_title = "Script File";
    }

//------------------------------------------------------------------------------
/**
 * Overrides add in GrObject.  Adds nothing.
 */
    public boolean add(GrObject gro) {

        return false;
    }

//------------------------------------------------------------------------------
/**
 * Reads the file from disk.
 * <p><b> Pre Conditions </b><br>
 *      The filename is not null.
 * @param filename  The name of the file to load.
 */
    public void read () throws IOException {

        if (m_grCon == null) return;
		m_filename = m_file.getAbsolutePath();

		Graph graph = null;
        Curve curve = null;
        boolean bUnderstood;

        // Open the file.
        BufferedReader br = new BufferedReader(new FileReader(m_filename));
        StreamTokenizer st = new StreamTokenizer(br);
        st.eolIsSignificant(true);
        st.slashSlashComments(true);
        st.slashStarComments(true);

        List attribs = new ArrayList();

        String line = null;
        st.nextToken();
        // Read each line in from the file
        while (st.ttype != StreamTokenizer.TT_EOF) {
        	attribs.clear();
            // Load up the attribs list.
            do {
            	if (st.ttype == StreamTokenizer.TT_NUMBER) {
                	attribs.add(new Float(st.nval));
                }
            	else {
            		if (st.sval != null) {
                		attribs.add(st.sval);
                	}
            	}

            	st.nextToken();
        	}
        	while (st.ttype != StreamTokenizer.TT_EOL
        		&& st.ttype != StreamTokenizer.TT_EOF);

        	if (attribs.size() == 0) continue;
    		if (!(attribs.get(0) instanceof String)) continue;

        	// Look for the key as the first token.
        	String key = ((String)attribs.get(0)).toUpperCase();
        	attribs.remove(0);

        	bUnderstood = false;

        	// Process the line according to key.
        	if (key.equals("OPEN")) {
            	if (attribs.size() > 0) {
                	String openfile = (String)attribs.get(0);
                    File file = new File(openfile);
                    m_currentPath = file.getParent();
                    m_grCon.openFile(file);
                }
                bUnderstood = true;
        	}
        	if (key.equals("OVERLAY")) {
            	if (attribs.size() > 0) {
                	String openfile = (String)attribs.get(0);
                    File file = new File(openfile);
                    if (file.getParent() == null)
                    	openfile = m_currentPath + "/" + openfile;

                    m_grCon.overlayDataFile(openfile);
                }
                bUnderstood = true;
        	}
        	if (key.equals("SAVEASPS")) {
            	if (attribs.size() > 0)
                	m_grCon.printPage((String)attribs.get(0));
                bUnderstood = true;
        	}
        	if (key.equals("SAVEASXML")) {
                PageOfGraphs page = m_grCon.getPage();
                // Don't proceed if there is no current page.
                if (page != null && attribs.size() > 0) {
                    // Create an XML project file containing the page.
                    XMLProjectFile projFile = new XMLProjectFile();
                    projFile.setProperty("file", (String)attribs.get(0));
                    projFile.add(m_page);

                    // Write the file to disk.
                    try {
                        projFile.write();
                    }
                    catch (java.io.IOException ioe) {
                        ioe.printStackTrace();
                    }
                }

                bUnderstood = true;
        	}
        	if (key.equals("SAVEDATAAS")) {
            	if (attribs.size() > 0)
                	m_grCon.saveToDataFile((String)attribs.get(0));
                bUnderstood = true;
        	}
        	if (key.equals("OPENSCRIPT")) {
            	if (attribs.size() > 0) {
                	String openfile = (String)attribs.get(0);
                    File file = new File(openfile);
                    if (file.getParent() == null)
                    	openfile = m_currentPath + "/" + openfile;
                    m_grCon.openScript(new File(openfile));
                }
                bUnderstood = true;
        	}
        	if (key.equals("OPENTEMPLATE")) {
            	if (attribs.size() > 0) {
                	String openfile = (String)attribs.get(0);
                    File file = new File(openfile);
                    if (file.getParent() == null)
                    	openfile = m_currentPath + "/" + openfile;
                    m_grCon.openTemplate(openfile);
                }
                bUnderstood = true;
        	}
            if (key.equals("OPENXMLTEMPLATE")) {
                if (attribs.size() > 0) {
                    String openfile = (String)attribs.get(0);
                    File file = new File(openfile);
                    if (file.getParent() == null)
                        openfile = m_currentPath + "/" + openfile;
                    m_grCon.openXMLTemplate(new File(openfile));
                }
                bUnderstood = true;
            }
        	if (key.equals("SETFORMAT")) {
            	if (attribs.size() > 0)
                	m_grCon.setDataFormatClass((String)attribs.get(0));
                bUnderstood = true;
        	}
        	if (key.equals("SETPAGEFORMAT")) {
            	try {
                	if (attribs.size() > 0) {
                    	String s = (String)attribs.get(0);
                        PageFormatter f = (PageFormatter)
                        	(Class.forName(s).newInstance());
                        m_grCon.applyFormat(f);
                    }

                    bUnderstood = true;
                }
                catch (ClassNotFoundException e) {
                	JOptionPane.showMessageDialog(m_parentFrame,
                    "Could not find specified page formatter",
                    "Class Not Found Exception", JOptionPane.ERROR_MESSAGE);
                }
                catch (InstantiationException e2) {
                	e2.printStackTrace();
                    JOptionPane.showMessageDialog(m_parentFrame,
                    	"Could not instantiate page formatter",
                        "Instantiation Exception",
                        JOptionPane.ERROR_MESSAGE);
                }
                catch (IllegalAccessException e3) {
                	e3.printStackTrace();
                    JOptionPane.showMessageDialog(m_parentFrame,
                    	"Page format class is not publicly accessible.",
                        "Illegal Access Exception",
                        JOptionPane.ERROR_MESSAGE);
                }
        	}
        	if (key.equals("DESELECTALL")) {
            	m_grCon.deselectAll();
                bUnderstood = true;
        	}
        	if (key.equals("SELECTALL")) {
            	m_grCon.selectAll();
                bUnderstood = true;
        	}
        	if (key.equals("CUT")) {
            	m_grCon.cutSelected();
                bUnderstood = true;
            }
            if (key.equals("COPY")) {
            	m_grCon.copySelected();
            	bUnderstood = true;
            }
            if (key.equals("PASTE")) {
            	m_grCon.pasteSelected();
                bUnderstood = true;
            }
            if (key.equals("DELETE")) {
            	m_grCon.deleteSelected();
                bUnderstood = true;
            }
            if (key.equals("NEWGRAPH")) {
            	m_grCon.newGraph();
                bUnderstood = true;
            }
            if (key.equals("HIDEGRAPH")) {
            	m_grCon.hideGraph();
                bUnderstood = true;
            }
// This should be updated to work with GrObject children so the order
// is known for displaying in the tree and saving to XML.
            if (key.equals("MOVEGRAPH")) {
            	// Get the current page.  Give an error message if there
                // is no page.
        		m_page = m_grCon.getPage();
                if (m_page == null) {
            		JOptionPane.showMessageDialog(m_parentFrame,
            			"No page for command: " + key,
                		"Script File Error", JOptionPane.ERROR_MESSAGE);
                		System.out.println("No page for command: " + key);
                	break;
                }

            	if (attribs.size() > 0) {
                	// Find out which graph and curve to select.
                    String s = (String)attribs.get(0);
                    s = s.toUpperCase();
                    int graphNum = Integer.parseInt(s) - 1;

                    // Pull the selected graph out of the show list.
                    graph = null;
                    Graph pulledGraph = null;
                    synchronized(m_page) {
                        List graphList = m_page.getGraphList();
                        for (int i = 0; i < graphList.size(); i++) {
                            graph = (Graph)graphList.get(i);
                            if (graph.getSelected() && graph.getShow()) {
                                graphList.remove(i);
                                pulledGraph = graph;
                                break;
                            }
                        }

                        // Put it back in where it belongs.
                        int index = 0;
                        if (pulledGraph != null) {
                            index = 0;
                            for (int i = 0; i < graphList.size(); i++) {
                                if (index == graphNum) {
                                    graphList.add(i, pulledGraph);
                                    break;
                                }
                                graph = (Graph)graphList.get(i);
                                if (graph.getShow()) index++;
                            }
                            m_page.updateLists();
                            m_page.getFormatter().layoutGraphs();
                        }
                    }
                }
                bUnderstood = true;
            }
            if (key.equals("SELECTGRAPH")) {
            	// Get the current page.  Give an error message if there
                // is no page.
        		m_page = m_grCon.getPage();
                if (m_page == null) {
            		JOptionPane.showMessageDialog(m_parentFrame,
            			"No page for command: " + key,
                		"Script File Error", JOptionPane.ERROR_MESSAGE);
                		System.out.println("No page for command: " + key);
                	break;
                }

            	if (attribs.size() > 0) {
                	// Find out which graph and curve to select.
                    String s = (String)attribs.get(0);
                    s = s.toUpperCase();
                    int graphNum = Integer.parseInt(s) - 1;

                    // Select the graph.
                    m_page.selectShowingGraph(graphNum, true, true);
                }
                bUnderstood = true;
            }
            if (key.equals("SELECTCURVE")) {
            	// Get the current page.  Give an error message if there
                // is no page.
        		m_page = m_grCon.getPage();
                if (m_page == null) {
            		JOptionPane.showMessageDialog(m_parentFrame,
            			"No page for command: " + key,
                		"Script File Error", JOptionPane.ERROR_MESSAGE);
                		System.out.println("No page for command: " + key);
                	break;
                }

            	if (attribs.size() > 0) {
                	// Find out which graph and curve to select.
                    String s = (String)attribs.get(0);
                    s = s.toUpperCase();
                    int gIndex = s.indexOf('G');
                    int cIndex = s.indexOf('C');
                    if (gIndex >= 0 && cIndex >= 0) {
                    	int graphNum = Integer.parseInt(
                        	s.substring(gIndex + 1, cIndex).trim()) - 1;
                        int curveNum = Integer.parseInt(
                        	s.substring(cIndex + 1).trim()) - 1;

                        // Select the curve.
                        m_page.selectShowingGraph(graphNum, true, true);
                        graph = (Graph)m_page.getShowList().get(graphNum);
                        graph.selectCurve(curveNum, false);
                        m_page.updateSelectedSeriesList();
                    }
                }
                bUnderstood = true;
            }
            if (key.equals("SETGRAPHPROPERTY")) {
            	if (attribs.size() > 1) {
                    // Get the current page.  Give an error message if there
                    // is no page.
                    m_page = m_grCon.getPage();
                    if (m_page == null) {
                        JOptionPane.showMessageDialog(m_parentFrame,
                            "No page for command: " + key,
                            "Script File Error", JOptionPane.ERROR_MESSAGE);
                        System.out.println("No page for command: " + key);
                        break;
                    }

                    // Look at each selected graph.
                    List graphList = m_page.getSelectedGraphs();

                    for (Iterator iter = graphList.iterator(); iter.hasNext(); ) {
                        graph = (Graph) iter.next();

                        // Apply the property value to the graph.
                        graph.setProperty((String)attribs.get(0),
                            (String)attribs.get(1));
                    }
                }

                bUnderstood = true;
            }
            if (key.equals("SETCURVEPROPERTY")) {
            	if (attribs.size() > 1) {
                    // Get the current page.  Give an error message if there
                    // is no page.
                    m_page = m_grCon.getPage();
                    if (m_page == null) {
                        JOptionPane.showMessageDialog(m_parentFrame,
                            "No page for command: " + key,
                            "Script File Error", JOptionPane.ERROR_MESSAGE);
                        System.out.println("No page for command: " + key);
                        break;
                    }

                    // Look at each graph.
                    List graphList = m_page.getGraphList();

                    for (Iterator gIter = graphList.iterator();
                        gIter.hasNext();) {

                        graph = (Graph) gIter.next();

                        // Look at each curve.
                        List curveList = graph.getCurves();
                        for (Iterator cIter = curveList.iterator();
                            cIter.hasNext(); ) {

                            curve = (Curve)cIter.next();

                            // If the curve's series has any points selected,
                            // apply the property value to the curve.
                            if (curve.getSeries().getNumSelected() > 0) {
                                curve.setProperty((String) attribs.get(0),
                                                  (String) attribs.get(1));
                            }
                        }
                    }
                }

                bUnderstood = true;
            }
            if (key.equals("GODIN")) {
            	// Get the current page.  Give an error message if there
                // is no page.
        		m_page = m_grCon.getPage();
                if (m_page == null) {
            		JOptionPane.showMessageDialog(m_parentFrame,
            			"No page for command: " + key,
                		"Script File Error", JOptionPane.ERROR_MESSAGE);
                		System.out.println("No page for command: " + key);
                	break;
                }

            	List dataList = m_page.getSelectedSeriesList();
                DataSnapShot snapShot = new DataSnapShot(dataList);

                gov.usgs.sfhydro.gr.tools.GodinFilter gf =
                	new gov.usgs.sfhydro.gr.tools.GodinFilter();
                gf.performAction(dataList);

                m_grCon.saveDataSnapShot(snapShot);
                bUnderstood = true;
            }
            if (key.equals("MODIFY")) {
            	ModifyDialog dialog =
                	new ModifyDialog(m_parentFrame, "", true);
                dialog.readFromPage(m_grCon);
                DataEditorPanel editorPanel= dialog.getEditorPanel();
                if (editorPanel != null)
                    editorPanel.applyModification(attribs);
                dialog.dispose();
                bUnderstood = true;
            }
            if (key.equals("SYSTEM")) {
            	if (attribs.size() > 0)
                	Runtime.getRuntime().exec(
                    	(String)attribs.get(0));
                bUnderstood = true;
            }

            if (!bUnderstood) {
            	JOptionPane.showMessageDialog(m_parentFrame,
            		"Did not understand command: " + key,
                	"Script File Error", JOptionPane.ERROR_MESSAGE);
                	System.out.println("Did not understand command: " + key);
                break;
            }
        }

        // Close the file.
        br.close();
	}
}
