/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2002-10-15
 * Date started:    2002-01-09
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr.data;

import gov.usgs.gr.*;
import java.io.*;
import java.util.*;
import javax.xml.parsers.*;
import org.xml.sax.*;

//==============================================================================
/**
 * Reads an XML file with configuration information for GrObject classes.
 * Rather than storing instances of the objects described in the file, this
 * class stores a list of attributes for each class specified in the file.
 * The attributes can then be applied as default to new instances of those
 * classes.
 * @see gov.usgs.gr.ConfigSAXHandler
 */
public class ConfigXMLFile extends XMLProjectFile {

	/** The list of class names and associated properties. */
    GrHashList m_classProps = new GrHashList();

//------------------------------------------------------------------------------
/**
 * Creates an empty file object.
 */
    public ConfigXMLFile () {

    	super();
        m_title = "Config XML File";
    }

//------------------------------------------------------------------------------
/**
 * Reads the file from disk.
 * <p><b> Pre Conditions </b><br>
 *      The filename is not null.
 * @throws IOException
 */
    public void read () throws IOException {

        readWithSAX();
    }

//------------------------------------------------------------------------------
/**
 * Reads the file from disk using the SAX parser.  Uses ConfigSAXHandler
 * so that the classes described in the file will not be instantiated,
 * but rather stored as class names and default property values.
 * @throws IOException
 */
    public void readWithSAX () throws IOException {

    	ConfigSAXHandler csh = new ConfigSAXHandler();
        csh.read(m_file.getAbsolutePath());
        m_classProps = csh.getClassProps();
    }

//------------------------------------------------------------------------------
/**
 * Return the hashmap of class properties.
 * @return the hash map of classes and and their default values.
 */
    public GrHashList getClassProps () {

        return m_classProps;
    }
}



