/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2006-06-06
 * Date started:    1998-06-11
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr.data;

import gov.usgs.gr.*;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.event.*;
import java.io.*;
import java.text.*;
import java.util.*;
import javax.swing.*;
import java.awt.EventQueue;
import java.lang.reflect.InvocationTargetException;

//==============================================================================
/**
 * Read, stores, and writes data to/from files that stores data in columns
 * delimited by characters such as spaces or commas.
 */
public class DataSeriesCollection extends gov.usgs.gr.GrObject {

    /** The title of the data set. */
    protected String m_title = "";
    /**
     * A list of data series which which were read from or will be written
     * to the file.
     */
    public List m_seriesList = null;
    /** An array containing all the series' data arrays. */
    protected double[][] m_seriesArrays;
    /** The number of data columns in the file. */
    protected int m_numChannels = 1;
    /** The number of dimensions in each data series. */
    protected int m_numDimensions = 2;
    protected String[] m_commandTokens = null;
    protected double[][] m_columnArray = null;
    /** The tree icon to use for this class. */
	public static ArrayList TREE_ICON_HOLDER = new ArrayList(1);
    /** The location of the tree icon. */
    protected String m_iconFile = "GrObject/GrFile.gif";

//------------------------------------------------------------------------------
/**
 * Creates an empty data file.
 */
    public DataSeriesCollection () {

    	super();
        m_title = "Data Series Collection";
    }

//------------------------------------------------------------------------------
/**
 * Adds any type of object as an effigy of itself.  Instances of object effigy
 * are added as-is only if their depictProperties variable is false.  Effigies
 * are used to discourage editing objects or viewing their properties as they
 * appear within the file and to make it clear that what appears within the file
 * are links to objects elsewhere in the tree.
 * @param gro   The object to add.
 * @return  Whether the object was successfully added.
 */
    public boolean add(GrObject gro) {

        if (gro instanceof ObjectEffigy) {
        	if (((ObjectEffigy)gro).depictProperties == false) {
        		m_children.add(gro);
            }
        	else {
        		m_children.add(new ObjectEffigy(gro));
        	}
        }
        else {
        	m_children.add(new ObjectEffigy(gro));
        }

        return true;
    }

//------------------------------------------------------------------------------
/**
 * Provide the list of properties with the latest values.
 */
    public GrHashList getProperties() {

    	GrHashList props = super.getProperties();

        return props;
    }

//------------------------------------------------------------------------------
/**
 * Set the values of one or more of this object's properties.
 * @param props  A list of GrProperty objects specifying the new values for
 *               certain properties.
 */
    public void setProperties(HashMap props) {

		super.setProperties(props);

        GrProperty p;

        Iterator iter = props.keySet().iterator();
        while (iter.hasNext()) {
            String key = (String)iter.next();
            Object obj = props.get(key);
            if (obj instanceof GrProperty) {
                p = (GrProperty)obj;

            }
            if (obj instanceof String) {
                String value = (String)obj;

                setProperty(key, value);
            }
        }
    }

//------------------------------------------------------------------------------
/**
 * Set a single property.
 * @see gov.usgs.gr.data.DataDimension#getProperties getProperties() for a list
 * of accepted properties.
 * @param key   The property name.
 * @param value The new value for the property.
 */
    public void setProperty(String key, String value) {

		super.setProperty(key, value);

    }

//------------------------------------------------------------------------------
/**
 * Accessor method.
 * @return An ImageIcon to use when representing this object in a tree.
 */
    public javax.swing.ImageIcon getTreeIcon() {

        if (TREE_ICON_HOLDER.size() <= 0)
            setTreeIcon();

    	return (javax.swing.ImageIcon)TREE_ICON_HOLDER.get(0);
    }

//------------------------------------------------------------------------------
/**
 * Loads the tree icon from a file and stores it in the TREE_ICON_HOLDER.
 */
    protected javax.swing.ImageIcon setTreeIcon() {

        javax.swing.ImageIcon icon = loadTreeIcon(m_iconFile);

        if (icon == null)
            icon = super.setTreeIcon();

        TREE_ICON_HOLDER.clear();
        TREE_ICON_HOLDER.add(icon);

        return icon;
    }

//------------------------------------------------------------------------------
/**
 *
 */
    public void setSeriesInput(String[] commandTokens, double[][] columnArray) {

        m_commandTokens = commandTokens;;
        m_columnArray = columnArray;;
        m_seriesList = null;
    }

//------------------------------------------------------------------------------
/**
 * Creates a column list from the data series list.
 */
    public List getCurrentColumnList() {
/*
    /////// Lifted from the DelimitedDataFile writeData() method...

        // Find and compile a list of data arrays contained in child DataSeries.
        int numSeries = m_seriesList.size();
        double[][] seriesArrays = new double[numSeries][];
        DataDimension[][] seriesDims = new DataDimension[numSeries][];
        int[][] associatedX = new int[numSeries][];
        int[] numDims = new int[numSeries];
        int longest = 0;
        int index = 0;
        Iterator iter = m_seriesList.iterator();
        while (iter.hasNext()) {
            DoubleDataSeries series = (DoubleDataSeries)iter.next();

            if (series.getNumPoints() > longest)
                longest = series.getNumPoints();
            seriesArrays[numSeries] = series.getArray();
            numDims[numSeries] = series.getNumDimensions();
            seriesDims[index] = new DataDimension[numDims[numSeries]];
            associatedX[index] = new int[numDims[numSeries]];
            for (int i = 0; i < numDims[numSeries]; i++) {
                associatedX[index][i] = -1;
            }
            index++;
        }

        double[][] seriesArrays = new double[numSeries][];

        int[] xDims = new int[numSeries];
        Arrays.fill(xDims, -1);
        for (int i = 0; i < numSeries; i++) {
            for (int j = numDims[i] - 1; j >= 0; j--) {
                if (!seriesDims[i][j].getSorted()) {
                    xDims[i] = j;
                }
            }
            for (int j = 0; j < numDims[i]; j ++) {
                if (!seriesDims[i][j].getSorted()) {
                    associatedX[i][j] = xDims[i];
                }
            }
        }
 */
 return null;
    }

//------------------------------------------------------------------------------
/**
 * Generate the source column list by parsing an input string describing it.
 */
    public List getSeriesList() {

        if (m_seriesList != null)
            return m_seriesList;

        m_seriesList = new ArrayList();

        for (int i = 0; i < m_commandTokens.length; i++) {
            if (m_commandTokens[i].indexOf('d') >= 0) {
                int[][] columnIndexArray = determineColumnIndexes(
                    m_commandTokens[i], m_columnArray.length);
                DoubleDataSeries series =
                    new DoubleDataSeries(columnIndexArray.length);

                // Create the array for the series.
                int numDims = columnIndexArray.length;
                int numX = columnIndexArray[0].length;
                double[] newArray = new double[numDims * numX];

                // Fill the array values.
                for (int xIndex = 0; xIndex < numX; xIndex++) {
                    for (int dimIndex = 0; dimIndex < numDims; dimIndex++) {
                        int columnIndex = columnIndexArray[i][dimIndex] - 1;
                        newArray[xIndex * numDims + dimIndex] =
                            m_columnArray[columnIndex][xIndex];
                    }
                }

                // Make the array current.
                series.setArray(newArray);

                // Add the array to the list.
                m_seriesList.add(series);
            }
        }

        return m_seriesList;
    }

//------------------------------------------------------------------------------
/**
 * Parse an input string to determine which column indexes to use for creating
 * data series.
 */
    public static int[][] determineColumnIndexes (String inString,
        int numColumns) {

        int index1 = inString.indexOf('(');
        int index2 = inString.indexOf(')');
        if (index1 < 0 || index2 < 0)
            return null;

        String dsString = inString.substring(index1 + 1, index2);
        String sourceString = inString.substring(index2 + 1);

        List sourceColumnList =
            generateSourceColumnList(sourceString, numColumns);
        int[][] columnIndexArray =
            generateColumnIndexArray(dsString, sourceColumnList);

        return columnIndexArray;
    }

//------------------------------------------------------------------------------
/**
 * Parse an input string describing the column placement and use it to generate
 * the column index array for creating data series.
 */
    public static int[][] generateColumnIndexArray (String inString,
        List sourceColumnList) {

        int numSeries = sourceColumnList.size();
        int[][] columnIndexArray = new int[numSeries][];

        for (int seriesNum = 0; seriesNum < numSeries; seriesNum++) {
            // Replace the variable n with the integer number of columns.
            String explicitString = inString.replaceAll("n", "" + seriesNum);

            List columnIndexList = new ArrayList();

            // Parse the string and add each specified index to the list.
            String[] commaTokens =
                AsciiIO.parseLine(explicitString.trim(), ",");
            for (int i = 0; i < commaTokens.length; i++) {
                int[] rangeTokens = AsciiIO.parseLineAsInts(
                    commaTokens[i].trim(), ": ");

                // If it is a single value, add it.
                if (rangeTokens.length == 1) {
                    columnIndexList.add(rangeTokens[0]);
                }

                // If it is a range of values, add all of them, inclusive.
                if (rangeTokens.length == 2) {
                    for (int j = rangeTokens[0]; j <= rangeTokens[1]; j++) {
                        columnIndexList.add(j);
                    }
                }
            }

            // Create the column index array for this data series.
            columnIndexArray[seriesNum] = new int[columnIndexList.size()];
            int index = 0;

            // Add the integer indexes from the list to the array.
            Iterator iter = columnIndexList.iterator();
            while (iter.hasNext()) {
                columnIndexArray[seriesNum][index] = ((Integer)iter.next());
                index++;
            }
        }

        return columnIndexArray;
    }

//------------------------------------------------------------------------------
/**
 * Generate the source column list by parsing an input string describing it.
 */
    public static List generateSourceColumnList (String inString,
        int numColumns) {

        List sourceColumnList = new ArrayList();

        // Replace the variable n with the integer number of columns.
        inString = inString.replaceAll("n", "" + numColumns);

        String[] commaTokens = AsciiIO.parseLine(inString.trim(), ",");
        for (int i = 0; i < commaTokens.length; i++) {
            int[] rangeTokens = AsciiIO.parseLineAsInts(
                commaTokens[i].trim(), ": ");

            // If it is a single value, add it.
            if (rangeTokens.length == 1) {
                sourceColumnList.add(rangeTokens[0]);
            }

            // If it is a range of values, add all of them, inclusive.
            if (rangeTokens.length == 2) {
                for (int j = rangeTokens[0]; j <= rangeTokens[1]; j++) {
                    sourceColumnList.add(j);
                }
            }
        }

        return sourceColumnList;
    }

//------------------------------------------------------------------------------
/**
 * Adds up a string of numbers with + and - signs between them.
 */
    public int findSumTotal (String addSubString) {

        int total = 0;

        // Break the string up into tokens that should be added up.
        String[] plusTokens = AsciiIO.parseLine(addSubString.trim(), "+");
        for (int i = 0; i < plusTokens.length; i++) {
            // Break each token into subtokens that should be subtracted.
            int[] minusTokens = AsciiIO.parseLineAsInts(
                plusTokens[i].trim(), "- ");
            int subTotal = minusTokens[0];

            // Perform subtraction.
            for (int j = 1; j < minusTokens.length; j++) {
                subTotal -= minusTokens[j];
            }

            // Perform addition.
            total += subTotal;
        }

        return total;
    }

//------------------------------------------------------------------------------
/**
 * Removes the specified series from this file.
 * @param series	The series that was removed.  Null if it wasn't found.
 */
    public void removeSeries(DataSeries series) {

    	// Find the series' location in the list.
    	Iterator iter = m_seriesList.iterator();
        int index = 0;
        int location = -1;
        while (iter.hasNext()) {
        	DataSeries thisSeries = (DataSeries)iter.next();
            if (thisSeries == series) location = index;
            index++;
        }

        // Remove the series if it was found.
        if (location >= 0) {
        	m_seriesList.remove(location);
            m_numChannels--;
        }
    }

//------------------------------------------------------------------------------
/**
 * Adds the specified series to this file.
 * @param series	The series to add.
 */
    public void addSeries(DataSeries series) {

    	m_seriesList.add(series);
        m_numChannels++;
    }

//------------------------------------------------------------------------------
/**
 * Checks if this file contains the series.
 * @param series	The series to look for.
 * @return	True if the specified series is in this file.
 */
    public boolean containsSeries(DataSeries series) {

    	boolean bFound = false;

    	Iterator iter = m_seriesList.iterator();
        while (iter.hasNext()) {
        	DataSeries thisSeries = (DataSeries)iter.next();
            if (thisSeries == series) bFound = true;
        }
        return bFound;
    }

//------------------------------------------------------------------------------
/**
 * Accessor method.
 * @return The title of the data set.
 */
    public String getTitle() {
    	return m_title;
    }
}
