/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2006-06-06
 * Date started:    2000-08-22
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr.data;

import gov.usgs.gr.*;
import java.util.*;

//==============================================================================
/**
 * Describes a location in space.  An instance of this class can be attached
 * as a child to some other object.
 */
public class Location extends gov.usgs.gr.GrObject {

    /**
     * The XML element name that will identify this class and classes that
     * are extended from it.
     */
    static String XML_ELEMENT_NAME = "location";
    /** The X offset from the lat/long location. */
    protected double m_x = 0;
    /** The y offset from the lat/long location. */
    protected double m_y = 0;
    /** The Z offset from the lat/long location. */
    protected double m_z = 0;
    /** Another Location object that this one is measured from. */
    protected Location m_relativeTo = null;
    /** The name of this location. */
    protected String m_name = "";
    /** The descriptive title of this location. */
    protected String m_title = "";
    /** The code for this location. */
    protected String m_code = "";


//------------------------------------------------------------------------------
/**
 * Constructor for creating a blank object.
 */
    public Location() {

    }

//------------------------------------------------------------------------------
/**
 * Doesn't add any children.  Could be implemented to add times or lat/long.
 * @param gro   The object to add.
 * @return  Whether the object was added successfully.
 */
    public boolean add(GrObject gro) {

    	boolean bAdded = false;

        // If gro is an object effigy, get the eventual subject and try
        // adding that.
        while (gro instanceof gov.usgs.gr.ObjectEffigy)
        	gro = ((ObjectEffigy)gro).getSubject();

/*
        // Only add instances of Location or DateTime.
        if (gro instanceof gov.usgs.gr.data.Latitude ||
        	gro instanceof gov.usgs.gr.data.Longitude) {

        	// When adding objects, replace existing ones.
        	ListIterator iter = m_children.listIterator();
        	int i = 0;
        	while (iter.hasNext()) {
        		GrObject oldGro = (GrObject)iter.next();
                if (oldGro.getClass().isInstance(gro)) {
                    // First deregister this object as an observer of
                    // the child.  Then register as an observer of the new.
                    oldGro.deleteObserver(this);
                    gro.addObserver(this);
                    iter.set(gro);
                    m_bHasChanged = true;
                    bAdded = true;
                    break;
                }
            }

            if (!bAdded)
            	return super.add(gro);
        }
*/
    	return bAdded;
    }

//------------------------------------------------------------------------------
/**
 * Accessor method.
 * @return An XML name of this element.
 */
    public String getXmlElementName() {
    	return "location";
    }

//------------------------------------------------------------------------------
/**
 * Create a string representation of this object.
 * @return  The string that describes this object.
 */
    public String toString() {

        return ("Location");
    }

//------------------------------------------------------------------------------
/**
 * Provide the list of properties with the latest values.
 * Properties include: <p>
 * x Double "X Coordinate" <p>
 * y Double "Y Coordinate" <p>
 * z Double "Z Coordinate" <p>
 * @return A list of GrProperty objects describing this object's properties.
 */
    public GrHashList getProperties() {

    	GrHashList props = super.getProperties();

    	props.put("x", new GrProperty(m_x, "X Coordinate"));
    	props.put("y", new GrProperty(m_y, "Y Coordinate"));
    	props.put("z", new GrProperty(m_z, "Z Coordinate"));

        return props;
    }

//------------------------------------------------------------------------------
/**
 * Set the values of one or more of this object's properties.
 * @see gov.usgs.gr.data.Location#getProperties getProperties() for a list
 * of accepted properties.
 * @param props A list of GrProperty objects specifying the new values for
 *              certain properties.
 */
    public void setProperties(GrHashList props) {

    	GrProperty p;

        Iterator iter = props.keySet().iterator();
        while (iter.hasNext()) {
        	String key = (String)iter.next();
            Object obj = props.get(key);
            if (obj instanceof GrProperty) {
				p = (GrProperty)obj;

        		if (key.equals("x"))
                    m_x = (Double)p.value;
        		if (key.equals("y"))
                    m_y = (Double)p.value;
        		if (key.equals("z"))
                    m_z = (Double)p.value;
            }
            if (obj instanceof String) {
            	String value = (String)obj;

                setProperty(key, value);
            }
        }
    }

//------------------------------------------------------------------------------
/**
 * Set a single property.
 * @see gov.usgs.gr.data.Location#getProperties getProperties() for a list
 * of accepted properties.
 * @param key   The property name.
 * @param value The new value for the property.
 */
    public void setProperty(String key, String value) {

        if (key.equals("x"))
            m_x = Double.parseDouble(value);
        if (key.equals("y"))
            m_y = Double.parseDouble(value);
        if (key.equals("z"))
            m_z = Double.parseDouble(value);
    }
}
