/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2006-04-06
 * Date started:    2006-03-13
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr.data;

import gov.usgs.gr.*;
import java.util.*;

//==============================================================================
/**
 * A dimension within a DataSeries.
 */
public class SeriesMetaData extends gov.usgs.gr.GrObject {

    /** The tree icon to use for this class. */
	public static ArrayList TREE_ICON_HOLDER = new ArrayList(1);
    /** The location of the tree icon. */
    protected String m_iconFile = "GrObject/DataDimension.gif";

//------------------------------------------------------------------------------
/**
 * Constructor for creating a generic dimension.
 */
    public SeriesMetaData() {

    }

//------------------------------------------------------------------------------
/**
 * Provide the list of properties with the latest values.  Properties include:
 * <p>
 * @return A list of GrProperty objects describing this object's properties.
 */
    public GrHashList getProperties() {

        GrHashList props = super.getProperties();

        return props;
    }

//------------------------------------------------------------------------------
/**
 * Set the values of one or more of this object's properties.
 * @see gov.usgs.gr.data.SeriesMetaData#getProperties getProperties() for a list
 * of accepted properties.
 * @param props  A list of GrProperty objects specifying the new values for
 *               certain properties.
 */
    public void setProperties(HashMap props) {

        GrProperty p;

        Iterator iter = props.keySet().iterator();
        while (iter.hasNext()) {
            String key = (String)iter.next();
            Object obj = props.get(key);
            if (obj instanceof GrProperty) {
                p = (GrProperty)obj;

            }
            if (obj instanceof String) {
                String value = (String)obj;

                setProperty(key, value);
            }
        }
    }

//------------------------------------------------------------------------------
/**
 * Set a single property.
 * @see gov.usgs.gr.data.SeriesMetaData#getProperties getProperties() for a list
 * of accepted properties.
 * @param key   The property name.
 * @param value The new value for the property.
 */
    public void setProperty(String key, String value) {


    }

//------------------------------------------------------------------------------
/**
 * Create a string representation of this object.
 * @return  A string that describes this object.
 */
    public String toString() {

        return ("Series Meta Data");
    }

//------------------------------------------------------------------------------
/**
 * Accessor method.
 * @return An ImageIcon to use when representing this object in a tree.
 */
    public javax.swing.ImageIcon getTreeIcon() {

        if (TREE_ICON_HOLDER.size() <= 0)
            setTreeIcon();

    	return (javax.swing.ImageIcon)TREE_ICON_HOLDER.get(0);
    }

//------------------------------------------------------------------------------
/**
 * Loads the tree icon from a file and stores it in the TREE_ICON_HOLDER.
 */
    protected javax.swing.ImageIcon setTreeIcon() {

        javax.swing.ImageIcon icon = loadTreeIcon(m_iconFile);

        if (icon == null)
            icon = super.setTreeIcon();

        TREE_ICON_HOLDER.clear();
        TREE_ICON_HOLDER.add(icon);

        return icon;
    }

//------------------------------------------------------------------------------
/**
 * Overrides add in GrObject.  Prevents children from being added.
 * @param gro The object to add.
 * @return Whether the object was successfully added.  Always false.
 */
    public boolean add(GrObject gro) {

    	return false;
    }

//------------------------------------------------------------------------------
/**
 * Accessor method.
 * @return The DataSeries object that contains this dimension object.
 */
    public DataSeries getAssociatedSeries() {

        Iterator iter = m_observers.listIterator();
        while (iter.hasNext()) {
            GrObject gro = (GrObject)iter.next();
            if (gro instanceof DataSeries)
                return (DataSeries)gro;
        }

        return null;
    }
}
