/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2007-05-03
 * Date started:    2003-11-07
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr.tools;

import gov.usgs.gr.data.*;
import gov.usgs.gr.*;
import gov.usgs.gr.visual.*;
import gov.usgs.gr.visual.gl.*;
import java.awt.BorderLayout;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

//==============================================================================
/**
 *
 */
public class KeyFrameHandler extends gov.usgs.gr.GrObject
    implements gov.usgs.gr.visual.gui.GuiPanelProvider {


    /** An instance of the controller associated with this object. */
    GrController m_grCon = null;
    /** An instance of the page associated with this object. */
    PageOfGraphs m_page = null;
    protected JPanel m_panel = new JPanel();
    protected JButton m_addButton = new JButton("Add");
    protected JButton m_deleteButton = new JButton("Delete");
    protected JButton m_nextButton = new JButton("Next");
    protected JButton m_prevButton = new JButton("Prev");
    protected int m_currentIndex = -1;
    protected boolean m_bInterpolateBetweenFrames = false;
    protected boolean m_bParabolicInterpolation = true;

    protected DoubleDataSeries m_series = new DoubleDataSeries(0,
        "Time - ModelView - Projection Matrix", 33);

//------------------------------------------------------------------------------
	public KeyFrameHandler() {

        add(m_series);

        createPanel();
	}

//------------------------------------------------------------------------------
/**
 *
 */
    public boolean add(GrObject gro) {

        // If gro is an object effigy, get the eventual subject and try
        // adding that.
        while (gro instanceof gov.usgs.gr.ObjectEffigy)
            gro = ((ObjectEffigy)gro).getSubject();

        if (m_children.contains(gro)) return false;

        boolean bAdded = false;

        if (gro instanceof DataSeries) {
            if (((DataSeries)gro).getNumDimensions() == 33) {
                remove(m_series);
                m_series = (DoubleDataSeries)gro;
            }
            bAdded = true;
        }

        super.add(gro);

        return bAdded;
    }

//------------------------------------------------------------------------------
/**
 * Provide the list of properties with the current values for this object.
 * Properties include: <p>
 * interpolateBetweenFrames  Boolean  "Interpolate Between Frames"                                         <p>
 * parabolicInterpolation    Boolean  "Use Parabolic Interpolation"       <p>
 * @return  A list of GrProperty objects.
 */
    public GrHashList getProperties() {

        // First add properties from the super class.
        GrHashList props = super.getProperties();

        props.put("interpolateBetweenFrames", new GrProperty(
            m_bInterpolateBetweenFrames, "Interpolate Between Frames", false));
        props.put("parabolicInterpolation", new GrProperty(
            m_bParabolicInterpolation, "Use Parabolic Interpolation", false));

        return props;
    }

//------------------------------------------------------------------------------
/**
 * Set the values of one or more of this object's properties.
 * @see gov.usgs.gr.tools.KeyFrameHandler#getProperties getProperties() for a
 * list of accepted properties.
 * @param props A list of GrProperty objects specifying the new values for
 *              certain properties.
 */
    public void setProperties(HashMap props) {

        GrProperty p;

        // For each property with a string value, pass it to the setProperty()
        // method to be handled there.
        Iterator iter = props.keySet().iterator();
        while (iter.hasNext()) {
            String key = (String)iter.next();
            Object obj = props.get(key);
            if (obj instanceof GrProperty) {
                p = (GrProperty)obj;

                // Handle string properties by passing them to the
                // setProperty() method.
                if (p.value instanceof String) {
                    setProperty(key, (String)p.value);
                    continue;
                }

                if (key.equals("interpolateBetweenFrames"))
                    m_bInterpolateBetweenFrames = (Boolean)p.value;
                if (key.equals("parabolicInterpolation"))
                    m_bParabolicInterpolation = (Boolean)p.value;
            }
            // Handle string properties by passing them to the
            // setProperty() method.
            if (obj instanceof String) {
                setProperty(key, (String)obj);
            }
        }
    }

//------------------------------------------------------------------------------
/**
 * Set a single property.
 * @see gov.usgs.gr.data.DataSeries#getProperties getProperties() for a list
 * of accepted properties.
 * @param key   The property name.
 * @param value The new value for the property.
 */
    public void setProperty(String key, String value) {

        if (key.equals("interpolateBetweenFrames"))
            m_bInterpolateBetweenFrames = Boolean.parseBoolean(value);
        if (key.equals("parabolicInterpolation"))
            m_bParabolicInterpolation = Boolean.parseBoolean(value);
    }

//------------------------------------------------------------------------------
/**
 * Tells this interface what the current controller and page are.
 */
    public void setControllerAndPage(GrController grCon, PageOfGraphs page) {

        m_grCon = grCon;
        m_page = page;
    }

//------------------------------------------------------------------------------
    public JPanel getPanel() {

        return m_panel;
    }

//------------------------------------------------------------------------------
    protected void jbInit() throws Exception {
    }

//------------------------------------------------------------------------------
    public void createPanel() {


        java.awt.event.ActionListener keysAL =
            new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                if (e.getActionCommand().equals("Add")) {
                    // First look for the parent page of this object.
                    // If it is a Gl3dPage, continue.
                    if (m_page == null)
                        return;

                    Gl3dGraph graph = (Gl3dGraph)m_page.firstChildInstanceOf(
                            "gov.usgs.gr.visual.gl.Gl3dGraph", false);
                    if (graph == null)
                        return;

                    if (!graph.getShow())
                        return;

                    // Get the OpenGL modelview and projection matrices.
                    double[] modelMatrix =
                        graph.getModelMatrix();
                    double[] projMatrix =
                        graph.getProjectionMatrix();

                    // Don't proceed without the matrices.
                    if (modelMatrix == null || projMatrix == null)
                        return;

                    // Find the current time for the time marker.
                    TimeMarker tm = (TimeMarker)m_page.firstChildInstanceOf(
                        "gov.usgs.gr.tools.TimeMarker", false);
                    double time = 0;
                    if (tm != null)
                        time = (double)tm.getCurrentValue();

                    // Look for the index where the new time should be
                    // inserted to keep the array sorted by time.
                    int index = m_series.search(time, 0);
                    double[] oldArray = m_series.getArray();
                    int numDims = m_series.getNumDimensions();
                    int newCount = m_series.getCount() + 1;
                    double[] newArray = new double[newCount * numDims];

                    // Copy the existing values up to the insertion point
                    // from the old array to the new one.
                    for (int i = 0; i <= index; i++) {
                        for (int j = 0; j < numDims; j++) {
                            newArray[i * numDims + j] =
                               oldArray[i * numDims + j];
                        }
                    }

                    // Insert the modelview matrix into the dataseries.
                    newArray[(index + 1) * numDims] = time;
                    for (int j = 1; j < 17; j++) {
                        newArray[(index + 1) * numDims + j]
                            = (double)modelMatrix[j - 1];
                        System.out.print("" + modelMatrix[j - 1] + " ");
                    }
                    System.out.println("");

                    // Insert the projection matrix into the dataseries.
                    for (int j = 17; j < 33; j++) {
                        newArray[(index + 1) * numDims + j]
                            = (double)projMatrix[j - 17];
                        System.out.print(""+ projMatrix[j - 17] + " ");
                    }
                    System.out.println("");

                    m_currentIndex = index + 1;

                    for (int i = (index + 2); i < newCount; i++) {
                        for (int j = 0; j < numDims; j++) {
                            newArray[i * numDims + j] =
                               oldArray[i * numDims + j];
                        }
                    }
                    m_series.setArray(newArray);
                }
                if (e.getActionCommand().equals("Delete")) {
                }
                if (e.getActionCommand().equals("Next")) {
                    // First look for the parent page of this object.
                    // If it is a Gl3dPage, continue.
                    if (m_page == null)
                        return;

                    Gl3dGraph graph = (Gl3dGraph)m_page.firstChildInstanceOf(
                            "gov.usgs.gr.visual.gl.Gl3dGraph", false);
                    if (graph == null)
                        return;

                    if (!graph.getShow())
                        return;

                    // Find the dataseries index from which to retrieve
                    // the matrices.
                    int numDims = m_series.getNumDimensions();
                    int seriesCount = m_series.getCount();
                    if (seriesCount > 0) {
                        m_currentIndex++;
                        if (m_currentIndex < 0 ||
                            m_currentIndex >= seriesCount) {

                            m_currentIndex = 0;
                        }

                        // Create empty arrays to store the numbers from
                        // the data series.
                        double[] modelMatrix = new double[16];
                        double[] projMatrix = new double[16];
                        double[] seriesArray = m_series.getArray();

                        // Fill the modelview matrix.
                        System.out.print("" + seriesArray[
                            m_currentIndex * numDims] + " ");
                        for (int j = 1; j < 17; j++) {
                            modelMatrix[j - 1]
                                = seriesArray[m_currentIndex * numDims + j];
                            System.out.print("" + modelMatrix[j - 1] + " ");
                        }
                        System.out.println("");

                        // Fill the projection matrix.
                        System.out.print("PROJ  ");
                        for (int j = 17; j < 33; j++) {
                            projMatrix[j - 17]
                                = seriesArray[m_currentIndex * numDims + j];
                            System.out.print(""+ projMatrix[j - 17] + " ");
                        }
                        System.out.println("");

                        // Pass the matrices to the page.
                        graph.setViewMatrices(modelMatrix,
                                projMatrix);
                    }
                }
                if (e.getActionCommand().equals("Prev")) {
                }
            }
        };


        m_panel.setBorder(BorderFactory.createTitledBorder("Key Frames"));
        BorderLayout mainLayout = new BorderLayout();
        m_panel.setLayout(new BoxLayout(m_panel, BoxLayout.Y_AXIS));

        m_addButton.addActionListener(keysAL);
        m_addButton.setActionCommand("Add");
        m_addButton.setToolTipText("Add");
        byte[] byteArray = {0};
        m_addButton.setIcon(new ImageIcon(byteArray));
        m_addButton.setBorderPainted(true);
        m_panel.add(m_addButton);
        m_deleteButton.addActionListener(keysAL);
        m_deleteButton.setActionCommand("Delete");
        m_deleteButton.setToolTipText("Delete");
        m_deleteButton.setIcon(new ImageIcon(byteArray));
        m_deleteButton.setBorderPainted(true);
        m_panel.add(m_deleteButton);
        m_nextButton.addActionListener(keysAL);
        m_nextButton.setActionCommand("Next");
        m_nextButton.setToolTipText("Show Next Key Frame");
//        m_nextButton.setIcon(new ImageIcon(byteArray));
        m_nextButton.setBorderPainted(true);
        m_panel.add(m_nextButton);
        m_prevButton.addActionListener(keysAL);
        m_prevButton.setActionCommand("Prev");
        m_prevButton.setToolTipText("Show Previous Key Frame");
//        m_prevButton.setIcon(new ImageIcon(byteArray));
        m_prevButton.setBorderPainted(true);
        m_panel.add(m_prevButton);
    }

//------------------------------------------------------------------------------
/**
 * Overrides the Observer update() in GrObject.
 */
    public void updateTime(double time) {

        if (m_bInterpolateBetweenFrames) {
            // First look for the parent page of this object.
            // If it is a Gl3dPage, continue.
            if (m_page == null)
                return;

            Gl3dGraph graph = (Gl3dGraph)m_page.firstChildInstanceOf(
                    "gov.usgs.gr.visual.gl.Gl3dGraph", false);
            if (graph == null)
                return;

            if (!graph.getShow())
                return;

            // Find the dataseries index from which to retrieve
            // the matrices.
            int numDims = m_series.getNumDimensions();
            int seriesCount = m_series.getCount();
            if (seriesCount > 0) {
                m_currentIndex++;
                if (m_currentIndex < 0 ||
                    m_currentIndex >= seriesCount) {

                    m_currentIndex = 0;
                }

                // Create empty arrays to store the numbers from
                // the data series.
                double[] modelMatrix = new double[16];
                double[] projMatrix = new double[16];

                m_series.m_bParabolicInterpolation =
                    m_bParabolicInterpolation;

                // Fill the modelview matrix.
//                    System.out.print("I: " + time + " ");
                for (int j = 1; j < 17; j++) {
                    modelMatrix[j - 1]
                        = m_series.getInterpolatedValue(time, 0, j);
//                        System.out.print("" + modelMatrix[j - 1] + " ");
                }
//                    System.out.println("");

                // Fill the projection matrix.
//                    System.out.print("I: PROJ  ");
                for (int j = 17; j < 33; j++) {
                    projMatrix[j - 17]
                        = m_series.getInterpolatedValue(time, 0, j);
//                        System.out.print(""+ projMatrix[j - 17] + " ");
                }
//                    System.out.println("");

                // Pass the matrices to the page.
                graph.setViewMatrices(modelMatrix,
                        projMatrix);
            }
        }
    }

//------------------------------------------------------------------------------
/**
 * Overrides default update() in GrObject.  Called by an observed GrObject's
 * notifyObservers() method to handle changes to the observed object.
 * May use the notifyObservers() method to propagate the event handling up to
 * this object's observers.
 * @param gro   The observed object that changed.
 * @param arg   Additional information that the updated object may choose to
 *              send.
 */
    public void update(GrObject gro, Object arg) {

        // Handle animation from the time marker.
        if (gro instanceof DataSeries && gro != m_series) {
            // Find the page's time marker.
            if (m_page == null)
                return;

            TimeMarker tm = (TimeMarker)m_page.firstChildInstanceOf(
                "gov.usgs.gr.tools.TimeMarker", false);
            if (tm == null)
                return;

            double time = (double)tm.getCurrentValue();

            updateTime(time);
        }

        m_bHasChanged = true;
        notifyObservers();
    }

//------------------------------------------------------------------------------
/**
 * Create a string representation of this object.
 * @return  The string that describes this object.
 */
    public String toString() {

        String s = "Key Frames";

        return s;
     }
}

