/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2007-03-13
 * Date started:    2003-10-28
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr.tools;

import gov.usgs.gr.*;
import gov.usgs.gr.data.*;
import gov.usgs.gr.tools.*;
import gov.usgs.gr.visual.*;
import gov.usgs.gr.visual.gl.*;
import gov.usgs.sfhydro.si3d.*;
import java.awt.*;
import java.util.*;

//==============================================================================
/**
 * Performs work on ThreadWorker objects using a multiple threads.
 */
public class MultiThreader implements Runnable {

    protected ThreadWorker m_threadWorker = null;
    protected int m_numThreadsFinished = 0;
    protected int m_numThreads = 1;
    protected String m_description = "";
    protected boolean m_bOomeOccurred = false;

//------------------------------------------------------------------------------
/**
 * 
 */
    public MultiThreader (ThreadWorker threadWorker, int numThreads,
        String description) {

        m_threadWorker = threadWorker;
        m_numThreads = numThreads;
        if (description != null)
            m_description = description;
    }

//------------------------------------------------------------------------------
/**
 * Start the threads.
 */
    public long runThreads() {

        long startTimeMillis = System.currentTimeMillis();

        m_numThreadsFinished = 0;
//System.out.println(Thread.currentThread().getName() + "   "
//    + Thread.currentThread().getPriority());
        m_bOomeOccurred = false;
        for (int threadNum = 0; threadNum < m_numThreads; threadNum++) {
            Thread t1 = new Thread(this, m_description + "-" + (threadNum + 1));
//System.out.println(t1.getName() + "   " + t1.getPriority());
            t1.start();
        }

        boolean bDone = false;
        do {
            synchronized (this) {
                if (m_numThreadsFinished == m_numThreads)
                    bDone = true;

                if (!bDone) {
                    try {
                        wait(10000);
                    }
                    catch (InterruptedException ie) {
                    }
                }
            }
        } while (!bDone);

        long endTimeMillis = System.currentTimeMillis();

        long elapsedTimeMillis = endTimeMillis - startTimeMillis;

        return elapsedTimeMillis;
    }

//------------------------------------------------------------------------------
/**
 * The method that is called by a newly created thread that is set to run
 * this object.
 */
    public void run () {
        
        try {

            while(m_threadWorker.performThreadWork());

            synchronized(this) {
                m_numThreadsFinished++;
                notify();
            }
        } catch (OutOfMemoryError e) {
            m_bOomeOccurred = true;
//            System.err.println("\noome in MultiThreader:");
//            e.printStackTrace();
            synchronized(this) {
                m_numThreadsFinished++;
                notify();
            }
        }
    }
    
//------------------------------------------------------------------------------
/**
 */
    public boolean getOomeOccurred () {

        return m_bOomeOccurred;
    }
}






