/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2008-10-14
 * Date started:    1999-05-19
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr.visual;

//==============================================================================

import gov.usgs.gr.GrHashList;
import java.io.PrintWriter;
import java.util.ArrayList;

/**
 * An x-y graph with ticks, labels, a grid, and multiple curves drawn inside.
 * This class and the components withing in it should be extended for drawing
 * to a particular output device.
 */
public class LineMarkup extends gov.usgs.gr.visual.Markup {
    /** The x coordinate of this mark up. */
    public double x2 = 0;
    /** The y coordinate of this mark up. */
    public double y2 = 0;
    /** The tree icon to use for this class. */
	public static ArrayList TREE_ICON_HOLDER = new ArrayList(1);
    /** The location of the tree icon. */
    protected String m_iconFile = "GrObject/LineMarkup.gif";

//------------------------------------------------------------------------------
/**
 * Construct an empty markup.
 *
 */
    public LineMarkup () {
        m_xCoordinateLevel = 2;
        m_yCoordinateLevel = 2;

        x = 0.3f;
        y = 0;
        x2 = 0.3f;
        y2 = 1;

        m_bUserChangeable = false;
    }

//------------------------------------------------------------------------------
/**
 * Provide the list of properties with the latest values.
 */
    public GrHashList getProperties() {

    	GrHashList props = super.getProperties();

        props.remove("title");
        props.remove("showTitle");

        return props;
    }

//------------------------------------------------------------------------------
/**
 * Accessor method.
 * @return An ImageIcon to use when representing this object in a tree.
 */
    public javax.swing.ImageIcon getTreeIcon() {

        if (TREE_ICON_HOLDER.size() <= 0)
            setTreeIcon();

    	return (javax.swing.ImageIcon)TREE_ICON_HOLDER.get(0);
    }

//------------------------------------------------------------------------------
/**
 * Loads the tree icon from a file and stores it in the TREE_ICON_HOLDER.
 */
    protected javax.swing.ImageIcon setTreeIcon() {

        javax.swing.ImageIcon icon = loadTreeIcon(m_iconFile);

        if (icon == null)
            icon = super.setTreeIcon();

        TREE_ICON_HOLDER.clear();
        TREE_ICON_HOLDER.add(icon);

        return icon;
    }

//------------------------------------------------------------------------------
/**
 * Should draw this mark up to an OpenGL context.
 */
    public void drawGL() {

        // Before drawing, check if this object is supposed to be shown.
        if (!m_bShow) return;

        	// Save the current position
        	gl.glPushMatrix();
            gl.glPushAttrib(gl.GL_LINE_BIT | gl.GL_CURRENT_BIT);
            gl.glColor3f(1, 1, 1);
            gl.glLineWidth(2);

            gl.glBegin(gl.GL_LINES);
                gl.glVertex2d(transformX(x), transformY(y));
                gl.glVertex2d(transformX(x2), transformY(y2));
            gl.glEnd();

            gl.glPopAttrib();
        	// Restore the previous position.
        	gl.glPopMatrix();
    }

//------------------------------------------------------------------------------
/**
 * Should draw this mark up to a PostScript file..
 */
    public void drawPS(PrintWriter ps) {

    }

//------------------------------------------------------------------------------
/**
 * Create a string representation of this object.
 * @return  The string that describes this object.
 */
    public String toString() {

    	String s = "Line Markup";

        return s;
    }
}




