/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2003-04-13
 * Date started:    2003-04-13
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr.visual;

import java.awt.geom.*;
import java.util.*;


//==============================================================================
/**
 * An interface for graphical elements that need to have portion selected.
 */
public interface SelectableGE {


//------------------------------------------------------------------------------
/**
 * Selects or deselects all points within the zoom window.
 * @param selectRectMin	The lower left corner of the rectangle describing the
 *                      selected area.
 * @param selectRectMax	The upper right corner of the rectangle describing the
 *                      selected area.
 * @param bAdding	True if selecting points, false if deselecting.
 * @param bExclusive	True if only one curve should be selected or,
 *						if multiple curves are already selected, only
 *						the selected curves can have more points selected.
 * @param bDeselectFirst	True if old selection is deselected before selecting
 *							new points.
 * @return	An unmodifiable list of selected data series.
 */
    public List selectWindow(Point2D.Double selectRectMin,
        Point2D.Double selectRectMax, boolean bAdding,
        boolean bExclusive, boolean bDeselectFirst);
}

