/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2006-04-06
 * Date started:    1999-05-19
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr.visual;

import com.sun.opengl.util.*;
import gov.usgs.gr.*;
import java.awt.*;
import java.awt.geom.*;
import java.io.*;
import java.util.*;
import javax.media.opengl.*;
import javax.media.opengl.glu.*;

//==============================================================================
/**
 * An x-y graph with ticks, labels, a grid, and multiple curves drawn inside.
 * This class and the components withing in it should be extended for drawing
 * to a particular output device.
 */
public class TextMarkup extends Markup {
	/** The text to be printed on the page. */
    protected String m_text = "";
    /** The default font index into the m_fontChoices array. */
    protected int m_fontIndex = GLUT.BITMAP_HELVETICA_10;
    /** The array of class names for GLUT bitmap fonts. */
    protected String[] m_fontChoices = {"glutBitmap8By13", "glutBitmap9By15",
        "glutBitmapHelvetica10", "glutBitmapHelvetica12",
        "glutBitmapHelvetica18", "glutBitmapTimesRoman10",
        "glutBitmapTimesRoman24"};
	/** The font to use when drawing to a PostScript file. */
    protected String m_psFont = "Helvetica-Bold";
	/** The size of the PostScript font. */
    protected float m_psFontSize = 8;
    /** The color to use when drawing. */
    protected float[] m_color = {1, 1, 1};
    /** An x offset that is applied before drawing the text in PostScript. */
    public float xPointOffset = 0;
    /** An y offset that is applied before drawing the text in PostScript. */
    public float yPointOffset = 0;
    /** An x offset that is applied before drawing the text on screen. */
    public float xScreenOffset = 0;
    /** An y offset that is applied before drawing the text on screen. */
    public float yScreenOffset = 0;
    /**
     * The justification of the string.  0 = left, 1 = center, 2 = right.
     * This currently only works for PostScript output.
     */
    public int xJustification = 0;
    /** The tree icon to use for this class. */
	public static ArrayList TREE_ICON_HOLDER = new ArrayList(1);
    /** The location of the tree icon. */
    protected String m_iconFile = "GrObject/TextMarkup.gif";

//------------------------------------------------------------------------------
/**
 * Construct an empty markup.
 *
 */
    public TextMarkup () {

        setDefaults("gov.usgs.gr.visual.TextMarkup");
    }

//------------------------------------------------------------------------------
/**
 * Provide the list of properties with the latest values.
 */
    public GrHashList getProperties() {

    	GrHashList props = super.getProperties();

        props.remove("title");
        props.remove("showTitle");

    	props.put("text", new GrProperty(new String(m_text),
        	"Text", m_bUserChangeable, m_bUserChangeable, m_bUserChangeable));

        // Give the user some line type choices...
        GrProperty p = new GrProperty("", "Screen Font");
        javax.swing.JComboBox jcb = new javax.swing.JComboBox();

        // Fill the screen font combo box.
        for (int i = 0; i < m_fontChoices.length; i++) {
        	jcb.addItem(m_fontChoices[i]);
        }

        int comboIndex = 0;
        switch (m_fontIndex) {
            case GLUT.BITMAP_8_BY_13:
                comboIndex = 0;
                break;
            case GLUT.BITMAP_9_BY_15:
                comboIndex = 1;
                break;
            case GLUT.BITMAP_HELVETICA_10:
                comboIndex = 2;
                break;
            case GLUT.BITMAP_HELVETICA_12:
                comboIndex = 3;
                break;
            case GLUT.BITMAP_HELVETICA_18:
                comboIndex = 4;
                break;
            case GLUT.BITMAP_TIMES_ROMAN_10:
                comboIndex = 5;
                break;
            case GLUT.BITMAP_TIMES_ROMAN_24:
                comboIndex = 6;
                break;
        }

        jcb.setSelectedIndex(comboIndex);
        p.value = m_fontChoices[comboIndex];
        if (comboIndex == 2) p.save = false;

        p.component = jcb;
		props.put("screenFont", p);

        if (m_bUserChangeable) {
            ColorProperty cp = new ColorProperty();
            cp.description = "Color";
            cp.setColor(m_color);
            props.put("color", cp);
        }

    	props.put("psFont", new GrProperty(new String(m_psFont),
        	"PS Font", false, false, false));
    	props.put("psFontSize", new GrProperty(m_psFontSize,
        	"PS Font Size", false, false, false));
    	props.put("xPointOffset", new GrProperty(xPointOffset,
        	"X Point Offset", false, false, false));
    	props.put("yPointOffset", new GrProperty(yPointOffset,
        	"Y Point Offset", false, false, false));
    	props.put("xScreenOffset", new GrProperty(xScreenOffset,
        	"X Screen Offset", false, false, false));
    	props.put("yScreenOffset", new GrProperty(yScreenOffset,
        	"Y Screen Offset", false, false, false));
    	props.put("xJustification", new GrProperty(xJustification,
        	"X Justification", false, false, false));

        return props;
    }

//------------------------------------------------------------------------------
/**
 * Provide the list of properties with the latest values.
 */
    public void setProperties(HashMap props) {

		super.setProperties(props);

    	GrProperty p;

        Iterator iter = props.keySet().iterator();
        while (iter.hasNext()) {
        	String key = (String)iter.next();
            Object obj = props.get(key);

            if (obj instanceof GrProperty) {
				p = (GrProperty)obj;

            	if (key.equals("text"))
                    m_text = (String)p.value;

            	if (key.equals("screenFont"))
                    setProperty(key, (String)p.value);
            	if (key.equals("color")) {
                    ColorProperty cp = (ColorProperty)p;
                    m_color = cp.getColorArray();
                }
            	if (key.equals("psFont"))
                    m_psFont = (String)p.value;
            	if (key.equals("psFontSize"))
                    m_psFontSize = (Float)p.value;
            	if (key.equals("xPointOffset"))
                    xPointOffset = (Float)p.value;
            	if (key.equals("yPointOffset"))
                    yPointOffset = (Float)p.value;
            	if (key.equals("xScreenOffset"))
                    xScreenOffset = (Float)p.value;
            	if (key.equals("yScreenOffset"))
                    yScreenOffset = (Float)p.value;
            	if (key.equals("xJustification"))
                    xJustification = (Integer)p.value;
            }
            if (obj instanceof String) {
            	String value = (String)obj;

                setProperty(key, value);
			}
        }

        m_bNeedsRedraw = true;
        m_bHasChanged = true;
        notifyObservers();
    }

//------------------------------------------------------------------------------
/**
 * Set a single property.
 */
    public void setProperty(String key, String value) {

    	super.setProperty(key, value);

        if (key.equals("text"))
            m_text = value;
        if (key.equals("screenFont")) {
            // Check each name.
            for (int i = 0; i < m_fontChoices.length; i++) {
                if (value.equals(m_fontChoices[i])) {
                    switch (i) {
                        case 0:
                            m_fontIndex = GLUT.BITMAP_8_BY_13;
                            break;
                        case 1:
                            m_fontIndex = GLUT.BITMAP_9_BY_15;
                            break;
                        case 2:
                            m_fontIndex = GLUT.BITMAP_HELVETICA_10;
                            break;
                        case 3:
                            m_fontIndex = GLUT.BITMAP_HELVETICA_12;
                            break;
                        case 4:
                            m_fontIndex = GLUT.BITMAP_HELVETICA_18;
                            break;
                        case 5:
                            m_fontIndex = GLUT.BITMAP_TIMES_ROMAN_10;
                            break;
                        case 6:
                            m_fontIndex = GLUT.BITMAP_TIMES_ROMAN_24;
                            break;
                    }
                    break;
                }
            }
        }
        if (key.equals("color")) {
            m_color = ColorProperty.getColorArray(value);
        }
        if (key.equals("psFont"))
            m_psFont = value;
        if (key.equals("psFontSize"))
            m_psFontSize = Float.parseFloat(value);
        if (key.equals("xPointOffset"))
            xPointOffset = Float.parseFloat(value);
        if (key.equals("yPointOffset"))
            yPointOffset = Float.parseFloat(value);
        if (key.equals("xScreenOffset"))
            xScreenOffset = Float.parseFloat(value);
        if (key.equals("yScreenOffset"))
            yScreenOffset = Float.parseFloat(value);
        if (key.equals("xJustification"))
            xJustification = Integer.parseInt(value);

        m_bNeedsRedraw = true;
    }

//------------------------------------------------------------------------------
/**
 * Should draw this mark up to an OpenGL context.
 */
    public void drawGL() {

        // Before drawing anything, check if the curve is supposed to be shown.
        if (!m_bShow) return;

        // Save the current position
        gl.glPushMatrix();
        gl.glPushAttrib(gl.GL_LINE_BIT | gl.GL_CURRENT_BIT);
        gl.glColor3fv(m_color, 0);

        gl.glRasterPos2d(transformX(x) + xScreenOffset,
            transformY(y) + yScreenOffset);
//            m_glFont.drawString(m_text);
        glut.glutBitmapString(m_fontIndex, m_text);

        gl.glPopAttrib();
        // Restore the previous position.
        gl.glPopMatrix();
    }

//------------------------------------------------------------------------------
/**
 * Should draw this mark up to a PostScript file..
 */
    public void drawPS(PrintWriter ps) {

        // Set the font.
		ps.println("\n% Set the font for an annotation");
		ps.println("/" + m_psFont + " findfont " + m_psFontSize
        	+ " scalefont setfont");

        String justification = "leftshow";
        if (xJustification == 1) justification = "centershow";
        if (xJustification == 2) justification = "rightshow";

        // Locate and draw the text.
        String text = m_text.replaceAll("\\(", "\\\\050")
            .replaceAll("\\)", "\\\\051");
        ps.println("(" + text + ") " + (transformX(x) + xPointOffset) + " "
        	+ (transformY(y) + yPointOffset) + " " + justification);
    }

//------------------------------------------------------------------------------
/**
 * Attribute modification method.
 * @param text	The new text to be drawn.
 */
    public void setText(String text) {
    	m_text = text;
    }

//------------------------------------------------------------------------------
/**
 * Attribute modification method.
 * @param font	The PostScript font to use for printed output.
 */
    public void setPsFont(String font) {
    	m_psFont = font;
    }

//------------------------------------------------------------------------------
/**
 * Attribute modification method.
 * @param size	The size of the PostScript font.
 */
    public void setPsFontSize(float size) {
    	m_psFontSize = size;
    }

//------------------------------------------------------------------------------
/**
 * Create a string representation of this object.
 * @return  The string that describes this object.
 */
    public String toString() {

        String s = null;
        if (m_bShow)
            s = "Text Markup:  " + m_text;
        else
            s = "(Text Markup):  " + m_text;

        return s;
    }

//------------------------------------------------------------------------------
/**
 * Accessor method.
 * @return An ImageIcon to use when representing this object in a tree.
 */
    public javax.swing.ImageIcon getTreeIcon() {

        if (TREE_ICON_HOLDER.size() <= 0)
            setTreeIcon();

    	return (javax.swing.ImageIcon)TREE_ICON_HOLDER.get(0);
    }

//------------------------------------------------------------------------------
/**
 * Loads the tree icon from a file and stores it in the TREE_ICON_HOLDER.
 */
    protected javax.swing.ImageIcon setTreeIcon() {

        javax.swing.ImageIcon icon = loadTreeIcon(m_iconFile);

        if (icon == null)
            icon = super.setTreeIcon();

        TREE_ICON_HOLDER.clear();
        TREE_ICON_HOLDER.add(icon);

        return icon;
    }
}




