package gov.usgs.gr.visual;

/*
 * (c) Copyright 1993, 1994, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED
 * Permission to use, copy, modify, and distribute this software for
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission.
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * US Government Users Restricted Rights
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor
 * clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the
 * United States.  Contractor/manufacturer is Silicon Graphics,
 * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
 *
 * OpenGL(TM) is a trademark of Silicon Graphics, Inc.
 */
/*
 * Trackball code:
 *
 * Implementation of a virtual trackball.
 * Implemented by Gavin Bell, lots of ideas from Thant Tessman and
 *   the August '88 issue of Siggraph's "Computer Graphics," pp. 121-129.
 *
 * Vector manip code:
 *
 * Original code from:
 * David M. Ciemiewicz, Mark Grossman, Henry Moreton, and Paul Haeberli
 *
 * Much mucking with by:
 * Gavin Bell
 *
 * Converted to Java by John M. Donovan, U.S. Geological Survey 2001-08-14
 */

public class Trackball {

    /*
     * This size should really be based on the distance from the center of
     * rotation to the point on the object underneath the mouse.  That
     * point would then track the mouse as closely as possible.  This is a
     * simple example, though, so that is left as an Exercise for the
     * Programmer.
     */
//    protected static float TRACKBALLSIZE = 1;
    protected double m_trackBallSize = 1;

    /*
     * Given two rotations, e1 and e2, expressed as quaternion rotations,
     * figure out the equivalent single rotation and stuff it into dest.
     *
     * This routine also normalizes the result every RENORMCOUNT times it is
     * called, to keep error from creeping in.
     *
     * NOTE: This routine is written so that q1 or q2 may be the same
     * as dest (or each other).
     */

    protected static int RENORMCOUNT = 97;

    public double[] m_currentQuaternion = new double[4];
    public double[] m_stepQuaternion = new double[4];
    protected boolean m_bMoved = false;

//------------------------------------------------------------------------------
/**
 * Initialize the object.
 *
 */
    public Trackball () {

        trackball(m_currentQuaternion, 0, 0, 0, 0);
        trackball(m_stepQuaternion, 0, 0, 0, 0);
    }

//------------------------------------------------------------------------------
    public void vzero(double[] v) {

        v[0] = 0;
        v[1] = 0;
        v[2] = 0;
    }

//------------------------------------------------------------------------------
    public void vset(double[] v, double x, double y, double z) {

        v[0] = x;
        v[1] = y;
        v[2] = z;
    }

//------------------------------------------------------------------------------
    public void vsub(double[] src1, double[] src2, double[] dst) {

        dst[0] = src1[0] - src2[0];
        dst[1] = src1[1] - src2[1];
        dst[2] = src1[2] - src2[2];
    }

//------------------------------------------------------------------------------
    public void vcopy(double[] v1, double[] v2) {

        for (int i = 0 ; i < 3 ; i++)
            v2[i] = v1[i];
    }

//------------------------------------------------------------------------------
    public void vcross(double[] v1, double[] v2, double[] cross) {

        double[] temp = new double[3];

        temp[0] = (v1[1] * v2[2]) - (v1[2] * v2[1]);
        temp[1] = (v1[2] * v2[0]) - (v1[0] * v2[2]);
        temp[2] = (v1[0] * v2[1]) - (v1[1] * v2[0]);
        vcopy(temp, cross);
    }

//------------------------------------------------------------------------------
    public double vlength(double[] v) {

        double d = Math.sqrt(v[0] * v[0] + v[1] * v[1] + v[2] * v[2]);
        return d;
    }

//------------------------------------------------------------------------------
    public void vscale(double[] v, double div) {

        v[0] *= div;
        v[1] *= div;
        v[2] *= div;
    }

//------------------------------------------------------------------------------
    public void vnormal(double[] v) {

        vscale(v, 1 / vlength(v));
    }

//------------------------------------------------------------------------------
    public double vdot(double[] v1, double[] v2) {

        return v1[0]*v2[0] + v1[1]*v2[1] + v1[2]*v2[2];
    }

//------------------------------------------------------------------------------
    public void vadd(double[] src1, double[] src2, double[] dst) {

        dst[0] = src1[0] + src2[0];
        dst[1] = src1[1] + src2[1];
        dst[2] = src1[2] + src2[2];
    }

//------------------------------------------------------------------------------
/*
 * Ok, simulate a track-ball.  Project the points onto the virtual
 * trackball, then figure out the axis of rotation, which is the cross
 * product of P1 P2 and O P1 (O is the center of the ball, 0,0,0)
 * Note:  This is a deformed trackball-- is a trackball in the center,
 * but is deformed into a hyperbolic sheet of rotation away from the
 * center.  This particular function was chosen after trying out
 * several variations.
 *
 * It is assumed that the arguments to this routine are in the range
 * (-1.0 ... 1.0)
 */
//------------------------------------------------------------------------------
    public void trackball(double[] q, double p1x, double p1y, double p2x,
        double p2y) {

        double[] a = new double[3]; /* Axis of rotation */
        double phi;  /* how much to rotate about axis */
        double[] p1 = new double[3];
        double[] p2 = new double[3];
        double[]  d = new double[3];
        double t;

        if (p1x == p2x && p1y == p2y) {
            /* Zero rotation */
            vzero(q);
            q[3] = 1;
            return;
        }

        // First, figure out z-coordinates for projection of P1 and P2 to
        // deformed sphere.
        vset(p1,p1x,p1y,tb_project_to_sphere(m_trackBallSize,p1x,p1y));
        vset(p2,p2x,p2y,tb_project_to_sphere(m_trackBallSize,p2x,p2y));

        //
        // Now, we want the cross product of P1 and P2
        vcross(p2, p1, a);

        // Figure out how much to rotate around that axis.
        vsub(p1, p2, d);
        t = vlength(d) / (2 * m_trackBallSize);

        // Avoid problems with out-of-control values...
        if (t > 1) t = 1;
        if (t < -1) t = -1;
        phi = 2 * Math.asin(t);

        axis_to_quat(a,phi,q);
    }

//------------------------------------------------------------------------------
/*
 * Given an axis and angle, compute quaternion.
 */
    public void axis_to_quat(double[] a, double phi, double[] q) {

        vnormal(a);
        vcopy(a,q);
        vscale(q, Math.sin(phi / 2));
        q[3] = Math.cos(phi / 2);
    }

//------------------------------------------------------------------------------
/*
 * Project an x,y pair onto a sphere of radius r OR a hyperbolic sheet
 * if we are away from the center of the sphere.
 */
    private static double tb_project_to_sphere(double r, double x,
        double y) {

        double d, t, z;

        d = Math.sqrt(x*x + y*y);
        if (d < r * 0.70710678118654752440) {    /* Inside sphere */
            z = Math.sqrt(r*r - d*d);
        } else {           /* On hyperbola */
            t = r / 1.41421356237309504880;
            z = t*t / d;
        }
        return z;
    }

//------------------------------------------------------------------------------
    public void add_quats(double[] q1, double[] q2, double[] dest) {

        int count=0;
        double[] t1 = new double[4];
        double[] t2 = new double[4];
        double[] t3 = new double[4];
        double[] tf = new double[4];

        vcopy(q1,t1);
        vscale(t1,q2[3]);

        vcopy(q2,t2);
        vscale(t2,q1[3]);

        vcross(q2,q1,t3);
        vadd(t1,t2,tf);
        vadd(t3,tf,tf);
        tf[3] = q1[3] * q2[3] - vdot(q1,q2);

        dest[0] = tf[0];
        dest[1] = tf[1];
        dest[2] = tf[2];
        dest[3] = tf[3];

        if (++count > RENORMCOUNT) {
            count = 0;
            normalize_quat(dest);
        }
    }

//------------------------------------------------------------------------------
/*
 * Quaternions always obey:  a^2 + b^2 + c^2 + d^2 = 1.0
 * If they don't add up to 1.0, dividing by their magnitued will
 * renormalize them.
 *
 * Note: See the following for more information on quaternions:
 *
 * - Shoemake, K., Animating rotation with quaternion curves, Computer
 *   Graphics 19, No 3 (Proc. SIGGRAPH'85), 245-254, 1985.
 * - Pletinckx, D., Quaternion calculus as a basic tool in computer
 *   graphics, The Visual Computer 5, 2-13, 1989.
 */
    private static void normalize_quat(double[] q) {

        int i;
        double mag;

        mag = (q[0]*q[0] + q[1]*q[1] + q[2]*q[2] + q[3]*q[3]);
        for (i = 0; i < 4; i++) q[i] /= mag;
    }

//------------------------------------------------------------------------------
/*
 * Build a rotation matrix, given a quaternion rotation.
 */
    public void build_rotmatrix(double[][] m, double[] q) {

        m[0][0] = 1 - 2 * (q[1] * q[1] + q[2] * q[2]);
        m[0][1] = 2 * (q[0] * q[1] - q[2] * q[3]);
        m[0][2] = 2 * (q[2] * q[0] + q[1] * q[3]);
        m[0][3] = 0;

        m[1][0] = 2 * (q[0] * q[1] + q[2] * q[3]);
        m[1][1]= 1 - 2 * (q[2] * q[2] + q[0] * q[0]);
        m[1][2] = 2 * (q[1] * q[2] - q[0] * q[3]);
        m[1][3] = 0;

        m[2][0] = 2 * (q[2] * q[0] - q[1] * q[3]);
        m[2][1] = 2 * (q[1] * q[2] + q[0] * q[3]);
        m[2][2] = 1 - 2 * (q[1] * q[1] + q[0] * q[0]);
        m[2][3] = 0;

        m[3][0] = 0;
        m[3][1] = 0;
        m[3][2] = 0;
        m[3][3] = 1;
    }

//------------------------------------------------------------------------------
/*
 * Resets the current quaternion to zero.
 */
    public void reset() {

        trackball(m_currentQuaternion, 0, 0, 0, 0);
    }

//------------------------------------------------------------------------------
/*
 * Resets the current quaternion to zero.
 */
    public void setStepSize(double p1x, double p1y, double p2x, double p2y) {

        trackball(m_stepQuaternion, p1x, p1y, p2x, p2y);
    }

//------------------------------------------------------------------------------
/*
 * Sets the size of the trackball.
 */
    public void setBallSize(double size) {

        m_trackBallSize = size;
    }

//------------------------------------------------------------------------------
/*
 * Resets the current quaternion to zero.
 */
    public void nextStep() {

        add_quats(m_stepQuaternion, m_currentQuaternion, m_currentQuaternion);
        m_bMoved = true;
    }

//------------------------------------------------------------------------------
/*
 * Resets the current quaternion to zero.
 */
    public double[] getRotationMatrix() {

        double[][] m = new double[4][4];
        build_rotmatrix(m, m_currentQuaternion);
        double[] m16 = new double[16];
        for (int i = 0; i < 4; i++) {
            for (int j = 0; j < 4; j++) {
                m16[i * 4 + j] = m[i][j];
            }
        }

        m_bMoved = false;

        return m16;
    }

//------------------------------------------------------------------------------
/*
 * Resets the current quaternion to zero.
 */
    public boolean moved() {

        return m_bMoved;
    }

}

