/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2006-06-06
 * Date started:    2000-09-15
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr.visual;

import gov.usgs.gr.*;
import java.util.*;

//==============================================================================
/**
 * Describes a translation, scaling, or rotation operation.
 */
public class TransformationOperation extends gov.usgs.gr.GrObject {

    /** The array of names for the operation types. */
    protected String[] m_typeChoices = {"Translation", "Scaling", "Rotation"};
    /** The type of operation.  0 = translate, 1 = scaling, 2 = rotate. */
    public int type = 0;
    /** The x value for the operation. */
    public double x = 0;
    /** The y value for the operation. */
    public double y = 0;
    /** The z value for the operation. */
    public double z = 0;
    /** The angle in degrees if this is a rotation operation. */
    public double angle = 0;

//------------------------------------------------------------------------------
/**
 * Construct an empty object.
 */
    public TransformationOperation () {

        setDefaults("gov.usgs.gr.visual.TransformationOperation");
    }

//------------------------------------------------------------------------------
/**
 * Copy Constructor.
 */
    public TransformationOperation (TransformationOperation to) {

    	setProperties(to.getProperties());
    }

//------------------------------------------------------------------------------
/**
 * Create a string representation of this object.
 * @return  The string that describes this object.
 */
    public String toString() {

    	String s = "Transformation Operation";
        for (int i = 0; i < m_typeChoices.length; i++) {
           if (type == i) {
                s += ": ";
                s += m_typeChoices[i];
                break;
            }
        }

        return s;
    }

//------------------------------------------------------------------------------
/**
 * Provide the list of properties with the latest values.
 */
    public GrHashList getProperties() {

    	GrHashList props = super.getProperties();


        // Give the user some X Coordinate Level choices...
        GrProperty p = new GrProperty("", "Type");
        javax.swing.JComboBox jcb = new javax.swing.JComboBox();

        // Fill the type combo box.
        for (int i = 0; i < m_typeChoices.length; i++) {
            jcb.addItem(m_typeChoices[i]);
        }
        jcb.setSelectedIndex(type);

        p.value = jcb.getItemAt(type);
        p.component = jcb;
        props.put("type", p);

        props.put("x", new GrProperty(x, "X"));
        props.put("y", new GrProperty(y, "Y"));
        props.put("z", new GrProperty(z, "Z"));
        props.put("angle", new GrProperty(angle, "Angle in Degrees"));

        return props;
    }

//------------------------------------------------------------------------------
/**
 * Provide the list of properties with the latest values.
 */
    public void setProperties(HashMap props) {

		super.setProperties(props);

    	GrProperty p;

        Iterator iter = props.keySet().iterator();
        while (iter.hasNext()) {
        	String key = (String)iter.next();
            Object obj = props.get(key);
            if (obj instanceof GrProperty) {
				p = (GrProperty)obj;

                if (key.equals("type")) {
                    // Check each name.
                    for (int i = 0; i < m_typeChoices.length; i++) {
                        String name = (String)p.value;
                        if (name.equals(m_typeChoices[i])) {
                            type = i;
                        }
                    }
                }
            	if (key.equals("x"))
            		x = (Double)p.value;
            	if (key.equals("y"))
            		y = (Double)p.value;
            	if (key.equals("z"))
            		z = (Double)p.value;
            	if (key.equals("angle"))
            		angle = (Double)p.value;
			}
            if (obj instanceof String) {
                String value = (String)obj;
                setProperty(key, value);
            }
        }
    }

//------------------------------------------------------------------------------
/**
 * Set a single property.
 */
    public void setProperty(String key, String value) {

    	super.setProperty(key, value);

        if (key.equals("type")) {
            // Check each name.
            for (int i = 0; i < m_typeChoices.length; i++) {
                String name = value;
                if (name.equals(m_typeChoices[i])) {
                    type = i;
                }
            }
        }
        if (key.equals("x"))
        	x = Double.parseDouble(value);
        if (key.equals("y"))
        	y = Double.parseDouble(value);
        if (key.equals("z"))
        	z = Double.parseDouble(value);
        if (key.equals("angle"))
        	angle = Double.parseDouble(value);
    }

//------------------------------------------------------------------------------
/**
 * Overrides add in GrObject.  Don't add any children.
 */
    public boolean add(GrObject gro) {

        return false;
    }
}




