/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2006-06-06
 * Date started:    2004-12-02
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr.visual.gl;

import gov.usgs.gr.*;
import gov.usgs.gr.data.*;
import gov.usgs.gr.tools.*;
import gov.usgs.gr.visual.*;
import java.awt.Point;
import java.awt.event.*;
import java.awt.geom.*;
import java.util.*;
import javax.media.opengl.*;

//==============================================================================
/**
 * An x-y graph with ticks, labels, a grid, and multiple curves drawn inside.
 */
public class Gl3dViewParameters extends gov.usgs.gr.GrObject {

    public double m_fieldOfView = 60;
    public double m_nearFactor = 0.1;
    public double m_farFactor = 2;
    public double m_eyeX = 0;
    public double m_eyeY = 0;
    public double m_eyeZ = 1.5;
    public double m_centerX = -0.5;
    public double m_centerY = -0.5;
    public double m_centerZ = 0;

//------------------------------------------------------------------------------
/**
 * Construct a time series graph and have it create its own x-axis zoom.
 *
 */
    public Gl3dViewParameters() {

        super();

        setDefaults("gov.usgs.gr.visual.gl.Gl3dViewParameters");
    }

//------------------------------------------------------------------------------
/**
 * Provide the list of properties with the latest values.
 */
    public GrHashList getProperties() {

        GrHashList props = super.getProperties();

        props.put("fieldOfView", new GrProperty(m_fieldOfView,
            "Field Of View (0-180)"));
        props.put("nearFactor", new GrProperty(m_nearFactor,
            "Near Clipping Factor"));
        props.put("farFactor", new GrProperty(m_farFactor,
            "Far Clipping Factor"));
        props.put("eyeX", new GrProperty(m_eyeX, "Eye X"));
        props.put("eyeY", new GrProperty(m_eyeY, "Eye Y"));
        props.put("eyeZ", new GrProperty(m_eyeZ, "Eye Z"));
        props.put("centerX", new GrProperty(m_centerX, "Center X"));
        props.put("centerY", new GrProperty(m_centerY, "Center Y"));
        props.put("centerZ", new GrProperty(m_centerZ, "Center Z"));

        return props;
    }

//------------------------------------------------------------------------------
/**
 * Provide the list of properties with the latest values.
 */
    public synchronized void setProperties(HashMap props) {

        super.setProperties(props);

        GrProperty p;

        Iterator iter = props.keySet().iterator();
        while (iter.hasNext()) {
            String key = (String)iter.next();
            Object obj = props.get(key);
            if (obj instanceof GrProperty) {
                p = (GrProperty)obj;

                if (key.equals("fieldOfView"))
                    m_fieldOfView = (Double)p.value;
                if (key.equals("nearFactor"))
                    m_nearFactor = (Double)p.value;
                if (key.equals("farFactor"))
                    m_farFactor = (Double)p.value;
                if (key.equals("eyeX"))
                    m_eyeX = (Double)p.value;
                if (key.equals("eyeY"))
                    m_eyeY = (Double)p.value;
                if (key.equals("eyeZ"))
                    m_eyeZ = (Double)p.value;
                if (key.equals("centerX"))
                    m_centerX = (Double)p.value;
                if (key.equals("centerY"))
                    m_centerY = (Double)p.value;
                if (key.equals("centerZ"))
                    m_centerZ = (Double)p.value;
            }
            if (obj instanceof String) {
                String value = (String)obj;
                setProperty(key, value);
            }
        }

        m_bHasChanged = true;
        notifyObservers();
    }

//------------------------------------------------------------------------------
/**
 * Set a single property.
 */
    public synchronized void setProperty(String key, String value) {

        super.setProperty(key, value);

        if (key.equals("fieldOfView"))
            m_fieldOfView = Double.parseDouble(value);
        if (key.equals("nearFactor"))
            m_nearFactor = Double.parseDouble(value);
        if (key.equals("farFactor"))
            m_farFactor = Double.parseDouble(value);
        if (key.equals("eyeX"))
            m_eyeX = Double.parseDouble(value);
        if (key.equals("eyeY"))
            m_eyeY = Double.parseDouble(value);
        if (key.equals("eyeZ"))
            m_eyeZ = Double.parseDouble(value);
        if (key.equals("centerX"))
            m_centerX = Double.parseDouble(value);
        if (key.equals("centerY"))
            m_centerY = Double.parseDouble(value);
        if (key.equals("centerZ"))
            m_centerZ = Double.parseDouble(value);
    }

//------------------------------------------------------------------------------
/**
 * Create a string representation of this object.
 * @return  The string that describes this object.
 */
    public String toString() {

    	String s = "3D View Parameters";

        return s;
    }
}



