/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2004-12-10
 * Date started:    1998-10-20
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr.visual.gl;

import gov.usgs.gr.visual.gl.*;
import gov.usgs.gr.*;
import gov.usgs.gr.visual.gl.*;
import java.awt.*;
import java.awt.event.*;
import java.lang.Math;
import java.util.*;
import javax.media.opengl.*;
import javax.media.opengl.glu.*;

//==============================================================================
/**
 * A panel holding the x-y graphs. Extends GlPanel and thus inherits its OpenGL
 * drawing capabilities.
 */
public class GlGrPanel implements gov.usgs.gr.visual.gui.PageProxy,
    java.awt.event.MouseListener, java.awt.event.MouseMotionListener,
    javax.media.opengl.GLEventListener {
    
    protected GLCanvas m_canvas = null;
    protected GL gl = null;
    protected GLU glu = null;
    /** A page of graphs contained within this panel. */
    protected PageOfGlGraphs m_page;
    /** The parent visual to pass mouse events to. */
    GrController m_grCon = null;
//    GlOlympicRings rings = new GlOlympicRings();
    
//------------------------------------------------------------------------------
    /**
     * Create a GrPanel through the super class.
     */
    public GlGrPanel() {
	
	m_canvas = new GLCanvas();
	
	// Use debug pipeline
//    m_canvas.setGL(new DebugGL(m_canvas.getGL()));
//    System.err.println("CANVAS GL IS: " + m_canvas.getGL().getClass().getName());
//    System.err.println("CANVAS GLU IS: " + m_canvas.getGLU().getClass().getName());
	
	m_canvas.addGLEventListener(this);
    }
    
//------------------------------------------------------------------------------
    /**
     * Create a GrPanel connected to a controller.
     */
    public GlGrPanel(GrController con) {
	
	this();
	
	m_grCon = con;
    }
    
//------------------------------------------------------------------------------
    /**
     * Create a GrPanel as a copy of another page of graphs.  Share the data.
     */
    public GlGrPanel(PageOfGlGraphs page, GrController con) {
	
	this();
	
	m_page = page;
	m_grCon = con;
    }
    
//------------------------------------------------------------------------------
    /**
     *
     */
    public void init(GLAutoDrawable drawable) {
	
	drawable.addMouseListener(this);
	drawable.addMouseMotionListener(this);
    }
    
//------------------------------------------------------------------------------
    /**
     * Clears the drawing area and redraws the contents.
     */
    public void display(GLAutoDrawable drawable) {
	
	gl = drawable.getGL();
	glu = new GLU();
	
	try {
	    if (m_page == null) {
		// Clear the window.
		gl.glDisable(gl.GL_SCISSOR_TEST);
		// USGS Green
		//gl.glClearColor(0, 0.433, 0.254, 0);
		// USGS Blue
		
		gl.glClearColor(0f, 0.379f, 0.639f, 0f);
		gl.glClear(gl.GL_COLOR_BUFFER_BIT);
		
		//            rings.setOpenGL((GLAutoDrawable)this, gl, glu);
		//            rings.drawGL();
		
		// Return to the default arrow cursor.
		((Component)GrController.getParentFrame(m_canvas))
		.setCursor(Cursor.getDefaultCursor());
		
		return;
	    }
	    
	    m_page.setOpenGL(drawable, gl, glu);
	    
	    m_page.drawGL();
	    
	    // Return to the default arrow cursor.
	    ((Component)GrController.getParentFrame(m_canvas))
	    .setCursor(Cursor.getDefaultCursor());
	} catch (Exception e) {
	    System.err.println("Exception caught in GlGrPanel.display():");
	    e.printStackTrace();
	}
    }
    
//------------------------------------------------------------------------------
    public void displayChanged(GLAutoDrawable drawable, boolean modeChanged,
	boolean deviceChanged) {
    }
    
//------------------------------------------------------------------------------
    /**
     * Changes the size and shape of the panel's associated drawing area and the
     * objects that are drawn on it.
     * <p><b> Post Conditions </b><br>
     *      Changes won't be seen until the panel is redrawn.
     * @param component Required to implement GLComponentListener.  Not used.
     * @param width     The new width for the drawing area.
     * @param heicht    The new height for the drawing area.
     */
    public void reshape(GLAutoDrawable drawable, int x, int y,
	int width, int height ) {
	
	gl = drawable.getGL();
	glu = new GLU();
	
	gl.glViewport(0, 0, width, height);
	
	// Also reshape the page of graphs contained in this panel.
	if (m_page != null)
	    m_page.reshape(width, height);
    }
    
//------------------------------------------------------------------------------
    /**
     * Attribute modification method.
     * @param page	A new page to use for drawing.
     */
    public void setPage(gov.usgs.gr.visual.PageOfGraphs page) {
	
	if (page instanceof PageOfGlGraphs)
	    m_page = (PageOfGlGraphs)page;
	if (page == null)
	    m_page = null;
    }
    
//------------------------------------------------------------------------------
    /**
     * Attribute modification method.
     * @param con	The new controller.
     */
    public void setController(GrController con) {
	m_grCon = con;
    }
    
//------------------------------------------------------------------------------
    public gov.usgs.gr.visual.PageOfGraphs setNewPage() {
	
	m_page = new PageOfGlGraphs();
	
	return m_page;
    }
    
//------------------------------------------------------------------------------
    /**
     * Pass the mouse event on to the GrVisual.
     * @param e     The mouse event object containing the mouse position.
     */
    public void mouseMoved(MouseEvent e) {
	
	if (m_grCon != null) m_grCon.mouseMoved(e);
    }
    
//------------------------------------------------------------------------------
    /**
     * Pass the mouse event on to the GrVisual.
     * @param e     The mouse event object containing the mouse position.
     */
    public void mouseDragged(MouseEvent e) {
	
	if (m_grCon != null) m_grCon.mouseDragged(e);
    }
    
//------------------------------------------------------------------------------
    /**
     * Pass the mouse event on to the GrVisual.
     * @param e     The mouse event object containing the mouse position.
     */
    public void mousePressed(MouseEvent e) {
	
	if (m_grCon != null) m_grCon.mousePressed(e);
    }
    
//------------------------------------------------------------------------------
    /**
     * Pass the mouse event on to the GrVisual.
     * @param e     The mouse event.
     */
    public void mouseReleased(MouseEvent e) {
	
	if (m_grCon != null) m_grCon.mouseReleased(e);
    }
    
//------------------------------------------------------------------------------
    /**
     * Pass the mouse event on to the GrVisual.
     * @param e     The mouse event.
     */
    public void mouseExited(MouseEvent e) {
	
	if (m_grCon != null) m_grCon.mouseExited(e);
    }
    
//------------------------------------------------------------------------------
    /**
     * Tells whether OpenGL is available.
     * @param g	The graphics context.
     */
    public static boolean openGlIsAvailable() {
	
	return true;
    }
    
//------------------------------------------------------------------------------
// Mouse methods...
    public void mouseEntered(MouseEvent e) {}
    public void mouseClicked(MouseEvent e) {}
    
//------------------------------------------------------------------------------
    public void repaint() {
	
	m_canvas.repaint();
    }
    
//------------------------------------------------------------------------------
    public Component getComponent() {
	
	return m_canvas;
    }
    
}


