/**
 * @(#) olympicCvs.java
 * from GL4Java Demo
 */
package gov.usgs.gr.visual.gl;

import java.awt.*;
import java.lang.Math;
import javax.media.opengl.*;
import javax.media.opengl.glu.*;

public class GlOlympicRings {

    /** The OpenGL GLAutoDrawable component. */
    protected GLAutoDrawable gld = null;
    /**
     * The main OpenGL object with all the gl drawing methods.  For
     * readability, the m_ prefix wasn't used.
     */
    protected GL gl = null;
    /** The OpenGL utility class with all the glu drawing methods. */
    protected GLU glu = null;
    boolean m_bShallWeRender = false;
    boolean m_bInitialized = false;

	static final double M_PI= 3.141592654;
	static final double M_2PI= 2*3.141592654;

	static final int
		XSIZE=   100,
		YSIZE=   75,
		RINGS= 5,
		BLUERING= 0,
		BLACKRING= 1,
		REDRING= 2,
		YELLOWRING =3,
		GREENRING =4,
		BACKGROUND =8,
		BLACK = 0,
		RED=1,
		GREEN=2,
		YELLOW=3,
		BLUE=4,
		MAGENTA=5,
		CYAN=6,
		WHITE=7;

	byte rgb_colors[][];
	int mapped_colors[];
	float dests[][];
	float offsets[][];
	float angs[];
	float rotAxis[][];
	int iters[];
	int theTorus;

    float lmodel_ambient[] = {0, 0, 0, 0};
    float lmodel_twoside[] = {0, 0, 0, 0};
    float lmodel_local[] = {0, 0, 0, 0};
    float light0_ambient[] = {0.1f, 0.1f, 0.1f, 1};
    float light0_diffuse[] = {1, 1, 1, 0};
    float light0_position[] = {0.8660254f, 0.5f, 1, 0};
    float light0_specular[] = {1, 1, 1, 0};
    float bevel_mat_ambient[] = {0, 0, 0, 1};
    float bevel_mat_shininess[] = {40, 0, 0, 0};
    float bevel_mat_specular[] = {1, 1, 1, 0};
    float bevel_mat_diffuse[] = {1, 0, 0, 0};

//------------------------------------------------------------------------------
	public GlOlympicRings () {
	}

//------------------------------------------------------------------------------
    public void init()  {

		rgb_colors=new byte[RINGS][3];
		mapped_colors=new int [RINGS];
		dests=new float [RINGS][3];
		offsets=new float[RINGS][3];
		angs=new float[RINGS];
		rotAxis=new float[RINGS][3];
		iters=new int[RINGS];

		int i;
		float top_y = 0.5f;
		float bottom_y = -0.5f;
		float top_z = 0.15f;
		float bottom_z = 0.69f;
		float spacing = 2.5f;

//		reInit();
		for (i = 0; i < RINGS; i++) {
		  rgb_colors[i][0] = rgb_colors[i][1] = rgb_colors[i][2] = (byte)0;
		}
		rgb_colors[BLUERING][2] = (byte)255;
		rgb_colors[REDRING][0] = (byte)255;
		rgb_colors[GREENRING][1] = (byte)255;
		rgb_colors[YELLOWRING][0] = (byte)255;
		rgb_colors[YELLOWRING][1] = (byte)255;
		mapped_colors[BLUERING] = BLUE;
		mapped_colors[REDRING] = RED;
		mapped_colors[GREENRING] = GREEN;
		mapped_colors[YELLOWRING] = YELLOW;
		mapped_colors[BLACKRING] = BLACK;

		dests[BLUERING][0] = -spacing;
		dests[BLUERING][1] = top_y;
		dests[BLUERING][2] = top_z;

		dests[BLACKRING][0] = 0;
		dests[BLACKRING][1] = top_y;
		dests[BLACKRING][2] = top_z;

		dests[REDRING][0] = spacing;
		dests[REDRING][1] = top_y;
		dests[REDRING][2] = top_z;

		dests[YELLOWRING][0] = -spacing / 2;
		dests[YELLOWRING][1] = bottom_y;
		dests[YELLOWRING][2] = bottom_z;

		dests[GREENRING][0] = spacing / 2;
		dests[GREENRING][1] = bottom_y;
		dests[GREENRING][2] = bottom_z;

        m_bInitialized = true;
    }

//------------------------------------------------------------------------------
    public void setup()  {

		theTorus = gl.glGenLists(1);
		gl.glNewList(theTorus, gl.GL_COMPILE);
		FillTorus(0.1f, 8, 1, 25);
		gl.glEndList();

		gl.glEnable(gl.GL_CULL_FACE);
		gl.glCullFace(gl.GL_BACK);
		gl.glEnable(gl.GL_DEPTH_TEST);
		gl.glClearDepth(1.0);

		gl.glLightfv(gl.GL_LIGHT0, gl.GL_AMBIENT, light0_ambient, 0);
		gl.glLightfv(gl.GL_LIGHT0, gl.GL_DIFFUSE, light0_diffuse, 0);
		gl.glLightfv(gl.GL_LIGHT0, gl.GL_SPECULAR, light0_specular, 0);
		gl.glLightfv(gl.GL_LIGHT0, gl.GL_POSITION, light0_position, 0);
		gl.glEnable(gl.GL_LIGHT0);

		gl.glLightModelfv(gl.GL_LIGHT_MODEL_LOCAL_VIEWER, lmodel_local, 0);
		gl.glLightModelfv(gl.GL_LIGHT_MODEL_TWO_SIDE, lmodel_twoside, 0);
		gl.glLightModelfv(gl.GL_LIGHT_MODEL_AMBIENT, lmodel_ambient, 0);
		gl.glEnable(gl.GL_LIGHTING);

		gl.glClearColor(1, 1, 1, 0);

		gl.glMaterialfv(gl.GL_FRONT, gl.GL_AMBIENT, bevel_mat_ambient, 0);
		gl.glMaterialfv(gl.GL_FRONT, gl.GL_SHININESS, bevel_mat_shininess, 0);
		gl.glMaterialfv(gl.GL_FRONT, gl.GL_SPECULAR, bevel_mat_specular, 0);
		gl.glMaterialfv(gl.GL_FRONT, gl.GL_DIFFUSE, bevel_mat_diffuse, 0);

		gl.glColorMaterial(gl.GL_FRONT_AND_BACK, gl.GL_DIFFUSE);
		gl.glEnable(gl.GL_COLOR_MATERIAL);
		gl.glShadeModel(gl.GL_SMOOTH);
    }

//------------------------------------------------------------------------------
/**
 * Sets the OpenGL objects to use for accessing drawing commands.
 * @param gld   The OpenGL GLAutoDrawable component.
 * @param gl    The OpenGL object containing the drawing commands.
 * @param glu   The OpenGL utilites object.
 */
    public void setOpenGL(GLAutoDrawable newgld, GL newgl, GLU newglu) {
        gld = newgld;
      	gl = newgl;
		glu = newglu;
    }

//------------------------------------------------------------------------------
    public void drawGL() {
        int i;

        if (!m_bInitialized) init();
        gl.glPushAttrib(gl.GL_ENABLE_BIT);
        setup();

		gl.glMatrixMode(gl.GL_PROJECTION);
        gl.glLoadIdentity();
		glu.gluPerspective(45, 1.33, 0.1, 100);
		gl.glMatrixMode(gl.GL_MODELVIEW);

        gl.glPushMatrix();

		gl.glClearColor(1, 1, 1, 0);
	    gl.glClear(gl.GL_COLOR_BUFFER_BIT | gl.GL_DEPTH_BUFFER_BIT);
	    glu.gluLookAt(0, 0, 7, 0, 0, 0, 0, 1, 0);

	    for (i = 0; i < RINGS; i++) {
	        gl.glColor3ubv(rgb_colors[i], 0);
	        gl.glPushMatrix();

/*
	        gl.glTranslatef(dests[i][0] + offsets[i][0],
                dests[i][1] + offsets[i][1],
	            dests[i][2] + offsets[i][2]);
	        gl.glRotatef(angs[i], rotAxis[i][0], rotAxis[i][1], rotAxis[i][2]);
*/

	        gl.glTranslatef(dests[i][0], dests[i][1], dests[i][2]);
//	        gl.glRotatef(angs[i], rotAxis[i][0], rotAxis[i][1], rotAxis[i][2]);
	        gl.glCallList(theTorus);
	        gl.glPopMatrix();
	    }

	    gl.glPopMatrix();
//	    animationCalc();
        gl.glPopAttrib();
    }
/*
//------------------------------------------------------------------------------
	public void animationCalc() {
	          int i, j;

		  for (i = 0; i < RINGS; i++) {
		    if (iters[i]!=0) {
		      for (j = 0; j < 3; j++) {
			offsets[i][j] = Clamp(iters[i], offsets[i][j]);
			offsets[i][j] = 0;
		      }
		      angs[i] = Clamp(iters[i], angs[i]);
		      iters[i]--;
		    }
		  }
	          if (iters[0]==0)
		  {
			  m_bShallWeRender=false;
		  }
//          else {
//            display();
//            }
	}

//------------------------------------------------------------------------------
	public void reInit() {
        int i;
	    float deviation;

	    deviation = MyRand() / 2;
	    deviation = deviation * deviation;
	    for (i = 0; i < RINGS; i++) {
	        offsets[i][0] = MyRand();
	        offsets[i][1] = MyRand();
	        offsets[i][2] = MyRand();
	        angs[i] = (float) (260.0 * MyRand());
	        rotAxis[i][0] = MyRand();
	        rotAxis[i][1] = MyRand();
	        rotAxis[i][2] = MyRand();
	        iters[i] = ( int ) (deviation * MyRand() + 60.0);
	        iters[i] = 0;
	    }
	}
*/
//------------------------------------------------------------------------------
	public void FillTorus(float rc, int numc, float rt, int numt) {
	    int i, j, k;
	    double s, t;
	    float x, y, z;

	    for (i = 0; i < numc; i++) {
	        gl.glBegin(gl.GL_QUAD_STRIP);
	        for (j = 0; j <= numt; j++) {
	            for (k = 1; k >= 0; k--) {
		            s = (i + k) % numc + 0.5;
		            t = j % numt;

		            x = (float) Math.cos(t * M_2PI / numt) * (float) Math.cos(s * M_2PI / numc);
		            y = (float) Math.sin(t * M_2PI / numt) * (float) Math.cos(s * M_2PI / numc);
		            z = (float) Math.sin(s * M_2PI / numc);
		            gl.glNormal3f(x, y, z);

		            x = (rt + rc * (float) Math.cos(s * M_2PI / numc)) * (float) Math.cos(t * M_2PI / numt);
		            y = (rt + rc * (float) Math.cos(s * M_2PI / numc)) * (float) Math.sin(t * M_2PI / numt);
		            z = rc * (float) Math.sin(s * M_2PI / numc);
		            gl.glVertex3f(x, y, z);
	            }
	        }
	        gl.glEnd();
	    }
	}
/*
//------------------------------------------------------------------------------
	public float Clamp(int iters_left, float t) {

	    if (iters_left < 3) {
	        return 0.0;
	    }
	    return (iters_left - 2) * t / iters_left;
	}

//------------------------------------------------------------------------------
	public float MyRand() {
	    // return 10.0 * (drand48() - 0.5);
	    return (float) ( 10.0 * (Math.random() - 0.5) );
	}
*/
}

