/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2006-06-06
 * Date started:    2001-02-17
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr.visual.gl;

import com.sun.opengl.util.*;
import gov.usgs.gr.*;
import gov.usgs.gr.data.*;
import gov.usgs.gr.visual.*;
import java.util.*;
import javax.media.opengl.*;
import javax.media.opengl.glu.*;

//==============================================================================
/**
 * A series of discontinuous triangle fan surfaces.
 */
public class GlParticleAnimation
	extends gov.usgs.gr.visual.DataAnimation implements GlElement {

    /** The OpenGL GLAutoDrawable component. */
    protected GLAutoDrawable gld = null;
    /**
     * The main OpenGL object with all the gl drawing methods.  For
     * readability, the m_ prefix wasn't used.
     */
    protected GL gl = null;
    /** The OpenGL utility class with all the glu drawing methods. */
    protected GLU glu = null;
    /** The OpenGL utility toolkit class with all the glut methods. */
    protected GLUT glut = null;

    protected ArrayList m_trailIndexes = new ArrayList();
    protected int m_numTrails = 20;
    protected double m_trailScale = 1;
    protected double m_trailAlpha = 1;
    protected int m_lastBegin = 0;
    protected int m_lastEnd = 0;
    double[] m_theta = {0, 0, 0, 0};
    protected boolean m_bDrawingCurrent = false;
    protected float[] m_circlePoints = null;
    protected float m_trailAlphaScale = 0.3f;
    protected float m_trailSizeScale = 0.8f;
    protected int m_trailLineWidth = 0;

    GlTexture m_texture = null;
    /** The tree icon to use for this class. */
	public static ArrayList TREE_ICON_HOLDER = new ArrayList(1);
    /** The location of the tree icon. */
    protected String m_iconFile = "GrObject/GlParticleAnimation.gif";

//------------------------------------------------------------------------------
/**
 * Constructor for creating a blank data series.
 */
    public GlParticleAnimation(){

        initialize();

        setDefaults("gov.usgs.gr.visual.gl.GlParticleAnimation");
    }

//------------------------------------------------------------------------------
/**
 * Constructor for creating a curve from a data series.
 */
    public GlParticleAnimation(DoubleDataSeries series){

        initialize();

        setDefaults("gov.usgs.gr.visual.gl.GlParticleAnimation");

        add(series);
   }

//------------------------------------------------------------------------------
/**
 * Initialize all the member variables.
 */
    public void initialize() {

        m_numDimsAllowed = 5;
        m_symbolTypes = new String[] {"Two Tone Square", "Square",
            "Two Tone Fish", "Fish", "Lit Sphere", "Bull's Eye", "Circle"};

        m_dataArrays = new double[m_numDimsAllowed][];
        m_seriesArray = new DoubleDataSeries[m_numDimsAllowed + 1];
        m_seriesNums = new int[] {1, 1, 1, 0, 0};
        m_dimNums = new int[] {0, 1, 2, -1, -1};
        m_seriesNumDims = new int[] {0, 0, 0, 0, 0};
        m_dimScales = new double[] {1, 1, 1, 1, 1};
        double[] array = new double[m_numDimsAllowed];
        m_constantSeries = new DoubleDataSeries(array, m_numDimsAllowed);
        m_color = new float[] {1, 0.2f, 0.2f};

        float len0 = 0.5f;
        float len1 = 0.25f;
        float len2 = 0.43301270189221932338186158537647f;
        m_circlePoints = new float[28];
        Arrays.fill(m_circlePoints, 0);
        m_circlePoints[3] = len0;
        m_circlePoints[4] = -len1;
        m_circlePoints[5] = len2;
        m_circlePoints[6] = -len2;
        m_circlePoints[7] = len1;
        m_circlePoints[8] = -len0;
        m_circlePoints[10] = -len2;
        m_circlePoints[11] = -len1;
        m_circlePoints[12] = -len1;
        m_circlePoints[13] = -len2;
        m_circlePoints[15] = -len0;
        m_circlePoints[16] = len1;
        m_circlePoints[17] = -len2;
        m_circlePoints[18] = len2;
        m_circlePoints[19] = -len1;
        m_circlePoints[20] = len0;
        m_circlePoints[22] = len2;
        m_circlePoints[23] = len1;
        m_circlePoints[24] = len1;
        m_circlePoints[25] = len2;
        m_circlePoints[27] = len0;
    }


//------------------------------------------------------------------------------
/**
 * Add a child object.  Accepts DoubleDataSeries, TimeMarker.  Only holds one DS at
 * a time.
 */
    public boolean add(GrObject gro) {

        // If gro is an object effigy, get the eventual subject and try
        // adding that.
        while (gro instanceof gov.usgs.gr.ObjectEffigy)
            gro = ((ObjectEffigy)gro).getSubject();

        if (gro instanceof GlTexture) {
            if (m_children.contains(gro)) return false;
            if (gro == this) return false;

            if (m_texture != null)
                remove(m_texture);
            m_texture = (GlTexture)gro;
            m_children.add(gro);
            gro.addObserver(this);

            m_bHasChanged = true;

            return true;
        }
        
/* A quick calculation of the distance travelled by the particle...
 if (gro instanceof DoubleDataSeries) {
            DoubleDataSeries series = (DoubleDataSeries)gro;
            int numDims = series.getNumDimensions();
            if (numDims == 4) {
                double[] array = series.getArray();
                int numPoints = series.getNumPoints();
                int numParticles = 2;
                numPoints /= numParticles;
                numDims *= numParticles;
                double totalDist = 0;
                double startTime = Double.NaN;
                double endTime = Double.NaN;
                double lastX = 0;
                double lastY = 0;
                for (int i = 0; i < numPoints; i++) {
                    double time = array[i * numDims];
                    double x = array[i * numDims + 1];
                    double y = array[i * numDims + 2];
                    if ((x != y || x < -1.000001 || x > 0.99999) && i < (numPoints - 1)) {
                        if (Double.isNaN(startTime)) {
                            startTime = time;
                        }
                        else {
                            double xDiff = x - lastX;
                            double yDiff = y - lastY;
                            double dist = Math.sqrt(
                                (xDiff * xDiff) + (yDiff * yDiff));
                            totalDist += dist;
                        }
                        endTime = time;
                        lastX = x;
                        lastY = y;
                    }
                    else {
                        if (totalDist > 0) {
                            System.out.println("Total dist between " + startTime
                                + " and " + endTime + " is " + totalDist);
                        }
                        
                        startTime = Double.NaN;
                        totalDist = 0;
                    }
                }

            }
        }
*/
    
        return super.add(gro);
    }
//------------------------------------------------------------------------------
/**
 * Overrides clear() in GrObject.
 */
    public synchronized void clear() {

        // Reset member variables to null.
        m_texture = null;

        super.clear();
    }

//------------------------------------------------------------------------------
/**
 * Overrides GrObject.remove() so member variable references to children can be
 * made null when they are removed.
 * @param i     The index of the object to remove.
 * @return      The object that was removed.
 */
    public GrObject remove(int i) {

        // Let the super class handle removal.
        GrObject child = super.remove(i);

        // Reset any associated member variable to null.
        if (child == m_texture) m_texture = null;

        return child;
    }

//------------------------------------------------------------------------------
/**
 * Overrides GrObject.remove() so member variable references to children can be
 * made null when they are removed.
 * @param gro   The object to be removed.
 * @return      Whether the object was removed.
 */
    public boolean remove(GrObject gro) {

        // Reset any associated member variable to null.
        if (gro == m_texture) m_texture = null;

        // Let the super class handle removal.
        return super.remove(gro);
    }
//------------------------------------------------------------------------------
/**
 * Provide the list of properties with the latest values.
 *
 * The numeric code of the symbol to be used on this curve.
 *  0 = single pixel.
 *  1 = plus sign.
 *  2 = X
 *  3 = tick mark
 *  4 = hollow circle
 *  5 = hollow square
 *  6 = hollow upward-pointing triangle
 *  7 = hollow downward-pointing triangle
 *  8 = hollow diamond
 *  9 = filled circle
 * 10 = filled square
 * 11 = filled upward-pointing triangle
 * 12 = filled downward-pointing triangle
 * 13 = filled diamond
 *
 */
    public GrHashList getProperties() {

    	GrHashList props = super.getProperties();

/*        props.put("scale", new GrProperty(
        	m_scale, "Scale"));

        // Give the user some symbol choices...
        GrProperty p = new GrProperty("", "Symbol Type");
        javax.swing.JComboBox jcb = new javax.swing.JComboBox();

        String[] symbolTypes = {"Two Tone Square", "Square",
            "Two Tone Fish", "Fish", "Lit Sphere"
            };

        // Fill the symbol type combo box.
        for (int i = 0; i < symbolTypes.length; i++) {
        	String name = symbolTypes[i];
        	jcb.addItem(name);
            if (m_symbolIndex == i) {
				jcb.setSelectedIndex(i);
                p.value = symbolTypes[i];
            }
        }

        p.component = jcb;
		props.put("symbolType", p);

        ColorProperty cp = new ColorProperty();
        cp.description = "Color";
        cp.setColor(m_color);
        props.put("color", cp);
*/
        props.put("numTrails", new GrProperty(
        	m_numTrails, "Number of Trailers"));
        props.put("alphaFade", new GrProperty(
            m_trailAlphaScale, "Trail Alpha Scale"));
        props.put("scaleFade", new GrProperty(
            m_trailSizeScale, "Trail Size Scale"));
        props.put("trailLineWidth", new GrProperty(
        	m_trailLineWidth, "Trail Line Width"));

        return props;
    }

//------------------------------------------------------------------------------
/**
 * Provide the list of properties with the latest values.
 */
    public void setProperties(HashMap props) {

		super.setProperties(props);

    	GrProperty p;

        Iterator iter = props.keySet().iterator();
        while (iter.hasNext()) {
        	String key = (String)iter.next();
            Object obj = props.get(key);
            if (obj instanceof GrProperty) {
				p = (GrProperty)obj;

//            	if (key.equals("scale"))
//            		m_scale = (Float)p.value;
            	if (key.equals("numTrails")) {
            		m_numTrails = ((Integer)p.value);
                    while (m_trailIndexes.size() > (m_numTrails * 2)) {
                        m_trailIndexes.remove(0);
                        m_trailIndexes.remove(0);
                    }
                }
            	if (key.equals("alphaFade"))
            		m_trailAlphaScale = (Float)p.value;
            	if (key.equals("scaleFade"))
            		m_trailSizeScale = (Float)p.value;
            	if (key.equals("trailLineWidth"))
            		m_trailLineWidth = ((Integer)p.value);
                if (p.value instanceof String) {
				    String value = (String)p.value;
                    setProperty(key, value);
                }
            }
            if (obj instanceof String) {
				String value = (String)obj;
                setProperty(key, value);
            }
        }

        m_bNeedsRedraw = true;
        m_bHasChanged = true;
        notifyObservers();
    }

//------------------------------------------------------------------------------
/**
 * Set a single property.
 */
    public void setProperty(String key, String value) {

    	super.setProperty(key, value);

/*
if (key.equals("scale"))
            m_scale = Float.parseFloat(value);
        if (key.equals("symbolType")) {
        	String[] symbolTypes = {"two tone square", "square",
            "two tone fish", "fish", "lit sphere"
        	};
        	// Check each name.
            for (int i = 0; i < symbolTypes.length; i++) {
            	String name = value.trim().toLowerCase();
                if (name.equals(symbolTypes[i])) {
                	m_symbolIndex = i;
                    break;
                }
            }
        }
        if (key.equals("color")) {
            m_color = ColorProperty.getColorArray(value);
        }
*/
        if (key.equals("numTrails")) {
            m_numTrails = Integer.parseInt(value);
            while (m_trailIndexes.size() > (m_numTrails * 2)) {
                m_trailIndexes.remove(0);
                m_trailIndexes.remove(0);
            }
        }
        if (key.equals("alphaFade"))
            m_trailAlphaScale = Float.parseFloat(value);
        if (key.equals("scaleFade"))
            m_trailSizeScale = Float.parseFloat(value);
        if (key.equals("trailLineWidth"))
            m_trailLineWidth = Integer.parseInt(value);
    }

//------------------------------------------------------------------------------
/**
 * Accessor method.
 * @return An ImageIcon to use when representing this object in a tree.
 */
    public javax.swing.ImageIcon getTreeIcon() {

        if (TREE_ICON_HOLDER.size() <= 0)
            setTreeIcon();

    	return (javax.swing.ImageIcon)TREE_ICON_HOLDER.get(0);
    }

//------------------------------------------------------------------------------
/**
 * Loads the tree icon from a file and stores it in the TREE_ICON_HOLDER.
 */
    protected javax.swing.ImageIcon setTreeIcon() {

        javax.swing.ImageIcon icon = loadTreeIcon(m_iconFile);

        if (icon == null)
            icon = super.setTreeIcon();

        TREE_ICON_HOLDER.clear();
        TREE_ICON_HOLDER.add(icon);

        return icon;
    }

//------------------------------------------------------------------------------
/**
 * Sets the OpenGL objects to use for accessing drawing commands.
 * @param gld   The OpenGL GLAutoDrawable component.
 * @param gl    The OpenGL object containing the drawing commands.
 * @param glu   The OpenGL utilites object.
 */
    public void setOpenGL(GLAutoDrawable newgld, GL newgl, GLU newglu) {

        if (glut == null || newgl != gl || newglu != glu)
            glut = new GLUT();

        gld = newgld;
      	gl = newgl;
		glu = newglu;
    }

//------------------------------------------------------------------------------
/**
 *
 */
    public boolean checkForValidData() {

        if (m_seriesNums[1] != m_seriesNums[2])
            return false;

        if (m_seriesNums[1] != m_seriesNums[3]) {
            m_seriesNums[3] = -1;
            m_dimNums[3] = -1;
        }

        return true;
    }

//------------------------------------------------------------------------------
/**
 * Issue the OpenGL commands to draw this series.
 */
    public void drawGL() {

        // Before drawing, check if this object is supposed to be shown.
        if (!m_bShow) return;

        // Calculate the number of iterations for the
        // draw loop and the offsets for the T, Loc, and Vec arrays.
        int locOffset = 0;
        int endOffset = 0;
        DoubleDataSeries timeSeries = m_seriesArray[m_seriesNums[0]];
        if (timeSeries == null)
            return;

        if (m_texture != null)
            m_texture.setOpenGL(gld, gl, glu);

        synchronized (timeSeries) {
            if (!checkForValidData()) return;

            if (compareCounts(0, 1) != 0) {
                // Not a one-to-one relationship between times and locations.
                locOffset = timeSeries.getBegin()
                    * m_seriesNumDims[m_seriesNums[1]];
                endOffset = timeSeries.getEnd()
                    * m_seriesNumDims[m_seriesNums[1]];

                // Check that the color series has the right number of points.
                if ((pointCount(0) * pointCount(1)) != pointCount(4)) {
                    m_dimNums[4] = -1;
                }
            }
            else {
                // One-to-one relationship between times and locations.
                locOffset = timeSeries.getBegin();
                endOffset = timeSeries.getEnd();
            }
        }

        m_theta = new double[m_seriesNumDims[m_seriesNums[1]] - 1];

//        synchronized (m_series) {
            // Clear the trails if the index moved backwards.
            int currentBegin = locOffset;
            if (m_trailIndexes.size() >= 2) {
                int lastBegin = ((Integer)m_trailIndexes.
                    get(m_trailIndexes.size() - 2));
                if (currentBegin < lastBegin) {
                    m_trailIndexes.clear();
                }
            }

            // Draw the old trails from oldest to newest.
            m_bDrawingCurrent = false;
            int numTrails = m_trailIndexes.size() / 2;
            int trailNum = 1;
            Iterator iter = m_trailIndexes.iterator();
            while (iter.hasNext()) {
                int begin = ((Integer)iter.next());
                if (!iter.hasNext()) break;
                int end = ((Integer)iter.next());

                // Make the older trail markers smaller and more faint.
                m_trailAlpha = ((double)trailNum / (double)numTrails)
                    * m_trailAlphaScale;
                m_trailScale = ((double)trailNum / (double)numTrails)
                    * m_trailSizeScale;

                drawParticles(begin, end);
                trailNum++;
            }

            // Draw the current position at full size and opacity.
            m_trailAlpha = 1;
            m_trailScale = 1;
            m_bDrawingCurrent = true;
            drawParticles(currentBegin, endOffset);

            // Add the current indexes to the list of trails.
            m_trailIndexes.add(currentBegin);
            m_trailIndexes.add(endOffset);

            // Trim the list of trails to be no longer than the maximum length.
            while (m_trailIndexes.size() > (m_numTrails * 2)) {
                m_trailIndexes.remove(0);
                m_trailIndexes.remove(0);
            }
//        }
    }

//------------------------------------------------------------------------------
/**
 * Issue the OpenGL commands to draw this series.
 */
    public void drawParticles(int begin, int end) {

        // Save the previous line color and pattern.
        gl.glPushAttrib(gl.GL_LINE_BIT | gl.GL_CURRENT_BIT | gl.GL_ENABLE_BIT);

        gl.glEnable(gl.GL_POINT_SMOOTH);
        gl.glEnable(gl.GL_BLEND);
        gl.glBlendFunc(gl.GL_SRC_ALPHA, gl.GL_ONE_MINUS_SRC_ALPHA);

        if (m_symbolIndex == 4) turnOnLighting();

///////////////
        int numActivePoints = m_seriesNumDims[m_seriesNums[1]];
        if (m_seriesNums[0] != m_seriesNums[1]) {
            numActivePoints = pointCount(1) / m_seriesNumDims[m_seriesNums[1]]
                / pointCount(0);
                // Account for (xy, xy) or (xyxy, xyxy) or (xyzxyz, xyzxyz)
        }

        int highestDimNum = -1;
        int lowestDimNum = Integer.MAX_VALUE;
        for (int i = 1; i < m_dimNums.length; i ++) {
            if (m_dimNums[i] > highestDimNum)
                highestDimNum = m_dimNums[i];
            if (m_dimNums[i] < lowestDimNum && m_dimNums[i] >= 0)
                lowestDimNum = m_dimNums[i];
        }
        int dimStep = highestDimNum - lowestDimNum + 1;

        int lastPrtDimOffset =
            m_seriesNumDims[m_seriesNums[1]] - lowestDimNum;


        int xDim = 0;
        int yDim = 1;

        for (int i = begin; i <= end; i++) {
            int offset = i;
            int lastOffset = offset;
            if ((m_lastEnd - m_lastBegin) == (end - begin)
                && (m_lastBegin != begin)) {

                lastOffset = offset - (begin - m_lastBegin);
            }

            for (int dimOffset = 0; dimOffset < lastPrtDimOffset;
                dimOffset += dimStep) {

                double lastX = 0;
                double lastY = 0;
                double lastZ = 0;
                double u = 0;
                double v = 0;
                double w = 0;

                double x = getValueAt(i, 1, dimOffset + m_dimNums[1]);
                double y = getValueAt(i, 2, dimOffset + m_dimNums[2]);
                double z = 0;
                if (m_zScale != 0 && m_dimNums[3] >= 0)
                    z = getValueAt(i, 3, dimOffset + m_dimNums[3]);

                if (lastOffset != offset && m_seriesNums[0] == m_seriesNums[1]){
                    lastX = getValueAt(lastOffset, 1, dimOffset + 1);
                    lastY = getValueAt(lastOffset, 2, dimOffset + 2);
                    if (m_zScale != 0 && m_dimNums[3] >= 0)
                        lastZ = getValueAt(lastOffset, 3,
                        dimOffset + m_dimNums[3]);

                    u = x - lastX;
                    v = y - lastY;
                    w = z - lastZ;

                    if (u != 0 || v != 0) {
                        m_theta[dimOffset] = Math.atan2(v, u);
                        m_theta[dimOffset] *= (180 / Math.PI);
                    }
                }

                if (m_texture == null) {
                    float[] color = {m_color[0], m_color[1], m_color[2],
                            (float)m_trailAlpha};
                    // Set the color.
                    if (m_colorScheme == null || m_dimNums[4] < 0) {

                        gl.glColor4fv(color, 0);
                        color[3] *= 3;
                        gl.glMaterialfv(gl.GL_FRONT, gl.GL_AMBIENT_AND_DIFFUSE,
                            color, 0);
                    }
                    else {
// 1 redraw behind?
//float[] csColor = m_colorScheme.getColor(
//                            getValueAt(lastOffset, 4,
//                            dimOffset + m_dimNums[4]));
                        float[] csColor = m_colorScheme.getColor(
                            getValueAt(i, 4,
                            dimOffset + m_dimNums[4]));
                        color[0] = csColor[0];
                        color[1] = csColor[1];
                        color[2] = csColor[2];

                        gl.glColor4fv(color, 0);
                        color[3] *= 3;
                        gl.glMaterialfv(gl.GL_FRONT, gl.GL_AMBIENT_AND_DIFFUSE,
                            color, 0);
                    }

                    // For drawing a line connecting the tail points...
                    if (m_trailLineWidth > 0) {
                        if (lastX != lastY && x != y && offset > lastOffset) {
                            if (!Double.isNaN(lastX) && !Double.isNaN(lastY) &&
                                !Double.isNaN(lastZ) && !Double.isNaN(x) &&
                                !Double.isNaN(y) && !Double.isNaN(z)) {
				
				gl.glLineWidth(m_trailLineWidth);
				gl.glBegin(GL.GL_LINES);
				gl.glVertex3d(lastX, lastY, lastZ);
				gl.glVertex3d(x, y, z);
				gl.glEnd();
			    }
                        }
                    }

                    gl.glPushMatrix();
                gl.glScaled(1, 1, m_zScale);
                gl.glTranslated(x, y, z);
// rot                if (m_theta.length == (numDims - 1))
// rot                    gl.glRotatef((float)m_theta[j], 0, 0, 1);
                gl.glRotated(m_theta[dimOffset], 0, 0, 1);
// lit sphere                if (m_series.getNumDimensions() < 4)
                    //gl.glScalef(m_scale, m_scale, 1);
                gl.glScaled(m_scale, m_scale, m_scale * m_zStretch);
// lit sphere                else
// lit sphere                    gl.glScalef(m_scale, m_scale, 8);

                // Set the color.
//                gl.glColor4(m_color[0], m_color[1], m_color[2], m_trailAlpha);
// lit sphere                int z = 3;
// lit sphere                if (m_series.getNumDimensions() < 4)
// lit sphere                    z = -1;
                // Set the color.
// lit sphere                if (m_colorScheme == null || z < 0) {
// old color                    gl.glColor4(m_color[0], m_color[1], m_color[2],
// old color                        m_trailAlpha);
// lit sphere                }
// lit sphere                else {
// lit sphere                    float[] color = m_colorScheme.getColor(z);
// lit sphere                    gl.glColor4(color[0], color[1], color[2], m_trailAlpha);
// lit sphere                }



                    if (m_bDrawingCurrent || m_trailLineWidth <= 0) {
                        switch (m_symbolIndex) {
                            case 0:
                                gl.glScaled(m_trailScale, m_trailScale, 1);
                                drawTwoToneSquare();
                                break;
                            case 1:
                                gl.glScaled(m_trailScale, m_trailScale, 1);
                                drawSquare();
                                break;
                            case 2:
                                gl.glScaled(m_trailScale, m_trailScale, 1);
                                drawTwoToneFish();
                                break;
                            case 3:
                                gl.glScaled(m_trailScale, m_trailScale, 1);
                                drawFish();
                                break;
                            case 4:
                                gl.glScaled(m_trailScale, m_trailScale, 1);
                                drawLitSphere();
                                break;
                            case 5:
                                 gl.glScaled(m_trailScale, m_trailScale, 1);
                               drawBullsEye();
                                break;
                            case 6:
                                gl.glScaled(m_trailScale, m_trailScale, 1);
                                drawCircle();
                                break;
                            default:
                                gl.glScaled(m_trailScale, m_trailScale, 1);
                                drawTwoToneSquare();
                        }
                    }
                }
                else {
                    gl.glScaled(m_trailScale, m_trailScale, 1);
                    m_texture.drawGL();
                }

                // Draw markups if this is the current position.
                if (m_bDrawingCurrent) {
                    Iterator iter2 = m_children.iterator();
                    while (iter2.hasNext()) {
                        GrObject gro = (GrObject)iter2.next();
                        if (gro instanceof Markup) {
                            Markup m = (Markup)gro;
                            m.setOpenGL(gld, gl, glu);
                            m.drawGL();
                        }
                    }
                }

                gl.glPopMatrix();
            }
        }

        // Restore the previous line attributes.
        gl.glPopAttrib();

        if (m_symbolIndex == 4) turnOffLighting();

        m_lastBegin = begin;
        m_lastEnd = end;

        m_bNeedsRedraw = false;
    }

//------------------------------------------------------------------------------
/**
 * Set up lighting and turn it on.
 */
    public void turnOnLighting() {

        gl.glPushAttrib(gl.GL_POLYGON_BIT);
        gl.glPolygonMode(gl.GL_FRONT, gl.GL_FILL);
        gl.glCullFace(gl.GL_BACK);
        gl.glEnable(gl.GL_CULL_FACE);
//        gl.glEnable(gl.GL_DEPTH_TEST);

        float mat_specular[] = {1, 1, 1, 1};
//        float shininess[] = {50};
        float light_position[] = {-1, -1, 0.2f, 0};
        float white_light[] = {0.004f, 0.004f, 0.004f, 1};
        float lmodel_ambient[] = {0.6f, 0.6f, 0.6f, 1};

        gl.glMaterialfv(gl.GL_FRONT, gl.GL_SPECULAR, mat_specular, 0);
        gl.glMaterialf(gl.GL_FRONT, gl.GL_SHININESS, 100);
        gl.glLightfv(gl.GL_LIGHT0, gl.GL_POSITION, light_position, 0);
        gl.glLightfv(gl.GL_LIGHT0, gl.GL_DIFFUSE, white_light, 0);
        gl.glLightfv(gl.GL_LIGHT0, gl.GL_SPECULAR, white_light, 0);
        gl.glLightModelfv(gl.GL_LIGHT_MODEL_AMBIENT, lmodel_ambient, 0);

        gl.glEnable(gl.GL_LIGHTING);
        gl.glEnable(gl.GL_LIGHT0);

//        gl.glClear(gl.GL_DEPTH_BUFFER_BIT);
    }

//------------------------------------------------------------------------------
/**
 * Turn off lighting.
 */
    public void turnOffLighting() {

        gl.glDisable(gl.GL_LIGHTING);
        gl.glDisable(gl.GL_LIGHT0);

        gl.glPopAttrib();
        gl.glDisable(gl.GL_CULL_FACE);
//        gl.glDisable(gl.GL_DEPTH_TEST);
    }

//------------------------------------------------------------------------------
/**
 * Issue the OpenGL commands to draw one lit sphere.
 */
    public void drawLitSphere() {

//                        glut.glutSolidSphere(0.5, 6, 4);
        double scale = m_trailScale;
        if (scale <= 0.99)
            scale *= 0.7;
        glut.glutSolidSphere(0.5 * scale, 16, 8);
//        glut.glutSolidSphere(0.5 * scale, 3, 3);
//        glut.glutSolidTorus(0.3 * m_trailScale, 0.5 * m_trailScale, 16, 16);
    }

//------------------------------------------------------------------------------
/**
 * Issue the OpenGL commands to draw one two-tone square.
 */
    public void drawTwoToneSquare() {

        drawSquare();

        gl.glScalef(0.6f, 0.6f, 1);

        // Set the color.
        gl.glPushAttrib(gl.GL_CURRENT_BIT);
        gl.glColor4f(0, 0, 0, (float)m_trailAlpha);

        drawSquare();

        gl.glPopAttrib();
    }

//------------------------------------------------------------------------------
/**
 * Issue the OpenGL commands to draw one two-tone square.
 */
    public void drawBullsEye() {

        drawCircle();

        gl.glScalef(0.6f, 0.6f, 1);

        // Set the color.
        gl.glPushAttrib(gl.GL_CURRENT_BIT);
        gl.glColor4f(0, 0, 0, (float)m_trailAlpha);

        drawCircle();

        gl.glPopAttrib();
    }

//------------------------------------------------------------------------------
/**
 * Issue the OpenGL commands to draw one fish.
 */
    public void drawFish() {

        gl.glBegin(gl.GL_TRIANGLES);
        // Tail fin.
        gl.glVertex2f(-0.15f, 0);
        gl.glVertex2f(-0.25f, 0.092f);
        gl.glVertex2f(-0.25f, -0.092f);
        // Lower fin.
        gl.glVertex2f(0.2f, -0.05f);
        gl.glVertex2f(0.1f, -0.05f);
        gl.glVertex2f(0.05f, -0.15f);
        // Top fin.
        gl.glVertex2f(0.2f, 0.05f);
        gl.glVertex2f(0.1f, 0.05f);
        gl.glVertex2f(0.05f, 0.15f);
        gl.glEnd();

        // Body.
        gl.glBegin(gl.GL_TRIANGLE_FAN);
        gl.glVertex2f(-0.5f, 0);
        gl.glVertex2f(-0.2f, -0.04f);
        gl.glVertex2f(0.4f, -0.096f);
        gl.glVertex2f(0.5f, 0);
        gl.glVertex2f(0.4f, 0.096f);
        gl.glVertex2f(-0.2f, 0.04f);
        gl.glEnd();
    }

//------------------------------------------------------------------------------
/**
 * Issue the OpenGL commands to draw one two-tone fish.
 */
    public void drawTwoToneFish() {

        drawFish();

        // Draw black interior.
        // Set the color.
        gl.glPushAttrib(gl.GL_CURRENT_BIT);
        gl.glColor4f(0, 0, 0, (float)m_trailAlpha);

        gl.glBegin(gl.GL_TRIANGLES);
        // Tail fin.
        gl.glVertex2f(-0.16f, 0);
        gl.glVertex2f(-0.237f, 0.06f);
        gl.glVertex2f(-0.237f, -0.06f);
        // Lower fin.
        gl.glVertex2f(0.2f, -0.033f);
        gl.glVertex2f(0.1f, -0.096f);
        gl.glVertex2f(0.12f, -0.033f);
        // Top fin.
        gl.glVertex2f(0.2f, 0.033f);
        gl.glVertex2f(0.1f, 0.096f);
        gl.glVertex2f(0.12f, 0.033f);
        gl.glEnd();
        gl.glEnd();

        gl.glBegin(gl.GL_TRIANGLE_FAN);
        // Body.
        gl.glVertex2f(-0.42f, 0);
        gl.glVertex2f(-0.2f, -0.03f);
        gl.glVertex2f(0.38f, -0.048f);
        gl.glVertex2f(0.43f, 0);
        gl.glVertex2f(0.38f, 0.048f);
        gl.glVertex2f(-0.2f, 0.03f);
        gl.glEnd();

        gl.glPopAttrib();
    }

//------------------------------------------------------------------------------
/**
 * Issue the OpenGL commands to draw one square.
 */
    public void drawSquare() {

        gl.glBegin(gl.GL_QUADS);
        gl.glVertex2f(-0.5f, -0.5f);
        gl.glVertex2f(-0.5f, 0.5f);
        gl.glVertex2f(0.5f, 0.5f);
        gl.glVertex2f(0.5f, -0.5f);
        gl.glEnd();
    }

//------------------------------------------------------------------------------
/**
 * Issue the OpenGL commands to draw one square.
 */
    public void drawCircle() {

        int numPoints = m_circlePoints.length / 2;

        gl.glBegin(gl.GL_TRIANGLE_FAN);
        for (int i = 0; i < numPoints; i++) {
            gl.glVertex2f(m_circlePoints[i * 2], m_circlePoints[i * 2 + 1]);
        }
        gl.glEnd();
    }

//------------------------------------------------------------------------------
/**
 * Create a string representation of this object.
 * @return  The string that describes this object.
 */
    public String toString(){

        String s = null;
        if (m_bShow)
            s = "GL Particle: " + m_title;
        else
            s = "(GL Particle): " + m_title;

        return s;
    }
}






