/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2006-06-06
 * Date started:    2003-12-09
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr.visual.gl;

import com.sun.opengl.util.*;
import gov.usgs.gr.*;
import gov.usgs.gr.data.*;
import gov.usgs.gr.visual.*;
import java.util.*;
import javax.media.opengl.*;
import javax.media.opengl.glu.*;

//==============================================================================
/**
 * A series of discontinuous triangle fan surfaces.
 */
public class GlPrimitiveAnimation
	extends gov.usgs.gr.visual.DataAnimation implements GlElement {

    /** The OpenGL GLAutoDrawable component. */
    protected GLAutoDrawable gld = null;
    /**
     * The main OpenGL object with all the gl drawing methods.  For
     * readability, the m_ prefix wasn't used.
     */
    protected GL gl = null;
    /** The OpenGL utility class with all the glu drawing methods. */
    protected GLU glu = null;
    /** The OpenGL utility toolkit class with all the glut methods. */
    protected GLUT glut = null;

    /** The index of the OpenGL primitive to draw.
     *  0 = Points, 1 = Lines, 2 = Triangles, 3 = Quads
     */
    protected int m_primitiveIndex = 0;
    /** The thickness of the line or point in pixels. */
    protected double m_thickness = 1;
    /** Whether to use shading. */
    protected boolean m_bUseShading = true;
    /** Whether to use 3D. */
    protected boolean m_bUse3D = false;
    /** The tree icon to use for this class. */
	public static ArrayList TREE_ICON_HOLDER = new ArrayList(1);
    /** The location of the tree icon. */
    protected String m_iconFile = "GrObject/GlPrimitiveAnimation.gif";

//------------------------------------------------------------------------------
/**
 * Constructor for creating a blank data series.
 */
    public GlPrimitiveAnimation(){

        initialize();

        setDefaults("gov.usgs.gr.visual.gl.GlParticleAnimation");
    }

//------------------------------------------------------------------------------
/**
 * Constructor for creating a curve from a data series.
 */
    public GlPrimitiveAnimation(DataSeries series){

        initialize();

        setDefaults("gov.usgs.gr.visual.gl.GlParticleAnimation");

        add(series);
   }

//------------------------------------------------------------------------------
/**
 * Initialize all the member variables.
 */
    public void initialize() {

        m_numDimsAllowed = 5;
        m_symbolTypes = new String[] {"Points", "Lines", "Triangles", "Quads"};

        m_dataArrays = new double[m_numDimsAllowed][];
        m_seriesArray = new DoubleDataSeries[m_numDimsAllowed + 1];
        m_seriesNums = new int[] {1, 1, 1, 0, 0};
        m_dimNums = new int[] {0, 1, 2, -1, -1};
        m_seriesNumDims = new int[] {0, 0, 0, 0, 0};
        m_dimScales = new double[] {1, 1, 1, 1, 1};
        double[] array = new double[m_numDimsAllowed];
        m_constantSeries = new DoubleDataSeries(array, m_numDimsAllowed);

        m_color = new float[] {1, 0.2f, 0.2f};
    }

//------------------------------------------------------------------------------
/**
 * Provide the list of properties with the latest values.
 */
    public GrHashList getProperties() {

    	GrHashList props = super.getProperties();

        props.put("thickness", new GrProperty(m_thickness, "Thickness"));
        props.put("bUseShading", new GrProperty(m_thickness, "Use Shading"));
        props.put("bUse3D", new GrProperty(m_bUse3D, "Use 3D"));

        return props;
    }

//------------------------------------------------------------------------------
/**
 * Provide the list of properties with the latest values.
 */
    public void setProperties(HashMap props) {

		super.setProperties(props);

    	GrProperty p;

        Iterator iter = props.keySet().iterator();
        while (iter.hasNext()) {
        	String key = (String)iter.next();
            Object obj = props.get(key);
            if (obj instanceof GrProperty) {
				p = (GrProperty)obj;

            	if (key.equals("thickness")) {
            		m_thickness = (Double)p.value;
                }
                if (key.equals("useShading")) {
                    m_bUseShading = (Boolean)p.value;
                }
                if (key.equals("use3D")) {
                    m_bUse3D = (Boolean)p.value;
                }
                if (p.value instanceof String) {
				    String value = (String)p.value;
                    setProperty(key, value);
                }
            }
            if (obj instanceof String) {
				String value = (String)obj;
                setProperty(key, value);
            }
        }

        m_bNeedsRedraw = true;
        m_bHasChanged = true;
        notifyObservers();
    }

//------------------------------------------------------------------------------
/**
 * Set a single property.
 */
    public void setProperty(String key, String value) {

    	super.setProperty(key, value);

        if (key.equals("thickness")) {
            m_thickness = Double.parseDouble(value);
        }
        if (key.equals("useShading")) {
            m_bUseShading = Boolean.parseBoolean(value);
        }
        if (key.equals("use3D")) {
            m_bUse3D = Boolean.parseBoolean(value);
        }
    }

//------------------------------------------------------------------------------
/**
 * Accessor method.
 * @return An ImageIcon to use when representing this object in a tree.
 */
    public javax.swing.ImageIcon getTreeIcon() {

        if (TREE_ICON_HOLDER.size() <= 0)
            setTreeIcon();

    	return (javax.swing.ImageIcon)TREE_ICON_HOLDER.get(0);
    }

//------------------------------------------------------------------------------
/**
 * Loads the tree icon from a file and stores it in the TREE_ICON_HOLDER.
 */
    protected javax.swing.ImageIcon setTreeIcon() {

        javax.swing.ImageIcon icon = loadTreeIcon(m_iconFile);

        if (icon == null)
            icon = super.setTreeIcon();

        TREE_ICON_HOLDER.clear();
        TREE_ICON_HOLDER.add(icon);

        return icon;
    }

//------------------------------------------------------------------------------
/**
 * Sets the OpenGL objects to use for accessing drawing commands.
 * @param gld   The OpenGL GLAutoDrawable component.
 * @param gl    The OpenGL object containing the drawing commands.
 * @param glu   The OpenGL utilites object.
 */
    public void setOpenGL(GLAutoDrawable newgld, GL newgl, GLU newglu) {

        if (glut == null || newgl != gl || newglu != glu)
            glut = new GLUT();

        gld = newgld;
      	gl = newgl;
		glu = newglu;
    }

//------------------------------------------------------------------------------
/**
 *
 */
    public boolean checkForValidData() {

        if (m_seriesNums[1] != m_seriesNums[2])
            return false;

        if (m_seriesNums[1] != m_seriesNums[3]) {
            m_seriesNums[3] = -1;
            m_dimNums[3] = -1;
        }

        return true;
    }

//------------------------------------------------------------------------------
/**
 * Issue the OpenGL commands to draw this series.
 */
    public void drawGL() {

        // Before drawing, check if this object is supposed to be shown.
        if (!m_bShow) return;

        // Save the previous line color and pattern.
        gl.glPushAttrib(gl.GL_LINE_BIT | gl.GL_CURRENT_BIT);

        // Calculate the number of iterations for the
        // draw loop and the offsets for the T, Loc, and Vec arrays.
        int locOffset = 0;
        int vecOffset = 0;
        int endOffset = 0;
        DataSeries timeSeries = m_seriesArray[m_seriesNums[0]];
        synchronized (timeSeries) {
            if (!checkForValidData()) return;

            if (compareCounts(0, 1) != 0 || compareCounts(1, 4) != 0) {
                // Not a one-to-one relationship between times, locations,
                // and vectors.
                if (pointCount(4) % (pointCount(0) * pointCount(1)) != 0) {
                    // Number of times multiplied by number of locations is
                    // not equal to number of vectors, so don't draw anything.
                    return;
                }
                else {
                    // Number of times multiplied by number of locations is
                    // equal to number of elements, so proceed with drawing.
                    vecOffset = timeSeries.getBegin() * pointCount(1);
                    endOffset = pointCount(1) - 1;
                }

                // Check that the color series has the right number of points.
                if ((pointCount(0) * pointCount(1)) != pointCount(4)) {
                    m_dimNums[4] = -1;
                }
            }
            else {
                // One-to-one relationship between times, locations,
                // and vectors.
                locOffset = timeSeries.getBegin();
                vecOffset = 0;
                endOffset = timeSeries.getEnd();
            }
        }

        int numVertexes = m_symbolIndex + 1;
        if (numVertexes <= 0 || numVertexes > 4)
            return;

        gl.glPushMatrix();

        switch (m_symbolIndex) {
            case 0:
                gl.glBegin(gl.GL_POINTS);
                break;
            case 1:
                gl.glBegin(gl.GL_LINES);
                break;
            case 2:
                gl.glBegin(gl.GL_TRIANGLES);
                break;
            case 3:
                gl.glBegin(gl.GL_QUADS);
                break;
            default:
                gl.glBegin(gl.GL_POINTS);
                break;
        }

        for (int i = locOffset; i <= endOffset; i += numVertexes) {

            for (int vNum = 0; vNum < numVertexes; vNum++) {
                double x = getValueAt(i + vNum, 1);
                double y = getValueAt(i + vNum, 2);
                double z = 0;
                if (m_dimNums[3] >= 0) {
                    z = getValueAt(i + vNum, 3);
                }

                // Set the color.
                if (m_colorScheme == null || m_dimNums[4] < 0) {
                    gl.glColor3fv(m_color, 0);
                }
                else {
                    gl.glColor3fv(m_colorScheme.getColor(
                        getValueAt(i + vNum, 4)), 0);
                }

                gl.glVertex3d(x, y, z);
            }
        }
        gl.glEnd();
        gl.glPopMatrix();

        // Restore the previous line attributes.
        gl.glPopAttrib();

        m_bNeedsRedraw = false;
    }

//------------------------------------------------------------------------------
/**
 * Set up lighting and turn it on.
 */
    public void turnOnLighting() {

        gl.glPushAttrib(gl.GL_POLYGON_BIT);
        gl.glPolygonMode(gl.GL_FRONT, gl.GL_FILL);
        gl.glCullFace(gl.GL_BACK);
        gl.glEnable(gl.GL_CULL_FACE);
//        gl.glEnable(gl.GL_DEPTH_TEST);

        float mat_specular[] = {1, 1, 1, 1};
//        float shininess[] = {50};
        float light_position[] = {-1, -1, 0.2f, 0};
        float white_light[] = {0.004f, 0.004f, 0.004f, 1};
        float lmodel_ambient[] = {0.6f, 0.6f, 0.6f, 1};

        gl.glMaterialfv(gl.GL_FRONT, gl.GL_SPECULAR, mat_specular, 0);
        gl.glMaterialf(gl.GL_FRONT, gl.GL_SHININESS, 100);
        gl.glLightfv(gl.GL_LIGHT0, gl.GL_POSITION, light_position, 0);
        gl.glLightfv(gl.GL_LIGHT0, gl.GL_DIFFUSE, white_light, 0);
        gl.glLightfv(gl.GL_LIGHT0, gl.GL_SPECULAR, white_light, 0);
        gl.glLightModelfv(gl.GL_LIGHT_MODEL_AMBIENT, lmodel_ambient, 0);

        gl.glEnable(gl.GL_LIGHTING);
        gl.glEnable(gl.GL_LIGHT0);

//        gl.glClear(gl.GL_DEPTH_BUFFER_BIT);
    }

//------------------------------------------------------------------------------
/**
 * Turn off lighting.
 */
    public void turnOffLighting() {

        gl.glDisable(gl.GL_LIGHTING);
        gl.glDisable(gl.GL_LIGHT0);

        gl.glPopAttrib();
        gl.glDisable(gl.GL_CULL_FACE);
//        gl.glDisable(gl.GL_DEPTH_TEST);
    }

//------------------------------------------------------------------------------
/**
 * Create a string representation of this object.
 * @return  The string that describes this object.
 */
    public String toString(){

        String s = null;
        if (m_bShow)
            s = "GL Primitive: " + m_title;
        else
            s = "(GL Primitive): " + m_title;

        return s;
    }
}






