/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2006-06-06
 * Date started:    2001-08-30
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr.visual.gl;

import gov.usgs.gr.*;
import gov.usgs.gr.data.*;
import gov.usgs.gr.tools.*;
import gov.usgs.gr.visual.*;
import java.util.*;
import javax.media.opengl.*;
import javax.media.opengl.glu.*;

//==============================================================================
/**
 * A series of discontinuous triangle fan surfaces.
 */
public class GlVelocityProfile
	extends gov.usgs.gr.visual.GraphicalElement implements GlElement {

    /** The OpenGL GLAutoDrawable component. */
    protected GLAutoDrawable gld = null;
    /**
     * The main OpenGL object with all the gl drawing methods.  For
     * readability, the m_ prefix wasn't used.
     */
    protected GL gl = null;
    /** The OpenGL utility class with all the glu drawing methods. */
    protected GLU glu = null;

    public DoubleDataSeries m_series = new DoubleDataSeries(4);
    public DoubleDataSeries m_rotatedSeries = new DoubleDataSeries(4);
    protected double m_prinDir = 90;
    protected double m_scale = 0;
    protected double m_xPos = 0;
    protected double m_yPos = 0;
    protected double m_width = 0;
    protected double m_height = 0;
    protected float[] m_color = {1, 0.2f, 0.2f};
    protected float[] m_bgColor = {0, 0, 0};
    protected boolean m_bDrawBG = true;
    protected boolean m_bDrawProjection = false;
    protected boolean m_bScaleGrid = true;
    protected boolean m_bUseU = true;
    protected boolean m_bUseV = false;
    protected boolean m_bUseW = false;
    protected boolean m_bUseZ = true;
    /** The thickness of the line in pixels. */
    protected float m_lineWidth = 2;

    protected TimeMarker m_timeMarker = new TimeMarker();;
    protected ColorScheme m_colorScheme = null;
    protected Gl3dAxisGrid m_3dAxisGrid = null;
    /**
     * The numeric code of the symbol to be used.
     *  0 = simple arrow.
     *  1 = arrow with black inner area.
     *  2 = bar
     */
    protected int m_symbolIndex = 1;
    /** The tree icon to use for this class. */
	public static ArrayList TREE_ICON_HOLDER = new ArrayList(1);
    /** The location of the tree icon. */
    protected String m_iconFile = "GrObject/GlVelocityProfile.gif";

//------------------------------------------------------------------------------
/**
 * Constructor for creating a blank data series.
 */
    public GlVelocityProfile(){

        setDefaults("gov.usgs.gr.visual.gl.GlVelocityProfile");
    }

//------------------------------------------------------------------------------
/**
 * Constructor for creating a curve from a data series.
 */
    public GlVelocityProfile(DoubleDataSeries series){

        add(series);

        setDefaults("gov.usgs.gr.visual.gl.GlVelocityProfile");
   }

//------------------------------------------------------------------------------
/**
 * Add a child object.  Accepts DoubleDataSeries, TimeMarker.  Only holds one DS at
 * a time.
 */
    public boolean add(GrObject gro) {

        // If gro is an object effigy, get the eventual subject and try
        // adding that.
        while (gro instanceof gov.usgs.gr.ObjectEffigy)
        	gro = ((ObjectEffigy)gro).getSubject();

    	if (m_children.contains(gro)) return false;

    	if (gro instanceof DoubleDataSeries) {
            remove(m_series);
            super.add(gro);
            m_series = (DoubleDataSeries)gro;
            generateRotatedSeries();

            return true;
        }
        else {
        	if (gro instanceof TimeMarker) {
            	if (m_timeMarker != null)
                	remove(m_timeMarker);
                super.add(gro);
                m_timeMarker = (TimeMarker)gro;

                return true;
        	}
            if (gro instanceof ColorScheme) {
                if (m_colorScheme != null)
                    remove((GrObject)m_colorScheme);
                super.add(gro);
                m_colorScheme = (ColorScheme)gro;

                return true;
            }
            if (gro instanceof Gl3dAxisGrid) {
                if (m_3dAxisGrid != null)
                    remove((GrObject)m_3dAxisGrid);
                super.add(gro);
                m_3dAxisGrid = (Gl3dAxisGrid)gro;

                return true;
            }
        }

        return false;
    }

//------------------------------------------------------------------------------
/**
 * Accessor method.
 * @return An ImageIcon to use when representing this object in a tree.
 */
    public javax.swing.ImageIcon getTreeIcon() {

        if (TREE_ICON_HOLDER.size() <= 0)
            setTreeIcon();

    	return (javax.swing.ImageIcon)TREE_ICON_HOLDER.get(0);
    }

//------------------------------------------------------------------------------
/**
 * Loads the tree icon from a file and stores it in the TREE_ICON_HOLDER.
 */
    protected javax.swing.ImageIcon setTreeIcon() {

        javax.swing.ImageIcon icon = loadTreeIcon(m_iconFile);

        if (icon == null)
            icon = super.setTreeIcon();

        TREE_ICON_HOLDER.clear();
        TREE_ICON_HOLDER.add(icon);

        return icon;
    }

//------------------------------------------------------------------------------
/**
 * Provide the list of properties with the latest values.
 */
    public GrHashList getProperties() {

    	GrHashList props = super.getProperties();

        props.put("prinDir", new GrProperty(m_prinDir, "Principal Direction"));
        props.put("scale", new GrProperty(m_scale, "Scale"));
        props.put("xPos", new GrProperty(m_xPos, "X Position"));
        props.put("yPos", new GrProperty(m_yPos, "Y Position"));
        props.put("width", new GrProperty(m_width, "Width"));
        props.put("height", new GrProperty(m_height, "Height"));
        props.put("drawBG", new GrProperty(m_bDrawBG, "Draw Background"));
        props.put("drawProjection", new GrProperty(m_bDrawProjection,
            "Draw Projection"));
        props.put("scaleGrid", new GrProperty(m_bScaleGrid,
            "Scale Grid to Profile"));
        props.put("useV", new GrProperty(m_bUseV, "Use V"));

        ColorProperty cp = new ColorProperty();
        cp.description = "Color";
        cp.setColor(m_color);
        props.put("color", cp);
        props.put("lineWidth", new GrProperty(m_lineWidth, "Line Width"));

        return props;
    }

//------------------------------------------------------------------------------
/**
 * Provide the list of properties with the latest values.
 */
    public void setProperties(HashMap props) {

		super.setProperties(props);

    	GrProperty p;

        Iterator iter = props.keySet().iterator();
        while (iter.hasNext()) {
        	String key = (String)iter.next();
            Object obj = props.get(key);
            if (obj instanceof GrProperty) {
				p = (GrProperty)obj;

            	if (key.equals("prinDir")) {
            		m_prinDir = (Double)p.value;
                    while (m_prinDir > 360) m_prinDir -= 360;
                    while (m_prinDir < -360) m_prinDir += 360;
                    generateRotatedSeries();
                }
            	if (key.equals("scale")) {
            		m_scale = (Double)p.value;
                    generateRotatedSeries();
            	}
            	if (key.equals("xPos"))
            		m_xPos = (Double)p.value;
            	if (key.equals("yPos"))
            		m_yPos = (Double)p.value;
            	if (key.equals("width"))
            		m_width = (Double)p.value;
            	if (key.equals("height"))
            		m_height = (Double)p.value;
                if (key.equals("drawBG"))
                    m_bDrawBG = (Boolean)p.value;
                if (key.equals("drawProjection"))
                    m_bDrawProjection = (Boolean)p.value;
                if (key.equals("scaleGrid"))
                    m_bScaleGrid = (Boolean)p.value;
                if (key.equals("useV"))
                    m_bUseV = (Boolean)p.value;
            	if (key.equals("color")) {
                    ColorProperty cp = (ColorProperty)p;
                    m_color = cp.getColorArray();
                }
            	if (key.equals("lineWidth"))
            		m_lineWidth = (Float)p.value;
            }
            if (obj instanceof String) {
				String value = (String)obj;
                setProperty(key, value);
            }
        }

        m_bNeedsRedraw = true;
        m_bHasChanged = true;
        notifyObservers();
    }

//------------------------------------------------------------------------------
/**
 * Set a single property.
 */
    public void setProperty(String key, String value) {

    	super.setProperty(key, value);

        if (key.equals("prinDir")) {
            m_prinDir = Double.parseDouble(value);
            while (m_prinDir > 360) m_prinDir -= 360;
            while (m_prinDir < -360) m_prinDir += 360;
            generateRotatedSeries();
        }
        if (key.equals("scale")) {
            m_scale = Double.parseDouble(value);
            generateRotatedSeries();
        }
        if (key.equals("xPos"))
            m_xPos = Double.parseDouble(value);
        if (key.equals("yPos"))
            m_yPos = Double.parseDouble(value);
        if (key.equals("width"))
            m_width = Double.parseDouble(value);
        if (key.equals("height"))
            m_height = Double.parseDouble(value);
        if (key.equals("drawBG"))
            m_bDrawBG = Boolean.parseBoolean(value);
        if (key.equals("drawProjection"))
            m_bDrawProjection = Boolean.parseBoolean(value);
        if (key.equals("scaleGrid"))
            m_bScaleGrid = Boolean.parseBoolean(value);
        if (key.equals("useV"))
            m_bUseV = Boolean.parseBoolean(value);
        if (key.equals("color")) {
            m_color = ColorProperty.getColorArray(value);
        }
        if (key.equals("lineWidth"))
            m_lineWidth = Float.parseFloat(value);
    }

//------------------------------------------------------------------------------
/**
 * Hide or show this series.
 * @param  bShow    True if this curve should be drawn.
 */
    public void show(boolean bShow){
        m_bShow = bShow;
    }

//------------------------------------------------------------------------------
/**
 * Sets the OpenGL objects to use for accessing drawing commands.
 * @param gld   The OpenGL GLAutoDrawable component.
 * @param gl    The OpenGL object containing the drawing commands.
 * @param glu   The OpenGL utilites object.
 */
    public void setOpenGL(GLAutoDrawable newgld, GL newgl, GLU newglu) {
        gld = newgld;
      	gl = newgl;
		glu = newglu;
    }

//------------------------------------------------------------------------------
/**
 * Issue the OpenGL commands to draw this series.
 */
    public void drawGL() {

        // Before drawing, check if this object is supposed to be shown.
        if (!m_bShow) return;

        synchronized (m_series) {
            // Save the previous line color and pattern.
            gl.glPushAttrib(gl.GL_LINE_BIT | gl.GL_CURRENT_BIT);

            int tDim = 0;
            int zDim = 1;
            int uDim = 2;
            int vDim = 3;

            double scale = m_scale;
            if (m_scale == 0) {
                double rightSide = Math.abs(m_rotatedSeries.getMax(uDim));
                double leftSide = Math.abs(m_rotatedSeries.getMin(uDim));
                double bigger = (rightSide > leftSide) ? rightSide : leftSide;
                scale = m_width / 2 / bigger;
            }

            double zScale = m_height / (m_rotatedSeries.getMax(zDim) + 1);

            // Draw a black background.
            if (m_bDrawBG) {
                gl.glColor3fv(m_bgColor, 0);
                gl.glBegin(gl.GL_QUADS);
                gl.glVertex2d(m_xPos, m_yPos);
                gl.glVertex2d(m_xPos + m_width, m_yPos);
                gl.glVertex2d(m_xPos + m_width, m_yPos + m_height);
                gl.glVertex2d(m_xPos, m_yPos + m_height);
                gl.glVertex2d(m_xPos, m_yPos);
                gl.glEnd();
            }

            gl.glLineWidth(m_lineWidth);

            int numDims = m_rotatedSeries.getNumDimensions();
            double[] array = m_rotatedSeries.getArray();

            gl.glPushMatrix();
            if (m_bDrawBG) {
                gl.glRotated(-90, 1, 0, 0);
            }
            gl.glScaled(scale, scale, scale);
            gl.glTranslated(m_xPos, m_yPos, 0);

            if (m_3dAxisGrid != null) {
                m_3dAxisGrid.setOpenGL(gld, gl, glu);
                m_3dAxisGrid.drawGL();
            }

            // Draw the arrow lines.
            gl.glBegin(gl.GL_LINES);
            for (int i = m_series.getBegin(); i <= m_series.getEnd(); i++) {
                int offset = i * numDims;
                double z = array[offset + zDim];
                double u = array[offset + uDim];
                double v = (m_bUseV ? array[offset + vDim] : 0);

                // Set the color.
                if (m_colorScheme == null) {
                    gl.glColor3fv(m_color, 0);
                }
                else {
                    gl.glColor3fv(m_colorScheme.getColor(z), 0);
                }

                gl.glVertex3d(0, 0, z * zScale);
                gl.glVertex3d(u, v, z * zScale);
//                gl.glVertex3d(0, 0, z * zScale);
//                gl.glVertex3d(u, 0, z * zScale);
            }
            gl.glEnd();

            gl.glColor3f(0.2f, 1, 0.5f);
            gl.glBegin(gl.GL_LINE_STRIP);
            for (int i = m_series.getBegin(); i <= m_series.getEnd(); i++) {
                int offset = i * numDims;
                double z = array[offset + zDim];
                double u = array[offset + uDim];
                double v = (m_bUseV ? array[offset + vDim] : 0);

                gl.glVertex3d(u, v, z * zScale);
            }
            gl.glEnd();

            if (m_bDrawProjection) {
                // Draw the projection.
                gl.glBegin(gl.GL_LINES);
                for (int i = m_series.getBegin(); i <= m_series.getEnd(); i++) {
                    int offset = i * numDims;
                    double z = array[offset + zDim];;
                    double u = array[offset + uDim];
                    double v = (m_bUseV ? array[offset + vDim] : 0);

                    // Set the color.
                    if (m_colorScheme == null) {
                        gl.glColor3fv(m_color, 0);
                    }
                    else {
                        gl.glColor3fv(m_colorScheme.getColor(z), 0);
                    }

                    gl.glVertex3d(0, 0, 0);
                    gl.glVertex3d(u, v, 0);
                }
                gl.glEnd();
            }

            // Draw a vertical stick line at the anchor point.
            gl.glColor3f(1 - m_bgColor[0], 1 - m_bgColor[1], 1 - m_bgColor[2]);
            gl.glBegin(gl.GL_LINES);
            gl.glVertex3d(0, 0, 0);
            gl.glVertex3d(0, 0, m_height);
            gl.glEnd();

            gl.glPopMatrix();

            // Restore the previous line attributes.
            gl.glPopAttrib();
        }

        m_bNeedsRedraw = false;
    }

//------------------------------------------------------------------------------
/**
 * Overrides default update() in GrObject.  Called by an observed GrObject's
 * notifyObservers() method to handle changes to the observed object.
 * May use the notifyObservers() method to propagate the event handling up to
 * this object's observers.
 * @param gro   The observed object that changed.
 * @param arg   Additional information that the updated object may choose to
 *              send.
 */
    public void update(GrObject gro, Object arg) {

    	if (gro == m_series) generateRotatedSeries();

        // Check if the object is a graphical element that needs to be redrawn.
        if (gro instanceof DoubleDataSeries) {
            m_bNeedsRedraw = true;
        }

        m_bHasChanged = true;
        notifyObservers();
    }

//------------------------------------------------------------------------------
/**
 * Rotate the source series to the principal direction.
 */
    public void generateRotatedSeries() {

    	double[] sourceArray = m_series.getArray();
        double[] rotatedArray = new double[m_series.getCount() * 4];

        int sourceTdim = 0;
        int sourceZdim = 1;
        int sourceUdim = 2;
        int sourceVdim = 3;
        int rotTdim = 0;
        int rotZdim = 1;
        int rotUdim = 2;
        int rotVdim = 3;

        for (int i = 0; i < m_series.getCount(); i++) {
            rotatedArray[i * 4 + rotTdim] = sourceArray[i * 4 + sourceTdim];
            rotatedArray[i * 4 + rotZdim] = sourceArray[i * 4 + sourceZdim];

            double u = sourceArray[i * 4 + sourceUdim];
            double v = sourceArray[i * 4 + sourceVdim];
            double len = Math.sqrt((u * u) + (v * v));
            double theta = Math.atan2(v, u);
            double correctedRotation = (m_prinDir - 90) * (Math.PI / 180);
            double newU = 0;
            double newV = 0;
            if (m_bUseV) {
                double cosVal = Math.cos(correctedRotation);
                double sinVal = Math.sin(correctedRotation);
                newU = u * cosVal - v * sinVal;
                newV = v * cosVal + u * sinVal;
            }
            else {
                theta += correctedRotation;
                newU = Math.cos(theta) * len;
            }

            rotatedArray[i * 4 + rotUdim] = newU;
            rotatedArray[i * 4 + rotVdim] = newV;
        }

        m_rotatedSeries.setArray(rotatedArray);

        if (m_3dAxisGrid != null && m_bScaleGrid) {
            Zoom xZoom = m_3dAxisGrid.getXZoom();
            Zoom yZoom = m_3dAxisGrid.getYZoom();
            xZoom.setContentLimits(m_rotatedSeries.getMin(2),
                m_rotatedSeries.getMax(2));
            yZoom.setContentLimits(m_rotatedSeries.getMin(3),
                m_rotatedSeries.getMax(3));
            xZoom.zoomAll(2);
            yZoom.zoomAll(2);
        }
    }

//------------------------------------------------------------------------------
/**
 * Create a string representation of this object.
 * @return  The string that describes this object.
 */
    public String toString(){

        String s = null;
        if (m_bShow)
            s = "GL Velocity Profile: " + m_title;
        else
            s = "(GL Velocity Profile): " + m_title;

        return s;
    }
}






