/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2006-04-06
 * Date started:    1999-01-16
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr.visual.gl;

import gov.usgs.gr.visual.*;
import java.awt.*;
import java.text.*;
import java.util.*;
import javax.media.opengl.*;
import com.sun.opengl.util.*;

//==============================================================================
/**
 * The Y version of the axis object.
 */
public class GlYAxis extends GlAxis {

    /** The tree icon to use for this class. */
	public static ArrayList TREE_ICON_HOLDER = new ArrayList(1);
    /** The location of the tree icon. */
    protected String m_iconFile = "GrObject/GlYAxis.gif";

//------------------------------------------------------------------------------
/**
 * Empty constructor.
 */
    public GlYAxis () {

    	m_dimension = 1;

        setDefaults("gov.usgs.gr.visual.gl.GlYAxis");
    }

//------------------------------------------------------------------------------
/**
 * Comment.
 * @param limits	The limits object telling what segment of the axis should
 *					be represented.
 * @param opposingLimits	The opposing axis' limits.  Used to determine the
 *							how the length of tick marks when drawing in
 *							graph coordinates.
 */
    public GlYAxis (Zoom zoom, Zoom opposingZoom) {

        super(zoom, opposingZoom);
    	m_dimension = 1;

        setDefaults("gov.usgs.gr.visual.gl.GlYAxis");
    }

//------------------------------------------------------------------------------
/**
 * Create a new axis by copying an existing one.
 * @param axis	The axis to be copied.
 */
    public GlYAxis (Axis axis) {

    	super(axis);
    	m_dimension = 1;
    }

//------------------------------------------------------------------------------
/**
 * Accessor method.
 * @return An ImageIcon to use when representing this object in a tree.
 */
    public javax.swing.ImageIcon getTreeIcon() {

        if (TREE_ICON_HOLDER.size() <= 0)
            setTreeIcon();

    	return (javax.swing.ImageIcon)TREE_ICON_HOLDER.get(0);
    }

//------------------------------------------------------------------------------
/**
 * Loads the tree icon from a file and stores it in the TREE_ICON_HOLDER.
 */
    protected javax.swing.ImageIcon setTreeIcon() {

        javax.swing.ImageIcon icon = loadTreeIcon(m_iconFile);

        if (icon == null)
            icon = super.setTreeIcon();

        TREE_ICON_HOLDER.clear();
        TREE_ICON_HOLDER.add(icon);

        return icon;
    }

//------------------------------------------------------------------------------
/**
 * Overrides the draw() function from Axis.  Rotates the axis so it will
 * be drawing vertically.
 * <p><b> Pre Conditions </b><br>
 *      gl has been set to a valid OpenGL drawing object.
 */
    public void drawGL() {

    	// Get the latest from the axis limits objects.
        update();
        if (m_majorTickInc <= 0) return;

        // Save the current transformation and then move the axis to its
        // proper location rotated 90 degrees.
        gl.glPushMatrix();
        gl.glTranslated(m_oppMin, -m_min, 0);
        gl.glRotated(90, 0, 0, 1);
        gl.glTranslated(m_min, -m_oppMax, 0);

        // Draw the axis restore the previous transformation.
        super.drawGL();
        gl.glPopMatrix();

        m_bNeedsRedraw = false;
    }

//------------------------------------------------------------------------------
/**
 * Draws labels in magenta beside each major tick mark.
 * <p><b> Pre Conditions </b><br>
 *      gl is a valid OpenGL context.
 */
    public void drawText(double xOrig, double yOrig, double axisDisplayWidth,
    	int fontIndex) {
//    	GLSolidFont font) {

        if (!m_bShowText || m_majorTickInc <= 0) return;

        double x;

//        Graphics2D g = (Graphics2D)((GLComponent)gld).getGraphics();

        // Save the current position
        gl.glPushMatrix();

        yOrig -= 4;
        if (yOrig < 0) yOrig = 0;
        gl.glTranslated(xOrig, yOrig, 0);
        double displayScale = axisDisplayWidth / (m_max - m_min);

        NumberFormat form = new DecimalFormat(m_zoom.getFormatString());

        // Loop through and draw each label.
        for (x = m_majorStart; (x - m_max) <= (0.01 * m_majorTickInc);
        	 x += m_majorTickInc) {

            if (Math.abs(x - 0) <= (0.01 * m_majorTickInc)) x = 0;
        	String output = form.format(x);
            double hPos = -7	- (5 * output.length());
            if (hPos < -xOrig) hPos = -xOrig;
            gl.glRasterPos2d(hPos, ((x - m_min) * displayScale));
///            font.drawString(output);
            glut.glutBitmapString(fontIndex, output);
        }

        // Restore the previous position.
        gl.glPopMatrix();

        // Draw the axis title.
        if (m_bShowTitle) {
        	// Save the current position
        	gl.glPushMatrix();

        	gl.glTranslated(0, ((m_max - m_min) * displayScale) + yOrig + 9, 0);
            StringTokenizer st = new StringTokenizer(m_title, "" + '\n');

            while (st.hasMoreTokens()) {
            	gl.glRasterPos2f(0f, 0f);
///            	font.drawString(st.nextToken());
                glut.glutBitmapString(fontIndex, st.nextToken());
        		gl.glTranslated(0, -10, 0);
            }

        	// Restore the previous position.
        	gl.glPopMatrix();
        }
    }

//------------------------------------------------------------------------------
/**
 * Create a string representation of this object.
 * @return  The string that describes this object.
 */
    public String toString() {

    	String s = "Y Axis";

        return s;
 	}
}



