/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2007-04-26
 * Date started:    2000-01-11
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr.visual.gui;

import gov.usgs.gr.data.*;
import gov.usgs.gr.*;
import gov.usgs.gr.visual.*;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.text.*;
import java.util.*;
import javax.swing.event.*;

//==============================================================================
/**
 * A panel that acts as a status bar by displaying labels.
 */
public class CurveStatusBar extends GrStatusBar {

	GrController m_grCon = null;
    Curve m_curve = null;
    DataSeries m_series = null;
	public boolean m_bSelectWithClick = false;
    float[] m_color = {0f, 0f, 0f};

	BorderLayout borderLayout0 = new BorderLayout();
    FlowLayout flowLayout1 = new FlowLayout();

    GridBagLayout gridBagLayout1W = new GridBagLayout();
	GridBagLayout gridBagLayout1E = new GridBagLayout();
	GridBagLayout gridBagLayout2W = new GridBagLayout();
	GridBagLayout gridBagLayout2E = new GridBagLayout();
    BorderLayout borderLayout1 = new BorderLayout();
    BorderLayout borderLayout2 = new BorderLayout();

    JPanel panel1 = new JPanel();
    JPanel panel2 = new JPanel();
    JPanel panel1W = new JPanel();
    JPanel panel1E = new JPanel();
    JPanel panel2W = new JPanel();
    JPanel panel2E = new JPanel();

	JTextField formatTextField = new JTextField(7);
	JToggleButton dim1LockedButton = new JToggleButton();
	JToggleButton dim2LockedButton = new JToggleButton();

    JPanel selectMethodPanel = new JPanel();
	GridLayout selectMethodLayout = new GridLayout(2, 1);
    ButtonGroup selectMethodGroup = new ButtonGroup();
    JRadioButton underMouseRadio = new JRadioButton("Under Mouse");
    JRadioButton clickRadio = new JRadioButton("With Click");

	JLabel row1DimLabel = new JLabel();
	JLabel row2DimLabel = new JLabel();
	JLabel col3HeadLabel = new JLabel();
	JLabel col4HeadLabel = new JLabel();
	JTextField minXLabel = new JTextField(8);
	JTextField minYLabel = new JTextField(8);
	JLabel col5HeadLabel = new JLabel();
	JTextField maxXLabel = new JTextField(8);
	JTextField maxYLabel = new JTextField(8);

	JCheckBox showCheckBox = new JCheckBox();
	JCheckBox sortedXCheckBox = new JCheckBox();
	JCheckBox sortedYCheckBox = new JCheckBox();

	JComboBox curveListCombo = new JComboBox();
	JTextField curveNameTextField = new JTextField(70);
	JTextField numPointsLabel = new JTextField(8);

	JCheckBox lineCheckBox = new JCheckBox();
	JComboBox linePatternCombo = new JComboBox();
    JTextField lineThicknessTextField = new JTextField(4);

	JCheckBox symbolsCheckBox = new JCheckBox();
	JComboBox lineSymbolsCombo = new JComboBox();
    JTextField symbolSizeTextField = new JTextField(4);

	JSlider redSlider = new JSlider(0, 100);
	JSlider greenSlider = new JSlider(0, 100);
	JSlider blueSlider = new JSlider(0, 100);

	JLabel redLabel = new JLabel();
	JLabel greenLabel = new JLabel();
	JLabel blueLabel = new JLabel();
    JPanel colorPanel = new JPanel();

	JLabel axesLabel = new JLabel();
	JLabel dataLockHeadingLabel = new JLabel();
    JButton applyButton = new JButton("Apply");

//------------------------------------------------------------------------------
/**
 * Create an default status bar.
 */
    public CurveStatusBar(GrController grCon) {

        m_grCon = grCon;

        try  {
            jbInit();
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }

        updateCurveList();
    }

//------------------------------------------------------------------------------
/**
 * Sets up the contents of the status bar.  This method can be used by
 * Borland JBuilder to visually design the panel.
 */
    void jbInit() throws Exception {

        // Set coordinate label font to monospaced so the characters won't
        // jump around as the numbers change.
        nearestXLabel.setFont(new Font("Monospaced", Font.PLAIN, 11));
        nearestYLabel.setFont(new Font("Monospaced", Font.PLAIN, 11));

        // Add a mouse listener so the user can cycle the date format
        // with Ctrl-Click.
        nearestXLabel.addMouseListener(new java.awt.event.MouseListener() {
            public void mousePressed(MouseEvent e) {
                boolean bCtrl = ((e.getModifiers() & e.CTRL_MASK) != 0);
                if (bCtrl) {
                    m_dateFormatIndex++;
                    if (m_dateFormatIndex >= m_dateFormats.length)
                        m_dateFormatIndex = 0;
                    dateFormat = m_dateFormats[m_dateFormatIndex];
                    nearestXLabel.setText(dateFormat);
                }
            }
            public void mouseEntered(MouseEvent e) {}
            public void mouseClicked(MouseEvent e) {}
            public void mouseExited(MouseEvent e) {}
            public void mouseReleased(MouseEvent e) {}
        });

        // Add a mouse listener so the user can cycle the coordinate mode
        // with Ctrl-Click.
        nearestYLabel.addMouseListener(new java.awt.event.MouseListener() {
            public void mousePressed(MouseEvent e) {
                boolean bCtrl = ((e.getModifiers() & e.CTRL_MASK) != 0);
                if (bCtrl) {
                    m_coordLevelIndex++;
                    if (m_coordLevelIndex >= m_coordLevels.length)
                        m_coordLevelIndex = 0;
                    String text = m_coordLevels[m_coordLevelIndex];
                    nearestYLabel.setText(text);
                    nearestXLabel.setToolTipText("X Value of "
                        + m_coordDescriptions[m_coordLevelIndex]);
                    nearestYLabel.setToolTipText("Y Value of "
                        + m_coordDescriptions[m_coordLevelIndex]);
                }
            }
            public void mouseEntered(MouseEvent e) {}
            public void mouseClicked(MouseEvent e) {}
            public void mouseExited(MouseEvent e) {}
            public void mouseReleased(MouseEvent e) {}
        });

		this.setLayout(borderLayout0);
        applyButton.addActionListener(new java.awt.event.ActionListener() {

			public void actionPerformed(ActionEvent e) {
				applyButton_actionPerformed(e);
			}
		});
		this.add(panel1, BorderLayout.NORTH);
        this.add(panel2, BorderLayout.SOUTH);

        panel1.setLayout(borderLayout1);
        panel1.add(panel1W, BorderLayout.WEST);
        panel1W.setLayout(gridBagLayout1W);
        panel1.add(panel1E, BorderLayout.EAST);
        panel1E.setLayout(gridBagLayout1E);
        panel2.setLayout(borderLayout2);
        panel2.add(panel2W, BorderLayout.WEST);
        panel2W.setLayout(gridBagLayout2W);
//        panel2.add(panel2E, BorderLayout.EAST);
        panel2E.setLayout(gridBagLayout2E);

        selectMethodPanel.setBorder(
        	BorderFactory.createTitledBorder("Choose Curve"));
		selectMethodPanel.setLayout(selectMethodLayout);
		underMouseRadio.addChangeListener(new javax.swing.event.ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				underMouseRadio_stateChanged(e);
			}
		});
		selectMethodPanel.add(underMouseRadio);
        underMouseRadio.setToolTipText(
        	"Lets the user choose a curve by moving the mouse over it");
		selectMethodPanel.add(clickRadio);
        clickRadio.setToolTipText(
        	"Lets the user choose a curve by clicking it");
    	selectMethodGroup.add(underMouseRadio);
    	selectMethodGroup.add(clickRadio);
        underMouseRadio.setSelected(true);

		formatTextField.setText("0.00000");
        formatTextField.setToolTipText("Coordinate format");

		modeLabel.setText("Pan");
        modeLabel.setToolTipText("Current mode");

		switchBarButton.setText("> Less <");
        switchBarButton.setToolTipText("Switch to Default Status Bar");
        switchBarButton.setActionCommand("switchBarButton");
		switchBarButton.addActionListener(new java.awt.event.ActionListener() {

			public void actionPerformed(ActionEvent e) {
				switchBarButton_actionPerformed(e);
			}
		});

		axesLabel.setText("Axes");
		axis1LockButton.setText("U");
        axis1LockButton.setToolTipText("Whether X axes are locked");
		axis2LockButton.setText("U");
        axis2LockButton.setToolTipText("Whether Y axes lre locked");
		axis1LockButton.addChangeListener(
        	new javax.swing.event.ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				axis1LockButton_stateChanged(e);
			}
		});
		axis2LockButton.addChangeListener(
        	new javax.swing.event.ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				axis2LockButton_stateChanged(e);
			}
		});

		dataLockHeadingLabel.setText("Data");

		dim1LockedButton.setText("L");
        dim1LockedButton.setToolTipText("Whether X data are locked");
		dim1LockedButton.addActionListener(new java.awt.event.ActionListener() {

			public void actionPerformed(ActionEvent e) {
				dim1LockedButton_actionPerformed(e);
			}
		});

		dim2LockedButton.setText("U");
        dim2LockedButton.setToolTipText("Whether Y data are locked");
		dim2LockedButton.addActionListener(new java.awt.event.ActionListener() {

			public void actionPerformed(ActionEvent e) {
				dim2LockedButton_actionPerformed(e);
			}
		});

		row1DimLabel.setText("X");
        row1DimLabel.setForeground(Color.black);
		row2DimLabel.setText("Y");
        row2DimLabel.setForeground(Color.black);

		col3HeadLabel.setForeground(new java.awt.Color(0, 0.4f, 0));
		col3HeadLabel.setText("Nearest");
		nearestXLabel.setForeground(new java.awt.Color(0, 0.4f, 0));
		nearestXLabel.setText("0");
        nearestXLabel.setEditable(false);
        nearestXLabel.setHorizontalAlignment(JTextField.RIGHT);
        nearestXLabel.setToolTipText("X value of nearest point to mouse");
		nearestYLabel.setForeground(new java.awt.Color(0, 0.4f, 0));
		nearestYLabel.setText("0");
        nearestYLabel.setEditable(false);
        nearestYLabel.setHorizontalAlignment(JTextField.RIGHT);
        nearestYLabel.setToolTipText("Y value of nearest point to mouse");
		col4HeadLabel.setForeground(new java.awt.Color(0, 0, 0.4f));
		col4HeadLabel.setText("Min");
		minXLabel.setForeground(new java.awt.Color(0, 0, 0.4f));
		minXLabel.setText("0");
        minXLabel.setEditable(false);
        minXLabel.setHorizontalAlignment(JTextField.RIGHT);
        minXLabel.setToolTipText("Smallest X value of series");
		minYLabel.setForeground(new java.awt.Color(0, 0, 0.4f));
		minYLabel.setText("0");
        minYLabel.setEditable(false);
        minYLabel.setHorizontalAlignment(JTextField.RIGHT);
        minYLabel.setToolTipText("Smallest Y value of series");
		col5HeadLabel.setForeground(new java.awt.Color(0.4f, 0, 0));
		col5HeadLabel.setText("Max");
		maxXLabel.setForeground(new java.awt.Color(0.4f, 0, 0));
		maxXLabel.setText("0");
        maxXLabel.setEditable(false);
        maxXLabel.setHorizontalAlignment(JTextField.RIGHT);
        maxXLabel.setToolTipText("Largest X value of series");
		maxYLabel.setForeground(new java.awt.Color(0.4f, 0, 0));
		maxYLabel.setText("0");
        maxYLabel.setEditable(false);
        maxYLabel.setHorizontalAlignment(JTextField.RIGHT);
        maxYLabel.setToolTipText("Largest Y value of series");

		showCheckBox.setText("Show");
        showCheckBox.setToolTipText("Show or hide the curve");

		sortedXCheckBox.setText("Sorted");
		sortedXCheckBox.setEnabled(false);
        sortedXCheckBox.setToolTipText(
        	"Whether X is sorted in ascending order");
		sortedYCheckBox.setText("Sorted");
		sortedYCheckBox.setEnabled(false);
        sortedYCheckBox.setToolTipText(
        	"Whether Y is sorted in ascending order");

        curveListCombo.setToolTipText("The curve being described");
     	curveListCombo.setLightWeightPopupEnabled(false);
		curveListCombo.addActionListener(new java.awt.event.ActionListener() {

			public void actionPerformed(ActionEvent e) {
				curveListCombo_actionPerformed(e);
			}
		});

		curveNameTextField.setText("Curve Name");
        curveNameTextField.setMinimumSize(new Dimension(200, 100));
        curveNameTextField.setToolTipText("Title of series");
		numPointsLabel.setText("Num Points");
        numPointsLabel.setEditable(false);
        numPointsLabel.setHorizontalAlignment(JTextField.RIGHT);
        numPointsLabel.setToolTipText("Number of data points in series");

		lineCheckBox.setText("Line");
        lineCheckBox.setToolTipText(
        	"Whether to draw lines between data points");

     	linePatternCombo.setLightWeightPopupEnabled(false);
        linePatternCombo.setToolTipText("Line Pattern");
        linePatternCombo.addItem("Solid");
        linePatternCombo.addItem("Dashed");
        linePatternCombo.addItem("Dotted");
        linePatternCombo.addItem("Dash Dot");
        linePatternCombo.addItem("Loose Dashed");
        linePatternCombo.addItem("Loose Dotted");
        linePatternCombo.addItem("Dash Dot Dash");
        linePatternCombo.addItem("Tight Dashed");
        linePatternCombo.addItem("Tight Dotted");

        lineThicknessTextField.setText("00.00");
        lineThicknessTextField.setToolTipText("Thickness of line");

		symbolsCheckBox.setText("Symbols");
        symbolsCheckBox.setToolTipText(
        	"Whether to draw symbols at the data points");

        lineSymbolsCombo.setToolTipText("Symbol type");
     	lineSymbolsCombo.setLightWeightPopupEnabled(false);
     	lineSymbolsCombo.addItem("Single Pixel");
     	lineSymbolsCombo.addItem("Plus Sign");
     	lineSymbolsCombo.addItem("X");
     	lineSymbolsCombo.addItem("Tick Mark");
     	lineSymbolsCombo.addItem("Hollow Circle");
     	lineSymbolsCombo.addItem("Hollow Square");
     	lineSymbolsCombo.addItem("Hollow Up Triangle");
     	lineSymbolsCombo.addItem("Hollow Down Triangle");
     	lineSymbolsCombo.addItem("Hollow Diamond");
     	lineSymbolsCombo.addItem("Filled Circle");
     	lineSymbolsCombo.addItem("Filled Square");
     	lineSymbolsCombo.addItem("Filled Up Triangle");
     	lineSymbolsCombo.addItem("Filled Down Triangle");
     	lineSymbolsCombo.addItem("Filled Diamond");

        symbolSizeTextField.setText("00.00");
        symbolSizeTextField.setToolTipText("Size of symbols");

        colorPanel.setBorder(
        	BorderFactory.createEtchedBorder());
        JLabel colorLabel = new JLabel("Color");
        colorLabel.setForeground(Color.black);
        colorPanel.add(colorLabel);

        redSlider.setBackground(Color.red);
        redSlider.setToolTipText("Amount of red in RGB color");
		redSlider.addChangeListener(new javax.swing.event.ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				redSlider_stateChanged(e);
			}
		});

        greenSlider.setBackground(Color.green);
        greenSlider.setToolTipText("Amount of green in RGB color");
		greenSlider.addChangeListener(new javax.swing.event.ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				greenSlider_stateChanged(e);
			}
		});

        blueSlider.setBackground(Color.blue);
        blueSlider.setToolTipText("Amount of blue in RGB color");
		blueSlider.addChangeListener(new javax.swing.event.ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				blueSlider_stateChanged(e);
			}
		});

		redLabel.setText("0.00");
        redLabel.setBackground(Color.red);
        redLabel.setForeground(Color.black);
		greenLabel.setText("0.00");
        greenLabel.setBackground(Color.green);
        greenLabel.setForeground(Color.black);
		blueLabel.setText("0.00");
        blueLabel.setBackground(Color.blue);
        blueLabel.setForeground(Color.black);

		panel1W.add(selectMethodPanel, new GridBagConstraints(0, 0, 1, 3, 0, 0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE,
            new Insets(0, 0, 0, 0), 0, 0));
		panel1W.add(formatTextField, new GridBagConstraints(1, 0, 1, 1, 0, 0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE,
            new Insets(0, 0, 0, 0), 0, 0));
		panel1W.add(modeLabel, new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE,
            new Insets(0, 0, 0, 0), 0, 0));
		panel1W.add(switchBarButton, new GridBagConstraints(1, 2, 1, 1, 0, 0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE,
            new Insets(0, 0, 0, 0), 0, 0));

		panel1E.add(row1DimLabel, new GridBagConstraints(1, 1, 1, 1, 0, 0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE,
            new Insets(0, 5, 5, 0), 0, 0));
		panel1E.add(row2DimLabel, new GridBagConstraints(1, 2, 1, 1, 0, 0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE,
            new Insets(0, 5, 5, 0), 0, 0));
		panel1E.add(axesLabel, new GridBagConstraints(2, 0, 1, 1, 0, 0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE,
            new Insets(0, 2, 2, 0), 0, 0));
		panel1E.add(axis1LockButton, new GridBagConstraints(2, 1, 1, 1, 0, 0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE,
            new Insets(0, 0, 0, 0), 0, 0));
		panel1E.add(axis2LockButton, new GridBagConstraints(2, 2, 1, 1, 0, 0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE,
            new Insets(0, 0, 0, 0), 0, 0));
		panel1E.add(dataLockHeadingLabel, new GridBagConstraints(3, 0, 1, 1, 0, 0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE,
            new Insets(0, 2, 2, 0), 0, 0));
		panel1E.add(dim1LockedButton, new GridBagConstraints(3, 1, 1, 1, 0, 0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE,
            new Insets(0, 0, 0, 0), 0, 0));
		panel1E.add(dim2LockedButton, new GridBagConstraints(3, 2, 1, 1, 0, 0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE,
            new Insets(0, 0, 0, 0), 0, 0));
		panel1E.add(col3HeadLabel, new GridBagConstraints(6, 0, 1, 1, 0, 0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE,
            new Insets(0, 2, 2, 0), 0, 0));
		panel1E.add(nearestXLabel, new GridBagConstraints(6, 1, 1, 1, 0, 0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE,
            new Insets(0, 2, 2, 0), 0, 0));
		panel1E.add(nearestYLabel, new GridBagConstraints(6, 2, 1, 1, 0, 0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE,
            new Insets(0, 2, 2, 0), 0, 0));
		panel1E.add(col4HeadLabel, new GridBagConstraints(7, 0, 1, 1, 0, 0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE,
            new Insets(0, 2, 2, 0), 0, 0));
		panel1E.add(minXLabel, new GridBagConstraints(7, 1, 1, 1, 0, 0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE,
            new Insets(0, 2, 2, 0), 0, 0));
		panel1E.add(minYLabel, new GridBagConstraints(7, 2, 1, 1, 0, 0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE,
            new Insets(0, 2, 2, 0), 0, 0));
		panel1E.add(col5HeadLabel, new GridBagConstraints(8, 0, 1, 1, 0, 0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE,
            new Insets(0, 2, 2, 0), 0, 0));
		panel1E.add(maxXLabel, new GridBagConstraints(8, 1, 1, 1, 0, 0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE,
            new Insets(0, 2, 2, 0), 0, 0));
		panel1E.add(maxYLabel, new GridBagConstraints(8, 2, 1, 1, 0, 0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE,
            new Insets(0, 2, 2, 0), 0, 0));
		panel1E.add(showCheckBox, new GridBagConstraints(9, 0, 1, 1, 0, 0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE,
            new Insets(0, 3, 3, 0), 0, 0));
		panel1E.add(sortedXCheckBox, new GridBagConstraints(9, 1, 1, 1, 0, 0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE,
            new Insets(0, 3, 3, 0), 0, 0));
		panel1E.add(sortedYCheckBox, new GridBagConstraints(9, 2, 1, 1, 0, 0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE,
            new Insets(0, 3, 3, 0), 0, 0));

		panel2W.add(curveListCombo, new GridBagConstraints(0, 0, 1, 1, 0, 0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE,
            new Insets(0, 0, 0, 0), 0, 0));
		panel2W.add(curveNameTextField, new GridBagConstraints(1, 0, 14, 1, 0, 0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE,
            new Insets(0, 0, 0, 0), 0, 0));
		panel2W.add(lineCheckBox, new GridBagConstraints(0, 1, 1, 1, 0, 0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE,
            new Insets(0, 0, 0, 0), 0, 0));
		panel2W.add(linePatternCombo, new GridBagConstraints(1, 1, 2, 1, 0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE,
            new Insets(0, 0, 0, 0), 0, 0));
		panel2W.add(lineThicknessTextField, new GridBagConstraints(3, 1, 1, 1, 0, 0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE,
            new Insets(0, 0, 0, 0), 0, 0));
		panel2W.add(symbolsCheckBox, new GridBagConstraints(0, 2, 1, 1, 0, 0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE,
            new Insets(0, 0, 0, 0), 0, 0));
		panel2W.add(lineSymbolsCombo, new GridBagConstraints(1, 2, 2, 1, 0, 0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE,
            new Insets(0, 0, 0, 0), 0, 0));
		panel2W.add(symbolSizeTextField, new GridBagConstraints(3, 2, 1, 1, 0, 0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE,
            new Insets(0, 0, 0, 0), 0, 0));

		panel2E.add(colorPanel, new GridBagConstraints(10, 0, 1, 3, 0, 0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE,
            new Insets(0, 12, 0, 0), 0, 0));
		panel2E.add(redSlider, new GridBagConstraints(13, 0, 1, 1, 0, 0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE,
            new Insets(0, 0, 0, 0), 0, 0));
		panel2E.add(greenSlider, new GridBagConstraints(13, 1, 1, 1, 0, 0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE,
            new Insets(0, 0, 0, 0), 0, 0));
		panel2E.add(blueSlider, new GridBagConstraints(13, 2, 1, 1, 0, 0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE,
            new Insets(0, 0, 0, 0), 0, 0));
		panel2E.add(redLabel, new GridBagConstraints(14, 0, 1, 1, 0, 0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE,
            new Insets(0, 2, 2, 12), 0, 0));
		panel2E.add(greenLabel, new GridBagConstraints(14, 1, 1, 1, 0, 0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE,
            new Insets(0, 2, 2, 12), 0, 0));
		panel2E.add(blueLabel, new GridBagConstraints(14, 2, 1, 1, 0, 0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE,
            new Insets(0, 2, 2, 12), 0, 0));

        panel2W.add(panel2E, new GridBagConstraints(4, 1, 1, 2, 0, 0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE,
            new Insets(0, 0, 0, 0), 0, 0));

        panel2W.add(numPointsLabel, new GridBagConstraints(5, 1, 1, 1, 0, 0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE,
            new Insets(0, 0, 0, 0), 0, 0));

        panel2W.add(applyButton, new GridBagConstraints(5, 2, 1, 1, 0, 0
            ,GridBagConstraints.WEST, GridBagConstraints.NONE,
            new Insets(0, 0, 0, 0), 0, 0));

    }

//------------------------------------------------------------------------------
/**
 * Returns the format in the format text field.
 */
    public String getFormat() {

    	return formatTextField.getText();
    }

//------------------------------------------------------------------------------
/**
 * Sets the curve that should have it's information displayed on this bar.
 */
    public void setCurve(Curve curve) {

        if (curve == null) {
            m_series = null;
            return;
        }

    	m_curve = curve;
        m_series = curve.getSeries();
        if (m_series == null) {
        	return;
        }

        if (m_series.getDimensionLocked(0)) {
            dim1LockedButton.setSelected(true);
        	dim1LockedButton.setText("L");
        }
        else {
            dim1LockedButton.setSelected(false);
        	dim1LockedButton.setText("U");
        }
        if (m_series.getDimensionLocked(1)) {
            dim1LockedButton.setSelected(true);
            dim2LockedButton.setText("L");
        }
        else {
            dim1LockedButton.setSelected(false);
            dim2LockedButton.setText("U");
        }

        DecimalFormat form = new DecimalFormat(formatTextField.getText());
    	minXLabel.setText(form.format(m_series.getMin(0)));
    	minYLabel.setText(form.format(m_series.getMin(1)));
    	maxXLabel.setText(form.format(m_series.getMax(0)));
    	maxYLabel.setText(form.format(m_series.getMax(1)));
        curveNameTextField.setText("" + curve.getTitle());
        curveNameTextField.setToolTipText("" + curve.getTitle());
        numPointsLabel.setText("" + m_series.getNumPoints() + " Points");
        symbolsCheckBox.setSelected(curve.getShowSymbols());
        lineCheckBox.setSelected(curve.getShowLine());
        showCheckBox.setSelected(curve.getShow());
        sortedXCheckBox.setSelected(m_series.getSorted(0));
        sortedYCheckBox.setSelected(m_series.getSorted(1));

        PageOfGraphs page = m_grCon.getPage();
        if (page == null) {
        	m_curve = null;
            return;
        }

        // Choose the correct entry in the curve combo to match the curve
        // that is being described.
        int index = 0;
        List graphList = page.getGraphList();
        synchronized(page) {
            Iterator graphIter = graphList.iterator();
            outer:
            while (graphIter.hasNext()) {
                Graph graph = (Graph)graphIter.next();
                List curveList = graph.getCurves();
                Iterator curveIter = curveList.iterator();
                while (curveIter.hasNext()) {
                    Curve curve2 = (Curve)curveIter.next();
                    if (curve == curve2) {
                        if (curveListCombo.getItemCount() > index)
                            curveListCombo.setSelectedIndex(index);
                        break outer;
                    }
                    index++;
                }
            }
        }


        short pattern = m_curve.getLinePattern();
        index = 0;
        switch (pattern) {
			case Gr.SOLID_LINE_PATTERN: index = 0; break;
			case Gr.DASHED_LINE_PATTERN: index = 1; break;
            case Gr.DOTTED_LINE_PATTERN: index = 2; break;
            case Gr.DASH_DOT_LINE_PATTERN: index = 3; break;
            case Gr.LOOSE_DASHED_LINE_PATTERN: index = 4; break;
            case Gr.LOOSE_DOTTED_LINE_PATTERN: index = 5; break;
            case Gr.DASH_DOT_DASH_LINE_PATTERN: index = 6; break;
            case Gr.TIGHT_DASHED_LINE_PATTERN: index = 7; break;
            case Gr.TIGHT_DOTTED_LINE_PATTERN: index = 8; break;
        }
        linePatternCombo.setSelectedIndex(index);

        form = new DecimalFormat("0.###");
        lineThicknessTextField.setText(form.format(m_curve.getLineWidth()));

        lineSymbolsCombo.setSelectedIndex(m_curve.getSymbolIndex());
        symbolSizeTextField.setText(form.format(m_curve.getSymbolSize()));

        float[] color = m_curve.getColor();
        redSlider.setValue(Math.round(color[0] * 100));
        greenSlider.setValue(Math.round(color[1] * 100));
        blueSlider.setValue(Math.round(color[2] * 100));

        colorPanel.setBackground(new Color(color[0], color[1], color[2]));

        form = new DecimalFormat("0.00");
        redLabel.setText(form.format(color[0]));
        greenLabel.setText(form.format(color[1]));
//        blueLabel.setText(form.format(color[2]));
    }

//------------------------------------------------------------------------------
/**
 * Checks the page and fills in the combo box with the most up to date
 * list of curves.
 */
    public void updateCurveList() {

        try{
        	curveListCombo.removeAllItems();
        }
        catch (IndexOutOfBoundsException e) {};

        boolean ourCurveIsStale = true;

        // Fill in all the available curves.
        PageOfGraphs page = m_grCon.getPage();
        if (page == null) {
        	m_curve = null;
            m_series = null;
            return;
        }

        List graphList = page.getGraphList();
        ArrayList curveEntryList = new ArrayList();
        synchronized(page) {
            Iterator graphIter = graphList.iterator();
            int graphNum = 1;
            while (graphIter.hasNext()) {
                Graph graph = (Graph)graphIter.next();
                List curveList = graph.getCurves();
                Iterator curveIter = curveList.iterator();
                int curveNum = 1;
                while (curveIter.hasNext()) {
                    Curve curve = (Curve)curveIter.next();
                    String curveEntry = "G" + graphNum + " C" + curveNum;
                    curveEntryList.add(curveEntry);
                    if (curve == m_curve) ourCurveIsStale = false;
                    curveNum++;
                }
                graphNum++;
            }
        }

        // Add these items separately to avoid deadlock.
        Iterator iter = curveEntryList.iterator();
        while (iter.hasNext()) {
            String curveEntry = (String)iter.next();
            curveListCombo.addItem(curveEntry);
        }

        if (ourCurveIsStale) m_curve = null;
    }

    //---------------------------------------------------------------------------
	void switchBarButton_actionPerformed(ActionEvent e) {
        m_grCon.switchStatusBar();
	}

    //---------------------------------------------------------------------------
	void dim1LockedButton_actionPerformed(ActionEvent e) {

    	if (dim1LockedButton.isSelected()) {
        	dim1LockedButton.setText("L");
            if (m_series != null) m_series.setDimensionLocked(0, true);
        }
        else {
        	dim1LockedButton.setText("U");
            if (m_series != null) m_series.setDimensionLocked(0, false);
        }

	}

    //---------------------------------------------------------------------------
	void dim2LockedButton_actionPerformed(ActionEvent e) {

    	if (dim2LockedButton.isSelected()) {
        	dim2LockedButton.setText("L");
            if (m_series != null) m_series.setDimensionLocked(1, true);
        }
        else {
        	dim2LockedButton.setText("U");
            if (m_series != null) m_series.setDimensionLocked(1, false);
        }

	}

    //---------------------------------------------------------------------------
	void curveListCombo_actionPerformed(ActionEvent e) {

    	int index = curveListCombo.getSelectedIndex();

        PageOfGraphs page = m_grCon.getPage();
        if (page == null) {
        	m_curve = null;
            return;
        }

        List graphList = page.getGraphList();
        synchronized(page) {
            Iterator graphIter = graphList.iterator();
            outer:
            while (graphIter.hasNext()) {
                Graph graph = (Graph)graphIter.next();
                List curveList = graph.getCurves();
                    Iterator curveIter = curveList.iterator();
                    while (curveIter.hasNext()) {
                        Curve curve = (Curve)curveIter.next();
                        if (index <= 0) {
                            setCurve(curve);
                            break outer;
                        }
                        index--;
                    }
            }
        }
    }

    //---------------------------------------------------------------------------
	void redSlider_stateChanged(ChangeEvent e) {

        m_color[0] = (float)(redSlider.getValue()) / 100;

        colorPanel.setBackground(new Color(m_color[0], m_color[1], m_color[2]));

        DecimalFormat form = new DecimalFormat("0.00");
        redLabel.setText(form.format(m_color[0]));


	}

    //---------------------------------------------------------------------------
	void greenSlider_stateChanged(ChangeEvent e) {

        m_color[1] = (float)(greenSlider.getValue()) / 100;

        colorPanel.setBackground(new Color(m_color[0], m_color[1], m_color[2]));

        DecimalFormat form = new DecimalFormat("0.00");
        greenLabel.setText(form.format(m_color[1]));

	}

    //---------------------------------------------------------------------------
	void blueSlider_stateChanged(ChangeEvent e) {

        m_color[2] = (float)(blueSlider.getValue()) / 100;

        colorPanel.setBackground(new Color(m_color[0], m_color[1], m_color[2]));

        DecimalFormat form = new DecimalFormat("0.00");
        blueLabel.setText(form.format(m_color[2]));

	}

//------------------------------------------------------------------------------
	void axis1LockButton_stateChanged(ChangeEvent e) {
        if (axis1LockButton.isSelected()) {
        	axis1LockButton.setText("L");
            axis1LockButton.setForeground(Color.red);
            m_grCon.m_bLockX = true;
        }
        else {
        	axis1LockButton.setText("U");
            axis1LockButton.setForeground(Color.blue);
            m_grCon.m_bLockX = false;
        }
	}

//------------------------------------------------------------------------------
	void axis2LockButton_stateChanged(ChangeEvent e) {
        if (axis2LockButton.isSelected()) {
        	axis2LockButton.setText("L");
            axis2LockButton.setForeground(Color.red);
            m_grCon.m_bLockY = true;
        }
        else {
        	axis2LockButton.setText("U");
            axis2LockButton.setForeground(Color.blue);
            m_grCon.m_bLockY = false;
        }

	}

//------------------------------------------------------------------------------
	void underMouseRadio_stateChanged(ChangeEvent e) {

    	m_bSelectWithClick = !underMouseRadio.isSelected();
	}

//------------------------------------------------------------------------------
	void applyButton_actionPerformed(ActionEvent e) {

    	if (m_curve == null) {
            updateCurveList();
            return;
        }

        m_grCon.savePageSnapShot();
    	m_curve.setShow(showCheckBox.isSelected());
    	m_curve.setTitle(curveNameTextField.getText());
        curveNameTextField.setToolTipText("Title of series: " + curveNameTextField.getText());
    	m_curve.setSymbolIndex(lineSymbolsCombo.getSelectedIndex());
    	m_curve.setLinePattern(linePatternCombo.getSelectedIndex());
    	m_curve.setShowLine(lineCheckBox.isSelected());
    	m_curve.setShowSymbols(symbolsCheckBox.isSelected());
        m_curve.setColor(m_color);

    	float val = 1;
        boolean badVal = false;
        try {
        	val = Float.parseFloat(symbolSizeTextField.getText());
        }
        catch(NumberFormatException nfe) {
        	badVal = true;
        }

        if (!badVal) {
        	if (val < 0.01 || val > 100) val = 1;
     		m_curve.setSymbolSize(val);
        }

        val = 1;
        try {
        	val = Float.parseFloat(lineThicknessTextField.getText());
        }
        catch(NumberFormatException nfe) {
        	badVal = true;
        }

        if (!badVal) {
        	if (val < 0.01 || val > 100) val = 5;
     		m_curve.setLineWidth(val);
        }

        m_grCon.triggerRepaint();
	}
}

