/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2007-06-28
 * Date started:    2003-04-29
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr.visual.gui;

import gov.usgs.gr.data.*;
import gov.usgs.gr.*;
import gov.usgs.gr.visual.*;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;

//==============================================================================
/**
 *
 */
public class DataEditorPanel extends JPanel {

    protected GrController m_grCon;
    protected PageOfGraphs m_page;
    protected List<DataSeries> m_selectedSeriesList;
    protected JDialog m_dialog = null;

    protected boolean m_enabling = false;

    protected BorderLayout mainLayout = new BorderLayout();
    protected JPanel mainLeftPanel = new JPanel();
    protected BorderLayout mainLeftLayout = new BorderLayout();
    protected JPanel mainLeftUpperPanel = new JPanel();
    protected BorderLayout mainLeftUpperLayout = new BorderLayout();
    protected JPanel operationPanel = new JPanel();
    protected BorderLayout operationLayout = new BorderLayout();

    protected JPanel operationSimplePanel = new JPanel();
    protected GridLayout operationSimpleLayout = new GridLayout(3, 3);
    protected ButtonGroup operationGroup = new ButtonGroup();
    protected JRadioButton noneRadio = new JRadioButton("None");
    protected JRadioButton setRadio = new JRadioButton("Set Equal");
    protected JRadioButton addRadio = new JRadioButton("Add");
    protected JRadioButton subRadio = new JRadioButton("Subtract");
    protected JRadioButton mulRadio = new JRadioButton("Multiply");
    protected JRadioButton divRadio = new JRadioButton("Divide");
    protected JRadioButton powRadio = new JRadioButton("Power");
    protected JRadioButton logRadio = new JRadioButton("Log 10");
    protected JRadioButton absRadio = new JRadioButton("Abs Val");

    protected JPanel operandPanel = new JPanel();
    protected BorderLayout operandLayout = new BorderLayout();
    protected JPanel operandConstantPanel = new JPanel();
    protected BorderLayout operandConstantLayout = new BorderLayout();
    protected JTextField constantValueTextField = new JTextField();

    protected JPanel mainLeftLowerPanel = new JPanel();
    protected BorderLayout mainLeftLowerLayout = new BorderLayout();

    protected JPanel dimensionPanel = new JPanel();
    protected GridLayout dimensionLayout = new GridLayout(1, 2);
    protected JTextField dimensionTextField = new JTextField();

    protected JPanel interpolationPanel = new JPanel();
    protected GridLayout interpolationLayout = new GridLayout(1, 2);
    protected ButtonGroup interpolationGroup = new ButtonGroup();
    protected JRadioButton interpLinearRadio = new JRadioButton("Linear");
    protected JRadioButton interpParabRadio = new JRadioButton("Parabolic");

    protected JPanel okCancelPanel = new JPanel();
    protected GridLayout okCancelLayout = new GridLayout(1, 5);
    protected JButton okButton = new JButton();
    protected JButton applyButton = new JButton();
    protected JButton undoButton = new JButton();
    protected JButton redoButton = new JButton();
    protected JButton cancelButton = new JButton();

//------------------------------------------------------------------------------
    public DataEditorPanel() {
        
        init();
    }

//------------------------------------------------------------------------------
    public DataEditorPanel(boolean bInitialize) {
        
        if (bInitialize) 
            init();
    }

//------------------------------------------------------------------------------
    protected void init() {

        // Set up button listeners.
        okButton.setText("OK");
        okButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                okButton_actionPerformed(e);
            }
        });
        applyButton.setText("Apply");
        applyButton.addActionListener(new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                applyButton_actionPerformed(e);
            }
        });
        undoButton.setForeground(Color.blue);
        undoButton.setText("Undo");
        undoButton.addActionListener(new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                undoButton_actionPerformed(e);
            }
        });
        redoButton.setForeground(Color.blue);
        redoButton.setText("Redo");
        redoButton.addActionListener(new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                redoButton_actionPerformed(e);
            }
        });
        cancelButton.setForeground(Color.red);
        cancelButton.setText("Close");
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                cancelButton_actionPerformed(e);
            }
        });

        setRadio.addChangeListener(new javax.swing.event.ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                setRadio_stateChanged(e);
            }
        });
        addRadio.addChangeListener(new javax.swing.event.ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                addRadio_stateChanged(e);
            }
        });
        subRadio.addChangeListener(new javax.swing.event.ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                subRadio_stateChanged(e);
            }
        });
        mulRadio.addChangeListener(new javax.swing.event.ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                mulRadio_stateChanged(e);
            }
        });
        divRadio.addChangeListener(new javax.swing.event.ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                divRadio_stateChanged(e);
            }
        });
        powRadio.addChangeListener(new javax.swing.event.ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                powRadio_stateChanged(e);
            }
        });
        logRadio.addChangeListener(new javax.swing.event.ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                logRadio_stateChanged(e);
            }
        });
        absRadio.addChangeListener(new javax.swing.event.ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                absRadio_stateChanged(e);
            }
        });
        noneRadio.addChangeListener(new javax.swing.event.ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                noneRadio_stateChanged(e);
            }
        });

        this.setLayout(mainLayout);
        this.add(mainLeftPanel, BorderLayout.WEST);
        mainLeftPanel.setLayout(mainLeftLayout);
        mainLeftPanel.add(mainLeftUpperPanel, BorderLayout.NORTH);
        mainLeftUpperPanel.setLayout(mainLeftUpperLayout);
        mainLeftUpperPanel.add(operationPanel, BorderLayout.NORTH);
        operationPanel.setBorder(
            BorderFactory.createTitledBorder("Operation"));
        operationPanel.setLayout(operationLayout);
        operationPanel.add(operationSimplePanel, BorderLayout.NORTH);

        // Operation simple panel...
        operationSimplePanel.setBorder(BorderFactory.createEtchedBorder());
        operationSimplePanel.setLayout(operationSimpleLayout);
        operationSimplePanel.add(addRadio);
        operationSimplePanel.add(mulRadio);
        operationSimplePanel.add(setRadio);
        operationSimplePanel.add(subRadio);
        operationSimplePanel.add(divRadio);
        operationSimplePanel.add(noneRadio);
        operationSimplePanel.add(powRadio);
        operationSimplePanel.add(logRadio);
        operationSimplePanel.add(absRadio);
        operationGroup.add(noneRadio);
        operationGroup.add(setRadio);
        operationGroup.add(addRadio);
        operationGroup.add(subRadio);
        operationGroup.add(mulRadio);
        operationGroup.add(divRadio);
        operationGroup.add(powRadio);
        operationGroup.add(logRadio);
        operationGroup.add(absRadio);
        noneRadio.setSelected(true);

        // Operand constant panel...
        mainLeftUpperPanel.add(operandPanel, BorderLayout.SOUTH);
        operandPanel.setBorder(BorderFactory.createTitledBorder("Operand"));
        operandPanel.setLayout(operandLayout);
        operandPanel.add(operandConstantPanel, BorderLayout.NORTH);
        operandConstantPanel.setLayout(operandConstantLayout);
        operandConstantPanel.add(constantValueTextField, BorderLayout.CENTER);
        constantValueTextField.setText("0          ");

        // Main left lower panel...
        mainLeftPanel.add(mainLeftLowerPanel, BorderLayout.SOUTH);
        mainLeftLowerPanel.setLayout(mainLeftLowerLayout);
        mainLeftLowerPanel.add(dimensionPanel, BorderLayout.NORTH);

        // Dimension panel...
        dimensionPanel.setBorder(BorderFactory.createTitledBorder("Dimension"));
        dimensionPanel.setLayout(dimensionLayout);
        dimensionPanel.add(dimensionTextField, BorderLayout.CENTER);
        dimensionTextField.setText("0          ");

        // Interpolation panel...
//        mainLeftLowerPanel.add(interpolationPanel, BorderLayout.SOUTH);
//        interpolationPanel.setBorder(
//            BorderFactory.createTitledBorder("Interpolation"));
//        interpolationPanel.setLayout(interpolationLayout);
//        interpolationPanel.add(interpLinearRadio);
//        interpolationPanel.add(interpParabRadio);
//        interpolationGroup.add(interpLinearRadio);
//        interpolationGroup.add(interpParabRadio);
//        interpLinearRadio.setSelected(true);

        // OK, Cancel buttons...
        this.add(okCancelPanel, BorderLayout.SOUTH);
        okCancelPanel.add(applyButton);
        okCancelPanel.add(undoButton);
        okCancelPanel.add(redoButton);
        okCancelPanel.add(cancelButton);
    }

//------------------------------------------------------------------------------
/**
 * Read the values from the page and put them in the dialog.
 * <p><b> Pre Conditions </b><br>
 *      The page has been set up.
 */
    public void readFromPage(GrController grCon) {

        m_grCon = grCon;
        m_page = m_grCon.getPage();
        synchronized (m_page) {
            m_selectedSeriesList = m_page.getSelectedSeriesList();
            if (m_selectedSeriesList.size() == 0)
                constantValueTextField.setText("Nothing selected");
        }
    }

//------------------------------------------------------------------------------
/**
 * Opens the specified script file and applies it to the data.
 */
    public void applyModification(List attribs) {

        Iterator iter = attribs.iterator();
        while (iter.hasNext()) {
            String s = (String)iter.next();
            String key = s.substring(0, s.indexOf('=')).trim().toUpperCase();
            String value = s.substring(s.indexOf('=') + 1).trim();
            value = value.toUpperCase();

            // Handle the key and value.  Each key and value corresponds
            // to a choice on this dialog box.
            if (key.equals("ACTION")) {
                if (value.equals("NONE")) noneRadio.setSelected(true);
                if (value.equals("ADD")) addRadio.setSelected(true);
                if (value.equals("SUB")) subRadio.setSelected(true);
                if (value.equals("MUL")) mulRadio.setSelected(true);
                if (value.equals("DIV")) divRadio.setSelected(true);
                if (value.equals("SET")) setRadio.setSelected(true);
                if (value.equals("POW")) powRadio.setSelected(true);
                if (value.equals("LOG")) logRadio.setSelected(true);
                if (value.equals("ABS")) absRadio.setSelected(true);
            }
///            if (key.equals("DIM")) {
///                if (value.equals("0") || value.equals("X"))
///                    xDimRadio.setSelected(true);
///                if (value.equals("1") || value.equals("Y"))
///                    yDimRadio.setSelected(true);
///            }
            if (key.equals("INPUT")) {
                constantValueTextField.setText(value);
            }
            if (key.equals("INTERP")) {
                if (value.equals("LINEAR")) interpLinearRadio.setSelected(true);
                if (value.equals("PARAB")) interpParabRadio.setSelected(true);
            }
        }
        writeToPage();
    }

//------------------------------------------------------------------------------
/**
 * Write values from the dialog back to the page.
 * <p><b> Pre Conditions </b><br>
 *      The page has been set up.
 */
    protected void writeToPage() {

        if (m_selectedSeriesList.size() == 0) return;

        // Show an hourglass until the job is complete.
        ((Component)getParentFrame()).setCursor(new Cursor(Cursor.WAIT_CURSOR));

        // Get a reference to the second curve if one is selected.
        Graph graph2;
        Curve curve2;
        DoubleDataSeries series2 = null;

        synchronized (m_page) {
            // Get the new value
            double value = Double.parseDouble(constantValueTextField.getText());

            // Determine if changing the x or y coordinates.
            int dimension = (int)(Double.parseDouble(
                dimensionTextField.getText()));

            List selectedSeriesList = null;
            // Use the original list of selected series instead of making
            // new copies.
            selectedSeriesList = m_selectedSeriesList;

            DataSnapShot snapShot = new DataSnapShot();
            Iterator iter = selectedSeriesList.iterator();
            while (iter.hasNext()) {
                // Get the series and set its data manipulation options.
                DataSeries series = (DataSeries)iter.next();
                synchronized (series) {

                    // Save the current state for later undo.
                    snapShot.store(series);

                    if (!noneRadio.isSelected()) {
                        // Determine which type of modification is being made.
                        int modificationType = -1;
                        if (setRadio.isSelected()) modificationType = 0;
                        if (addRadio.isSelected()) modificationType = 1;
                        if (subRadio.isSelected()) modificationType = 2;
                        if (mulRadio.isSelected()) modificationType = 3;
                        if (divRadio.isSelected()) modificationType = 4;
                        if (powRadio.isSelected()) modificationType = 5;
                        if (logRadio.isSelected()) modificationType = 6;
                        if (absRadio.isSelected()) modificationType = 7;

                        // Apply the modification using a constant.
                        if (series instanceof DoubleDataSeries)
                            ((DoubleDataSeries)series).modifySelectedPoints(
                                modificationType, dimension, value);
                        if (series instanceof FloatDataSeries)
                            ((FloatDataSeries)series).modifySelectedPoints(
                                modificationType, dimension, (float)value);
                        if (series instanceof IntDataSeries)
                            ((IntDataSeries)series).modifySelectedPoints(
                                modificationType, dimension, (float)value);
                    }
                }
            }
        }
    }

//------------------------------------------------------------------------------
/**
 * Look for the frame which contains this component by searching its parent
 * components until a Frame is encountered.
 * @return The frame that holds this component.
 */
    public Frame getParentFrame() {

        if (m_dialog == null)
            return null;

        Frame frame = null;
        Container container = m_dialog.getContentPane();

        // Ascend the hierarchy of containers until the parent frame is found.
        do {
            container = container.getParent();
            if (container instanceof Frame) frame = (Frame)container;
        } while (container != null && !(container instanceof Frame));

        return frame;
    }

//------------------------------------------------------------------------------
    protected void okButton_actionPerformed(ActionEvent e) {

        writeToPage();
        m_grCon.triggerRepaint();

        if (m_dialog != null)
            m_dialog.dispose();
    }

//------------------------------------------------------------------------------
    protected void applyButton_actionPerformed(ActionEvent e) {

        writeToPage();
        m_grCon.triggerRepaint();
    }

//------------------------------------------------------------------------------
    protected void undoButton_actionPerformed(ActionEvent e) {

        m_grCon.undoModification();
        m_grCon.triggerRepaint();
        readFromPage(m_grCon);
    }

//------------------------------------------------------------------------------
    protected void redoButton_actionPerformed(ActionEvent e) {

        m_grCon.redoModification();
        m_grCon.triggerRepaint();
        readFromPage(m_grCon);
    }

//------------------------------------------------------------------------------
    protected void cancelButton_actionPerformed(ActionEvent e) {

        if (m_dialog != null)
            m_dialog.dispose();
    }

//------------------------------------------------------------------------------
    protected void noneRadio_stateChanged(ChangeEvent e) {

        enableComponents();
    }

//------------------------------------------------------------------------------
    protected void setRadio_stateChanged(ChangeEvent e) {

        enableComponents();
    }

//------------------------------------------------------------------------------
    protected void addRadio_stateChanged(ChangeEvent e) {

        enableComponents();
    }

//------------------------------------------------------------------------------
    protected void subRadio_stateChanged(ChangeEvent e) {

        enableComponents();
    }

//------------------------------------------------------------------------------
    protected void mulRadio_stateChanged(ChangeEvent e) {

        enableComponents();
    }

//------------------------------------------------------------------------------
    protected void divRadio_stateChanged(ChangeEvent e) {

        enableComponents();
    }

//------------------------------------------------------------------------------
    protected void powRadio_stateChanged(ChangeEvent e) {

        enableComponents();
    }

//------------------------------------------------------------------------------
    protected void logRadio_stateChanged(ChangeEvent e) {

        enableComponents();
    }

//------------------------------------------------------------------------------
    protected void absRadio_stateChanged(ChangeEvent e) {

        enableComponents();
    }

//------------------------------------------------------------------------------
    protected void enableComponents() {

        if (m_enabling) return;
        else m_enabling = true;
        try {
            // Gray out input parameter options if the "None" operation is
            // selected.
            if (noneRadio.isSelected() || logRadio.isSelected() ||
                absRadio.isSelected()) {
                
                constantValueTextField.setEnabled(false);
            }
            else {
                // Dimension options should always be
                // available unless the operation is "None".

                // Enable the constant text field.
                constantValueTextField.setEnabled(true);
            }
        }
        catch (Exception ex) { m_enabling = false; }
        catch (Error er) { m_enabling = false; }

        m_enabling = false;
    }

//------------------------------------------------------------------------------
    protected void setDialog(JDialog dialog) {

        m_dialog = dialog;
    }
}



