/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2001-02-01
 * Date started:    1999-04-01
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr.visual.gui;

import gov.usgs.gr.data.*;
import gov.usgs.gr.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.table.*;
import java.text.*;
import java.util.*;

//==============================================================================
/**
 *
 */
public class DataSelectionDialog extends JDialog {

    private boolean m_bOkPressed = false;
    private boolean m_bAdding = false;
    private int[] m_dims = null;
    private double[] m_mins = null;
    private double[] m_maxes = null;

	BorderLayout mainBorderLayout = new BorderLayout();
	BorderLayout dimBorderLayout = new BorderLayout();
	BorderLayout minBorderLayout = new BorderLayout();
	BorderLayout maxBorderLayout = new BorderLayout();
	BorderLayout addBorderLayout = new BorderLayout();
	JPanel mainPanel = new JPanel();
	JPanel dimPanel = new JPanel();
	JPanel minPanel = new JPanel();
	JPanel maxPanel = new JPanel();
	JPanel addPanel = new JPanel();
	JPanel lowerPanel = new JPanel();
	JButton okButton = new JButton();
	JButton cancelButton = new JButton();

    JTextField dimText = new JTextField();
    JTextField minText = new JTextField();
    JTextField maxText = new JTextField();
    ButtonGroup addGroup = new ButtonGroup();
    JRadioButton addRadio = new JRadioButton("Add to Current Selection");
    JRadioButton removeRadio = new JRadioButton
        ("Remove from Current Selection");


//------------------------------------------------------------------------------
	public DataSelectionDialog(Frame frame, String title, boolean modal) {
		super(frame, title, modal);

		try  {
			jbInit();
			pack();
		}
		catch (Exception ex) {
			ex.printStackTrace();
		}
	}

//------------------------------------------------------------------------------
	public DataSelectionDialog() {
		this(null, "", false);
	}

//------------------------------------------------------------------------------
	void jbInit() throws Exception {

    	// Set up button listeners.
		okButton.setText("OK");
		okButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				okButton_actionPerformed(e);
			}
		});
		cancelButton.setText("Cancel");
		cancelButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				cancelButton_actionPerformed(e);
			}
		});

        // Set the layout managers for the panels.
        mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));

		dimPanel.setLayout(new BoxLayout(dimPanel, BoxLayout.X_AXIS));
        dimPanel.setBorder(BorderFactory.createTitledBorder("Dimensions"));
		dimPanel.add(dimText);
        dimPanel.add(Box.createRigidArea(new Dimension(5,0)));
		mainPanel.add(dimPanel);

        minPanel.setLayout(new BoxLayout(minPanel, BoxLayout.X_AXIS));
        minPanel.setBorder(BorderFactory.createTitledBorder("Min Values"));
		minPanel.add(minText);
        minPanel.add(Box.createRigidArea(new Dimension(5,0)));
		mainPanel.add(minPanel);

        maxPanel.setLayout(new BoxLayout(maxPanel, BoxLayout.X_AXIS));
        maxPanel.setBorder(BorderFactory.createTitledBorder("Max Values"));
		maxPanel.add(maxText);
        maxPanel.add(Box.createRigidArea(new Dimension(5,0)));
		mainPanel.add(maxPanel);

        addPanel.setLayout(new BoxLayout(addPanel, BoxLayout.Y_AXIS));
        addPanel.setBorder(BorderFactory.createEtchedBorder());
		addPanel.add(addRadio);
		addPanel.add(removeRadio);
    	addGroup.add(addRadio);
    	addGroup.add(removeRadio);
        addRadio.setSelected(true);
        addPanel.add(Box.createRigidArea(new Dimension(5,0)));
		mainPanel.add(addPanel);

		lowerPanel.setLayout(new BoxLayout(lowerPanel, BoxLayout.X_AXIS));

        lowerPanel.add(Box.createHorizontalGlue());

		lowerPanel.add(okButton);
		lowerPanel.add(cancelButton);

		mainPanel.add(lowerPanel);

		getContentPane().add(mainPanel);

        pack();
	}

//------------------------------------------------------------------------------
	public int[] getDims() {

		return m_dims;
	}

//------------------------------------------------------------------------------
	public double[] getMins() {

		return m_mins;
	}

//------------------------------------------------------------------------------
	public double[] getMaxes() {

		return m_maxes;
	}

//------------------------------------------------------------------------------
	public boolean getAdding() {

		return m_bAdding;
	}

//------------------------------------------------------------------------------
	public boolean getOkPressed() {

		return m_bOkPressed;
	}

//------------------------------------------------------------------------------
	void okButton_actionPerformed(ActionEvent e) {

    	m_bOkPressed = true;

// Sort all arrays by dimension number.
        m_dims = AsciiIO.parseLineAsInts(dimText.getText(), " \t\n\r\f,");
        m_mins = AsciiIO.parseLineAsDoubles(minText.getText(), " \t\n\r\f,");
        m_maxes = AsciiIO.parseLineAsDoubles(maxText.getText(), " \t\n\r\f,");

        m_bAdding = addRadio.isSelected();

    	dispose();
	}

//------------------------------------------------------------------------------
	void cancelButton_actionPerformed(ActionEvent e) {

    	m_bOkPressed = false;
    	dispose();
	}
}


