/**
 * @(#)             Gr graphing application
 * @author          Shannon Hickey, Sun, via example on Java.net
 *                  Modified by John M. Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2006-12-21
 * Date started:    2006-12-21
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr.visual.gui;

import gov.usgs.gr.GrController;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.File;
import java.io.IOException;
import javax.swing.JFrame;

public class FileDropHandler extends javax.swing.TransferHandler {
    
    GrController m_grCon = null;
    
    public FileDropHandler(JFrame frame, GrController grCon) {
        m_grCon = grCon;
        frame.setTransferHandler(this);
    }
    
    public boolean canImport(TransferSupport support) {
        // for the demo, we'll only support drops (not clipboard paste)
        if (!support.isDrop()) {
            return false;
        }
        
        // return false if the drop doesn't contain a list of files
        if (!support.isDataFlavorSupported(DataFlavor.javaFileListFlavor)) {
            return false;
        }
        
        // check to see if the source actions (a bitwise-OR of supported
        // actions) contains the COPY action
        boolean bCopySupported =
            (COPY & support.getSourceDropActions()) == COPY;
        
        /* if COPY is supported, choose COPY and accept the transfer */
        if (bCopySupported) {
            support.setDropAction(COPY);
            return true;
        }
        
        // COPY isn't supported, so reject the transfer.
        //
        // Note: If you want to accept the transfer with the default
        //       action anyway, you could instead return true.
        return false;
    }
    public boolean importData(TransferSupport support) {
        if (!canImport(support)) {
            return false;
        }
        
        // fetch the Transferable
        Transferable t = support.getTransferable();
        
        try {
            // fetch the data from the Transferable
            Object data = t.getTransferData(DataFlavor.javaFileListFlavor);
            
            // data of type javaFileListFlavor is a list of files
            final java.util.List<File> fileList = (java.util.List<File>)data;
            
	    Runnable runnable = new Runnable() {
		public void run() {
                    // loop through the files in the file list
                    for (File file : fileList) {

                        m_grCon.openFile(file);

                    }
                }
            };
            runnable.run();
        } catch (UnsupportedFlavorException e) {
            return false;
        } catch (IOException e) {
            return false;
        }
        
        return true;
    }
}
