/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2007-07-20
 * Date started:    1998-06-11
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr.visual.gui;

import gov.usgs.gr.*;
import gov.usgs.gr.data.*;
import gov.usgs.gr.tools.TimeMarker;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

//==============================================================================
/**
 * A menu bar to be displayed across the top of the application and contain
 * all general purpose functions.
 */
public class GrMenuBar extends JMenuBar {

    // Menus...
    JMenu m_fileMenu;
    JMenu m_editMenu;
    JMenu m_viewMenu;
    JMenu m_pageMenu;
    JMenu m_toolsMenu;
    JMenu m_helpMenu;
    JMenu m_gapMenu;
    JPanel m_dynamicPanel = new JPanel();
    JPanel m_bar = new JPanel();
    /** The parent frame. */
    JFrame m_frame;
    /** An instance of the interface back to the main Gr view. */
    GrController m_grCon;
    /** A list of actions performed when the menu items are selected. */
    ArrayList m_toolBarActions = new ArrayList();

    protected boolean m_bProviderPanelEmpty = true;


    protected JRadioButton m_panRadio = new JRadioButton("Pan");
    protected JRadioButton m_zoomRadio = new JRadioButton("Zoom", true);
    protected JRadioButton m_selectRadio = new JRadioButton("Select");

//------------------------------------------------------------------------------
/**
     * The constructor for the menu bar.
     * @param frame The frame that this menu bar is attached to.
     * @param grCon The interface back to the main Gr object.
     */
    public GrMenuBar(JFrame frame, GrController grCon) {

        // Save parameters into the member variables.
        m_frame = frame;
        m_grCon = grCon;

        try {
            jbInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

//------------------------------------------------------------------------------
/**
     * Layout the components within the dialog.  This method can be used by
     * Borland JBuilder for designing the layout visually.
     */
    void jbInit() throws Exception {

        AbstractAction action;
        JMenuItem mi;

        // Set up the File menu...
        m_fileMenu = new JMenu("File"); // ---> File Menu
        action = new OpenAction(m_grCon); // Open File
        mi = m_fileMenu.add(action); // Ctrl-Shift-O
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.CTRL_MASK | ActionEvent.SHIFT_MASK));
        m_toolBarActions.add(action);
//        action = new OverlayAction(m_grCon);	// Overlay Data
//        mi = m_fileMenu.add(action);            // Ctrl-Shift-Y
//		mi.setAccelerator(KeyStroke.getKeyStroke(
//        	KeyEvent.VK_Y, ActionEvent.CTRL_MASK | ActionEvent.SHIFT_MASK));
        action = new SaveAction(m_grCon); // Save Data As
//        m_toolBarActions.addElement(action);// Ctrl-S
        mi = m_fileMenu.add(action);
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
        action = new PrintAction(m_grCon); // Save PS As
//        m_toolBarActions.addElement(action);// Ctrl-P
        mi = m_fileMenu.add(action);
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P, ActionEvent.CTRL_MASK));
//        action = new OpenProjectAction(m_grCon);	// Open Project
//        mi = m_fileMenu.add(action);
        action = new SavePageAction(m_grCon); // Save As XML
        mi = m_fileMenu.add(action);
//        m_toolBarActions.addElement(action);
//        action = new OpenScriptAction(m_grCon);	// Open Script
//        mi = m_fileMenu.add(action);
//        action = new PreferencesAction(m_grCon);	// Preferences
//        mi = m_fileMenu.add(action);                // Ctrl-Shift-F
//		mi.setAccelerator(KeyStroke.getKeyStroke(
//        	KeyEvent.VK_F, ActionEvent.CTRL_MASK | ActionEvent.SHIFT_MASK));
        m_fileMenu.add(new ExitAction()); // Exit
        m_fileMenu.getPopupMenu().setLightWeightPopupEnabled(false);

        // Set up the Edit menu...
        m_editMenu = new JMenu("Edit"); // ---> Edit Menu
        action = new UndoModificationAction(m_grCon); // Undo
        mi = m_editMenu.add(action); // Ctrl-U
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_U, ActionEvent.CTRL_MASK));
        action = new RedoModificationAction(m_grCon); // Redo
        mi = m_editMenu.add(action); // Ctrl-Shift-U
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_U, ActionEvent.CTRL_MASK | ActionEvent.SHIFT_MASK));
        action = new SelectModeAction(m_grCon); // Select Mode
//        m_toolBarActions.addElement(action);            // Ctrl-Shift-S
        mi = m_editMenu.add(action);
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK | ActionEvent.SHIFT_MASK));
        action = new EditSelectedAction(m_grCon); // Modify
        mi = m_editMenu.add(action); // Ctrl-M
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_M, ActionEvent.CTRL_MASK));
//        m_toolBarActions.addElement(action);
        action = new CutSelectedAction(m_grCon); // Cut
        mi = m_editMenu.add(action); // Ctrl-X
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, ActionEvent.CTRL_MASK));
        action = new CopySelectedAction(m_grCon); // Copy
        mi = m_editMenu.add(action); // Ctrl-C
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, ActionEvent.CTRL_MASK));
        action = new PasteSelectedAction(m_grCon); // Paste
        mi = m_editMenu.add(action); // Ctrl-V
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V, ActionEvent.CTRL_MASK));
        action = new DeleteSelectedAction(m_grCon); // Delete
        mi = m_editMenu.add(action); // <Delete>
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0));
//        m_toolBarActions.addElement(action);
        action = new SelectAllAction(m_grCon); // Select All
        mi = m_editMenu.add(action); // Ctrl-A
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A, ActionEvent.CTRL_MASK));
        action = new DeselectAllAction(m_grCon); // Deselect All
        mi = m_editMenu.add(action); // Ctrl-D
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D, ActionEvent.CTRL_MASK));
        m_editMenu.getPopupMenu().setLightWeightPopupEnabled(false);

        // Set up the View menu...
        m_viewMenu = new JMenu("View"); // ---> View menu
//        action = new ExploreModeAction(m_grCon);	// Explore
//        m_toolBarActions.addElement(action);        // Ctrl-Shift-X
//        mi = m_viewMenu.add(action);
//		mi.setAccelerator(KeyStroke.getKeyStroke(
//        	KeyEvent.VK_X, ActionEvent.CTRL_MASK | ActionEvent.SHIFT_MASK));
        action = new ZoomInAction(m_grCon); // Zoom in
        mi = m_viewMenu.add(action); // Ctrl-Shift-Z
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, ActionEvent.CTRL_MASK | ActionEvent.SHIFT_MASK));
        action = new ZoomOutAction(m_grCon); // Zoom out
        mi = m_viewMenu.add(action); // Ctrl-Z
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, ActionEvent.CTRL_MASK));
        action = new ZoomAllAction(m_grCon); // Zoom all
        m_toolBarActions.add(action); // Ctrl-L
        mi = m_viewMenu.add(action);
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L, ActionEvent.CTRL_MASK));
        action = new LockXAction(m_grCon); // Lock X
        mi = m_viewMenu.add(action); // Ctrl-Alt-X
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, ActionEvent.CTRL_MASK | ActionEvent.ALT_MASK));
        action = new LockYAction(m_grCon); // Lock Y
        mi = m_viewMenu.add(action); // Ctrl-Alt-Y
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Y, ActionEvent.CTRL_MASK | ActionEvent.ALT_MASK));
        action = new PanRightAction(m_grCon); // Pan Right
        mi = m_viewMenu.add(action);
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT, ActionEvent.CTRL_MASK));
        action = new PanRightAction(m_grCon); // Pan a lot Right (Hidden)
        mi = m_viewMenu.add(action);
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT, ActionEvent.SHIFT_MASK));
//        mi.setVisible(false);
        action = new PanLeftAction(m_grCon); // Pan Left
        mi = m_viewMenu.add(action);
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_LEFT, ActionEvent.CTRL_MASK));
        action = new PanLeftAction(m_grCon); // Pan a lot Left (Hidden)
        mi = m_viewMenu.add(action);
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_LEFT, ActionEvent.SHIFT_MASK));
//        mi.setVisible(false);
        action = new PanDownAction(m_grCon); // Pan Down
        mi = m_viewMenu.add(action);
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_DOWN, ActionEvent.CTRL_MASK));
        action = new PanUpAction(m_grCon); // Pan Up
        mi = m_viewMenu.add(action);
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_UP, ActionEvent.CTRL_MASK));
        m_viewMenu.getPopupMenu().setLightWeightPopupEnabled(false);

        // Set up the Page menu...
        m_pageMenu = new JMenu("Page"); // ---> Page menu
        action = new NewGraphAction(m_grCon); // New graph
        mi = m_pageMenu.add(action); // Ctrl-N
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, ActionEvent.CTRL_MASK));
//        m_toolBarActions.addElement(action);
        action = new HideGraphAction(m_grCon); // Hide graph
//        m_toolBarActions.addElement(action);        // Ctrl-H
        mi = m_pageMenu.add(action);
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_H, ActionEvent.CTRL_MASK));
        action = new ShowMoreGraphsAction(m_grCon); // Show more graphs
//        m_toolBarActions.addElement(action);        // Alt-S
        mi = m_pageMenu.add(action);
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.ALT_MASK));
        action = new NewPageAction(m_grCon); // New page
        mi = m_pageMenu.add(action);
//        m_toolBarActions.addElement(action);
        action = new DeletePageAction(m_grCon); // Delete Page
        mi = m_editMenu.add(action);
//        m_toolBarActions.addElement(action);
        action = new NextPageAction(m_grCon); // Show Next Page
        mi = m_editMenu.add(action);
//        m_toolBarActions.addElement(action);
//        action = new DrawFastAction(m_grCon);		// Draw fast
//        m_toolBarActions.addElement(action);        // Ctrl-F
//        mi = m_pageMenu.add(action);
//		mi.setAccelerator(KeyStroke.getKeyStroke(
//        	KeyEvent.VK_F, ActionEvent.CTRL_MASK));
        // Open a classic template
        mi = m_pageMenu.add(new OpenClassicTemplateAction(m_grCon));
        // Open an XML template
        mi = m_pageMenu.add(new OpenXMLTemplateAction(m_grCon));
        action = new EditPropertiesAction(m_grCon); // Edit Properties
        m_toolBarActions.add(action); // Ctrl-Shift-R
        mi = m_pageMenu.add(action);
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R, ActionEvent.CTRL_MASK | ActionEvent.SHIFT_MASK));
        m_pageMenu.getPopupMenu().setLightWeightPopupEnabled(false);

        // Set up the Tools menu...
        m_toolsMenu = new JMenu("Tools"); // ---> Tools menu
        m_gapMenu = new JMenu("Fill Gaps"); // Gap filling menu
        m_gapMenu.getPopupMenu().setLightWeightPopupEnabled(false);
        mi = m_gapMenu.add(new FillLinearlyAction(m_grCon));
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L, ActionEvent.CTRL_MASK | ActionEvent.SHIFT_MASK));
        mi = m_gapMenu.add(new FillParabolicallyAction(m_grCon));
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P, ActionEvent.CTRL_MASK | ActionEvent.SHIFT_MASK));
        m_toolsMenu.add(m_gapMenu);

        JMenu dataToolSubMenu = new JMenu("Data Tools");
        dataToolSubMenu.getPopupMenu().setLightWeightPopupEnabled(false);
        mi = dataToolSubMenu.add(new GodinFilterAction(m_grCon));
        mi = dataToolSubMenu.add(new Con120i913FilterAction(m_grCon));
        mi = dataToolSubMenu.add(new Con51G113FilterAction(m_grCon));
        mi = dataToolSubMenu.add(new Con24m214FilterAction(m_grCon));
        mi = dataToolSubMenu.add(new DigitalFilterAction(m_grCon));
        mi = dataToolSubMenu.add(new MeanCalculatorAction(m_grCon));
        mi = dataToolSubMenu.add(new RmsCalculatorAction(m_grCon));
        m_toolsMenu.add(dataToolSubMenu);

        JMenu drawSubMenu = new JMenu("Draw");
        drawSubMenu.getPopupMenu().setLightWeightPopupEnabled(false);
        mi = drawSubMenu.add(new StartDrawingAction(m_grCon)); // Start drawing
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_W, ActionEvent.CTRL_MASK));
        mi = drawSubMenu.add(new StopDrawingAction(m_grCon)); // Stop drawing
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_W, ActionEvent.CTRL_MASK | ActionEvent.SHIFT_MASK));
        m_toolsMenu.add(drawSubMenu);

        JMenu animSubMenu = new JMenu("Animation");
        animSubMenu.getPopupMenu().setLightWeightPopupEnabled(false);
        mi = animSubMenu.add(new CreateAnimationAction(m_grCon)); // Create anim
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A, ActionEvent.CTRL_MASK | ActionEvent.SHIFT_MASK));
        mi = animSubMenu.add(new ToggleRunningAction(m_grCon)); // Toggle running
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R, ActionEvent.CTRL_MASK));
        mi = animSubMenu.add(new IncPriorityAction(m_grCon)); // Increase priority
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_GREATER, ActionEvent.CTRL_MASK));
        mi = animSubMenu.add(new DecPriorityAction(m_grCon)); // Decrease priority
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_LESS, ActionEvent.CTRL_MASK));
        mi = animSubMenu.add(new IncSleepAction(m_grCon)); // Increase sleep
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_PERIOD, ActionEvent.CTRL_MASK));
        mi = animSubMenu.add(new DecSleepAction(m_grCon)); // Decrease sleep
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_COMMA, ActionEvent.CTRL_MASK));
        mi = animSubMenu.add(new IncStepAction(m_grCon)); // Increase step
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_EQUALS, ActionEvent.CTRL_MASK));
        mi = animSubMenu.add(new DecStepAction(m_grCon)); // Decrease step
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_MINUS, ActionEvent.CTRL_MASK));
        mi = animSubMenu.add(new StepForwardAction(m_grCon)); // Step forward
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_CLOSE_BRACKET, ActionEvent.CTRL_MASK));
        mi = animSubMenu.add(new StepBackwardAction(m_grCon)); // Step backward
        mi.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_OPEN_BRACKET, ActionEvent.CTRL_MASK));
        m_toolsMenu.add(animSubMenu);

        JMenu formatSubMenu = new JMenu("Format");
        formatSubMenu.getPopupMenu().setLightWeightPopupEnabled(false);
        mi = formatSubMenu.add(new ApplyDefaultFormatAction(m_grCon)); // Default format
        mi = formatSubMenu.add(new ApplyWorkingFormatAction(m_grCon)); // Working format
        mi = formatSubMenu.add(new ApplyReportFormatAction(m_grCon)); // Report format
        mi = formatSubMenu.add(new ApplySlideFormatAction(m_grCon)); // Slide format
        m_toolsMenu.add(formatSubMenu);
        mi = m_toolsMenu.add(new ApplyTemplateAction(m_grCon)); // Apply template
        m_toolsMenu.getPopupMenu().setLightWeightPopupEnabled(false);

        // Set up the Help menu...
        m_helpMenu = new JMenu("Help"); // ---> Help menu
        m_helpMenu.add(new AboutAction()); // About box
        m_helpMenu.getPopupMenu().setLightWeightPopupEnabled(false);

        // Put each menu on the menu bar.
        add(m_fileMenu);
        add(m_editMenu);
        add(m_viewMenu);
        add(m_pageMenu);
        add(m_toolsMenu);
//        m_toolsMenu.add(m_gapMenu);
        add(m_helpMenu);
    }

//------------------------------------------------------------------------------
/**
     * Create a button bar.
     */
    public JPanel getButtonBar() {

        m_bar.removeAll();
        m_bar.setLayout(new BoxLayout(m_bar, BoxLayout.Y_AXIS));

        JPanel generalPanel = new JPanel();
        generalPanel.setBorder(BorderFactory.createTitledBorder("General"));
        generalPanel.setLayout(new GridLayout(2, 1));

        JButton openFileButton = new JButton("Open File...");
        openFileButton.addActionListener(new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                OpenAction action = new OpenAction(m_grCon);
                action.actionPerformed(e);
//				m_grCon.openFile();
            }
        });

        JButton propButton = new JButton("Properties...");
        propButton.addActionListener(new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                EditPropertiesAction action = new EditPropertiesAction(m_grCon);
                action.actionPerformed(e);
/*
                GrObjectPropertiesDialog dialog = new GrObjectPropertiesDialog(
                m_frame, "Edit Properties", true);
                dialog.readFromObject(new ObjectEffigy(m_grCon, true));
                dialog.setVisible(true);
                if (m_grCon.getPage() == null) m_grCon.triggerRepaint();
                 */
            }
        });

        openFileButton.setMargin(new Insets(0, 0, 0, 0));
        propButton.setMargin(new Insets(0, 0, 0, 0));
        generalPanel.add(openFileButton);
        generalPanel.add(propButton);

        JPanel savePanel = new JPanel();
        savePanel.setBorder(BorderFactory.createTitledBorder("Save"));
        savePanel.setLayout(new GridLayout(3, 1));

        JButton saveGSButton = new JButton("As GS...");
        saveGSButton.addActionListener(new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                SaveAction action = new SaveAction(m_grCon);
                action.actionPerformed(e);
//				m_grCon.saveToDataFile();
            }
        });

        JButton savePSButton = new JButton("As PS...");
        savePSButton.addActionListener(new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                PrintAction action = new PrintAction(m_grCon);
                action.actionPerformed(e);
//				m_grCon.printPage();
            }
        });
//		javax.swing.ImageIcon buttonIcon =
//    		new javax.swing.ImageIcon(System.getProperty("gov.usgs.gr.grhome")
//            + "/res/GrObject/PsOptions.gif");
//        savePSButton.setIcon(buttonIcon);
//        savePSButton.setText(null);
        JButton saveXMLButton = new JButton("As XML...");
        saveXMLButton.addActionListener(new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                SavePageAction action = new SavePageAction(m_grCon);
                action.actionPerformed(e);
//				m_grCon.savePageAs();
            }
        });

        saveGSButton.setMargin(new Insets(0, 0, 0, 0));
        savePSButton.setMargin(new Insets(0, 0, 0, 0));
        saveXMLButton.setMargin(new Insets(0, 0, 0, 0));
        savePanel.add(saveGSButton);
        savePanel.add(savePSButton);
        savePanel.add(saveXMLButton);

        JPanel selectionPanel = new JPanel();
        selectionPanel.setBorder(BorderFactory.createTitledBorder("Selection"));
        selectionPanel.setLayout(new GridLayout(2, 1));

        JButton selectionModifyButton = new JButton("Modify");
        selectionModifyButton.addActionListener(new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                EditSelectedAction action = new EditSelectedAction(m_grCon);
                action.actionPerformed(e);
/*
                ModifyDialog dialog = new ModifyDialog(
                m_frame, "Modify Selected Points", true);
                dialog.readFromPage(m_grCon);
                dialog.setVisible(true);
                 */
            }
        });
        JButton selectionDeleteButton = new JButton("Delete");
        selectionDeleteButton.addActionListener(new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                DeleteSelectedAction action = new DeleteSelectedAction(m_grCon);
                action.actionPerformed(e);
//				m_grCon.deleteSelected();
            }
        });
        selectionModifyButton.setMargin(new Insets(0, 0, 0, 0));
        selectionDeleteButton.setMargin(new Insets(0, 0, 0, 0));
        selectionPanel.add(selectionModifyButton);
        selectionPanel.add(selectionDeleteButton);




        JPanel graphPanel = new JPanel();
        graphPanel.setBorder(BorderFactory.createTitledBorder("Graph"));
        graphPanel.setLayout(new GridLayout(4, 1));

        JButton graphNewButton = new JButton("New");
        graphNewButton.addActionListener(new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                NewGraphAction action = new NewGraphAction(m_grCon);
                action.actionPerformed(e);
//				m_grCon.newGraph();
            }
        });
        JButton graphShowButton = new JButton("Show");
        graphShowButton.addActionListener(new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                ShowMoreGraphsAction action = new ShowMoreGraphsAction(m_grCon);
                action.actionPerformed(e);
//				m_grCon.showMoreGraphs();
            }
        });
        JButton graphHideButton = new JButton("Hide");
        graphHideButton.addActionListener(new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                HideGraphAction action = new HideGraphAction(m_grCon);
                action.actionPerformed(e);
//				m_grCon.hideGraph();
            }
        });

        JButton zoomAllButton = new JButton("Zoom All");
        zoomAllButton.addActionListener(new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                ZoomAllAction action = new ZoomAllAction(m_grCon);
                action.actionPerformed(e);
//				m_grCon.zoomAll();
            }
        });
        graphNewButton.setMargin(new Insets(0, 0, 0, 0));
        graphShowButton.setMargin(new Insets(0, 0, 0, 0));
        graphHideButton.setMargin(new Insets(0, 0, 0, 0));
        zoomAllButton.setMargin(new Insets(0, 0, 0, 0));
        graphPanel.add(graphNewButton);
        graphPanel.add(graphShowButton);
        graphPanel.add(graphHideButton);
        graphPanel.add(zoomAllButton);


        JPanel pagePanel = new JPanel();
        pagePanel.setBorder(BorderFactory.createTitledBorder("Page"));
        pagePanel.setLayout(new GridLayout(3, 1));

        JButton pageNewButton = new JButton("New");
        pageNewButton.addActionListener(new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                NewPageAction action = new NewPageAction(m_grCon);
                action.actionPerformed(e);
            }
        });
        JButton pageNextButton = new JButton("Next");
        pageNextButton.addActionListener(new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                NextPageAction action = new NextPageAction(m_grCon);
                action.actionPerformed(e);
            }
        });
        JButton pageDeleteButton = new JButton("Delete");
        pageDeleteButton.addActionListener(new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                DeletePageAction action = new DeletePageAction(m_grCon);
                action.actionPerformed(e);
            }
        });
        pageNewButton.setMargin(new Insets(0, 0, 0, 0));
        pageNextButton.setMargin(new Insets(0, 0, 0, 0));
        pageDeleteButton.setMargin(new Insets(0, 0, 0, 0));
        pagePanel.add(pageNewButton);
        pagePanel.add(pageNextButton);
        pagePanel.add(pageDeleteButton);


        JPanel modePanel = new JPanel();
        modePanel.setBorder(BorderFactory.createTitledBorder("Mode"));
        modePanel.setLayout(new GridLayout(3, 1));

        m_panRadio.setMargin(new Insets(0, 0, 0, 0));
        m_zoomRadio.setMargin(new Insets(0, 0, 0, 0));
        m_selectRadio.setMargin(new Insets(0, 0, 0, 0));
        modePanel.add(m_panRadio);
        modePanel.add(m_zoomRadio);
        modePanel.add(m_selectRadio);
        ButtonGroup modeGroup = new ButtonGroup();
        modeGroup.add(m_panRadio);
        modeGroup.add(m_zoomRadio);
        modeGroup.add(m_selectRadio);
        final GrController grCon = m_grCon;
        m_panRadio.addChangeListener(new javax.swing.event.ChangeListener() {

            public void stateChanged(javax.swing.event.ChangeEvent e) {
                if (m_panRadio.isSelected()) {
                    grCon.usePanMode(true);
                    grCon.triggerRepaint();
                }
            }
        });
        m_zoomRadio.addChangeListener(new javax.swing.event.ChangeListener() {

            public void stateChanged(javax.swing.event.ChangeEvent e) {
                if (m_zoomRadio.isSelected()) {
                    grCon.useZoomMode(true);
                    grCon.triggerRepaint();
                }
            }
        });
        m_selectRadio.addChangeListener(new javax.swing.event.ChangeListener() {

            public void stateChanged(javax.swing.event.ChangeEvent e) {
                if (m_selectRadio.isSelected()) {
                    grCon.useSelectMode(true);
                    grCon.triggerRepaint();
                }
            }
        });

        m_dynamicPanel.setLayout(new BoxLayout(m_dynamicPanel, BoxLayout.Y_AXIS));

        m_bar.add(generalPanel);
        m_bar.add(savePanel);
        m_bar.add(pagePanel);
        m_bar.add(graphPanel);
        m_bar.add(selectionPanel);
        m_bar.add(modePanel);
        m_bar.add(m_dynamicPanel);

        return m_bar;
    }

//------------------------------------------------------------------------------
    public void showPanMode(boolean val) {

        if (m_panRadio.isSelected() != val) {
            m_panRadio.setSelected(val);
        }
    }

//------------------------------------------------------------------------------
    public void showZoomMode(boolean val) {

        if (m_zoomRadio.isSelected() != val) {
            m_zoomRadio.setSelected(val);
        }
    }

//------------------------------------------------------------------------------
    public void showSelectMode(boolean val) {

        if (m_selectRadio.isSelected() != val) {
            m_selectRadio.setSelected(val);
        }
    }

//------------------------------------------------------------------------------
/**
     * Share the list of actions with the outside.
     * @return The list of actions that can be taken by items in this menu bar.
     */
    public List getToolBarActions() {
        return m_toolBarActions;
    }

//------------------------------------------------------------------------------
/**
     * Check if there is a time marker on the page.  Show the animation panel
     * if there is.
     */
    public void updateDynamicPanel() {

        // Check if there is a time marker on the page.
        gov.usgs.gr.visual.PageOfGraphs page = m_grCon.getPage();
        if (page == null) {
            return;
        }
        m_dynamicPanel.removeAll();
        boolean bProviderPanelEmpty = true;

        // Look for a TimeMarker object as a child of the current page.
        Enumeration enumeration = page.children();
        while (enumeration.hasMoreElements()) {
            GrObject gro = (GrObject) enumeration.nextElement();
            if (gro instanceof GuiPanelProvider) {
                GuiPanelProvider provider = (GuiPanelProvider) gro;

                // Place the animation panel above the others, if present.
                if (provider instanceof TimeMarker) {
                    m_dynamicPanel.add(provider.getPanel(), 0);
                } else {
                    m_dynamicPanel.add(provider.getPanel());
                }
                bProviderPanelEmpty = false;
            }
        }

        m_bProviderPanelEmpty = bProviderPanelEmpty;
        javax.swing.SwingUtilities.invokeLater(new Runnable() {

            public void run() {
                m_dynamicPanel.setVisible(false);
                m_dynamicPanel.setVisible(!m_bProviderPanelEmpty);
            }
        });
    }

//------------------------------------------------------------------------------
    //==========================================================================
    /** Action class within GrMenuBar for opening a data file. */
    class OpenAction extends AbstractAction {

        //----------------------------------------------------------------------
        public OpenAction(GrController grCon) {
            super("Open File...");
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            m_grCon.openFile();
        }
    }

//==========================================================================
    /** Action class within GrMenuBar for opening a data file. */
/*    class OverlayAction extends AbstractAction {
    //----------------------------------------------------------------------
    public OverlayAction(GrController grCon) {
    super( "Overlay Data..." );
    }
    //----------------------------------------------------------------------
    public void actionPerformed(ActionEvent e) {
    m_grCon.overlayDataFile();
    }
    }
     */
    //==========================================================================
    /** Action class within GrMenuBar for opening a data file. */
    class SaveAction extends AbstractAction {

        //----------------------------------------------------------------------
        public SaveAction(GrController grCon) {
            super("Save As GS...");
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            m_grCon.saveToDataFile();
        }
    }

//==========================================================================
    /**
     * Action class within GrMenuBar for creating printed output of the page
     * on the screen.
     */
    class PrintAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public PrintAction(GrController grCon) {
            super("Save As PS...");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            m_grCon.printPage();
        }
    }

//==========================================================================
    /**
     * Action class within GrMenuBar for creating printed output of the page
     * on the screen.
     */
/*    class OpenProjectAction extends AbstractAction {
    GrController m_grCon;
    //----------------------------------------------------------------------
    public OpenProjectAction(GrController grCon) {
    super( "Open Project..." );
    m_grCon = grCon;
    }
    //----------------------------------------------------------------------
    public void actionPerformed(ActionEvent e) {
    m_grCon.openProject();
    }
    }
     */
    //==========================================================================
    /**
     * Action class within GrMenuBar for creating printed output of the page
     * on the screen.
     */
    class SavePageAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public SavePageAction(GrController grCon) {
            super("Save As XML...");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            m_grCon.savePageAs();
        }
    }

//==========================================================================
    /** Action class within GrMenuBar for opening a data file. */
/*    class OpenScriptAction extends AbstractAction {
    //----------------------------------------------------------------------
    public OpenScriptAction(GrController grCon) {
    super( "Open Script..." );
    }
    //----------------------------------------------------------------------
    public void actionPerformed(ActionEvent e) {
    m_grCon.openScript();
    }
    }
     */
    //==========================================================================
    /** Action class within GrMenuBar for editing graph properties. */
/*
    class PreferencesAction extends AbstractAction {
    //----------------------------------------------------------------------
    public PreferencesAction(GrController grCon) {
    super( "Preferences" );
    }
    //----------------------------------------------------------------------
    public void actionPerformed(ActionEvent e) {
    FilePreferencesDialog dialog =
    new FilePreferencesDialog(m_frame, "Preferences", true);
    dialog.readFromController(m_grCon);
    dialog.setVisible(true);
    }
    }
     */
    //==========================================================================
    /** Action class within GrMenuBar for exiting the program. */
    class ExitAction extends AbstractAction {

        //----------------------------------------------------------------------
        public ExitAction() {
            super("Exit");
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            System.exit(0);
        }
    }

//==========================================================================
    /** Action class within GrMenuBar for undoing modifications to a series. */
    class UndoModificationAction extends AbstractAction {

        //----------------------------------------------------------------------
        public UndoModificationAction(GrController grCon) {
            super("Undo");
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            m_grCon.undoModification();
        }
    }

//==========================================================================
    /** Action class within GrMenuBar for redoing undone modifications. */
    class RedoModificationAction extends AbstractAction {

        //----------------------------------------------------------------------
        public RedoModificationAction(GrController grCon) {
            super("Redo");
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            m_grCon.redoModification();
        }
    }

//==========================================================================
    /** Action class within GrMenuBar for changing to edit mode. */
    class SelectModeAction extends AbstractAction {

        //----------------------------------------------------------------------
        public SelectModeAction(GrController grCon) {
            super("Select");
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            m_grCon.useSelectMode(true);
        }
    }

//==========================================================================
    /** Action class within GrMenuBar for editing graph properties. */
    class EditSelectedAction extends AbstractAction {

        //----------------------------------------------------------------------
        public EditSelectedAction(GrController grCon) {
            super("Modify");
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            final ModifyDialog dialog = new ModifyDialog(m_frame, "Modify Selected Points", true);
            dialog.readFromPage(m_grCon);

            javax.swing.SwingUtilities.invokeLater(new Runnable() {

                public void run() {
                    dialog.setVisible(true);
                }
            });
        }
    }

//==========================================================================
    /** Action class within GrMenuBar for editing graph properties. */
    class CutSelectedAction extends AbstractAction {

        //----------------------------------------------------------------------
        public CutSelectedAction(GrController grCon) {
            super("Cut");
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            m_grCon.cutSelected();
        }
    }

//==========================================================================
    /** Action class within GrMenuBar for editing graph properties. */
    class CopySelectedAction extends AbstractAction {

        //----------------------------------------------------------------------
        public CopySelectedAction(GrController grCon) {
            super("Copy");
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            m_grCon.copySelected();
        }
    }

//==========================================================================
    /** Action class within GrMenuBar for editing graph properties. */
    class PasteSelectedAction extends AbstractAction {

        //----------------------------------------------------------------------
        public PasteSelectedAction(GrController grCon) {
            super("Paste");
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            m_grCon.pasteSelected();
        }
    }

//==========================================================================
    /** Action class within GrMenuBar for editing graph properties. */
    class DeleteSelectedAction extends AbstractAction {

        //----------------------------------------------------------------------
        public DeleteSelectedAction(GrController grCon) {
            super("Delete");
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            m_grCon.deleteSelected();
        }
    }

//==========================================================================
    /** Action class within GrMenuBar for editing graph properties. */
    class SelectAllAction extends AbstractAction {

        //----------------------------------------------------------------------
        public SelectAllAction(GrController grCon) {
            super("Select All");
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            m_grCon.selectAll();
        }
    }

//==========================================================================
    /** Action class within GrMenuBar for editing graph properties. */
    class DeselectAllAction extends AbstractAction {

        //----------------------------------------------------------------------
        public DeselectAllAction(GrController grCon) {
            super("Deselect All");
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            m_grCon.deselectAll();
        }
    }

/*
    //==========================================================================
    // Action class within GrMenuBar for zooming in on window area.
    class ExploreModeAction extends AbstractAction {
    GrController m_grCon;
    //----------------------------------------------------------------------
    public ExploreModeAction(GrController grCon) {
    super( "Explore" );
    putValue(Action.SHORT_DESCRIPTION, "Explore");
    m_grCon = grCon;
    }
    //----------------------------------------------------------------------
    public void actionPerformed(ActionEvent e) {
    m_grCon.useExploreMode();
    }
    }
     */
    //==========================================================================
    /** Action class within GrMenuBar for reverting to the last zoom. */
    class PanRightAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public PanRightAction(GrController grCon) {
            super("Pan Right");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            m_grCon.panAllHorz(e, 1);
        }
    }

//==========================================================================
    /** Action class within GrMenuBar for reverting to the last zoom. */
    class PanLeftAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public PanLeftAction(GrController grCon) {
            super("Pan Left");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            m_grCon.panAllHorz(e, -1);
        }
    }

//==========================================================================
    /** Action class within GrMenuBar for reverting to the last zoom. */
    class PanDownAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public PanDownAction(GrController grCon) {
            super("Pan Down");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            m_grCon.panAllDown();
        }
    }

//==========================================================================
    /** Action class within GrMenuBar for reverting to the last zoom. */
    class PanUpAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public PanUpAction(GrController grCon) {
            super("Pan Up");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            m_grCon.panAllUp();
        }
    }

//==========================================================================
    /** Action class within GrMenuBar for zooming in on the data view. */
    class ZoomInAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public ZoomInAction(GrController grCon) {
            super("Zoom In");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            m_grCon.zoomIn();
        }
    }

//==========================================================================
    /** Action class within GrMenuBar for zooming out on the data view. */
    class ZoomOutAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public ZoomOutAction(GrController grCon) {
            super("Zoom Out");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            m_grCon.zoomOut();
        }
    }

//==========================================================================
    /** Action class within GrMenuBar for displaying all data in the view. */
    class ZoomAllAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public ZoomAllAction(GrController grCon) {
            super("Zoom All");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            m_grCon.zoomAll();
        }
    }

//==========================================================================
    /**
     * Action class within GrMenuBar for keeping the X axis from being
     * effected by zooms or pans.
     */
    class LockXAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public LockXAction(GrController grCon) {
            super("Lock X");
            putValue(Action.SHORT_DESCRIPTION, "Lock X");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            m_grCon.lockX();
        }
    }

//==========================================================================
    /**
     * Action class within GrMenuBar for keeping the Y axis from being
     * effected by zooms or pans.
     */
    class LockYAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public LockYAction(GrController grCon) {
            super("Lock Y");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            m_grCon.lockY();
        }
    }

//==========================================================================
    /** Action class within GrMenuBar for editing graph properties. */
    class NewGraphAction extends AbstractAction {

        //----------------------------------------------------------------------
        public NewGraphAction(GrController grCon) {
            super("New Graph");
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            m_grCon.newGraph();
        }
    }

//==========================================================================
    /** Action class within GrMenuBar for editing graph properties. */
    class EditPropertiesAction extends AbstractAction {

        //----------------------------------------------------------------------
        public EditPropertiesAction(GrController grCon) {
            super("Properties...");
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            final GrObjectPropertiesDialog dialog = new GrObjectPropertiesDialog(m_frame, "Edit Properties", true);
            dialog.readFromObject(new ObjectEffigy(m_grCon, true));

            javax.swing.SwingUtilities.invokeLater(new Runnable() {

                public void run() {
                    dialog.setVisible(true);
                }
            });

            if (m_grCon.getPage() == null) {
                m_grCon.triggerRepaint();
            }
            updateDynamicPanel();
        }
    }

//==========================================================================
    /** Action class within GrMenuBar for opening a template file. */
    class OpenClassicTemplateAction extends AbstractAction {

        //----------------------------------------------------------------------
        public OpenClassicTemplateAction(GrController grCon) {
            super("Open classic template...");
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            m_grCon.openTemplate();
        }
    }

//==========================================================================
    /** Action class within GrMenuBar for opening a template file. */
    class OpenXMLTemplateAction extends AbstractAction {

        //----------------------------------------------------------------------
        public OpenXMLTemplateAction(GrController grCon) {
            super("Open XML template...");
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            m_grCon.openXMLTemplate();
        }
    }

//==========================================================================
    /**
     * Action class within GrMenuBar for hiding the currently selected graphs.
     */
    class HideGraphAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public HideGraphAction(GrController grCon) {
            super("Hide Graph");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            m_grCon.hideGraph();
        }
    }

//==========================================================================
    /**
     * Action class within GrMenuBar for showing one of the hidden graphs.
     */
    class ShowMoreGraphsAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public ShowMoreGraphsAction(GrController grCon) {
            super("Show Graph");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            m_grCon.showMoreGraphs();
        }
    }

//==========================================================================
    /** Action class within GrMenuBar for deleting the current page. */
    class NewPageAction extends AbstractAction {

        //----------------------------------------------------------------------
        public NewPageAction(GrController grCon) {
            super("New Page");
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            m_grCon.newPage();
        }
    }

//==========================================================================
    /** Action class within GrMenuBar for deleting the current page. */
    class DeletePageAction extends AbstractAction {

        //----------------------------------------------------------------------
        public DeletePageAction(GrController grCon) {
            super("Delete Page");
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            // Have the user confirm whether selected graphs should be deleted.
            gov.usgs.gr.visual.PageOfGraphs page = m_grCon.getPage();
            if (page != null) {
                int choice = JOptionPane.showConfirmDialog(m_frame, "Delete this page?", "Confirm Delete", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
                if (choice == JOptionPane.YES_OPTION) {
                    m_grCon.remove(page);
                    m_grCon.triggerRepaint();
                }
            }
        }
    }

//==========================================================================
    /** Action class within GrMenuBar for deleting the current page. */
    class NextPageAction extends AbstractAction {

        //----------------------------------------------------------------------
        public NextPageAction(GrController grCon) {
            super("Next Page");
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {

            m_grCon.popBottom();
        }
    }

//==========================================================================
    /**
     * Action class within GrMenuBar for drawing fast with low quality or
     * slow with high quality.
     */
    class DrawFastAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public DrawFastAction(GrController grCon) {
            super("Draw Fast");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            m_grCon.toggleQuickDraw();
        }
    }

//==========================================================================
    /**
     * Action class within GrMenuBar for filling gaps in a linear fashion.
     */
    class FillLinearlyAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public FillLinearlyAction(GrController grCon) {
            super("Linearly");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {

            List dataList = m_grCon.getPage().getSelectedSeriesList();

            if (dataList.size() == 0) {
                javax.swing.JOptionPane.showMessageDialog(null, "No data series were selected", "Notification", javax.swing.JOptionPane.INFORMATION_MESSAGE);
                return;
            }

            // Set the series to linear interpolation.
            Iterator iter = dataList.iterator();
            while (iter.hasNext()) {
                DoubleDataSeries series = (DoubleDataSeries) iter.next();
                series.m_bParabolicInterpolation = false;
            }

            DataSnapShot snapShot = new DataSnapShot(dataList);

            gov.usgs.gr.tools.SeriesGapFiller filler = new gov.usgs.gr.tools.SeriesGapFiller();
            filler.performAction(dataList);

            m_grCon.saveDataSnapShot(snapShot);

            m_grCon.triggerRepaint();
        }
    }

//==========================================================================
    /**
     * Action class within GrMenuBar for filling gaps in a linear fashion.
     */
    class FillParabolicallyAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public FillParabolicallyAction(GrController grCon) {
            super("Parabolically");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {

            List dataList = m_grCon.getPage().getSelectedSeriesList();

            if (dataList.size() == 0) {
                javax.swing.JOptionPane.showMessageDialog(null, "No data series were selected", "Notification", javax.swing.JOptionPane.INFORMATION_MESSAGE);
                return;
            }

            // Set the series to parabolic interpolation.
            Iterator iter = dataList.iterator();
            while (iter.hasNext()) {
                DoubleDataSeries series = (DoubleDataSeries) iter.next();
                series.m_bParabolicInterpolation = true;
            }

            DataSnapShot snapShot = new DataSnapShot(dataList);

            gov.usgs.gr.tools.SeriesGapFiller filler = new gov.usgs.gr.tools.SeriesGapFiller();
            filler.performAction(dataList);

            m_grCon.saveDataSnapShot(snapShot);

            m_grCon.triggerRepaint();
        }
    }

//==========================================================================
    /**
     * Action class within GrMenuBar for running a Godin filter on a curve.
     */
    class GodinFilterAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public GodinFilterAction(GrController grCon) {
            super("Godin Filter");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {

            List dataList = m_grCon.getPage().getSelectedSeriesList();

            if (dataList.size() == 0) {
                javax.swing.JOptionPane.showMessageDialog(null, "No data series were selected", "Notification", javax.swing.JOptionPane.INFORMATION_MESSAGE);
                return;
            }

            DataSnapShot snapShot = new DataSnapShot(dataList);

            gov.usgs.sfhydro.gr.tools.GodinFilter gf = new gov.usgs.sfhydro.gr.tools.GodinFilter();
            gf.performAction(dataList);

            m_grCon.saveDataSnapShot(snapShot);

            m_grCon.triggerRepaint();
        }
    }

//==========================================================================
    /**
     * Action class within GrMenuBar for running a Godin filter on a curve.
     */
    class Con120i913FilterAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public Con120i913FilterAction(GrController grCon) {
            super("Convolution 120i913 Filter");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {

            List dataList = m_grCon.getPage().getSelectedSeriesList();

            if (dataList.size() == 0) {
                javax.swing.JOptionPane.showMessageDialog(null, "No data series were selected", "Notification", javax.swing.JOptionPane.INFORMATION_MESSAGE);
                return;
            }

            DataSnapShot snapShot = new DataSnapShot(dataList);

            gov.usgs.sfhydro.gr.tools.ConvolutionFilter cf =
                new gov.usgs.sfhydro.gr.tools.ConvolutionFilter();
            cf.setProperty("numK", "120");
            cf.performAction(dataList);

            m_grCon.saveDataSnapShot(snapShot);

            m_grCon.triggerRepaint();
        }
    }

//==========================================================================
    /**
     * Action class within GrMenuBar for running a Godin filter on a curve.
     */
    class Con51G113FilterAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public Con51G113FilterAction(GrController grCon) {
            super("Convolution 51G113 Filter");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {

            List dataList = m_grCon.getPage().getSelectedSeriesList();

            if (dataList.size() == 0) {
                javax.swing.JOptionPane.showMessageDialog(null, "No data series were selected", "Notification", javax.swing.JOptionPane.INFORMATION_MESSAGE);
                return;
            }

            DataSnapShot snapShot = new DataSnapShot(dataList);

            gov.usgs.sfhydro.gr.tools.ConvolutionFilter cf =
                new gov.usgs.sfhydro.gr.tools.ConvolutionFilter();
            cf.setProperty("numK", "51");
            cf.performAction(dataList);

            m_grCon.saveDataSnapShot(snapShot);

            m_grCon.triggerRepaint();
        }
    }

//==========================================================================
    /**
     * Action class within GrMenuBar for running a Godin filter on a curve.
     */
    class Con24m214FilterAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public Con24m214FilterAction(GrController grCon) {
            super("Convolution 24m214 Filter");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {

            List dataList = m_grCon.getPage().getSelectedSeriesList();

            if (dataList.size() == 0) {
                javax.swing.JOptionPane.showMessageDialog(null, "No data series were selected", "Notification", javax.swing.JOptionPane.INFORMATION_MESSAGE);
                return;
            }

            DataSnapShot snapShot = new DataSnapShot(dataList);

            gov.usgs.sfhydro.gr.tools.ConvolutionFilter cf =
                new gov.usgs.sfhydro.gr.tools.ConvolutionFilter();
            cf.setProperty("numK", "24");
            cf.performAction(dataList);

            m_grCon.saveDataSnapShot(snapShot);

            m_grCon.triggerRepaint();
        }
    }

//==========================================================================
    /**
     * Action class within GrMenuBar for running a Digital filter on a curve.
     */
    class DigitalFilterAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public DigitalFilterAction(GrController grCon) {
            super("Digital Filter");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {

            List dataList = m_grCon.getPage().getSelectedSeriesList();

            if (dataList.size() == 0) {
                javax.swing.JOptionPane.showMessageDialog(null, "No data series were selected", "Notification", javax.swing.JOptionPane.INFORMATION_MESSAGE);
                return;
            }

            DataSnapShot snapShot = new DataSnapShot(dataList);

            gov.usgs.sfhydro.gr.tools.DigitalFilter gf = new gov.usgs.sfhydro.gr.tools.DigitalFilter();
            gf.performAction(dataList);

            m_grCon.saveDataSnapShot(snapShot);

            m_grCon.triggerRepaint();
        }
    }

//==========================================================================
    /**
     * Action class within GrMenuBar.
     */
    class StartDrawingAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public StartDrawingAction(GrController grCon) {
            super("Start Drawing");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {

            m_grCon.useDrawMode(true);
        }
    }

//==========================================================================
    /**
     * Action class within GrMenuBar.
     */
    class StopDrawingAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public StopDrawingAction(GrController grCon) {
            super("Stop Drawing");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {

            m_grCon.useDrawMode(false);
        }
    }

//==========================================================================
    /**
     * Action class within GrMenuBar for finding the mean of a curve.
     */
    class MeanCalculatorAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public MeanCalculatorAction(GrController grCon) {
            super("Calculate Mean");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {

            List dataList = m_grCon.getPage().getSelectedSeriesList();

            if (dataList.size() == 0) {
                javax.swing.JOptionPane.showMessageDialog(null, "No data series were selected", "Notification", javax.swing.JOptionPane.INFORMATION_MESSAGE);
                return;
            }

            DataSnapShot snapShot = new DataSnapShot(dataList);

            gov.usgs.gr.tools.MeanCalculator mc = new gov.usgs.gr.tools.MeanCalculator();
            mc.performAction(dataList);

            m_grCon.saveDataSnapShot(snapShot);

            m_grCon.triggerRepaint();
        }
    }

//==========================================================================
    /**
     * Action class within GrMenuBar for finding the mean of a curve.
     */
    class RmsCalculatorAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public RmsCalculatorAction(GrController grCon) {
            super("Calculate RMS");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {

            List dataList = m_grCon.getPage().getSelectedSeriesList();

            if (dataList.size() == 0) {
                javax.swing.JOptionPane.showMessageDialog(null, "No data series were selected", "Notification", javax.swing.JOptionPane.INFORMATION_MESSAGE);
                return;
            }

            DataSnapShot snapShot = new DataSnapShot(dataList);

            gov.usgs.sfhydro.gr.tools.RmsCalculator rc = new gov.usgs.sfhydro.gr.tools.RmsCalculator();
            rc.performAction(dataList);

            m_grCon.saveDataSnapShot(snapShot);

            m_grCon.triggerRepaint();
        }
    }

//==========================================================================
    /**
     * Action class within GrMenuBar.
     */
    class CreateAnimationAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public CreateAnimationAction(GrController grCon) {
            super("Create");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {

            m_grCon.createAnimation();
        }
    }

//==========================================================================
    /**
     * Action class within GrMenuBar for.
     */
    class ToggleRunningAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public ToggleRunningAction(GrController grCon) {
            super("Run (Toggle)");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {

            // Don't go on if the page is null.
            gov.usgs.gr.visual.PageOfGraphs page = m_grCon.getPage();
            if (page == null) {
                return;
            }
            TimeMarker tm = (TimeMarker) page.firstChildInstanceOf(
                "gov.usgs.gr.tools.TimeMarker", false);
            if (tm != null) {
                // Toggle the state of the run property.
                tm.toggleRunning();
            }
        }
    }

//==========================================================================
    /**
     * Action class within GrMenuBar for.
     */
    class IncPriorityAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public IncPriorityAction(GrController grCon) {
            super("Increase Thread Priority");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {

            // Don't go on if the page is null.
            gov.usgs.gr.visual.PageOfGraphs page = m_grCon.getPage();
            if (page == null) {
                return;
            }
            // Look for a TimeMarker object as a child of the current page.
            Enumeration enumeration = page.children();
            while (enumeration.hasMoreElements()) {
                GrObject gro = (GrObject) enumeration.nextElement();
                if (gro instanceof gov.usgs.gr.tools.TimeMarker) {
                    // Toggle the state of the run property.
                    HashMap props = gro.getProperties();
                    GrProperty p = (GrProperty) props.get("priority");
                    String oldValue = (String) p.value;
                    String newValue = "maximum";
                    if (oldValue.trim().toLowerCase().equals("minimum")) {
                        newValue = "normal";
                    }
                    gro.setProperty("priority", newValue);
                }
            }
        }
    }

//==========================================================================
    /**
     * Action class within GrMenuBar.
     */
    class DecPriorityAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public DecPriorityAction(GrController grCon) {
            super("Decrease Thread Priority");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {

            // Don't go on if the page is null.
            gov.usgs.gr.visual.PageOfGraphs page = m_grCon.getPage();
            if (page == null) {
                return;
            }
            // Look for a TimeMarker object as a child of the current page.
            Enumeration enumeration = page.children();
            while (enumeration.hasMoreElements()) {
                GrObject gro = (GrObject) enumeration.nextElement();
                if (gro instanceof gov.usgs.gr.tools.TimeMarker) {
                    // Toggle the state of the run property.
                    HashMap props = gro.getProperties();
                    GrProperty p = (GrProperty) props.get("priority");
                    String oldValue = (String) p.value;
                    String newValue = "minimum";
                    if (oldValue.trim().toLowerCase().equals("maximum")) {
                        newValue = "normal";
                    }
                    gro.setProperty("priority", newValue);
                }
            }
        }
    }

//==========================================================================
    /**
     * Action class within GrMenuBar.
     */
    class IncSleepAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public IncSleepAction(GrController grCon) {
            super("Increase Sleep Duration");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {

            // Don't go on if the page is null.
            gov.usgs.gr.visual.PageOfGraphs page = m_grCon.getPage();
            if (page == null) {
                return;
            }
            // Look for a TimeMarker object as a child of the current page.
            Enumeration enumeration = page.children();
            while (enumeration.hasMoreElements()) {
                GrObject gro = (GrObject) enumeration.nextElement();
                if (gro instanceof gov.usgs.gr.tools.TimeMarker) {
                    // Increase the sleep duration.
                    HashMap props = gro.getProperties();
                    GrProperty p = (GrProperty) props.get("sleep");
                    int value = (Integer) p.value;
                    value++;
                    value *= 1.1;

                    gro.setProperty("sleep", new String("" + value));
                }
            }
        }
    }

//==========================================================================
    /**
     * Action class within GrMenuBar.
     */
    class DecSleepAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public DecSleepAction(GrController grCon) {
            super("Decrease Sleep Duration");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {

            // Don't go on if the page is null.
            gov.usgs.gr.visual.PageOfGraphs page = m_grCon.getPage();
            if (page == null) {
                return;
            }
            // Look for a TimeMarker object as a child of the current page.
            Enumeration enumeration = page.children();
            while (enumeration.hasMoreElements()) {
                GrObject gro = (GrObject) enumeration.nextElement();
                if (gro instanceof gov.usgs.gr.tools.TimeMarker) {
                    // Decrease the sleep duration.
                    HashMap props = gro.getProperties();
                    GrProperty p = (GrProperty) props.get("sleep");
                    int value = (Integer) p.value;
                    value--;
                    value *= 0.9;
                    if (value <= 2) {
                        value = 2;
                    }
                    gro.setProperty("sleep", new String("" + value));
                }
            }
        }
    }

//==========================================================================
    /**
     * Action class within GrMenuBar.
     */
    class IncStepAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public IncStepAction(GrController grCon) {
            super("Increase Step");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {

            // Don't go on if the page is null.
            gov.usgs.gr.visual.PageOfGraphs page = m_grCon.getPage();
            if (page == null) {
                return;
            }
            // Look for a TimeMarker object as a child of the current page.
            Enumeration enumeration = page.children();
            while (enumeration.hasMoreElements()) {
                GrObject gro = (GrObject) enumeration.nextElement();
                if (gro instanceof gov.usgs.gr.tools.TimeMarker) {
                    // Increase the step.
                    HashMap props = gro.getProperties();
                    GrProperty p = (GrProperty) props.get("stepSize");
                    int value = (Integer) p.value;
                    value++;

                    gro.setProperty("stepSize", new String("" + value));
                }
            }
        }
    }

//==========================================================================
    /**
     * Action class within GrMenuBar.
     */
    class DecStepAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public DecStepAction(GrController grCon) {
            super("Decrease Step");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {

            // Don't go on if the page is null.
            gov.usgs.gr.visual.PageOfGraphs page = m_grCon.getPage();
            if (page == null) {
                return;
            }
            // Look for a TimeMarker object as a child of the current page.
            Enumeration enumeration = page.children();
            while (enumeration.hasMoreElements()) {
                GrObject gro = (GrObject) enumeration.nextElement();
                if (gro instanceof gov.usgs.gr.tools.TimeMarker) {
                    // Decrease the step.
                    HashMap props = gro.getProperties();
                    GrProperty p = (GrProperty) props.get("stepSize");
                    int value = (Integer) p.value;
                    value--;

                    gro.setProperty("stepSize", new String("" + value));
                }
            }
        }
    }

//==========================================================================
    /**
     * Action class within GrMenuBar.
     */
    class StepForwardAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public StepForwardAction(GrController grCon) {
            super("Step Forward");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {

            // Don't go on if the page is null.
            gov.usgs.gr.visual.PageOfGraphs page = m_grCon.getPage();
            if (page == null) {
                return;
            }
            TimeMarker tm = (TimeMarker) page.firstChildInstanceOf(
                "gov.usgs.gr.tools.TimeMarker", false);
            if (tm != null) {
                // Toggle the state of the run property.
                tm.stepForward();
            }
        }
    }

//==========================================================================
    /**
     * Action class within GrMenuBar.
     */
    class StepBackwardAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public StepBackwardAction(GrController grCon) {
            super("Step Backward");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {

            // Don't go on if the page is null.
            gov.usgs.gr.visual.PageOfGraphs page = m_grCon.getPage();
            if (page == null) {
                return;
            }
            TimeMarker tm = (TimeMarker) page.firstChildInstanceOf(
                "gov.usgs.gr.tools.TimeMarker", false);
            if (tm != null) {
                // Toggle the state of the run property.
                tm.stepBackward();
            }
        }
    }

//==========================================================================
    /**
     * Action class within GrMenuBar for applying a new format.
     */
    class ApplyDefaultFormatAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public ApplyDefaultFormatAction(GrController grCon) {
            super("Default Format");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            m_grCon.applyFormat(new gov.usgs.gr.visual.PageFormatter());
        }
    }

//==========================================================================
    /**
     * Action class within GrMenuBar for applying a new format.
     */
    class ApplyWorkingFormatAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public ApplyWorkingFormatAction(GrController grCon) {
            super("Working Format");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            m_grCon.applyFormat(new gov.usgs.sfhydro.gr.formats.WorkingFormatter());
        }
    }

//==========================================================================
    /**
     * Action class within GrMenuBar for applying a new format.
     */
    class ApplyReportFormatAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public ApplyReportFormatAction(GrController grCon) {
            super("Report Format");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            m_grCon.applyFormat(new gov.usgs.sfhydro.gr.formats.ReportFormatter());
        }
    }

//==========================================================================
    /**
     * Action class within GrMenuBar for applying a new format.
     */
    class ApplySlideFormatAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public ApplySlideFormatAction(GrController grCon) {
            super("Slide Format");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            m_grCon.applyFormat(new gov.usgs.sfhydro.gr.formats.SlideFormatter());
        }
    }

//==========================================================================
    /**
     * Action class within GrMenuBar for applying the current template.
     */
    class ApplyTemplateAction extends AbstractAction {

        GrController m_grCon;

        //----------------------------------------------------------------------
        public ApplyTemplateAction(GrController grCon) {
            super("Apply template");
            m_grCon = grCon;
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            m_grCon.applyXMLTemplate();
        }
    }

//==========================================================================
    /** Action class within GrMenuBar for displaying product info. */
    class AboutAction extends AbstractAction {

        //----------------------------------------------------------------------
        public AboutAction() {
            super("About");
        }

        //----------------------------------------------------------------------
        public void actionPerformed(ActionEvent e) {
            final HelpAboutDialog dialog = new HelpAboutDialog(m_frame, "About Gr", true);

            javax.swing.SwingUtilities.invokeLater(new Runnable() {

                public void run() {
                    dialog.setVisible(true);
                }
            });
        }
    }
}
