/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2006-06-06
 * Date started:    1999-03-15
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr.visual.gui;

import gov.usgs.gr.*;
import gov.usgs.gr.visual.*;
import gov.usgs.gr.visual.gui.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import java.text.*;
import java.util.*;

//==============================================================================
/**
 *
 */
public class GrObjectPropertiesDialog extends JDialog implements ActionListener {

	String[] m_graphTitles;
    boolean[] m_graphShowing;
    int m_graphIndex = 0;

    protected GrObject m_root;
    protected JTree m_tree;
    protected DefaultTreeModel m_treeModel;

    /** A list of available nodes. */
    protected GrHashList m_availableNodes = new GrHashList();
    /** A list of modified nodes. */
    protected GrHashList m_modifiedNodes = new GrHashList();
    /** A list of selected nodes. */
    protected ArrayList m_selectedNodes = new ArrayList();
    /** A list of cut or copied nodes. */
    protected ArrayList m_nodeClipBoard = new ArrayList();
    /** Whether to redepict the effigies after updating the originals. */
    boolean m_bRedepict = false;

	JPanel mainPanel = new JPanel();
	BorderLayout borderLayout1 = new BorderLayout();
	JTabbedPane tabbedPane = new JTabbedPane();
	JPanel topPanel = new JPanel();
 	BorderLayout lowerPanelLayout = new BorderLayout();
	JPanel okPanel = new JPanel();

	JPanel topPanel2 = new JPanel();
    JTree pageTree = new JTree();
    JScrollPane treeScrollPane = new JScrollPane();
    JScrollPane tableScrollPane = new JScrollPane();
	JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
    GrPropertiesPanel m_propPanel = new GrPropertiesPanel();

	JPanel lowerPanel = new JPanel();
	JButton okButton = new JButton();
	JButton applyButton = new JButton();
	JButton cancelButton = new JButton();

	JMenuItem applyMenuItem = new JMenuItem("Apply All Changes");
	JMenuItem newObjectMenuItem = new JMenuItem("New");
	JMenuItem cutMenuItem = new JMenuItem("Cut");
	JMenuItem copyMenuItem = new JMenuItem("Copy");
	JMenuItem pasteOriginalMenuItem = new JMenuItem("Paste Original");
	JMenuItem pasteNewCopyMenuItem = new JMenuItem("Paste New Copy");
    JMenuItem pasteComboDataSeries = new JMenuItem("Paste as DataSeries");

    JMenuItem newXMLProjFileMI = new JMenuItem("XML Project File");
    JMenuItem newDelimDataFileMI = new JMenuItem("Delimited Data File");
    JMenuItem newTimeDataFileMI = new JMenuItem("Time Data File");
    JMenuItem newPageMI = new JMenuItem("Page");
    JMenuItem newGraphMI = new JMenuItem("Graph");
    JMenuItem new3dGraphMI = new JMenuItem("3D Graph");
    JMenuItem newTextMarkupMI = new JMenuItem("Text Markup");
    JMenuItem newTextureMI = new JMenuItem("Image Texture");
    JMenuItem newFormulaCsMI = new JMenuItem("Formula Color Scheme");
    JMenuItem newInterpCsMI = new JMenuItem("Interpolating Color Scheme");
    JMenuItem newColorBarMI = new JMenuItem("Color Bar");
    JMenuItem newTransformContainerMI = new JMenuItem("Transformation Container");
    JMenuItem newTransformOpMI = new JMenuItem("Transformation Operation");
    JMenuItem newDimOrgMI = new JMenuItem("Dimension Organizer");
    JMenuItem newLPRMI = new JMenuItem("Line Point Reducer");
    JMenuItem newGodinMI = new JMenuItem("Godin Filter");
    JMenuItem newObjContainerMI = new JMenuItem("Object Container");
    JMenuItem newCsCalcMI = new JMenuItem("Cross Section Calculator");
    JMenuItem newVaCalcMI = new JMenuItem("Vector Field Animation Calculator");
    JMenuItem newPaCalcMI = new JMenuItem("Particle Animation Calculator");
    JMenuItem newPrtTrkMI = new JMenuItem("Particle Tracker");
    JMenuItem newTriFanMI = new JMenuItem("Triangle Fan Surface");
    JMenuItem newPartAnimMI = new JMenuItem("Particle Animation");
    JMenuItem newRectAnimMI = new JMenuItem("Rectangle Animation");
    JMenuItem newVectAnimMI = new JMenuItem("Vector Animation");
    JMenuItem newVelProfileMI = new JMenuItem("Velocity Profile");
    JMenuItem newLineMarkupMI = new JMenuItem("Line Markup");
    JMenuItem newTimeMarkerMI = new JMenuItem("Time Marker");
    JMenuItem newKeyFrameMI = new JMenuItem("Key Frame Handler");

//------------------------------------------------------------------------------
	public GrObjectPropertiesDialog(Frame frame, String title, boolean modal) {
		super(frame, title, modal);
		try  {
			jbInit();
			pack();
		}
		catch (Exception ex) {
			ex.printStackTrace();
		}
	}

//------------------------------------------------------------------------------
	public GrObjectPropertiesDialog() {
		this(null, "", false);
	}

//------------------------------------------------------------------------------
	void jbInit() throws Exception {

    	// Set up button listeners.
		okButton.setText("OK");
		okButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				okButton_actionPerformed(e);
			}
		});
		applyButton.setText("Apply");
		applyButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				applyButton_actionPerformed(e);
			}
		});
		cancelButton.setText("Cancel");
		cancelButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				cancelButton_actionPerformed(e);
			}
		});

        // Set the layout managers for the panels.
		mainPanel.setLayout(borderLayout1);
		lowerPanel.setLayout(lowerPanelLayout);


        // Add the components to the containers and the sub containers to
        // parent containers.
		getContentPane().add(mainPanel);
		mainPanel.add(splitPane, BorderLayout.CENTER);

        splitPane.setTopComponent(treeScrollPane);
        splitPane.setBottomComponent(tableScrollPane);
//        splitPane.setBottomComponent(m_propPanel);

		mainPanel.add(okPanel, BorderLayout.SOUTH);
		okPanel.add(okButton, null);
		okPanel.add(applyButton, null);
		okPanel.add(cancelButton, null);

        this.getRootPane().setMinimumSize(new Dimension(150, 150));


        applyMenuItem.addActionListener(this);
        cutMenuItem.addActionListener(this);
        copyMenuItem.addActionListener(this);
        pasteOriginalMenuItem.addActionListener(this);
        pasteNewCopyMenuItem.addActionListener(this);
        pasteComboDataSeries.addActionListener(this);
        newXMLProjFileMI.addActionListener(this);
        newDelimDataFileMI.addActionListener(this);
        newTimeDataFileMI.addActionListener(this);
        newPageMI.addActionListener(this);
        newGraphMI.addActionListener(this);
        new3dGraphMI.addActionListener(this);
        newTextMarkupMI.addActionListener(this);
        newTextureMI.addActionListener(this);
        newFormulaCsMI.addActionListener(this);
        newInterpCsMI.addActionListener(this);
        newColorBarMI.addActionListener(this);
        newTransformContainerMI.addActionListener(this);
        newTransformOpMI.addActionListener(this);
        newDimOrgMI.addActionListener(this);
        newLPRMI.addActionListener(this);
        newGodinMI.addActionListener(this);
        newObjContainerMI.addActionListener(this);
        newCsCalcMI.addActionListener(this);
        newVaCalcMI.addActionListener(this);
        newPaCalcMI.addActionListener(this);
        newPrtTrkMI.addActionListener(this);
        newTriFanMI.addActionListener(this);
        newPartAnimMI.addActionListener(this);
        newRectAnimMI.addActionListener(this);
        newVectAnimMI.addActionListener(this);
        newVelProfileMI.addActionListener(this);
        newLineMarkupMI.addActionListener(this);
        newTimeMarkerMI.addActionListener(this);
        newKeyFrameMI.addActionListener(this);
	}

//------------------------------------------------------------------------------
/**
 * Checks if the element is in the list of available nodes.  If it is not,
 * it loads its properties and adds them to the list.  It then adds the
 * node and its properties to the list of selected nodes.
 */
    protected void select(GrObject gro) {

    	GrHashList props = (GrHashList)m_availableNodes.get(gro);
        if (!m_availableNodes.containsKey(gro)) {
        	props = gro.getProperties();
            m_availableNodes.put(gro, props);
        }

        m_selectedNodes.add(gro);
    }

//------------------------------------------------------------------------------
/**
 *
 */
    protected GrHashList findMatchingProperties(Iterator iter) {

    	GrHashList matchers = null;

        while (iter.hasNext()) {
        	// Make the first map the one to check against.
            GrObject gro = (GrObject)iter.next();
        	GrHashList props = (GrHashList)
            	(((GrHashList)m_availableNodes.get(gro)).clone());
            if (matchers == null) {
            	matchers = props;
                continue;
            }

            // Check that each entry in matching list is also present in the
            // most recent map.
            Iterator matcherIter = matchers.listIterator();
            while (matcherIter.hasNext()) {
            	String matcherKey = (String)matcherIter.next();
                GrProperty grProp = (GrProperty)matchers.get(matcherKey);
                String matcherDescription = grProp.description;

                boolean foundMatch = false;
            	Iterator propsIter = props.listIterator();
                while (propsIter.hasNext()) {
            		String propsKey = (String)propsIter.next();
                	GrProperty grProp2 = (GrProperty)props.get(propsKey);
                	String propsDescription = grProp2.description;

                    // A match is when the key and description match.
                    if (propsKey.equals(matcherKey) &&
                    	propsDescription.equals(matcherDescription)) {

                        foundMatch = true;
                        break;
                    }
                }

                if (!foundMatch) matcherIter.remove();
            }
		}

        if (matchers == null) {
        	matchers = new GrHashList();
        }
        return matchers;
    }

//------------------------------------------------------------------------------
/**
 * Updates the list of modified node properties with the most recently
 * modified.
 */
    protected void updateModifiedList() {

        if (m_propPanel == null) return;

        HashMap newlyModifiedProps = m_propPanel.getModifiedProps();
        m_propPanel.clearModifiedProps();

        // Each selected node should have the modified properties
        // applied to it.
        ListIterator selectedIter = m_selectedNodes.listIterator(0);
        while (selectedIter.hasNext()) {
            // Get the node.
            GrObject selGro = (GrObject)selectedIter.next();

            // Get the property list.
            HashMap modProps = null;
            if (m_modifiedNodes.containsKey(selGro))
                modProps = (HashMap)m_modifiedNodes.get(selGro);
            else
                modProps = new HashMap();

            // Apply the change to each property in the list.
            int numNewlyModified = 0;

            Iterator newlyIter = newlyModifiedProps.keySet().iterator();
            while (newlyIter.hasNext()) {
                String key = (String)newlyIter.next();
                GrProperty p = (GrProperty)newlyModifiedProps.get(key);

                modProps.put(key, p);
                numNewlyModified++;
            }

            // Add the node and the property list to the modified nodes
            // group.
            if (numNewlyModified > 0)
                m_modifiedNodes.put(selGro, modProps);
        }
    }

//------------------------------------------------------------------------------
/**
 * Read the values from the page and put them in the dialog.
 * <p><b> Pre Conditions </b><br>
 *      The page has been set up.
 */
    public void readFromObject(GrObject gro) {

/////////////--------- Properties... --------->>>>>>>>>
        m_root = gro;

        m_treeModel = new DefaultTreeModel(gro);
        m_tree = new JTree(m_treeModel);
        m_tree.setCellRenderer(new JComponentCellRenderer());
        treeScrollPane.setViewportView(m_tree);
        tableScrollPane.setViewportView(m_propPanel);

    	Dimension d = getToolkit().getScreenSize();

        m_tree.setMinimumSize(new Dimension(100, 50));
        treeScrollPane.setMinimumSize(new Dimension(100, 50));
        tableScrollPane.setMinimumSize(new Dimension(100, 50));
        splitPane.setMinimumSize(new Dimension(100, 100));

        treeScrollPane.setPreferredSize(new Dimension(400,
        	(int)(0.5 * d.height)));
        tableScrollPane.setPreferredSize(new Dimension(400,
        	(int)(0.3 * d.height)));
        splitPane.setPreferredSize(new Dimension(450,
        	(int)(0.8 * d.height)));

		splitPane.setDividerLocation((int)(0.5 * d.height));

        //Add listener to components that can bring up popup menus.
        MouseListener popupListener = new PopupListener();
        treeScrollPane.addMouseListener(popupListener);
        m_tree.addMouseListener(popupListener);

// Use in the future.        m_tree.getSelectionModel().setSelectionMode
//                (TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION);

        //Listen for when the selection changes.
        m_tree.addTreeSelectionListener(new TreeSelectionListener() {
            public void valueChanged(TreeSelectionEvent e) {

                // Save the modified properties before creating a new list.
                updateModifiedList();

            	TreePath[] paths = m_tree.getSelectionPaths();
                m_selectedNodes.clear();

                if (paths != null) {
                	for (int i = 0; i < paths.length; i++) {
                		GrObject go =
                    		(GrObject)paths[i].getLastPathComponent();

                    	select(go);
                	}
                }

                GrHashList props = findMatchingProperties(
                	m_selectedNodes.iterator());

                // Create a new list of selected nodes and their common
                // properties for the table.

                m_propPanel.setProperties(props);
//                m_propPanel.revalidate();
                tableScrollPane.revalidate();
            }
		});

        m_tree.clearSelection();

        m_tree.setSelectionRow(0);

        // Compress the components before displaying.
        setSize(500, 400);
        pack();

	}

//------------------------------------------------------------------------------
/**
 * Write values from the dialog back to the real objects.
 */
    protected void updateObjects() {

    	// Save the modified properties before creating a new list.
        updateModifiedList();

        GrObject realRoot = m_root;
        while (realRoot instanceof gov.usgs.gr.ObjectEffigy)
        	realRoot = ((ObjectEffigy)realRoot).getSubject();
        realRoot.deferUpdates();

        Iterator modIter = m_modifiedNodes.keySet().iterator();
        while (modIter.hasNext()) {
        	GrObject gro = (GrObject)modIter.next();
            HashMap props = (HashMap)m_modifiedNodes.get(gro);

            gro.setProperties(props);
            if (gro instanceof ObjectEffigy) {
            	((ObjectEffigy)gro).updateSubject();
                ((ObjectEffigy)gro).getSubject().notifyObservers();
            }
            else {
            	gro.notifyObservers();
            }
        }

        m_modifiedNodes.clear();

        if (m_root instanceof ObjectEffigy && m_bRedepict) {
        	((ObjectEffigy)m_root).redepict();
// This updates the tree, but unfortunately collapses it also.
//            m_treeModel.nodeStructureChanged(m_root);
///            m_treeModel.reload();
        }

        realRoot.processDeferredUpdates();
	}

//------------------------------------------------------------------------------
/**
 * Cuts the selected nodes from the tree.
 */
    protected void cutSelected() {

    	m_nodeClipBoard.clear();

    	ListIterator selectedIter = m_selectedNodes.listIterator();
        while (selectedIter.hasNext()) {
        	// Get the node.
            GrObject selGro = (GrObject)selectedIter.next();

            // Add it to the clipboard and remove it from the available nodes.
            m_nodeClipBoard.add(selGro);
            m_availableNodes.remove(selGro);
            GrObject parent = m_root.findParentOf(selGro);
            if (parent == null) continue;
            parent.remove(selGro);

			modifiedChildrenOf(parent);
        }

        updateTreeStructure();
    }

//------------------------------------------------------------------------------
/**
 * Copies the selected nodes to this dialog's clipboard.
 */
    protected void copySelected() {

    	m_nodeClipBoard.clear();

    	ListIterator selectedIter = m_selectedNodes.listIterator();
        while (selectedIter.hasNext()) {
        	// Get the node.
            GrObject selGro = (GrObject)selectedIter.next();

            // Add it to the clipboard.
            m_nodeClipBoard.add(selGro);
        }
    }

//------------------------------------------------------------------------------
/**
 * Pastes a link of each object in the clipboard into each selected node.
 */
    protected void paste(boolean bNewCopy) {

    	ListIterator selectedIter = m_selectedNodes.listIterator();
        while (selectedIter.hasNext()) {
        	// Get the node.
            GrObject selGro = (GrObject)selectedIter.next();

    		ListIterator clipIter = m_nodeClipBoard.listIterator();
        	while (clipIter.hasNext()) {
        		// Get the node.
            	GrObject clipGro = (GrObject)clipIter.next();

            	// Add the object to each selected node.
                GrObject newGro = clipGro;
                while (newGro instanceof ObjectEffigy)
                	newGro = ((ObjectEffigy)newGro).getSubject();

                // Add either a copy or the original.
                if (bNewCopy) {
                    GrObject newCopy = newGro.createCopy(true);
                    selGro.add(newCopy);
                    newCopy.deleteObserver(selGro);
                }
                else {
                    selGro.add(newGro);
                    newGro.deleteObserver(selGro);
                }
            }

			modifiedChildrenOf(selGro);
        }

        updateTreeStructure();
    }

//------------------------------------------------------------------------------
/**
 * Pastes a DataSeries that is a combination of all DataDimensions in the
 * clipboard.
 */
    protected void pasteAsDataSeries() {

        gov.usgs.gr.tools.DimensionOrganizer organizer =
                new gov.usgs.gr.tools.DimensionOrganizer();
        ListIterator clipIter = m_nodeClipBoard.listIterator();
        while (clipIter.hasNext()) {
            // Get the node.
            GrObject clipGro = (GrObject)clipIter.next();

            // Add the object to each selected node.
            while (clipGro instanceof ObjectEffigy)
                clipGro = ((ObjectEffigy)clipGro).getSubject();

            if (clipGro instanceof gov.usgs.gr.data.DataDimension)
                organizer.add(clipGro);
        }

        gov.usgs.gr.data.DataSeries comboSeries =
                organizer.createCombinationSeries();

        if (comboSeries != null) {
            ListIterator selectedIter = m_selectedNodes.listIterator();
            while (selectedIter.hasNext()) {
                // Get the node.
                GrObject selGro = (GrObject)selectedIter.next();
                selGro.add(comboSeries);
                modifiedChildrenOf(selGro);
            }

            updateTreeStructure();
        }
    }

//------------------------------------------------------------------------------
/**
 * Pastes a link of each object in the clipboard into each selected node.
 */
    protected void addNew(String className) {

		ListIterator selectedIter = m_selectedNodes.listIterator();
        while (selectedIter.hasNext()) {
        	// Get the selected node.
            GrObject selGro = (GrObject)selectedIter.next();

            // Add the object to each selected node.
            try {
            	GrObject newGro =
                	(GrObject)Class.forName(className).newInstance();

                selGro.add(newGro);
            }
            catch (ClassNotFoundException cnfe) {
            	cnfe.printStackTrace();
            }
            catch (IllegalAccessException iae) {
            	iae.printStackTrace();
            }
            catch (InstantiationException ie) {
            	ie.printStackTrace();
            }

			modifiedChildrenOf(selGro);
        }

        updateTreeStructure();
    }

//------------------------------------------------------------------------------
/**
 * Add the parent and the property list to the modified nodes group.
 */
    protected void modifiedChildrenOf(GrObject parent) {

    	// Get the existing modified properties of the parent, if any.
        HashMap parentProps = null;
        if (m_modifiedNodes.containsKey(parent))
        	parentProps = (HashMap)m_modifiedNodes.get(parent);
        else
        	parentProps = new HashMap();

        // Add the parent and the property list to the modified nodes group.
        m_modifiedNodes.put(parent, parentProps);
    }

//------------------------------------------------------------------------------
/**
 * Updates the tree to reflect changes in the underlying structure.
 */
    protected void updateTreeStructure() {

        // Save tree state so it can be expanded.
		TreePath[] paths = new TreePath[m_tree.getRowCount()];
        for (int i = 0; i < paths.length; i++) {
			paths[i] = m_tree.getPathForRow(i);
        }

        m_treeModel.nodeStructureChanged(m_root);

        for (int i = 0; i < paths.length; i++) {
        	m_tree.makeVisible(paths[i]);
        }

        // Cause the effigies to be redepicted on next update (Apply).
        m_bRedepict = true;
    }

//------------------------------------------------------------------------------
/**
 * Return a float value converted from the passed object.
 * @param o	Either a Float or String object.
 */
    protected float floatVal(Object obj) {

    	float f = Float.NaN;

    	if (obj instanceof String)
        	f = Float.parseFloat((String)obj);
        if (obj instanceof Float)
        	f = (Float)obj;

        return f;

	}

//------------------------------------------------------------------------------
	void okButton_actionPerformed(ActionEvent e) {

        m_bRedepict = false;
    	updateObjects();
    	dispose();
	}

//------------------------------------------------------------------------------
	void applyButton_actionPerformed(ActionEvent e) {

    	updateObjects();
	}

//------------------------------------------------------------------------------
	void cancelButton_actionPerformed(ActionEvent e) {

    	dispose();
	}


//------------------------------------------------------------------------------
    // Returns just the class name -- no package info.
    protected String getClassName(Object o) {
        String classString = o.getClass().getName();
        int dotIndex = classString.lastIndexOf(".");
        return classString.substring(dotIndex+1);
    }

//------------------------------------------------------------------------------
    public void actionPerformed(ActionEvent e) {

        JMenuItem source = (JMenuItem)(e.getSource());

        if (source.getText().equals("Apply All Changes"))
        	updateObjects();
        if (source.getText().equals("Cut")) {
        	cutSelected();
        }
        if (source.getText().equals("Copy")) {
        	copySelected();
        }
        if (source.getText().equals("Paste Original")) {
        	paste(false);
        }
        if (source.getText().equals("Paste New Copy")) {
        	paste(true);
        }
        if (source.getText().equals("Paste as DataSeries")) {
            pasteAsDataSeries();
        }
        if (source.getText().equals("XML Project File")) {
        	addNew("gov.usgs.gr.data.XMLProjectFile");
        }
        if (source.getText().equals("Delimited Data File")) {
        	addNew("gov.usgs.gr.data.DelimitedDataFile");
        }
        if (source.getText().equals("Time Data File")) {
        	addNew("gov.usgs.gr.data.TimeDataFile");
        }
        if (source.getText().equals("Page")) {
        	addNew("gov.usgs.gr.visual.gl.PageOfGlGraphs");
        }
        if (source.getText().equals("Graph")) {
        	addNew("gov.usgs.gr.visual.gl.GlGraph");
        }
        if (source.getText().equals("3D Graph")) {
            addNew("gov.usgs.gr.visual.gl.Gl3dGraph");
        }
        if (source.getText().equals("Text Markup")) {
        	addNew("gov.usgs.gr.visual.TextMarkup");
        }
        if (source.getText().equals("Triangle Fan Surface")) {
        	addNew("gov.usgs.gr.visual.gl.GlDiscontinuousTriFanSurface");
        }
        if (source.getText().equals("Image Texture")) {
        	addNew("gov.usgs.gr.visual.gl.GlTexture");
        }
        if (source.getText().equals("Formula Color Scheme")) {
        	addNew("gov.usgs.gr.visual.FormulaColorScheme");
        }
        if (source.getText().equals("Interpolating Color Scheme")) {
            addNew("gov.usgs.gr.visual.InterpolatingColorScheme");
        }
        if (source.getText().equals("Color Bar")) {
            addNew("gov.usgs.gr.visual.gl.GlColorBar");
        }
        if (source.getText().equals("Transformation Container")) {
            addNew("gov.usgs.gr.visual.gl.GlTransformation");
        }
        if (source.getText().equals("Transformation Operation")) {
            addNew("gov.usgs.gr.visual.TransformationOperation");
        }
        if (source.getText().equals("Dimension Organizer")) {
        	addNew("gov.usgs.gr.tools.DimensionOrganizer");
        }
        if (source.getText().equals("Line Point Reducer")) {
        	addNew("gov.usgs.gr.tools.LinePointReducer");
        }
        if (source.getText().equals("Godin Filter")) {
        	addNew("gov.usgs.sfhydro.gr.tools.GodinFilter");
        }
        if (source.getText().equals("Object Container")) {
        	addNew("gov.usgs.gr.GrObjectContainer");
        }
        if (source.getText().equals("Cross Section Calculator")) {
        	addNew("gov.usgs.sfhydro.gr.tools.CrossSectionCalculator");
        }
        if (source.getText().equals("Vector Field Animation Calculator")) {
        	addNew("gov.usgs.sfhydro.gr.tools.VectAnimCalculator");
        }
        if (source.getText().equals("Particle Animation Calculator")) {
        	addNew("gov.usgs.sfhydro.gr.tools.ParticleAnimCalculator");
        }
        if (source.getText().equals("Time Marker")) {
        	addNew("gov.usgs.gr.tools.TimeMarker");
        }
        if (source.getText().equals("Key Frame Handler")) {
            addNew("gov.usgs.gr.tools.KeyFrameHandler");
        }
        if (source.getText().equals("Line Markup")) {
        	addNew("gov.usgs.gr.visual.LineMarkup");
        }
        if (source.getText().equals("Vector Animation")) {
        	addNew("gov.usgs.gr.visual.gl.GlVectorAnimation");
        }
        if (source.getText().equals("Particle Animation")) {
        	addNew("gov.usgs.gr.visual.gl.GlParticleAnimation");
        }
        if (source.getText().equals("Rectangle Animation")) {
        	addNew("gov.usgs.gr.visual.gl.GlRectAnimation");
        }
        if (source.getText().equals("Velocity Profile")) {
        	addNew("gov.usgs.gr.visual.gl.GlVelocityProfile");
        }
    }

//==============================================================================
	class PopupListener extends MouseAdapter {
    	public void mousePressed(MouseEvent e) {
        	maybeShowPopup(e);
        }

        public void mouseReleased(MouseEvent e) {
        	maybeShowPopup(e);
        }

        private void maybeShowPopup(MouseEvent e) {
        	if (e.isPopupTrigger()) {
                // Check for modified properties before displaying a menu.
                updateModifiedList();

                JPopupMenu popup = new JPopupMenu();
            	if (m_modifiedNodes.size() > 0) {
                	applyMenuItem.setForeground(new Color(0.5f, 0, 0));
                    popup.add(applyMenuItem);
                }
                // Only add each menu selection if it makes sense in
                // the current context.
                if (m_selectedNodes.size() > 0) {
                	popup.add(cutMenuItem);
                    cutMenuItem.setEnabled(true);
                    popup.add(copyMenuItem);
                    if (m_nodeClipBoard.size() > 0) {
                    	popup.add(pasteOriginalMenuItem);
                    	popup.add(pasteNewCopyMenuItem);

                        // If all items in the clipboard are DataDimensions
                        // add a menu item for pasting a combination data series
                        boolean bAllDims = true;
                        Iterator iter = m_nodeClipBoard.iterator();
                        while (iter.hasNext()) {
                            Object obj = iter.next();
                            while (obj instanceof ObjectEffigy)
                                obj = ((ObjectEffigy)obj).getSubject();
                            if (!(obj instanceof
                                    gov.usgs.gr.data.DataDimension)) {

                                bAllDims = false;
                                break;
                            }
                        }
                        if (bAllDims) {
                            popup.add(pasteComboDataSeries);
                        }
                    }

                    	JMenu newSubMenu = new JMenu("Add New");
                        newSubMenu.setForeground(new Color(0, 0, 0.5f));
                    	JMenu addFileSubMenu = new JMenu("File");
                    	JMenu addGeSubMenu = new JMenu("Graphical Element");
                    	JMenu addToolSubMenu = new JMenu("Tool");
                    	JMenu addAnimationSubMenu = new JMenu("Animation");
                        newSubMenu.add(addFileSubMenu);
                        newSubMenu.add(addGeSubMenu);
                        newSubMenu.add(addToolSubMenu);
                        newSubMenu.add(addAnimationSubMenu);
                    	addFileSubMenu.add(newXMLProjFileMI);
                    	addFileSubMenu.add(newDelimDataFileMI);
                    	addFileSubMenu.add(newTimeDataFileMI);
                    	addGeSubMenu.add(newPageMI);
                    	addGeSubMenu.add(newGraphMI);
                        addGeSubMenu.add(new3dGraphMI);
                    	addGeSubMenu.add(newTextMarkupMI);
                    	addGeSubMenu.add(newTextureMI);
                    	addGeSubMenu.add(newTriFanMI);
                    	addGeSubMenu.add(newFormulaCsMI);
                    	addGeSubMenu.add(newInterpCsMI);
                        addGeSubMenu.add(newColorBarMI);
                        addGeSubMenu.add(newTransformContainerMI);
                        addGeSubMenu.add(newTransformOpMI);
                    	addToolSubMenu.add(newDimOrgMI);
                    	addToolSubMenu.add(newLPRMI);
                    	addToolSubMenu.add(newGodinMI);
                    	addToolSubMenu.add(newObjContainerMI);
                    	addToolSubMenu.add(newVaCalcMI);
                    	addToolSubMenu.add(newPaCalcMI);
//                    	addToolSubMenu.add(newPrtTrkMI);
                    	addAnimationSubMenu.add(newTimeMarkerMI);
                    	addAnimationSubMenu.add(newVectAnimMI);
                    	addAnimationSubMenu.add(newPartAnimMI);
                    	addAnimationSubMenu.add(newRectAnimMI);
                    	addAnimationSubMenu.add(newVelProfileMI);
                        addAnimationSubMenu.add(newKeyFrameMI);
                    	addAnimationSubMenu.add(newLineMarkupMI);
                    	popup.add(newSubMenu);


                    boolean bSelectedAreSameClass = true;
                    String commonName = null;
                    Iterator iter = m_selectedNodes.iterator();
                    while (iter.hasNext()) {
                    	GrObject gro = (GrObject)iter.next();
                        if (gro == m_root) cutMenuItem.setEnabled(false);

                        while (gro instanceof ObjectEffigy)
                        	gro = ((ObjectEffigy)gro).getSubject();
                        String name = gro.getClass().getName();
                        if (commonName == null) {
                        	commonName = name;
                        }
                        else {
                        	if (!(name.equals(commonName))) {
                            	bSelectedAreSameClass = false;
                                break;
                            }
                        }
                    }

                    // Add custom commands
                    if (bSelectedAreSameClass && m_modifiedNodes.size() == 0) {
                    	GrObject gro = (GrObject)m_selectedNodes.get(0);
                    	JMenuItem individualItem = gro.getMenuCommands();
                        if (individualItem != null) {
                        	popup.addSeparator();
                    		popup.add(individualItem);
                        }
                    }
                }
                // Only display the menu if it is not empty.
                if (popup.getSubElements().length > 0) {
                	popup.show(e.getComponent(), e.getX(), e.getY());
                }
            }
        }
    }

}


