/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2006-08-03
 * Date started:    2002-10-21
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr.visual.gui;

import gov.usgs.gr.*;
import java.awt.Dimension;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

//==============================================================================
/**
 * A panel that shows property names and editable values in a two-column format.
 */
public class GrPropertiesPanel extends JPanel {

    GrHashList m_props = new GrHashList();
    List m_modifiedComps = new ArrayList();
    HashMap m_componentMap = new GrHashList();
    JSplitPane m_titleSplitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
    JPanel m_titlePanel = new JPanel();
//    JScrollPane m_scrollPane = new JScrollPane();
    int m_dividerLocation = 160;

//------------------------------------------------------------------------------
/**
 * Create an empty panel
 */
    public GrPropertiesPanel() {
        setupTitlePanel();
    }


//------------------------------------------------------------------------------
/**
 * Set up the title panel and split pane between the property and value labels.
 */
    public void setupTitlePanel() {

        m_titleSplitPane.setDividerLocation(m_dividerLocation);
        m_titleSplitPane.setDividerSize(2);
        m_titleSplitPane.setBackground(java.awt.Color.BLACK);

        JPanel leftPanel = new JPanel();
        leftPanel.setLayout(new BoxLayout(leftPanel, BoxLayout.X_AXIS));
        leftPanel.setBorder(new javax.swing.border.BevelBorder(
            javax.swing.border.BevelBorder.RAISED));
        leftPanel.add(Box.createRigidArea(new Dimension(5, 20)));
        leftPanel.add(new JLabel("Property"));
        leftPanel.add(Box.createRigidArea(new Dimension(10, 20)));
        m_titleSplitPane.setLeftComponent(leftPanel);

        JPanel rightPanel = new JPanel();
        rightPanel.setLayout(new BoxLayout(rightPanel, BoxLayout.X_AXIS));
        rightPanel.setBorder(new javax.swing.border.BevelBorder(
            javax.swing.border.BevelBorder.RAISED));
        rightPanel.add(Box.createRigidArea(new Dimension(5, 20)));
        rightPanel.add(new JLabel("Value"));
        rightPanel.add(Box.createRigidArea(new Dimension(10, 20)));
        m_titleSplitPane.setRightComponent(rightPanel);

        m_titleSplitPane.addPropertyChangeListener(
            new java.beans.PropertyChangeListener() {
                public void propertyChange(java.beans.PropertyChangeEvent e) {
                    if (e.getPropertyName().equals("dividerLocation")) {
                        m_dividerLocation = (Integer)e.getNewValue();
                        setProperties(m_props);
                        revalidate();
                    }
                }
            }
        );

        m_titlePanel.setLayout(new BoxLayout(m_titlePanel, BoxLayout.Y_AXIS));
        m_titlePanel.setBackground(java.awt.Color.BLACK);
        m_titlePanel.add(m_titleSplitPane);
        m_titlePanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
        m_titlePanel.setMinimumSize(new Dimension(50, 24));
        m_titlePanel.setPreferredSize(new Dimension(280, 26));
        m_titlePanel.setMaximumSize(new Dimension(Short.MAX_VALUE, 42));

    }

//------------------------------------------------------------------------------
/**
 *
 */
    public void setProperties(GrHashList props) {

        m_props = props;

        setBackground(java.awt.Color.WHITE);
        removeAll();
        repaint();
        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

        if (props.size() == 0) {
            add(new JLabel("No Properties"));
            add(Box.createVerticalGlue());
            return;
        }

        add(m_titlePanel);

        JPanel middlePanel = new JPanel();
        middlePanel.setLayout(new BoxLayout(middlePanel, BoxLayout.Y_AXIS));
        middlePanel.setBackground(java.awt.Color.BLACK);

//        middlePanel.setPreferredSize(new Dimension(400, 1));

//        m_scrollPane.removeAll();
//        m_scrollPane.setViewportView(middlePanel);
//        m_scrollPane.setMinimumSize(new Dimension(100, 50));
//        m_scrollPane.setPreferredSize(new Dimension(400, 400));
//        add(m_scrollPane);

        add(middlePanel);

        boolean bNoProps = true;

        Iterator iter = props.listIterator();
        while (iter.hasNext()) {
            String key = (String)iter.next();
            Object obj = props.get(key);
            if (obj instanceof GrProperty) {
                JPanel propertyPanel = addProperty((GrProperty)obj);

                if (propertyPanel != null) {
                    bNoProps = false;
                    middlePanel.add(propertyPanel);
                    middlePanel.add(Box.createRigidArea(new Dimension(1, 1)));
                }
            }
        }

        if (bNoProps) {
            add(new JLabel("  No Properties"));
        }

        add(Box.createVerticalGlue());

    }

//------------------------------------------------------------------------------
/**
 *
 */
    protected JPanel addProperty(GrProperty p) {

        if (p.display == false) return null;

        JPanel propPanel = new JPanel();

        propPanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
        propPanel.setMinimumSize(new Dimension(100, 22));
        propPanel.setPreferredSize(new Dimension(440, 24));
        propPanel.setMaximumSize(new Dimension(Short.MAX_VALUE, 42));

        propPanel.setBackground(java.awt.Color.WHITE);
        propPanel.setLayout(new BoxLayout(propPanel, BoxLayout.X_AXIS));
        propPanel.add(Box.createRigidArea(new Dimension(5, 20)));

        JLabel name = new JLabel(p.description);
        name.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
        name.setMinimumSize(new Dimension(m_dividerLocation - 5, 20));
        name.setPreferredSize(new Dimension(m_dividerLocation - 5, 22));
        name.setMaximumSize(new Dimension(m_dividerLocation - 5, 42));

        propPanel.add(name);

        JPanel spacer = new JPanel();
        spacer.setBackground(java.awt.Color.BLACK);
        spacer.setMinimumSize(new Dimension(1, 1));
        spacer.setPreferredSize(new Dimension(1, 1));
        spacer.setMaximumSize(new Dimension(1, 1600));
        propPanel.add(spacer);

        JPanel compPanel = new JPanel();
        propPanel.add(compPanel);
        compPanel.setLayout(new BoxLayout(compPanel, BoxLayout.X_AXIS));
        compPanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
///        compPanel.setMinimumSize(new Dimension(150, 21));
///        compPanel.setPreferredSize(new Dimension(280, 23));
///        compPanel.setMaximumSize(new Dimension(1500, 42));
        compPanel.add(Box.createRigidArea(new Dimension(5, 24)));

        Object value = p.value;
        JComponent comp = null;

        if (p.component != null) {
            if (p.component instanceof JComponent) {
                comp = p.component;
            }
        }
        if (comp == null) {
            if (p.value instanceof Boolean) {
                JCheckBox cb = new JCheckBox();
                cb.setSelected((Boolean)p.value);
                comp = cb;
            }
        }
        if (comp == null) {
            String text = null;
            if (p.value instanceof String) {
                text = filterNewlineForScreen((String)p.value);
            }
            if (p.value instanceof Number) {
                text = "" + p.value;
            }
            if (text != null) {
                if (p.edit) {
                    comp = new JTextField(text);
                }
                else {
                    comp = new JLabel(text);
                    compPanel.setBackground(java.awt.Color.WHITE);
                }
            }
        }
        if (comp != null) {
            final JComponent finalComp = comp;

            comp.addFocusListener(
                new java.awt.event.FocusListener() {
                    public void focusGained(FocusEvent e) {
                        if (!(m_modifiedComps.contains(finalComp))) {
                            m_modifiedComps.add(finalComp);
                        }
                    }
                    public void focusLost(FocusEvent e) {}
                }
            );
            if (comp instanceof java.awt.ItemSelectable) {
                ((java.awt.ItemSelectable)comp).addItemListener(
                    new java.awt.event.ItemListener() {
                        public void itemStateChanged(
                            java.awt.event.ItemEvent e) {

                            if (!(m_modifiedComps.contains(finalComp))) {
                                m_modifiedComps.add(finalComp);
                            }
                        }
                    }
                );

                // Disable if not editable
                comp.setEnabled(p.edit);
            }
            if (comp instanceof JComboBox) {
                java.awt.Component editor = ((JComboBox)comp).getEditor().
                    getEditorComponent();
                editor.addFocusListener(
                    new java.awt.event.FocusAdapter() {
                        public void focusGained(FocusEvent e) {
                            if (!(m_modifiedComps.contains(finalComp))) {
                                m_modifiedComps.add(finalComp);
                            }
                        }
                        public void focusLost(FocusEvent e) {}
                    }
                );
            }

            m_componentMap.put(comp, p);

            comp.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
            comp.setMinimumSize(new Dimension(50, 21));
            comp.setPreferredSize(new Dimension(278, 23));
            comp.setMaximumSize(new Dimension(Short.MAX_VALUE, 42));
            propPanel.add(comp);
            propPanel.add(Box.createHorizontalGlue());
        }

        add(propPanel);

        return propPanel;
    }

//------------------------------------------------------------------------------
    public HashMap getModifiedProps () {

        GrHashList modifiedProps = new GrHashList();

        synchronized (m_modifiedComps) {
            Iterator iter = m_modifiedComps.iterator();
            while (iter.hasNext()) {
                JComponent comp = (JComponent)iter.next();
                GrProperty p = (GrProperty)m_componentMap.get(comp);
                if (p == null) {
                    continue;
                }
                writeCompToProp(comp, p);

                Iterator iter2 = m_props.keySet().iterator();
                while (iter2.hasNext()) {
                    String key = (String)iter2.next();
                    Object obj = m_props.get(key);
                    if (obj == p) {
                        modifiedProps.put(key, p);
                    }
                }
            }
        }

        return (HashMap)modifiedProps;
    }

//------------------------------------------------------------------------------
    public void clearModifiedProps () {

        m_modifiedComps.clear();
    }

//------------------------------------------------------------------------------
    protected void writeCompToProp (JComponent comp, GrProperty p) {

        if (comp == null) return;

        if (comp instanceof JCheckBox) {
            JCheckBox cb = (JCheckBox)comp;
            p.value = cb.isSelected();
        }
        if (comp instanceof JTextField) {
            String text = ((JTextField)comp).getText();
            if (p.value instanceof String) {
                p.value = filterNewlineFromScreen(text);
            }
            try {
                if (p.value instanceof Integer) {
                    p.value = new Integer(text);
                }
                if (p.value instanceof Long) {
                    p.value = new Long(text);
                }
                if (p.value instanceof Float) {
                    p.value = new Float(text);
                }
                if (p.value instanceof Double) {
                    p.value = new Double(text);
                }
            }
            catch (NumberFormatException e) {}
        }
        if (comp instanceof JComboBox) {
            try {
                String text = (String)((JComboBox)comp).getSelectedItem();
                if (p.value instanceof String) {
                    p.value = filterNewlineFromScreen(text);
                }
            }
            catch (Exception e) {}
        }
    }

//------------------------------------------------------------------------------
/**
 * Converts newline characters to a backslash followed by an 'n'.
 */
    public String filterNewlineForScreen(String s) {

        String newString = "";
        int index1 = 0;

        while (index1 < s.length()) {
            int index2 = s.indexOf("" + '\n', index1);
            if (index2 < 0) index2 = s.length();

            newString += s.substring(index1, index2);
            index1 = index2 + 1;
            if (index1 < s.length()) newString += ("\\" + "n");
        }

        return newString;
    }

//------------------------------------------------------------------------------
/**
 * Converts a backslash followed by an 'n' to newline characters.
 */
    public String filterNewlineFromScreen(String s) {

        String newString = "";
        int index1 = 0;

        while (index1 < s.length()) {
            int index2 = s.indexOf("\\n", index1);
            if (index2 < 0) index2 = s.length();

            newString += s.substring(index1, index2);
            index1 = index2 + 2;
            if (index1 < s.length()) newString += '\n';
        }

        return newString;
    }
}
