/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2002-11-14
 * Date started:    1998-06-11
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr.visual.gui;

import gov.usgs.gr.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

//==============================================================================
/**
 * A panel that acts as a status bar by displaying labels.
 */
public class GrStatusBar extends JPanel {
    
    GrController m_grCon;
    
    public JTextField nearestXLabel = new JTextField(20);
    public JTextField nearestYLabel = new JTextField(15);
    public JLabel modeLabel = new JLabel();
    public JToggleButton axis1LockButton = new JToggleButton();
    public JToggleButton axis2LockButton = new JToggleButton();
    JButton switchBarButton = new JButton();
    protected int m_dateFormatIndex = 0;
    protected String[] m_dateFormats = {"0.00000", "yyyy/DDD HH:mm",
	"yy/DDD HH:mm:ss", "yyyy/MM/dd HH:mm", "yy/MM/dd HH:mm:ss",
	"EEE, MMM dd, yyyy", "hh:mm:ss aa"};
    public int m_coordLevelIndex = 0;
    protected String[] m_coordLevels = {"Nearest Pt.", "Under Mouse"};
    protected String[] m_coordDescriptions = {"Nearest Point to Mouse","Mouse"};
//    protected String[] m_coordLevels = {"Nearest", "User Space", "Frame 0-1",
//        "Graph 0-1", "Graph Pixels", "Page Pixels"};
//    protected String[] m_coordDescriptions = {"Nearest Point to Mouse",
//        "Mouse", "Mouse within Frame 0-1", "Mouse within Graph 0-1",
//        "Mouse within Graph in Pixels", "Mouse within Page in Pixels"};
    public String dateFormat = "";
    
//------------------------------------------------------------------------------
    /**
     * Empty constructor.
     */
    public GrStatusBar() {
    }
    
//------------------------------------------------------------------------------
    /**
     * Create an default status bar.
     */
    public GrStatusBar(GrController grCon) {
	
	m_grCon = grCon;
	
	try  {
	    jbInit();
	} catch (Exception ex) {
	    ex.printStackTrace();
	}
    }
    
//------------------------------------------------------------------------------
    /**
     * Sets up the contents of the status bar.  This method can be used by
     * Borland JBuilder to visually design the panel.
     */
    void jbInit() throws Exception {
	
	// Set coordinate label font to monospaced so the characters won't
	// jump around as the numbers change.
	nearestXLabel.setFont(new Font("Monospaced", Font.BOLD, 13));
	nearestYLabel.setFont(new Font("Monospaced", Font.BOLD, 13));
	
	// Add a mouse listener so the user can cycle the date format
	// with Ctrl-Click.
	nearestXLabel.addMouseListener(new java.awt.event.MouseListener() {
	    public void mousePressed(MouseEvent e) {
		boolean bCtrl = ((e.getModifiers() & e.CTRL_MASK) != 0);
		if (bCtrl) {
		    m_dateFormatIndex++;
		    if (m_dateFormatIndex >= m_dateFormats.length)
			m_dateFormatIndex = 0;
		    dateFormat = m_dateFormats[m_dateFormatIndex];
		    nearestXLabel.setText(dateFormat);
		}
	    }
	    public void mouseEntered(MouseEvent e) {}
	    public void mouseClicked(MouseEvent e) {}
	    public void mouseExited(MouseEvent e) {}
	    public void mouseReleased(MouseEvent e) {}
	});
	
	// Add a mouse listener so the user can cycle the coordinate mode
	// with Ctrl-Click.
	nearestYLabel.addMouseListener(new java.awt.event.MouseListener() {
	    public void mousePressed(MouseEvent e) {
		boolean bCtrl = ((e.getModifiers() & e.CTRL_MASK) != 0);
		if (bCtrl) {
		    m_coordLevelIndex++;
		    if (m_coordLevelIndex >= m_coordLevels.length)
			m_coordLevelIndex = 0;
		    String text = m_coordLevels[m_coordLevelIndex];
		    nearestYLabel.setText(text);
		    nearestXLabel.setToolTipText("X Value of "
			+ m_coordDescriptions[m_coordLevelIndex]);
		    nearestYLabel.setToolTipText("Y Value of "
			+ m_coordDescriptions[m_coordLevelIndex]);
		}
	    }
	    public void mouseEntered(MouseEvent e) {}
	    public void mouseClicked(MouseEvent e) {}
	    public void mouseExited(MouseEvent e) {}
	    public void mouseReleased(MouseEvent e) {}
	});
	
	// Initialize the labels and set their colors.
	switchBarButton.setActionCommand("switchBarButton");
	switchBarButton.setText("< More >");
	switchBarButton.setToolTipText("Switch to Detailed Status Bar");
	switchBarButton.addActionListener(new java.awt.event.ActionListener() {
	    
	    public void actionPerformed(ActionEvent e) {
		switchBarButton_actionPerformed(e);
	    }
	});
	
	modeLabel.setText("Status");
	modeLabel.setToolTipText("Current Mode");
	
	nearestXLabel.setToolTipText("X Value of "
	    + m_coordDescriptions[m_coordLevelIndex]);
	nearestXLabel.setEditable(false);
	nearestXLabel.setHorizontalAlignment(JTextField.RIGHT);
	nearestYLabel.setToolTipText("Y Value of "
	    + m_coordDescriptions[m_coordLevelIndex]);
	nearestYLabel.setEditable(false);
	nearestYLabel.setHorizontalAlignment(JTextField.RIGHT);
	
	axis1LockButton.setText("X Unlocked");
	axis1LockButton.setForeground(Color.blue);
	axis1LockButton.setToolTipText("Whether X Axes Are Locked");
	axis1LockButton.addChangeListener(
	    new javax.swing.event.ChangeListener() {
	    
	    public void stateChanged(ChangeEvent e) {
		axis1LockButton_stateChanged(e);
	    }
	});
	axis2LockButton.setText("Y Unlocked");
	axis2LockButton.setForeground(Color.blue);
	axis2LockButton.setToolTipText("Whether Y Axes Are Locked");
	axis2LockButton.addChangeListener(
	    new javax.swing.event.ChangeListener() {
	    
	    public void stateChanged(ChangeEvent e) {
		axis2LockButton_stateChanged(e);
	    }
	});
	
	// Place the labels in the panel.
	this.setLayout(new FlowLayout());
	this.add(switchBarButton, null);
	this.add(nearestXLabel, null);
	JLabel comma = new JLabel(", ");
	comma.setForeground(Color.black);
	this.add(comma, null);
	this.add(nearestYLabel, null);
	this.add(modeLabel, null);
	this.add(axis1LockButton, null);
	this.add(axis2LockButton, null);
    }
    
//------------------------------------------------------------------------------
    public void setDateFormatIndex(int dateFormatIndex) {
	
	if (dateFormatIndex < 0 || dateFormatIndex >= m_dateFormats.length)
	    return;
	
	m_dateFormatIndex = dateFormatIndex;
	dateFormat = m_dateFormats[m_dateFormatIndex];
	nearestXLabel.setText(dateFormat);
    }
    
//------------------------------------------------------------------------------
    public void showPanMode(boolean val) {
	
	if (val) modeLabel.setText("Pan");
	else modeLabel.setText(" ");
    }
    
//------------------------------------------------------------------------------
    public void showZoomMode(boolean val) {
	
	if (val) modeLabel.setText("Zoom");
	else modeLabel.setText(" ");
    }
    
//------------------------------------------------------------------------------
    public void showDrawMode(boolean val) {
	
	if (val) modeLabel.setText("Draw");
	else modeLabel.setText(" ");
    }
    
//------------------------------------------------------------------------------
    public void showSelectMode(boolean val) {
	
	if (val) modeLabel.setText("Select");
	else modeLabel.setText(" ");
    }
    
//------------------------------------------------------------------------------
    void switchBarButton_actionPerformed(ActionEvent e) {
	m_grCon.switchStatusBar();
    }
    
//------------------------------------------------------------------------------
    void axis1LockButton_stateChanged(ChangeEvent e) {
	if (axis1LockButton.isSelected()) {
	    axis1LockButton.setText("X Locked");
	    axis1LockButton.setForeground(Color.red);
	    m_grCon.m_bLockX = true;
	} else {
	    axis1LockButton.setText("X Unlocked");
	    axis1LockButton.setForeground(Color.blue);
	    m_grCon.m_bLockX = false;
	}
    }
    
//------------------------------------------------------------------------------
    void axis2LockButton_stateChanged(ChangeEvent e) {
	if (axis2LockButton.isSelected()) {
	    axis2LockButton.setText("Y Locked");
	    axis2LockButton.setForeground(Color.red);
	    m_grCon.m_bLockY = true;
	} else {
	    axis2LockButton.setText("Y Unlocked");
	    axis2LockButton.setForeground(Color.blue);
	    m_grCon.m_bLockY = false;
	}
	
    }
}


