/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2003-09-09
 * Date started:    1998-06-11
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr.visual.gui;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;

//==============================================================================
/**
 * A dialog box for displaying information about the program.
 */
public class HelpAboutDialog extends JDialog {

    /** The tree icon to use for this class. */
	public static javax.swing.ImageIcon USGS_ICON = new javax.swing.ImageIcon(
        Thread.currentThread().getContextClassLoader().getResource(
        "usgs2.gif"));

//------------------------------------------------------------------------------
/**
 * Construct a dialog tied to the specified frame and having a certain title.
 */
    public HelpAboutDialog(Frame frame, String title, boolean modal) {

        // Pass the parameters on to the super class.
        super(frame, title, modal);

        // Layout the contents of the dialog.
        try  {
            jbInit();
            pack();
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
    }


//------------------------------------------------------------------------------
/**
 * Construct the dialog without specifying a parent frame or title.
 */
    public HelpAboutDialog() {

        // Use the more specific constructor but pass it generic parameters.
        this(null, "", true);
    }

//------------------------------------------------------------------------------
/**
 * Layout the components within the container.  This method can be used by
 * Borland JBuilder for designing the layout visually.
 */
    void jbInit() throws Exception {

    	// Add the USGS logo to the top of the dialog box.
        JPanel logoPanel = new JPanel() {
            public void paintComponent(Graphics g) {
                setSize(USGS_ICON.getIconWidth(), USGS_ICON.getIconHeight());
                USGS_ICON.paintIcon(this, g, 0, 0);
            }
		};
        logoPanel.setPreferredSize(new Dimension(USGS_ICON.getIconWidth(),
        	USGS_ICON.getIconHeight()));

        // Add Gr credits.
        JLabel label1 = new JLabel("Gr Graphing Application, Rev. "
        	+ gov.usgs.gr.Gr.getVersion());
        JLabel label2 = new JLabel("John M. Donovan, 1999-2005");
        JLabel label3 = new JLabel("U.S. Geological Survey");
        JLabel label4 = new JLabel("Water Resources Division");
        JLabel label5 = new JLabel("Sacramento, California");

        JPanel mainPanel = new JPanel();
        mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));
		mainPanel.add(Box.createRigidArea(new Dimension(0,5)));
        mainPanel.add(logoPanel);
		mainPanel.add(Box.createRigidArea(new Dimension(0,5)));
        mainPanel.add(label1);
        mainPanel.add(label2);
        mainPanel.add(label3);
        mainPanel.add(label4);
        mainPanel.add(label5);
        mainPanel.add(Box.createRigidArea(new Dimension(0,5)));

        // Add a bordered area with library credits.
        JLabel credLabel1 =
        	new JLabel("Uses Sun Java 2, OpenGL, and JOGL");

        JPanel creditPanel = new JPanel();
        creditPanel.setLayout(new BoxLayout(creditPanel, BoxLayout.Y_AXIS));
        creditPanel.setBorder(BorderFactory.createEtchedBorder());
        creditPanel.add(credLabel1);

        mainPanel.add(creditPanel);
        mainPanel.add(Box.createRigidArea(new Dimension(0,5)));

        // Add the OK button to the bottom.
        JButton okButton = new JButton("OK");
        okButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dispose();
            }
        });

		// Add the OK button to the bottom.
        JPanel okPanel = new JPanel();
        okPanel.add(okButton);
        mainPanel.add(okPanel);

        getContentPane().add(mainPanel);
    }
}


