/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2007-06-28
 * Date started:    1999-04-01
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr.visual.gui;

import gov.usgs.gr.data.*;
import gov.usgs.gr.*;
import gov.usgs.gr.visual.*;
import java.awt.Container;
import java.awt.Frame;
import javax.swing.*;
import java.util.*;

//==============================================================================
/**
 *
 */
public class ModifyDialog extends JDialog {

    GrController m_grCon;
    PageOfGraphs m_page;
    List m_selectedSeriesList;
    String m_editorClassName = "gov.usgs.gr.visual.gui.XyEditorPanel";
    DataEditorPanel m_editorPanel = null;

//------------------------------------------------------------------------------
	public ModifyDialog(Frame frame, String title, boolean modal) {
		super(frame, title, modal);

	}

//------------------------------------------------------------------------------
	public ModifyDialog() {
		this(null, "", false);
	}

//------------------------------------------------------------------------------
/**
 * Read the values from the page and put them in the dialog.
 * <p><b> Pre Conditions </b><br>
 *      The page has been set up.
 */
    public void readFromPage(GrController grCon) {

        m_grCon = grCon;
        m_page = m_grCon.getPage();
        if (m_page != null) {
            synchronized (m_page) {
                m_selectedSeriesList = m_page.getSelectedSeriesList();

                GrHashList classNames = new GrHashList();
                Iterator selIter = m_selectedSeriesList.iterator();
                while (selIter.hasNext()) {
                    DataSeries series = (DataSeries) selIter.next();
                    String editorName = series.getEditorClassName();
                    if (editorName == null) {
                        if (DoubleDataSeries.countDims(series, 2))
                            editorName = "gov.usgs.gr.visual.gui.XyEditorPanel";
                        else
                            editorName =
                                "gov.usgs.gr.visual.gui.DataEditorPanel";
                    }
                    classNames.put(editorName, null);
                }

                if (classNames.size() > 0)
                    m_editorClassName = (String) classNames.getKey(0);
            }
        }

        // Compress the components before displaying.
        try {
            Class editorClass = Class.forName(m_editorClassName);
            Class defaultClass =
                    Class.forName("gov.usgs.gr.visual.gui.DataEditorPanel");
            if (defaultClass.isAssignableFrom(editorClass))
                m_editorPanel = (DataEditorPanel)editorClass.newInstance();
        }
        catch (Exception e) {}

        if (m_editorPanel == null)
            m_editorPanel = new XyEditorPanel();

        m_editorPanel.setDialog(this);
        m_editorPanel.readFromPage(grCon);
        getContentPane().add(m_editorPanel);
        setSize(500, 400);
        pack();
	}

//------------------------------------------------------------------------------
/**
 *
 */
    public void setVisible() {

        if (m_selectedSeriesList.size() == 0) {
            javax.swing.JOptionPane.showMessageDialog(getParentFrame(),
                "No data are selected for modification.",
                "Nothing Selected",
                javax.swing.JOptionPane.INFORMATION_MESSAGE);
        }
        else {
            super.setVisible(true);
        }
    }

//------------------------------------------------------------------------------
/**
 *
 */
    public DataEditorPanel getEditorPanel() {

        return m_editorPanel;
    }

//------------------------------------------------------------------------------
/**
 * Look for the frame which contains this component by searching its parent
 * components until a Frame is encountered.
 * @return The frame that holds this component.
 */
    public Frame getParentFrame() {

        Frame frame = null;
        Container container = this.getContentPane();

        // Ascend the hierarchy of containers until the parent frame is found.
        do {
			container = container.getParent();
			if (container instanceof Frame) frame = (Frame)container;
        } while (container != null && !(container instanceof Frame));

        return frame;
    }
}


