/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2007-04-23
 * Date started:    1999-04-01
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr.visual.gui;

import gov.usgs.gr.data.*;
import gov.usgs.gr.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.table.*;
import java.text.*;
import java.util.*;

//==============================================================================
/**
 *
 */
public class OpenConfirmationDialog extends JDialog {

    private Frame m_associatedFrame = null;
    private boolean m_bOkPressed = false;
    private boolean m_bOverlay = false;
    private String m_fileType = "gov.usgs.sfhydro.data.GsDataFile";
    ArrayList m_formatPresets = new ArrayList();
    GrFile m_grFile = null;

	BorderLayout mainBorderLayout = new BorderLayout();
	BorderLayout topBorderLayout = new BorderLayout();
	JPanel mainPanel = new JPanel();
	JPanel radioPanel = new JPanel();
	JPanel topPanel = new JPanel();
	JPanel lowerPanel = new JPanel();
	JButton okButton = new JButton("OK");
	JButton cancelButton = new JButton("Cancel");
    JButton optionsButton = new JButton("Options...");
	JComboBox formatComboBox = new JComboBox();
    JTextField formatText = new JTextField();

    ButtonGroup overlayGroup = new ButtonGroup();
    JRadioButton overlayRadio = new JRadioButton("Overlay");
    JRadioButton newPageRadio = new JRadioButton("New Page");

//------------------------------------------------------------------------------
	public OpenConfirmationDialog(Frame frame, String title, boolean modal) {
		super(frame, title, modal);

        m_associatedFrame = frame;

        m_formatPresets.add("gov.usgs.gr.data.XyDataFile");
        m_formatPresets.add("gov.usgs.gr.data.TimeDataFile");
        m_formatPresets.add("gov.usgs.sfhydro.data.GsDataFile");
        m_formatPresets.add("gov.usgs.sfhydro.data.GrGsDataFile");
        m_formatPresets.add("gov.usgs.sfhydro.data.CR10DataFile");
        m_formatPresets.add("gov.usgs.sfhydro.data.CR1000DataFile");
        m_formatPresets.add("gov.usgs.gr.nwis.adaps.rdb.RdbFile");
        m_formatPresets.add("gov.usgs.sfhydro.data.SFPortsDataFile");
        m_formatPresets.add("gov.usgs.sfhydro.data.VPVDataFile");
        m_formatPresets.add("gov.usgs.sfhydro.data.Dwr1DataFile");
        m_formatPresets.add("gov.usgs.sfhydro.data.Noaa1DataFile");
        m_formatPresets.add("gov.usgs.sfhydro.data.Noaa2DataFile");
        m_formatPresets.add("gov.usgs.sfhydro.data.Si3dBathymetryFile");
        m_formatPresets.add("gov.usgs.sfhydro.data.Si3dBarrierFile");
        m_formatPresets.add("gov.usgs.sfhydro.data.ListeningStationDataFile");
        m_formatPresets.add("gov.usgs.gr.ScriptFile99");
//        m_formatPresets.add("gov.usgs.gr.TemplateFile");
        m_formatPresets.add("gov.usgs.gr.data.XMLProjectFile");
//        m_formatPresets.add("gov.usgs.gr.data.CompressedProjectFile");
        m_formatPresets.add("gov.usgs.gr.data.DelimitedDataFile");

		try  {
			jbInit();
			pack();
		}
		catch (Exception ex) {
			ex.printStackTrace();
		}
	}

//------------------------------------------------------------------------------
	public OpenConfirmationDialog() {
		this(null, "", false);
	}

//------------------------------------------------------------------------------
	void jbInit() throws Exception {

    	// Set up button listeners.
		okButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				okButton_actionPerformed(e);
			}
		});
		cancelButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				cancelButton_actionPerformed(e);
			}
		});
		optionsButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				optionsButton_actionPerformed(e);
			}
		});
		formatComboBox.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				formatComboBox_actionPerformed(e);
			}
		});

        // Set the layout managers for the panels.
        mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));

		topPanel.setLayout(new BoxLayout(topPanel, BoxLayout.X_AXIS));
        topPanel.setBorder(BorderFactory.createTitledBorder("File Type"));

        Iterator iter = m_formatPresets.listIterator();
        while (iter.hasNext()) {
			formatComboBox.addItem(iter.next());
        }
        formatComboBox.setEditable(true);
		topPanel.add(formatComboBox);
        topPanel.add(Box.createRigidArea(new Dimension(5,0)));

        topPanel.add(optionsButton);

		mainPanel.add(topPanel);
		lowerPanel.setLayout(new BoxLayout(lowerPanel, BoxLayout.X_AXIS));

    	overlayGroup.add(overlayRadio);
    	overlayGroup.add(newPageRadio);
        overlayRadio.setSelected(m_bOverlay);
		radioPanel.setLayout(new BoxLayout(radioPanel, BoxLayout.X_AXIS));
        radioPanel.add(overlayRadio);
        radioPanel.add(newPageRadio);
        radioPanel.setBorder(BorderFactory.createBevelBorder(
            BevelBorder.LOWERED));

		lowerPanel.add(radioPanel);
        lowerPanel.add(Box.createHorizontalGlue());

		lowerPanel.add(okButton);
		lowerPanel.add(cancelButton);

		mainPanel.add(lowerPanel);

		getContentPane().add(mainPanel);
	}

//------------------------------------------------------------------------------
/**
 * Read the values from the GUI controller and put them in the dialog.
 * <p><b> Pre Conditions </b><br>
 *      The controller has been set up.
 */
    public void initialize(String bestGuessFormat, boolean bOverlay) {

        // Select the correct format combo box item.
        boolean bFoundFormat = false;
        int index = 0;
        Iterator iter = m_formatPresets.listIterator();
        while (iter.hasNext()) {
			String formatPreset = (String)iter.next();
        	if (bestGuessFormat.equals(formatPreset)) {

        		formatComboBox.setSelectedIndex(index);
                bFoundFormat = true;
                break;
            }
            index++;
        }

        // If the current format was not found, add it to the list.
        if (!bFoundFormat) {
        	formatComboBox.addItem(bestGuessFormat);
            formatComboBox.setSelectedIndex(m_formatPresets.size());
        }

        m_bOverlay = bOverlay;
        overlayRadio.setSelected(m_bOverlay);
        newPageRadio.setSelected(!m_bOverlay);

        // Compress the components before displaying.
        pack();

	}

//------------------------------------------------------------------------------
	public boolean getOkPressed() {

		return m_bOkPressed;
	}

//------------------------------------------------------------------------------
	public boolean getOverlay() {

		return m_bOverlay;
	}

//------------------------------------------------------------------------------
	public String getFileType() {

		return m_fileType;
	}

//------------------------------------------------------------------------------
	public GrFile getGrFile() {

		return m_grFile;
	}

//------------------------------------------------------------------------------
	public boolean setGrFile() {

        // Check if the chosen format is a type of GrFile.
        Class grFileClass = null;
        Class formatClass = null;
        String selectedClass = (String)formatComboBox.getSelectedItem();

        try {
        	grFileClass = Class.forName("gov.usgs.gr.GrFile");
            formatClass = Class.forName(selectedClass);

            if (!(grFileClass.isAssignableFrom(formatClass)))
                return false;

//            if (!(formatClass.isInstance(m_grFile)))
            if (m_grFile == null || !(formatClass.equals(m_grFile.getClass())))
                m_grFile = (GrFile)formatClass.newInstance();
        }
        catch (ClassNotFoundException cnfe) {
            JOptionPane.showMessageDialog(GrController.getParentFrame(this),
            	"The class " + selectedClass + " could not be found.",
                "Class Not Found", JOptionPane.ERROR_MESSAGE);
            return false;
        }
        catch (IllegalAccessException iae)  {
            JOptionPane.showMessageDialog(GrController.getParentFrame(this),
            	"The class " + selectedClass + " could not be accessed.",
                "Class Not Available", JOptionPane.ERROR_MESSAGE);
            return false;
        }
        catch (InstantiationException ie) {
            JOptionPane.showMessageDialog(GrController.getParentFrame(this),
            	"The class " + selectedClass + " could not be created.",
                "Class Not Available", JOptionPane.ERROR_MESSAGE);
            return false;
        }
        catch (Exception e2) {
            e2.printStackTrace();
//            JOptionPane.showMessageDialog(GrController.getParentFrame(this),
//            	"The file type is not valid.",
//                "Exception", JOptionPane.ERROR_MESSAGE);
            return false;
        }
        catch (Error e3) {
            JOptionPane.showMessageDialog(GrController.getParentFrame(this),
            	"The file type is not valid.",
                "Error", JOptionPane.ERROR_MESSAGE);
            return false;
        }

        return true;
	}

//------------------------------------------------------------------------------
	void okButton_actionPerformed(ActionEvent e) {

    	m_bOkPressed = true;
        m_bOverlay = overlayRadio.isSelected();
		m_fileType = (String)(formatComboBox.getSelectedItem());

    	dispose();
	}

//------------------------------------------------------------------------------
	void cancelButton_actionPerformed(ActionEvent e) {

    	m_bOkPressed = false;
    	dispose();
	}

//------------------------------------------------------------------------------
	void optionsButton_actionPerformed(ActionEvent e) {

        boolean bValid = setGrFile();
        if (m_grFile == null || !bValid) {
            optionsButton.setEnabled(false);
            return;
        }
        if (m_grFile.hasOptionsDialog()) {
            optionsButton.setEnabled(true);
            m_grFile.showOptionsDialog(m_associatedFrame);
        }
        else {
            optionsButton.setEnabled(false);
        }
	}

//------------------------------------------------------------------------------
	void formatComboBox_actionPerformed(ActionEvent e) {

        setGrFile();
        if (m_grFile == null) {
            optionsButton.setEnabled(false);
            return;
        }
        
        boolean bEnableOptionsButton = m_grFile.hasOptionsDialog();
        if (bEnableOptionsButton) {
            optionsButton.setEnabled(true);
        }
        else {
            optionsButton.setEnabled(false);
        }
    }
}


