/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2002-08-24
 * Date started:    2002-08-24
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr.visual.gui;

import gov.usgs.gr.data.*;
import gov.usgs.gr.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.text.*;
import java.util.*;
import javax.print.*;
import javax.print.attribute.*;
import javax.print.attribute.standard.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.table.*;

//==============================================================================
/**
 *
 */
public class PrintPsFileDialog extends JDialog {

    JList m_printerList = new JList();

    protected PrintService[] m_printServices = null;
    protected PrintService m_defaultService = null;
    protected File m_file = null;


//------------------------------------------------------------------------------
	public PrintPsFileDialog(Frame frame, String title, boolean modal,
        File file) {

		super(frame, title, modal);

        m_file = file;

        DocFlavor flavor = DocFlavor.INPUT_STREAM.POSTSCRIPT;
        PrintRequestAttributeSet aset =
            new HashPrintRequestAttributeSet();
        aset.add(MediaSizeName.ISO_A4);
        m_printServices =
            PrintServiceLookup.lookupPrintServices(flavor, aset);
        PrintService m_defaultService =
            PrintServiceLookup.lookupDefaultPrintService();

		try  {
			jbInit();
		}
		catch (Exception ex) {
			ex.printStackTrace();
		}
	}

//------------------------------------------------------------------------------
	public PrintPsFileDialog() {
		this(null, "", false, null);
	}

//------------------------------------------------------------------------------
	void jbInit() throws Exception {

        BorderLayout mainBorderLayout = new BorderLayout();
        BorderLayout dimBorderLayout = new BorderLayout();
        JPanel mainPanel = new JPanel();
        JPanel listPanel = new JPanel();
        JPanel lowerPanel = new JPanel();
        JButton printButton = new JButton();
        JButton closeButton = new JButton();

    	// Set up button listeners.
		printButton.setText("Send to Printer");
		printButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				printButton_actionPerformed(e);
			}
		});
		closeButton.setText("Close");
		closeButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				closeButton_actionPerformed(e);
			}
		});

        // Set the layout managers for the panels.
        mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));
        mainPanel.add(Box.createRigidArea(new Dimension(0,7)));

		listPanel.setLayout(new BoxLayout(listPanel, BoxLayout.X_AXIS));
        listPanel.add(Box.createRigidArea(new Dimension(7,0)));
        listPanel.setBorder(BorderFactory.createTitledBorder("Printers"));
        String[] printerNames = new String[m_printServices.length];
        for (int i = 0; i < printerNames.length; i++) {
            printerNames[i] = m_printServices[i].toString();
        }
        m_printerList.setListData(printerNames);
        m_printerList.setSelectedIndex(0);

        JScrollPane treeScrollPane = new JScrollPane();
        treeScrollPane.setViewportView(m_printerList);
        treeScrollPane.setMinimumSize(new Dimension(50, 50));
        treeScrollPane.setPreferredSize(new Dimension(300, 300));
		listPanel.add(treeScrollPane);
        listPanel.add(Box.createRigidArea(new Dimension(7,0)));
		mainPanel.add(listPanel);
        mainPanel.add(Box.createRigidArea(new Dimension(0,7)));

		lowerPanel.setLayout(new BoxLayout(lowerPanel, BoxLayout.X_AXIS));

        lowerPanel.add(Box.createHorizontalGlue());

		lowerPanel.add(printButton);
        lowerPanel.add(Box.createRigidArea(new Dimension(7,0)));
		lowerPanel.add(closeButton);

		mainPanel.add(lowerPanel);
        mainPanel.add(Box.createRigidArea(new Dimension(0,7)));

		getContentPane().add(mainPanel);

        pack();
	}

//------------------------------------------------------------------------------
	void printButton_actionPerformed(ActionEvent e) {

        if (m_printServices.length > 0 && m_file != null) {
            int selectedIndex = m_printerList.getSelectedIndex();
            DocPrintJob pj = m_printServices[selectedIndex].createPrintJob();
            // InputStreamDoc is an implementation of the Doc interface
            try {
                FileInputStream fis =
                        new FileInputStream(m_file.getAbsolutePath());
                DocFlavor flavor = DocFlavor.INPUT_STREAM.POSTSCRIPT;
                PrintRequestAttributeSet aset =
                    new HashPrintRequestAttributeSet();
                aset.add(MediaSizeName.ISO_A4);
                Doc doc = new SimpleDoc(fis, flavor, null);
                System.out.println("Sending print job to "
                                   + m_printServices[selectedIndex]);
                pj.print(doc, aset);
             }
             catch (javax.print.PrintException pe) {
             }
             catch (FileNotFoundException e2) {
             }
        }

    	dispose();
	}

//------------------------------------------------------------------------------
	void closeButton_actionPerformed(ActionEvent e) {

    	dispose();
	}
}


