/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2007-06-28
 * Date started:    2003-04-29
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr.visual.gui;

import gov.usgs.gr.data.*;
import gov.usgs.gr.*;
import gov.usgs.gr.tools.MovingAverageCalculator;
import gov.usgs.gr.visual.*;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.GridLayout;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;

//==============================================================================
/**
 *
 */
public class XyEditorPanel extends DataEditorPanel {
    
    protected JPanel operationMovAvgPanel = new JPanel();
    protected BorderLayout operationMovAvgLayout = new BorderLayout();
    protected JRadioButton movingAvgRadio = new JRadioButton("Moving Average");
    protected JPanel movingAvgSpanPanel = new JPanel();
    protected JLabel movingAvgSpanLabel = new JLabel("Span");
    protected JTextField movingAvgSpanTextField = new JTextField();
    protected JPanel movingAvgSamplesPanel = new JPanel();
    protected JLabel movingAvgSamplesLabel = new JLabel("Samples");
    protected JTextField movingAvgSamplesTextField = new JTextField();
    
    protected ButtonGroup operandGroup = new ButtonGroup();
    protected JRadioButton constantRadio = new JRadioButton("Constant");
    protected JPanel operandCurvePanel = new JPanel();
    protected BorderLayout operandCurveLayout = new BorderLayout();
    protected JRadioButton curveRadio = new JRadioButton("Curve");
    protected JComboBox curveCombo = new JComboBox();
    
    protected ButtonGroup dimensionGroup = new ButtonGroup();
    protected JRadioButton xDimRadio = new JRadioButton("Modify X");
    protected JRadioButton yDimRadio = new JRadioButton("Modify Y");
    
    protected JPanel interpolationPanel = new JPanel();
    protected GridLayout interpolationLayout = new GridLayout(1, 2);
    protected ButtonGroup interpolationGroup = new ButtonGroup();
    protected JRadioButton interpLinearRadio = new JRadioButton("Linear");
    protected JRadioButton interpParabRadio = new JRadioButton("Parabolic");
    
    protected JPanel outputPanel = new JPanel();
    protected BorderLayout outputLayout = new BorderLayout();
    protected JPanel newCurvePanel = new JPanel();
    protected BorderLayout newCurveLayout = new BorderLayout();
    protected ButtonGroup outputGroup = new ButtonGroup();
    protected JRadioButton outputToOriginalRadio = new JRadioButton("Modify Original");
    protected JRadioButton outputNewCurveRadio = new JRadioButton("Create New Curve");
    
    protected JPanel selectedPanel = new JPanel();
    protected GridLayout selectedLayout = new GridLayout(3, 1);
    protected ButtonGroup selectedGroup = new ButtonGroup();
    protected JRadioButton selectOldRadio = new JRadioButton("Old Curve");
    protected JRadioButton selectNewRadio = new JRadioButton("New Curve");
    protected JRadioButton selectBothRadio = new JRadioButton("Both Curves");
    
    protected JPanel intervalPanel = new JPanel();
    protected BorderLayout intervalLayout = new BorderLayout();
    protected ButtonGroup intervalGroup = new ButtonGroup();
    protected JRadioButton intervalC1Radio = new JRadioButton("Curve 1");
    protected JRadioButton intervalC2Radio = new JRadioButton("Curve 2");
    
    protected JPanel intervalNewPanel = new JPanel();
    protected BorderLayout intervalNewLayout = new BorderLayout();
    protected JPanel intervalRegularPanel = new JPanel();
    protected GridLayout intervalRegularLayout = new GridLayout(3, 1);
    protected JRadioButton intervalRegularRadio = new JRadioButton("Regular");
    protected JPanel regularIntervalPanel = new JPanel();
    protected JLabel regularIntervalLabel = new JLabel("Interval");
    protected JTextField regularIntervalTextField = new JTextField();
    protected JPanel regularOffsetPanel = new JPanel();
    protected JLabel regularOffsetLabel = new JLabel("Offset");
    protected JTextField regularOffsetTextField = new JTextField();
    
    protected JPanel resamplingMethodPanel = new JPanel();
    protected GridLayout resamplingMethodLayout = new GridLayout(2, 1);
    protected ButtonGroup resamplingMethodGroup = new ButtonGroup();
    protected JRadioButton resampInterpRadio = new JRadioButton("Interpolation");
    protected JRadioButton resampAvgRadio = new JRadioButton("Averaging");
    
//------------------------------------------------------------------------------
    public XyEditorPanel() {
        
        super(false);
        
        xyInit();
    }
    
//------------------------------------------------------------------------------
    protected void xyInit() {
        
        // Set up button listeners.
        okButton.setText("OK");
        okButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                okButton_actionPerformed(e);
            }
        });
        applyButton.setText("Apply");
        applyButton.addActionListener(new java.awt.event.ActionListener() {
            
            public void actionPerformed(ActionEvent e) {
                applyButton_actionPerformed(e);
            }
        });
        undoButton.setForeground(Color.blue);
        undoButton.setText("Undo");
        undoButton.addActionListener(new java.awt.event.ActionListener() {
            
            public void actionPerformed(ActionEvent e) {
                undoButton_actionPerformed(e);
            }
        });
        redoButton.setForeground(Color.blue);
        redoButton.setText("Redo");
        redoButton.addActionListener(new java.awt.event.ActionListener() {
            
            public void actionPerformed(ActionEvent e) {
                redoButton_actionPerformed(e);
            }
        });
        cancelButton.setForeground(Color.red);
        cancelButton.setText("Close");
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                cancelButton_actionPerformed(e);
            }
        });
        
        constantRadio.addChangeListener(new javax.swing.event.ChangeListener() {
            
            public void stateChanged(ChangeEvent e) {
                someComponent_stateChanged(e);
            }
        });
        outputToOriginalRadio.addChangeListener(new javax.swing.event.ChangeListener() {
            
            public void stateChanged(ChangeEvent e) {
                someComponent_stateChanged(e);
            }
        });
        outputNewCurveRadio.addChangeListener(new javax.swing.event.ChangeListener() {
            
            public void stateChanged(ChangeEvent e) {
                someComponent_stateChanged(e);
            }
        });
        intervalC1Radio.addChangeListener(new javax.swing.event.ChangeListener() {
            
            public void stateChanged(ChangeEvent e) {
                someComponent_stateChanged(e);
            }
        });
        intervalC2Radio.addChangeListener(new javax.swing.event.ChangeListener() {
            
            public void stateChanged(ChangeEvent e) {
                someComponent_stateChanged(e);
            }
        });
        intervalRegularRadio.addChangeListener(new javax.swing.event.ChangeListener() {
            
            public void stateChanged(ChangeEvent e) {
                someComponent_stateChanged(e);
            }
        });
        xDimRadio.addChangeListener(new javax.swing.event.ChangeListener() {
            
            public void stateChanged(ChangeEvent e) {
                someComponent_stateChanged(e);
            }
        });
        setRadio.addChangeListener(new javax.swing.event.ChangeListener() {
            
            public void stateChanged(ChangeEvent e) {
                someComponent_stateChanged(e);
            }
        });
        addRadio.addChangeListener(new javax.swing.event.ChangeListener() {
            
            public void stateChanged(ChangeEvent e) {
                someComponent_stateChanged(e);
            }
        });
        subRadio.addChangeListener(new javax.swing.event.ChangeListener() {
            
            public void stateChanged(ChangeEvent e) {
                someComponent_stateChanged(e);
            }
        });
        mulRadio.addChangeListener(new javax.swing.event.ChangeListener() {
            
            public void stateChanged(ChangeEvent e) {
                someComponent_stateChanged(e);
            }
        });
        divRadio.addChangeListener(new javax.swing.event.ChangeListener() {
            
            public void stateChanged(ChangeEvent e) {
                someComponent_stateChanged(e);
            }
        });
        noneRadio.addChangeListener(new javax.swing.event.ChangeListener() {
            
            public void stateChanged(ChangeEvent e) {
                someComponent_stateChanged(e);
            }
        });
        powRadio.addChangeListener(new javax.swing.event.ChangeListener() {
            
            public void stateChanged(ChangeEvent e) {
                someComponent_stateChanged(e);
            }
        });
        logRadio.addChangeListener(new javax.swing.event.ChangeListener() {
            
            public void stateChanged(ChangeEvent e) {
                someComponent_stateChanged(e);
            }
        });
        absRadio.addChangeListener(new javax.swing.event.ChangeListener() {
            
            public void stateChanged(ChangeEvent e) {
                someComponent_stateChanged(e);
            }
        });
        movingAvgRadio.addChangeListener(new javax.swing.event.ChangeListener() {
            
            public void stateChanged(ChangeEvent e) {
                someComponent_stateChanged(e);
            }
        });
        interpParabRadio.addChangeListener(
            new javax.swing.event.ChangeListener() {
            
            public void stateChanged(ChangeEvent e) {
                someComponent_stateChanged(e);
            }
        });
        
        this.setLayout(mainLayout);
        this.add(mainLeftPanel, BorderLayout.WEST);
        mainLeftPanel.setLayout(mainLeftLayout);
        mainLeftPanel.add(mainLeftUpperPanel, BorderLayout.NORTH);
        mainLeftUpperPanel.setLayout(mainLeftUpperLayout);
        mainLeftUpperPanel.add(operationPanel, BorderLayout.NORTH);
        operationPanel.setBorder(
            BorderFactory.createTitledBorder("Operation"));
        operationPanel.setLayout(operationLayout);
        operationPanel.add(operationSimplePanel, BorderLayout.NORTH);
        
        // Operation simple panel...
        operationSimplePanel.setBorder(BorderFactory.createEtchedBorder());
        operationSimplePanel.setLayout(operationSimpleLayout);
        operationSimplePanel.add(addRadio);
        operationSimplePanel.add(mulRadio);
        operationSimplePanel.add(setRadio);
        operationSimplePanel.add(subRadio);
        operationSimplePanel.add(divRadio);
        operationSimplePanel.add(noneRadio);
        operationSimplePanel.add(powRadio);
        operationSimplePanel.add(logRadio);
        operationSimplePanel.add(absRadio);
        operationGroup.add(noneRadio);
        operationGroup.add(setRadio);
        operationGroup.add(addRadio);
        operationGroup.add(subRadio);
        operationGroup.add(mulRadio);
        operationGroup.add(divRadio);
        operationGroup.add(powRadio);
        operationGroup.add(logRadio);
        operationGroup.add(absRadio);
        noneRadio.setSelected(true);
        
        // Operation moving average panel...
        operationPanel.add(operationMovAvgPanel, BorderLayout.CENTER);
        operationMovAvgPanel.setBorder(BorderFactory.createEtchedBorder());
        operationMovAvgPanel.setLayout(operationMovAvgLayout);
        operationMovAvgPanel.add(movingAvgRadio, BorderLayout.NORTH);
        operationGroup.add(movingAvgRadio);
        operationMovAvgPanel.add(movingAvgSpanPanel, BorderLayout.WEST);
        movingAvgSpanPanel.add(movingAvgSpanLabel, BorderLayout.EAST);
        movingAvgSpanLabel.setEnabled(false);
        movingAvgSpanPanel.add(movingAvgSpanTextField);
        movingAvgSpanTextField.setEnabled(false);
        movingAvgSpanTextField.setText("             0");
        operationMovAvgPanel.add(movingAvgSamplesPanel);
        movingAvgSamplesPanel.add(movingAvgSamplesLabel);
        movingAvgSamplesLabel.setEnabled(false);
        movingAvgSamplesPanel.add(movingAvgSamplesTextField);
        movingAvgSamplesTextField.setEnabled(false);
        movingAvgSamplesTextField.setText("             0");
        
        // Operand constant panel...
        mainLeftUpperPanel.add(operandPanel, BorderLayout.SOUTH);
        operandPanel.setBorder(BorderFactory.createTitledBorder("Operand"));
        operandPanel.setLayout(operandLayout);
        operandPanel.add(operandConstantPanel, BorderLayout.NORTH);
        operandConstantPanel.setLayout(operandConstantLayout);
        operandConstantPanel.add(constantRadio, BorderLayout.WEST);
        operandGroup.add(constantRadio);
        constantRadio.setSelected(true);
        operandConstantPanel.add(constantValueTextField, BorderLayout.CENTER);
        constantValueTextField.setText("0          ");
        
        // Operand curve panel...
        operandPanel.add(operandCurvePanel, BorderLayout.SOUTH);
        operandCurvePanel.setLayout(operandCurveLayout);
        operandCurvePanel.add(curveRadio, BorderLayout.WEST);
        operandGroup.add(curveRadio);
        operandCurvePanel.add(curveCombo, BorderLayout.EAST);
        curveCombo.setEnabled(false);
        
        // Main left lower panel...
        mainLeftPanel.add(mainLeftLowerPanel, BorderLayout.SOUTH);
        mainLeftLowerPanel.setLayout(mainLeftLowerLayout);
        mainLeftLowerPanel.add(dimensionPanel, BorderLayout.NORTH);
        
        // Dimension panel...
        dimensionPanel.setBorder(BorderFactory.createTitledBorder("Dimension"));
        dimensionPanel.setLayout(dimensionLayout);
        dimensionPanel.add(xDimRadio);
        dimensionPanel.add(yDimRadio);
        dimensionGroup.add(xDimRadio);
        dimensionGroup.add(yDimRadio);
        yDimRadio.setSelected(true);
        
        // Interpolation panel...
        mainLeftLowerPanel.add(interpolationPanel, BorderLayout.SOUTH);
        interpolationPanel.setBorder(
            BorderFactory.createTitledBorder("Interpolation"));
        interpolationPanel.setLayout(interpolationLayout);
        interpolationPanel.add(interpLinearRadio);
        interpolationPanel.add(interpParabRadio);
        interpolationGroup.add(interpLinearRadio);
        interpolationGroup.add(interpParabRadio);
        interpLinearRadio.setSelected(true);
        
        // Output panel...
        this.add(outputPanel, BorderLayout.EAST);
        outputPanel.setBorder(BorderFactory.createTitledBorder("Output"));
        outputPanel.setLayout(outputLayout);
        outputPanel.add(newCurvePanel, BorderLayout.NORTH);
        newCurvePanel.setLayout(newCurveLayout);
        newCurvePanel.add(outputToOriginalRadio, BorderLayout.NORTH);
        newCurvePanel.add(outputNewCurveRadio, BorderLayout.CENTER);
        outputGroup.add(outputToOriginalRadio);
        outputToOriginalRadio.setSelected(true);
        outputGroup.add(outputNewCurveRadio);
        
        // Selected panel...
        newCurvePanel.add(selectedPanel, BorderLayout.EAST);
        selectedPanel.setBorder(BorderFactory.createTitledBorder("Select"));
        selectedPanel.setLayout(selectedLayout);
        selectedPanel.add(selectOldRadio);
        selectedPanel.add(selectNewRadio);
        selectedPanel.add(selectBothRadio);
        selectedGroup.add(selectOldRadio);
        selectedGroup.add(selectNewRadio);
        selectedGroup.add(selectBothRadio);
        selectNewRadio.setSelected(true);
        
        // Interval panel...
        outputPanel.add(intervalPanel, BorderLayout.CENTER);
        intervalPanel.setBorder(BorderFactory.createTitledBorder("Interval"));
        intervalPanel.setLayout(intervalLayout);
        intervalPanel.add(intervalC1Radio, BorderLayout.NORTH);
        intervalC1Radio.setEnabled(false);
        intervalGroup.add(intervalC1Radio);
        intervalC1Radio.setSelected(true);
        
        // Interval new panel...
        intervalPanel.add(intervalNewPanel, BorderLayout.SOUTH);
        intervalNewPanel.setLayout(intervalNewLayout);
        intervalNewPanel.setBorder(BorderFactory.createEtchedBorder());
        
        // Curve 2 Radio...
        intervalNewPanel.add(intervalC2Radio, BorderLayout.NORTH);
        intervalC2Radio.setEnabled(false);
        intervalGroup.add(intervalC2Radio);
        
        // Interval Regular...
        intervalNewPanel.add(intervalRegularPanel, BorderLayout.CENTER);
        intervalRegularPanel.setLayout(intervalRegularLayout);
        intervalRegularPanel.setBorder(BorderFactory.createEtchedBorder());
        intervalRegularPanel.add(intervalRegularRadio);
        intervalGroup.add(intervalRegularRadio);
        intervalRegularRadio.setEnabled(false);
        intervalRegularPanel.add(regularIntervalPanel);
        regularIntervalPanel.add(regularIntervalLabel);
        regularIntervalLabel.setEnabled(false);
        regularIntervalPanel.add(regularIntervalTextField);
        regularIntervalTextField.setEnabled(false);
        regularIntervalTextField.setText("             0");
        intervalRegularPanel.add(regularOffsetPanel);
        regularOffsetPanel.add(regularOffsetLabel);
        regularOffsetLabel.setEnabled(false);
        regularOffsetPanel.add(regularOffsetTextField);
        regularOffsetTextField.setEnabled(false);
        regularOffsetTextField.setText("             0");
        
        // Resampling method panel...
        intervalNewPanel.add(resamplingMethodPanel, BorderLayout.EAST);
        resamplingMethodPanel.setBorder(
            BorderFactory.createTitledBorder("Resample With"));
        resamplingMethodPanel.setLayout(resamplingMethodLayout);
        resamplingMethodPanel.add(resampInterpRadio);
        resamplingMethodPanel.add(resampAvgRadio);
        resamplingMethodGroup.add(resampInterpRadio);
        resamplingMethodGroup.add(resampAvgRadio);
        resampInterpRadio.setSelected(true);
        
        // OK, Cancel buttons...
        this.add(okCancelPanel, BorderLayout.SOUTH);
        okCancelPanel.add(applyButton);
        okCancelPanel.add(undoButton);
        okCancelPanel.add(redoButton);
        okCancelPanel.add(cancelButton);
    }
    
//------------------------------------------------------------------------------
    /**
     * Read the values from the page and put them in the dialog.
     * <p><b> Pre Conditions </b><br>
     *      The page has been set up.
     */
    public void readFromPage(GrController grCon) {
        
        m_grCon = grCon;
        m_page = m_grCon.getPage();
        if (m_page == null) {
            constantValueTextField.setText("Page not available");
            return;
        }
        synchronized (m_page) {
            m_selectedSeriesList = m_page.getSelectedSeriesList();
            if (m_selectedSeriesList.size() == 0)
                constantValueTextField.setText("Nothing selected");
            
            // Add every curve to the combo box.
            try {
                curveCombo.removeAllItems();
            } catch (IndexOutOfBoundsException e) {}
            
            synchronized(m_page) {
                List graphList = m_page.getGraphList();
                Iterator graphIter = m_page.getGraphList().iterator();
                int graphNum = 1;
                while (graphIter.hasNext()) {
                    Graph graph = (Graph)graphIter.next();
                    List curveList = graph.getCurves();
                    Iterator curveIter = curveList.iterator();
                    int curveNum = 1;
                    while (curveIter.hasNext()) {
                        Curve curve = (Curve)curveIter.next();
                        String curveTitle = curve.getSeries().getTitle();
                        if (curveTitle.length() > 70)
                            curveTitle = curveTitle.substring(0, 70);
                        String curveEntry = "G" + graphNum + " C"
                            + curveNum + ": " + curveTitle;
                        curveCombo.addItem(curveEntry);
                        curveNum++;
                    }
                    graphNum++;
                }
            }
        }
    }
    
//------------------------------------------------------------------------------
    /**
     * Opens the specified script file and applies it to the data.
     */
    public void applyModification(List attribs) {
        
        Iterator iter = attribs.iterator();
        while (iter.hasNext()) {
            String s = (String)iter.next();
            String key = s.substring(0, s.indexOf('=')).trim().toUpperCase();
            String value = s.substring(s.indexOf('=') + 1, s.length()).trim();
            value = value.toUpperCase();
            
            // Handle the key and value.  Each key and value corresponds
            // to a choice on this dialog box.
            if (key.equals("ACTION")) {
                if (value.equals("NONE")) noneRadio.setSelected(true);
                if (value.equals("ADD")) addRadio.setSelected(true);
                if (value.equals("SUB")) subRadio.setSelected(true);
                if (value.equals("MUL")) mulRadio.setSelected(true);
                if (value.equals("DIV")) divRadio.setSelected(true);
                if (value.equals("SET")) setRadio.setSelected(true);
                if (value.equals("POW")) powRadio.setSelected(true);
                if (value.equals("LOG")) logRadio.setSelected(true);
                if (value.equals("ABS")) absRadio.setSelected(true);
                if (value.equals("AVG")) movingAvgRadio.setSelected(true);
            }
            if (key.equals("AVGSPAN")) {
                movingAvgSpanTextField.setText(value);
            }
            if (key.equals("AVGSAMPLES")) {
                movingAvgSamplesTextField.setText(value);
            }
            if (key.equals("DIM")) {
                if (value.equals("0") || value.equals("X"))
                    xDimRadio.setSelected(true);
                if (value.equals("1") || value.equals("Y"))
                    yDimRadio.setSelected(true);
            }
            if (key.equals("INPUT")) {
                int gIndex = s.indexOf('G');
                int cIndex = s.indexOf('C');
                if (gIndex >= 0 && cIndex >= 0) {
                    curveRadio.setSelected(true);
                    // Find the graph number and curve number specified
                    int graphNum1 = Integer.parseInt(
                        s.substring(gIndex + 1, cIndex).trim()) - 1;
                    int curveNum1 = Integer.parseInt(
                        s.substring(cIndex + 1).trim()) - 1;
                    
                    // Match the specified curve to one in the combo box.
                    for (int i = 0; i < curveCombo.getItemCount(); i++) {
                        String cs = (String)curveCombo.getItemAt(i);
                        int start = 1;
                        int end = cs.indexOf(' ');
                        int graphNum2 = Integer.parseInt(
                            cs.substring(start, end)) - 1;
                        start = cs.indexOf('C') + 1;
                        end = cs.indexOf(':');
                        int curveNum2 = Integer.parseInt(
                            cs.substring(start, end)) - 1;
                        if (graphNum1 == graphNum2 && curveNum1 == curveNum2) {
                            curveCombo.setSelectedIndex(i);
                            break;
                        }
                    }
                }
                // Use a constant instead of a curve as input.
                else {
                    constantRadio.setSelected(true);
                    constantValueTextField.setText(value);
                }
            }
            if (key.equals("INTERP")) {
                if (value.equals("LINEAR")) interpLinearRadio.setSelected(true);
                if (value.equals("PARAB")) interpParabRadio.setSelected(true);
            }
            if (key.equals("OUTPUT")) {
                if (value.equals("ORIGINAL"))
                    outputToOriginalRadio.setSelected(true);
                if (value.equals("NEW"))
                    outputNewCurveRadio.setSelected(true);
            }
            if (key.equals("INTERVAL")) {
                if (value.equals("C1"))
                    intervalC1Radio.setSelected(true);
                if (value.equals("C2"))
                    intervalC2Radio.setSelected(true);
                if (value.equals("REGULAR"))
                    intervalRegularRadio.setSelected(true);
            }
            if (key.equals("REGINTERVAL")) {
                regularIntervalTextField.setText(value);
            }
            if (key.equals("REGOFFSET")) {
                regularOffsetTextField.setText(value);
            }
            if (key.equals("RESAMP")) {
                if (value.equals("INTERP")) resampInterpRadio.setSelected(true);
                if (value.equals("AVG")) resampAvgRadio.setSelected(true);
            }
        }
        writeToPage();
    }
    
//------------------------------------------------------------------------------
    /**
     * Write values from the dialog back to the page.
     * <p><b> Pre Conditions </b><br>
     *      The page has been set up.
     */
    protected void writeToPage() {
        
        if (m_selectedSeriesList.size() == 0) return;
        
        // Show an hourglass until the job is complete.
        ((Component)getParentFrame()).setCursor(new Cursor(Cursor.WAIT_CURSOR));
        
        // Get a reference to the second curve if one is selected.
        Graph graph2;
        Curve curve2;
        DoubleDataSeries series2 = null;
        
        synchronized (m_page) {
            if (curveRadio.isSelected()) {
                String curveString = (String)curveCombo.getSelectedItem();
                int start = 1;
                int end = curveString.indexOf(' ');
                int graphNum = Integer.parseInt(
                    curveString.substring(start, end)) - 1;
                start = curveString.indexOf('C') + 1;
                end = curveString.indexOf(':');
                int curveNum = Integer.parseInt(
                    curveString.substring(start, end)) - 1;
                
                List graphList = m_page.getGraphList();
                if (graphList.size() > graphNum) {
                    graph2 = (Graph)graphList.get(graphNum);
                    List curveList = graph2.getCurves();
                    if (curveList.size() > curveNum) {
                        curve2 = (Curve)curveList.get(curveNum);
                        series2 = curve2.getSeries();
                    }
                }
            }
            
            // Get the new value
            double value = TimeReference.convertToDecimalDay(
                constantValueTextField.getText());
            
            // Determine if changing the x or y coordinates.
            int dimension;
            if (xDimRadio.isSelected()) dimension = 0;
            else dimension = 1;
            
            List selectedSeriesList = null;
            if (outputNewCurveRadio.isSelected()) {
                // Save the current page so the paste can be undone.
                m_grCon.savePageSnapShot();
                
                // Create new copies of the selected curves and paste them into
                // the same graphs as the originals.
                selectedSeriesList =
                    m_grCon.pasteToSameGraph(m_selectedSeriesList);
            } else {
                // Use the original list of selected series instead of making
                // new copies.
                selectedSeriesList = m_selectedSeriesList;
            }
            
            DataSnapShot snapShot = new DataSnapShot();
            Iterator iter = selectedSeriesList.iterator();
            while (iter.hasNext()) {
                // Get the series and set its data manipulation options.
                DoubleDataSeries series = (DoubleDataSeries)iter.next();
                synchronized (series) {
                    series.m_bParabolicInterpolation =
                        interpParabRadio.isSelected();
                    series.m_bInterpolateSamples =
                        resampInterpRadio.isSelected();
                    
                    // Save the current state for later undo.
                    snapShot.store(series);
                    
                    if (!noneRadio.isSelected()) {
                        // Determine which type of modification is being made.
                        int modificationType = -1;
                        if (setRadio.isSelected()) modificationType = 0;
                        if (addRadio.isSelected()) modificationType = 1;
                        if (subRadio.isSelected()) modificationType = 2;
                        if (mulRadio.isSelected()) modificationType = 3;
                        if (divRadio.isSelected()) modificationType = 4;
                        if (powRadio.isSelected()) modificationType = 5;
                        if (logRadio.isSelected()) modificationType = 6;
                        if (absRadio.isSelected()) modificationType = 7;
                        
                        // If applying a moving average, read the parameter text
                        // fields and call the method.
                        if (movingAvgRadio.isSelected()) {
                            double span;
                            int samples;
                            span = TimeReference.convertToDecimalDay(
                                movingAvgSpanTextField.getText());
                            samples = (int)(Float.parseFloat(
                                movingAvgSamplesTextField.getText()));
                            MovingAverageCalculator.applyMovingAverage(
				series, span, samples);
                        } else {
                            // Apply the modification either using a constant or
                            // another curve.
                            if (constantRadio.isSelected()) {
                                series.modifySelectedPoints(modificationType,
                                    dimension, value);
                            } else {
                                synchronized (series2) {
                                    series.modifySelectedPoints(
                                        modificationType, dimension, series2);
                                }
                            }
                        }
                    }
                    
                    // Resample to match the second curve.
                    if (intervalC2Radio.isSelected()) {
                        synchronized (series2) {
                            series.resample(series2);
                        }
                    }
                    
                    // Resample to match a fixed interval.
                    if (intervalRegularRadio.isSelected()) {
                        double interval = TimeReference.convertToDecimalDay(
                            regularIntervalTextField.getText());
                        double offset = TimeReference.convertToDecimalDay(
                            regularOffsetTextField.getText());
                        series.resample(interval, offset);
                    }
                }
            }
            
            // Store the list of old series for undo.
            if (outputToOriginalRadio.isSelected()) {
                m_grCon.saveDataSnapShot(snapShot);
            } else {
                // Choose which series should be deselected.
                if (!selectBothRadio.isSelected()) {
                    // Deselect All on either the old or new series...
                    int index = 0;
                    while (index < m_selectedSeriesList.size()) {
                        DoubleDataSeries series =
                            (DoubleDataSeries)m_selectedSeriesList.get(index);
                        
                        // Establish whether the series is old or new.
                        boolean bNew = false;
                        Iterator newSelectedIter =
                            selectedSeriesList.iterator();
                        while (newSelectedIter.hasNext()) {
                            DoubleDataSeries newSeries =
                                (DoubleDataSeries)newSelectedIter.next();
                            if (series == newSeries) {
                                bNew = true;
                            }
                        }
                        
                        // Deselect the series if it isn't in the right
                        // group.
                        boolean bDeselect = false;
                        if (selectOldRadio.isSelected()) {
                            if (bNew) bDeselect = true;
                        }
                        if (selectNewRadio.isSelected()) {
                            if (!bNew) bDeselect = true;
                        }
                        
                        if (bDeselect) {
                            synchronized (series) {
                                series.deselectAll();
                            }
                            m_selectedSeriesList.remove(index);
                        } else {
                            index++;
                        }
                        
                    }
                }
            }
        }
    }
    
//------------------------------------------------------------------------------
    protected void someComponent_stateChanged(ChangeEvent e) {
        
        enableComponents();
    }
    
//------------------------------------------------------------------------------
    protected void enableComponents() {
        
        if (m_enabling) return;
        else m_enabling = true;
        try {
            
            // Enable the moving average options if it is selected.
            if (movingAvgRadio.isSelected()) {
                movingAvgSpanLabel.setEnabled(true);
                movingAvgSpanTextField.setEnabled(true);
                if (interpParabRadio.isSelected()) {
                    movingAvgSamplesLabel.setEnabled(true);
                    movingAvgSamplesTextField.setEnabled(true);
                } else {
                    movingAvgSamplesLabel.setEnabled(false);
                    movingAvgSamplesTextField.setEnabled(false);
                }
                
                // Disable the option having to do with set, shift, and scale.
                constantRadio.setEnabled(false);
                curveRadio.setEnabled(false);
                constantValueTextField.setEnabled(false);
                curveCombo.setEnabled(false);
                xDimRadio.setEnabled(false);
            } else {
                // Disable the moving average options.
                movingAvgSpanLabel.setEnabled(false);
                movingAvgSpanTextField.setEnabled(false);
                movingAvgSamplesLabel.setEnabled(false);
                movingAvgSamplesTextField.setEnabled(false);
                
                // Gray out input parameter options if the "None" operation is
                // selected.
                if (noneRadio.isSelected() || absRadio.isSelected() ||
                    logRadio.isSelected()) {
                    
                    if (yDimRadio.isSelected()) movingAvgRadio.setEnabled(true);
                    constantRadio.setEnabled(false);
                    constantValueTextField.setEnabled(false);
                    curveRadio.setEnabled(false);
                    curveCombo.setEnabled(false);
                    if (noneRadio.isSelected())
                        xDimRadio.setEnabled(false);
                } else {
                    // Dimension options should always be
                    // available unless the operation is "None".
                    
                    // Enable the appropriate constant or curve options.
                    constantRadio.setEnabled(true);
                    if (xDimRadio.isSelected() || powRadio.isSelected()) {
                        
                        curveRadio.setEnabled(false);
                        if (curveRadio.isSelected())
                            constantRadio.setSelected(true);
                        
                        // Disable the moving average options.
                        movingAvgRadio.setEnabled(false);
                        movingAvgSpanLabel.setEnabled(false);
                        movingAvgSpanTextField.setEnabled(false);
                        movingAvgSamplesLabel.setEnabled(false);
                        movingAvgSamplesTextField.setEnabled(false);
                    } else {
                        curveRadio.setEnabled(true);
                        movingAvgRadio.setEnabled(true);
                    }
                    
                    if (constantRadio.isSelected()) {
                        // Enable the constant text field.
                        constantValueTextField.setEnabled(true);
                        curveCombo.setEnabled(false);
                        xDimRadio.setEnabled(true);
                    } else {
                        constantValueTextField.setEnabled(false);
                        curveCombo.setEnabled(true);
                        xDimRadio.setEnabled(false);
                    }
                }
            }
            
            intervalC1Radio.setEnabled(true);
            
            // Don't enable the choices that reference a second curve if
            // constant input was selected.
            if (constantRadio.isSelected() || movingAvgRadio.isSelected()) {
                intervalC2Radio.setEnabled(false);
                
                if (intervalC2Radio.isSelected()) {
                    intervalC1Radio.setSelected(true);
                }
            } else {
                // Enable the components that reference a second curve.
                intervalC2Radio.setEnabled(true);
            }
            
            intervalRegularRadio.setEnabled(true);
            // Enable the regular interval options if it is selected.
            if (intervalRegularRadio.isSelected()) {
                regularIntervalLabel.setEnabled(true);
                regularIntervalTextField.setEnabled(true);
                regularOffsetLabel.setEnabled(true);
                regularOffsetTextField.setEnabled(true);
            } else {
                regularIntervalLabel.setEnabled(false);
                regularIntervalTextField.setEnabled(false);
                regularOffsetLabel.setEnabled(false);
                regularOffsetTextField.setEnabled(false);
            }
            
            // If new curve will be resampled, enable resampling method radios.
            if (intervalC1Radio.isSelected()) {
                resampInterpRadio.setEnabled(false);
                resampAvgRadio.setEnabled(false);
            } else {
                resampInterpRadio.setEnabled(true);
                resampAvgRadio.setEnabled(true);
            }
            
            // If output to new curve is selected, enable choices regarding which
            // curve will be selected after the operation.
            if (outputNewCurveRadio.isSelected()) {
                selectOldRadio.setEnabled(true);
                selectNewRadio.setEnabled(true);
                selectBothRadio.setEnabled(true);
            } else {
                selectOldRadio.setEnabled(false);
                selectNewRadio.setEnabled(false);
                selectBothRadio.setEnabled(false);
            }
            
        } catch (Exception ex) { m_enabling = false; } catch (Error er) { m_enabling = false; }
        
        m_enabling = false;
    }
}


