/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2004-12-08
 * Date started:    1998-10-20
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr.visual.j2d;

import gov.usgs.gr.*;
import gov.usgs.gr.visual.j2d.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

//==============================================================================
/**
 * A panel holding the x-y graphs. Extends GlPanel and thus inherits its OpenGL
 * drawing capabilities.
 */
public class J2dGrPanel implements MouseListener,
	MouseMotionListener, gov.usgs.gr.visual.gui.PageProxy {

    /** A page of graphs contained within this panel. */
    protected PageOfJ2dGraphs m_page;
    /** The parent visual to pass mouse events to. */
    GrController m_grCon = null;
    JPanel m_panel = null;

//------------------------------------------------------------------------------
/**
 * Create a GrPanel through the super class.
 */
    public J2dGrPanel () {

        m_panel = new JPanel();
        m_panel.addMouseListener(this);
        m_panel.addMouseMotionListener(this);
//        addKeyListener(this);
    }

//------------------------------------------------------------------------------
/**
 * Create a GrPanel connected to a certain controller.
 */
    public J2dGrPanel (GrController con) {

        this();

        m_grCon = con;
    }

//------------------------------------------------------------------------------
/**
 * Create a GrPanel as a copy of another page of graphs.  Share the data.
 */
    public J2dGrPanel (PageOfJ2dGraphs page, GrController con) {

        this();

        m_page = page;
        m_grCon = con;
    }

//------------------------------------------------------------------------------
/**
 * Clears the drawing area and redraws the contents.
 * @param g	The graphics context.
 */
    public void paint(Graphics g) {

    	// Get the size of the drawing area.
        Dimension d = m_panel.getSize();

    	if (m_page == null) {
            g.setColor(new Color(0f, 0.433f, 0.254f));
            g.fillRect(0, 0, d.width, d.height);

            // Return to the default arrow cursor.
            ((Component)GrController.getParentFrame(m_panel))
                .setCursor(Cursor.getDefaultCursor());

			return;
        }

        m_page.reshape(d.width, d.height);

        // Cast the graphics context to a Graphics2D object.
        Graphics2D j2d = (Graphics2D) g;

        // Tell the page to draw the graphs.
        m_page.setGraphics2D(j2d);
        m_page.draw();

        j2d.dispose();

        // Return to the default arrow cursor.
        ((Component)GrController.getParentFrame(m_panel))
        	.setCursor(Cursor.getDefaultCursor());
    }

//------------------------------------------------------------------------------
/**
 * Attribute modification method.
 * @param con	The new controller.
 */
    public void setController(GrController con) {
		m_grCon = con;
    }

//------------------------------------------------------------------------------
/**
 * Attribute modification method.
 * @param page	A new page to use for drawing.
 */
    public void setPage(gov.usgs.gr.visual.PageOfGraphs page) {

		if (page instanceof PageOfJ2dGraphs)
        	m_page = (PageOfJ2dGraphs)page;
        if (page == null)
        	m_page = null;
    }

//------------------------------------------------------------------------------
    public gov.usgs.gr.visual.PageOfGraphs setNewPage() {

    	m_page = new PageOfJ2dGraphs();

        return m_page;
    }

//------------------------------------------------------------------------------
/**
 * Pass the mouse event on to the GrVisual.
 * @param e     The mouse event object containing the mouse position.
 */
    public void mouseMoved(MouseEvent e) {

        if (m_grCon != null) m_grCon.mouseMoved(e);
    }

//------------------------------------------------------------------------------
/**
 * Pass the mouse event on to the GrVisual.
 * @param e     The mouse event object containing the mouse position.
 */
    public void mouseDragged(MouseEvent e) {

        if (m_grCon != null) m_grCon.mouseDragged(e);
    }

//------------------------------------------------------------------------------
/**
 * Pass the mouse event on to the GrVisual.
 * @param e     The mouse event object containing the mouse position.
 */
    public void mousePressed(MouseEvent e) {

        if (m_grCon != null) m_grCon.mousePressed(e);
    }

//------------------------------------------------------------------------------
/**
 * Pass the mouse event on to the GrVisual.
 * @param e     The mouse event.
 */
    public void mouseReleased(MouseEvent e) {

        if (m_grCon != null) m_grCon.mouseReleased(e);
    }

//------------------------------------------------------------------------------
/**
 * Pass the key event on to the GrVisual.
 * @param e     The key event.
 */
//    public void keyPressed(KeyEvent e) {
//
//        if (m_grCon != null) m_grCon.keyPressed(e);
//    }

//------------------------------------------------------------------------------
/**
 * Look for the frame which contains this component by searching its parent
 * components until a Frame is encountered.
 * <p><b> Pre Conditions </b><br>
 *      None.
 * <p><b> Post Conditions </b><br>
 *      None.
 * @return The frame that holds this component.
 */
/*
    public Frame getParentFrame() {

        Frame frame = null;
        Container container = this;

        // Ascend the hierarchy of containers until the parent frame is found.
        do {
            container = container.getParent();
             if (container instanceof Frame) frame = (Frame)container;
        } while (container != null && !(container instanceof Frame));

        return frame;
    }
*/

//------------------------------------------------------------------------------
// Mouse methods...
    public void mouseExited(MouseEvent e) {}
    public void mouseEntered(MouseEvent e) {}
    public void mouseClicked(MouseEvent e) {}

//------------------------------------------------------------------------------
// Key methods...
    public void keyReleased(KeyEvent e) {}
    public void keyTyped(KeyEvent e) {}

//------------------------------------------------------------------------------
    public void repaint() {

        m_panel.repaint();
    }

//------------------------------------------------------------------------------
    public Component getComponent() {

        return m_panel;
    }
}


