/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2002-01-22
 * Date started:    1998-10-20
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.gr.visual.j2d;

import java.awt.*;
import java.awt.geom.*;
import java.util.*;
import gov.usgs.gr.visual.*;

//==============================================================================
/**
 * A group of graphs layed out on a page.  Draws itself to an Java 2D drawing
 * area.
 */
public class PageOfJ2dGraphs extends gov.usgs.gr.visual.PageOfGraphs
	implements J2dElement {

	Graphics2D j2d;

//------------------------------------------------------------------------------
/**
 * Create a blank page for graphs.
 */
    public PageOfJ2dGraphs () {

        setDefaults("gov.usgs.gr.visual.j2d.PageOfJ2dGraphs");
    }

//------------------------------------------------------------------------------
/**
 * Create a page of graphs cloned from an existing one.   Data is shared.
 */
    public PageOfJ2dGraphs (PageOfGraphs page) {
    	super (page);
    }

//------------------------------------------------------------------------------
/**
 * Sets the Graphics2D object to use for drawing.
 * @param g	The graphics context.
 */
    public void setGraphics2D(Graphics2D g) {

		if (!m_bDrawing) {
        	j2d = g;
        }
    }

//------------------------------------------------------------------------------
/**
 * Clears the drawing area and redraws the contents.
 */
    public synchronized void draw() {

        drawJ2d();
/*
    	// Check if this method is being executed by this object's own thread.
        // If it isn't, split a new thread off to do the work.
    	if (!m_bDrawing) {
        	start();
            return;
        }
*/
    }

//------------------------------------------------------------------------------
/**
 * Clears the drawing area and redraws the contents.
 */
    public synchronized void drawJ2d() {

        drawFromThread();

        m_bNeedsRedraw = false;
    }

//------------------------------------------------------------------------------
/**
 * Clears the drawing area and redraws the contents.
 */
    public synchronized void drawFromThread() {

		if (m_bNeedsReformat) m_formatter.reformat();

        // Don't bother to draw anything if the drawing area is too small.
        if (m_display.height <= 0) return;

        // Draw the rubber band if a region is being selected.
/*
        if (m_zoomRect.width > 0) {
        	gl.enable(GL.GL_COLOR_LOGIC_OP);
            gl.logicOp(GL.GL_XOR);
            drawZoomWindow();
            m_oldZoomRect = new Rectangle(m_zoomRect);
            gl.disable(GL.GL_COLOR_LOGIC_OP);
        	return;
        }
        else {
        	m_oldZoomRect = null;
        }
*/

		// Calculate the ratio between screen hieght
        double displayScale = 1 / (double)m_showList.size();

        // Clear the drawing area with black.
        j2d.setBackground(new Color(0, 0, 0));
        j2d.clearRect(0, 0, m_display.width, m_display.height);

        // Set up the viewing transformation for this section of the page.
        j2d.translate(0, m_display.height);
        j2d.scale(1, -1);
        j2d.translate(m_content.x, m_content.y);

        // Let each graph in the panel draw itself using the J2d
        // command objects.
        Iterator iter = m_graphList.iterator();
        while(iter.hasNext()) {
            J2dGraph graph = (J2dGraph) iter.next();
            if (graph.getShow() && graph != m_movingGraph) {
                graph.setGraphics2D(j2d);
                graph.drawJ2d();
            }
        }

        // Draw the moving graph over top of all the others.
        if (m_movingGraph != null) {
        	((J2dGraph)m_movingGraph).setGraphics2D(j2d);
            ((J2dElement)m_movingGraph).drawJ2d();
        }

        if (m_zoomRect.width > 0) drawZoomWindow();
    }

//------------------------------------------------------------------------------
/**
 * Draw a window around the zoom area.
 */
    protected void drawZoomWindow() {

    	// Draw a dashed cyan box around the zoom area.
        j2d.setColor(Color.white);
        j2d.draw(m_zoomRect);
    }

//------------------------------------------------------------------------------
/**
 * Create an OpenGL graph to be used in this page.
 */
    protected Graph newGraph() {
    	return new J2dGraph();
    }

//------------------------------------------------------------------------------
/**
 * Create an OpenGL graph as a clone of an existing graph.
 * @param graph	A template graph to use in creating this one.
 */
    protected Graph newGraph(Graph graph) {
    	return new J2dGraph(graph);
    }
}


