/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2006-06-02
 * Date started:    1998-06-11
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.sfhydro.data;

import gov.usgs.gr.*;
import gov.usgs.gr.data.*;
import java.io.*;

//==============================================================================
/**
 *
 */
public class DemDataFile extends gov.usgs.gr.data.DelimitedDataFile  {

    protected static int NUM_PROFILES = 1201;
    protected int m_stripWidth = 1, m_arraySize;
    protected int[] m_min;
    protected int[] m_max;
    protected int m_numXpoints = 0;
    protected int m_numYpoints = 0;
    protected double m_gridWidth = 1;


//------------------------------------------------------------------------------
/**
 *
 */
    public DemDataFile () {

    	super();
        m_title = "DEM Data File";

        m_min = new int[3];
        m_max = new int[3];

        m_max[0] = NUM_PROFILES - 1;
        m_max[1] = NUM_PROFILES - 1;
    }

//------------------------------------------------------------------------------
/**
 * Overrides add in GrObject.  Adds everything as an effigy.
 * @param gro   The object to add.
 * @return  Whether the object was successfully added.
 */
    public boolean add(GrObject gro) {

        if (gro instanceof ObjectEffigy) {
        	if (((ObjectEffigy)gro).depictProperties == false) {
        		m_children.add(gro);
            }
        	else {
        		m_children.add(new ObjectEffigy(gro));
        	}
        }
        else {
        	m_children.add(new ObjectEffigy(gro));
        }

        return true;
    }

//------------------------------------------------------------------------------
/**
 *
 */
    protected void readHeader ()  throws IOException {

        String line, title, s;
        int i;
        char[] buff = new char[40];
	    double blEastLong, blNorthLat, tlEastLong, tlNorthLat;
	    double trEastLong, trNorthLat, brEastLong, brNorthLat;
	    double BRX;

        try {
            m_br.skip(546);

            m_br.read(buff, 0, 24);
            line = new String(buff);
            s = line.trim().replace('D', 'E');
            blEastLong = Double.parseDouble(s);

            m_br.read(buff, 0, 24);
            line = new String(buff);
            s = line.trim().replace('D', 'E');
            blNorthLat = Double.parseDouble(s);

            m_br.read(buff, 0, 24);
            line = new String(buff);
            s = line.trim().replace('D', 'E');
            tlEastLong = Double.parseDouble(s);

            m_br.read(buff, 0, 24);
            line = new String(buff);
            s = line.trim().replace('D', 'E');
            tlNorthLat = Double.parseDouble(s);

            m_br.read(buff, 0, 24);
            line = new String(buff);
            s = line.trim().replace('D', 'E');
            trEastLong = Double.parseDouble(s);

            m_br.read(buff, 0, 24);
            line = new String(buff);
            s = line.trim().replace('D', 'E');
            trNorthLat = Double.parseDouble(s);

            m_br.read(buff, 0, 24);
            line = new String(buff);
            s = line.trim().replace('D', 'E');
            brEastLong = Double.parseDouble(s);

            m_br.read(buff, 0, 24);
            line = new String(buff);
            s = line.trim().replace('D', 'E');
            brNorthLat = Double.parseDouble(s);

            m_br.skip(130);
        }
        catch (IOException e) { System.out.println(e); }
    }

//------------------------------------------------------------------------------
/**
 *
 */
    protected void readData () throws IOException {

        String line, s;

        char[] buff = new char[20];
        double val;
        int i, j, buffIndex;
        boolean done = false;

        if (m_min[0] < 0) m_min[0] = 0;
        if (m_min[1] < 0) m_min[1] = 0;
        if (m_max[0] >= NUM_PROFILES) m_max[0] = NUM_PROFILES;
        if (m_max[1] >= NUM_PROFILES) m_max[1] = NUM_PROFILES;

        m_numXpoints = m_max[0] - m_min[0] + 1;
        m_numYpoints = m_max[1] - m_min[1] + 1;
        m_arraySize = NUM_PROFILES;
        double[] array = new double[m_numXpoints * m_numYpoints];


        for (j = m_min[1]; j < m_max[1] ; j++) {
            // Skip nine tokens
            for (int k = 0; k < 9; k++) nextToken();

            for (i = m_min[0]; i < m_max[0]; i++) {
                s = nextToken();
                val = Double.parseDouble(s);
                array[j * m_numXpoints + i] = val;
            }
            for (i = m_max[0]+1; i <= NUM_PROFILES; i++) {
                s = nextToken();
            }
        }

        DoubleDataSeries surfSeries = new DoubleDataSeries(array, 1);
        gov.usgs.gr.visual.gl.Gl3dRectangles glRects =
            new gov.usgs.gr.visual.gl.Gl3dRectangles();
        glRects.setProperty("numRows", "" + m_numXpoints);
        glRects.setProperty("numColumns", "" + m_numYpoints);
        glRects.add(surfSeries);

        glRects.setProperty("zScale", "0");
        glRects.setProperty("gridWidth", "30");
        float colors[][] = {{0,92,58}, {0,204,153}, {0,100,100},
            {0,186,210}, {95,37,181}, {154,139,232}, {135,88,113},
            {205,128,163}, {197,92,61}, {241,205,120}};
        gov.usgs.gr.visual.InterpolatingColorScheme ics =
                new gov.usgs.gr.visual.InterpolatingColorScheme();
        ics.regenerateColorScheme(0.0001, surfSeries.getMax(0), 0.4, colors);
        ics.setColorAtValue("(0,0,140)0");
        glRects.add(ics);

//        gov.usgs.gr.visual.gl.GlTexture glTex =
//            new gov.usgs.gr.visual.gl.GlTexture();
//        glTex.setProperty("file", "I:\\dem\\bayDeltaCrop.jpg");
//        glRects.add(glTex);

        gov.usgs.gr.visual.gl.GlGraph graph =
            new gov.usgs.gr.visual.gl.GlGraph();
        graph.add(glRects);
        gov.usgs.gr.visual.gl.PageOfGlGraphs page =
            new gov.usgs.gr.visual.gl.PageOfGlGraphs();
        page.add(graph);
        page.updateZoomContents();
        page.showMoreGraphs();
        page.setProperty("formatter",
            "gov.usgs.sfhydro.gr.formats.VariableSizeFormatter");
        gov.usgs.gr.visual.Axis xAxis = graph.getXAxis();
        gov.usgs.gr.visual.Axis yAxis = graph.getYAxis();
        xAxis.setProperty("showMajorTicks", "false");
        yAxis.setProperty("showMajorTicks", "false");
        xAxis.setProperty("showMinorTicks", "false");
        yAxis.setProperty("showMinorTicks", "false");
        xAxis.setProperty("showLabels", "false");
        yAxis.setProperty("showLabels", "false");
        xAxis.setProperty("showGridLines", "false");
        yAxis.setProperty("showGridLines", "false");
        xAxis.getZoom().setOuterLimits(0, m_numXpoints * 30);
        yAxis.getZoom().setOuterLimits(0, m_numYpoints * 30);
        xAxis.getZoom().zoomAll();
        yAxis.getZoom().zoomAll();
        xAxis.getZoom().setContentLimits(0, m_numXpoints * 30);
        yAxis.getZoom().setContentLimits(0, m_numYpoints * 30);
        add(page);

        System.out.println("Done loading DEM");
    }

//------------------------------------------------------------------------------
/**
 *
 */
    private String nextToken () {

        String token = " ", s;
        char[] buff = new char[30];
        double val;
        int i, j, buffIndex;
        boolean done = false;

        try {
            buffIndex = 0;
            do {
                buff[buffIndex] = ' ';
                m_br.read(buff, buffIndex, 1);

                if (buff[buffIndex] == ' ' && buffIndex != 0) {
                    done = true;
                }
                else {
                    if (buff[buffIndex] != ' ') buffIndex++;
                }
            }
            while (!done);
            token = (new String(buff)).trim();
        }
        catch (IOException e) { System.out.println(e); }

        return token;
    }

}



