/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2007-05-02
 * Date started:    2001-07-16
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.sfhydro.data;

import gov.usgs.gr.data.*;
import java.io.*;
import java.util.*;

//==============================================================================
/**
 * An array of data in one or more dimensions.
 * @see gov.usgs.data.XyData.
 */
public class LineDataFile extends gov.usgs.gr.data.DelimitedDataFile {

    double[] m_offsets;

//------------------------------------------------------------------------------
/**
 * Creates a data file.
 */
    public LineDataFile () {

        m_title = "Line Data File";

        m_numDimensions = 2;

        double[] offsets = {600000, 4200000};
        m_offsets = offsets;
    }

//------------------------------------------------------------------------------
/**
 * Read the data through a reader attached to an open file.  Called by read()
 * <p><b> Pre Conditions </b><br>
 *      The header has been read with readHeader()
 * <p><b> Post Conditions </b><br>
 *      The file needs to be closed.
 */
    protected void readData () throws IOException {

    	clear();

        // Make a list of column lists, one for each series.
        List<List<Double>> columnList = new ArrayList<List<Double>>();
        for (int i = 0; i < m_numChannels; i++) {
            columnList.add(new ArrayList());
        }

    	String[] tokens;

        // Read each line of the data file.
        while ((tokens = AsciiIO.parseLine(m_br.readLine(),
        	m_inputDelimiters)) != null) {

            if (tokens.length < 2) {
                if (tokens.length > 0) {
                    if (tokens[0].equals("END")) {
                        breakSeries(columnList);
                    }
                }
                continue;
            }

            // Add each number on the line to the appropriate column list.
            for (int i = 0; i < tokens.length; i++) {
            	List<Double> column = columnList.get(0);
//                int dIndex = tokens[i].indexOf('D');
                tokens[i] = tokens[i].replace('D', 'E');
                double d = Double.parseDouble(tokens[i]);
                d -= m_offsets[i];
                column.add(d);
            }
        }
    }

//------------------------------------------------------------------------------
/**
 * Break off what has been read so far and add it as a child DataSeries.
 * @param columnList    The data that has been read in so far since the
 *                      last break.
 */
    protected void breakSeries (List<List<Double>> columnList) {

        // Transfer the Floats to an array of floats and create data series.
        for (List column : columnList) {
            Object[] realObjects = column.toArray();
            if (realObjects.length < m_numDimensions) break;

            double[] vals = new double[realObjects.length];
            for (int i = 0; i < realObjects.length; i++) {
            	vals[i] = (Double)realObjects[i];
            }

            // Create a new data series from the array.
        	DoubleDataSeries series = null;
        	if (m_numDimensions == 2)
		    series = new DoubleDataSeries(vals, 2);
        	else series = new DoubleDataSeries(vals, m_numDimensions);
            add(series);
        }

        // Reset the column list.
        columnList.clear();
        for (int i = 0; i < m_numChannels; i++) {
            columnList.add(new ArrayList());
        }
    }

}
