/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2006-04-06
 * Date started:    1999-05-18
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.sfhydro.gr.formats;

import gov.usgs.gr.*;
import gov.usgs.gr.data.*;
import gov.usgs.sfhydro.data.*;
import gov.usgs.gr.visual.*;
import gov.usgs.gr.visual.ps.*;
import java.awt.Rectangle;
import java.awt.geom.*;
import java.util.*;

//==============================================================================
/**
 * Applies formatting to a PageOfGraphs.
 */
public class ReportFormatter extends gov.usgs.gr.visual.PageFormatter {

    /** The tree icon to use for this class. */
	public static ArrayList TREE_ICON_HOLDER = new ArrayList(1);
    /** The location of the tree icon. */
    protected String m_iconFile = "GrObject/ReportFormatter.gif";

//------------------------------------------------------------------------------
/**
 * Create an empty formatter.
 */
    public ReportFormatter () {

        setDefaults("gov.usgs.sfhydro.gr.formats.ReportFormatter");
    }

//------------------------------------------------------------------------------
/**
 * Create a page formatter and give it a page to format.
 */
    public ReportFormatter (PageOfGraphs page) {

    	m_page = page;
    }

//------------------------------------------------------------------------------
/**
 * Create a string representation of this object.
 * @return  The string that describes this object.
 */
    public String toString() {

    	String s = "Report Formatter";

        return s;
 	}

//------------------------------------------------------------------------------
/**
 * Accessor method.
 * @return An ImageIcon to use when representing this object in a tree.
 */
    public javax.swing.ImageIcon getTreeIcon() {

        if (TREE_ICON_HOLDER.size() <= 0)
            setTreeIcon();

    	return (javax.swing.ImageIcon)TREE_ICON_HOLDER.get(0);
    }

//------------------------------------------------------------------------------
/**
 * Loads the tree icon from a file and stores it in the TREE_ICON_HOLDER.
 */
    protected javax.swing.ImageIcon setTreeIcon() {

        javax.swing.ImageIcon icon = loadTreeIcon(m_iconFile);

        if (icon == null)
            icon = super.setTreeIcon();

        TREE_ICON_HOLDER.clear();
        TREE_ICON_HOLDER.add(icon);

        return icon;
    }

//------------------------------------------------------------------------------
/**
 * Makes a copy of this formatter.
 * @return	A copy of this formatter.
 */
    public PageFormatter getCopy() {

    	return new ReportFormatter();
    }

//------------------------------------------------------------------------------
/**
 * Applies the format to the page as well as making some one time changes.
 */
    public void initialize() {

    	synchronized(m_page) {
            char labelChar = (char)((int)'A' + m_page.getShowList().size() - 1);
            List graphList = m_page.getGraphList();
            Iterator iter = graphList.iterator();
            int i = 0;
            while (iter.hasNext()) {
                Graph graph = (Graph)iter.next();

                graph.getXAxis().setShowGridLines(false);
                graph.getYAxis().setShowGridLines(false);
                graph.getXAxis().setShowMinorTicks(false);
                graph.getYAxis().setShowMinorTicks(false);
                graph.setShowMainTitle(false);
                String yTitle = graph.getYAxis().getTitle();
                if (yTitle.trim().length() == 0)
                    graph.getYAxis().setTitle(graph.getTitle().toUpperCase());
                graph.getXAxis().setShowTitle(true);

                // Reference the start year in the x title.
                String dateString = "DAYS FROM JANUARY 1";
                if (i == 0) {
                    List curveList = graph.getCurves();
                    if (curveList.size() > 0) {
                        Curve curve = (Curve)curveList.get(0);
                        DataSeries series = curve.getSeries();
                        if (series instanceof TimeSeries) {
                            TimeSeries ts = (TimeSeries)series;
                            dateString += ", ";
                            dateString += ts.getReferenceYear();
                        }
                    }
                }

                String xTitle = graph.getXAxis().getTitle();
                if (xTitle.trim().length() == 0)
                    graph.getXAxis().setTitle(dateString);

                i++;
            }

            m_page.deleteAllMarkups();
            m_page.setTitle("");

            if (m_page.m_psOptions != null) {
                m_page.m_psOptions.setPsLandscape(false);
                m_page.m_psOptions.setPsFont("Helvetica-Bold");
                m_page.m_psOptions.setPsFontSize(8);
                m_page.m_psOptions.setPsColor(false);
                m_page.m_psOptions.setPsLineWidth(0.5f);
                m_page.m_psOptions.setPsUniformWidth(true);
            }

            reformat();
            m_page.notifyObservers();
    	}
    }

//------------------------------------------------------------------------------
/**
 * Applies the format to the page.
 */
    public void reformat() {

        synchronized(m_page) {
            char labelChar = (char)((int)'A' + m_page.getShowList().size() - 1);
            List showList = m_page.getShowList();
            Iterator graphIter = showList.iterator();
            while (graphIter.hasNext()) {
                Graph graph = (Graph)graphIter.next();

                graph.getYAxis().setShowTitle(true);
                if (m_page instanceof gov.usgs.gr.visual.ps.PageOfPsGraphs) {

                    // Set the resolution to 360 dpi for each curve.
                    List curveList = graph.getCurves();
                    Iterator curveIter = curveList.iterator();
                    while (curveIter.hasNext()) {
                        PsCurve curve = (PsCurve)curveIter.next();
                        curve.setResolution(0.2f);
                    }
                }

                // Add a different letter to each graph if there is more than
                // one graph showing.
                graph.deleteAllMarkups();
                if (m_page.getShowList().size() > 1) {
                    TextMarkup markup = new TextMarkup();
                    markup.setText("" + labelChar);
                    markup.setPsFont("Helvetica-BoldOblique");
                    markup.setPsFontSize(8);
                    markup.x = 0;
                    markup.y = 1;
                    markup.xPointOffset = 2;
                    markup.yPointOffset = -8;
                    markup.xScreenOffset = 4;
                    markup.yScreenOffset = -12;
                    markup.setCoordinateLevels(2, 2);
                    markup.setUserChangeable(false);
                    graph.add(markup);
                    labelChar--;
                }
            }

            layoutGraphs();

            m_page.m_bNeedsReformat = false;
        }
    }

//------------------------------------------------------------------------------
/**
 * Layout all showing graphs and reproportion each graph within the panel.
 */
    protected void layoutPsGraphs() {

        synchronized(m_page) {
            int xPos = 0, yPos = 0;
            List showList = m_page.getShowList();

            Rectangle contentRect = m_page.getContentRect();
            contentRect.x = 60;
            contentRect.y = 72;
            contentRect.width = 0;
            contentRect.height = 0;
            float fixedWidth = 430;
            float fixedHeight = 115;

            // Reproportion each graph within the panel.
            Iterator iter = showList.iterator();
            int i = 0;
            while (iter.hasNext()) {
                GrObject gro = (GrObject)iter.next();
                if (!(gro instanceof PsGraph))
                    continue;

                PsGraph graph = (PsGraph)gro;

                // Make sure only the bottom showing graph has x-axis labels.
                if (i == 0) graph.setShowXtext(true);
                else graph.setShowXtext(false);

                // Calculate how much to scale the graph so its frame will be
                // the same size as all the others.
                Rectangle2D.Double graphFrame = graph.getFrameRect();

                // Set the new size and position for the graph.
                int newWidth = 0, newHeight = 0;
                newWidth = 495;
                int yOffset = 8;
                if (graph.getXAxis().getShowText()) {
                    yOffset += 10;
                    if (graph.getXAxis().getShowTitle()) yOffset += 25;
                }

                newHeight = (int)(fixedHeight + yOffset + 8);

                graph.setSize(newWidth, newHeight);
                graph.setXPosition(xPos);
                graph.setYPosition(yPos);

    // Calculate frame offset based on font size and whether labels and titles
    // are being drawn.
    // If content exceeds maximum height, scale the frame down until it fits.
                graphFrame.x = 50;
                graphFrame.y = yOffset - 10;
                graphFrame.width = fixedWidth;
                graphFrame.height = fixedHeight;
                graph.setMediaFrameRect(graphFrame);

                yPos += newHeight;
                contentRect.height += newHeight;
                contentRect.width = newWidth;

                // Set the tick scale so both axes have the same size ticks.
                float xTickScale = 1;
                if (newHeight != 0) xTickScale = m_page.getHeight()
                    / (float)newHeight;
                graph.setXTickScale(xTickScale);
                float yTickScale = 1;
                if (newWidth != 0) yTickScale = m_page.getHeight()
                    / (float)newWidth;
                graph.setYTickScale(yTickScale);

                i++;
            }

            // Make sure the graph fits within the maximum boundary area.
            shrinkToFit();
        }
    }

//------------------------------------------------------------------------------
/**
 * Shrink the content area so it is no bigger than the maximum allowable size.
 * Reduce the size of the frames to achieve this.  If the content area already
 * fits, don't make any changes.
 */
    public void shrinkToFit() {

        synchronized(m_page) {
            int maxWidth = 495;
            int maxHeight = 650;
            float yPos = 0;
            List showList = m_page.getShowList();
            Rectangle contentRect = m_page.getContentRect();

            // Don't make any changes if everything already fits.
            if (contentRect.height < maxHeight) return;

            float avgDisplayHeight = (float)contentRect.height
                / (float)showList.size();
            float targetDisplayHeight = (float)maxHeight / (float)showList.size();
            float yTrim = avgDisplayHeight - targetDisplayHeight;

            // Change the height and y position of each graph.
            int i = 1;
            Iterator iter = showList.iterator();
            while (iter.hasNext()) {
                Graph graph = (Graph)iter.next();

                Rectangle2D.Double graphFrame = graph.getFrameRect();
                // Reduce the frame's height.
                graph.setHeight((int)(graph.getHeight() - yTrim));
                graph.setFrameRect(graphFrame);
                graph.setYPosition((int)yPos);

                yPos += graph.getHeight();
                i++;
            }

            // Update the content area size.
            contentRect.width = maxWidth;
            contentRect.height = maxHeight;
        }
    }
}
