/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2006-06-06
 * Date started:    2004-12-01
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.sfhydro.gr.formats;

import java.awt.Rectangle;
import java.awt.geom.*;
import java.text.*;
import java.util.*;
import gov.usgs.gr.*;
import gov.usgs.gr.data.*;
import gov.usgs.sfhydro.data.*;
import gov.usgs.gr.visual.*;
import gov.usgs.gr.visual.ps.*;

//==============================================================================
/**
 * Applies formatting to a PageOfGraphs.
 */
public class VariableShapeFormatter extends gov.usgs.gr.visual.PageFormatter {

    /** A list of sizes for graphs. */
    List m_shapeValues = new ArrayList();

    /** The minimum number of values passed out in getProperties. */
    protected int m_numVals = 1;
    /** The tree icon to use for this class. */
	public static ArrayList TREE_ICON_HOLDER = new ArrayList(1);
    /** The location of the tree icon. */
    protected String m_iconFile = "GrObject/VariableShapeFormatter.gif";

//------------------------------------------------------------------------------
/**
 * Create an empty formatter.
 */
    public VariableShapeFormatter () {

        setDefaults("gov.usgs.sfhydro.gr.formats.VariableShapeFormatter");

        m_shapeValues.add(new Rectangle2D.Float(0, 0, 100, 100));
    }

//------------------------------------------------------------------------------
/**
 * Create a page formatter and give it a page to format.
 */
    public VariableShapeFormatter (PageOfGraphs page) {

        setDefaults("gov.usgs.sfhydro.gr.formats.VariableSizeFormatter");

    	m_page = page;
        m_shapeValues.add(new Rectangle2D.Float(0, 0, 100, 100));
    }

//------------------------------------------------------------------------------
/**
 * Accessor method.
 * @return An ImageIcon to use when representing this object in a tree.
 */
    public javax.swing.ImageIcon getTreeIcon() {

        if (TREE_ICON_HOLDER.size() <= 0)
            setTreeIcon();

    	return (javax.swing.ImageIcon)TREE_ICON_HOLDER.get(0);
    }

//------------------------------------------------------------------------------
/**
 * Loads the tree icon from a file and stores it in the TREE_ICON_HOLDER.
 */
    protected javax.swing.ImageIcon setTreeIcon() {

        javax.swing.ImageIcon icon = loadTreeIcon(m_iconFile);

        if (icon == null)
            icon = super.setTreeIcon();

        TREE_ICON_HOLDER.clear();
        TREE_ICON_HOLDER.add(icon);

        return icon;
    }

//------------------------------------------------------------------------------
/**
 * Create a string representation of this object.
 * @return  The string that describes this object.
 */
    public String toString() {

    	String s = "Variable Shape Formatter";

        return s;
 	}

//------------------------------------------------------------------------------
/**
 * Makes a copy of this formatter.
 * @return	A copy of this formatter.
 */
    public PageFormatter getCopy() {

    	return new VariableSizeFormatter();
    }

//------------------------------------------------------------------------------
/**
 * Provide the list of properties with the latest values.
 */
    public GrHashList getProperties() {

    	GrHashList props = super.getProperties();

        props.put("numVals", new GrProperty(m_numVals, "Number of Values"));

        int valNum = 0;

        Iterator iter = m_shapeValues.iterator();
        while (iter.hasNext()) {
        	Rectangle2D.Float rect = (Rectangle2D.Float)iter.next();
            String sVal = "" + rect.x + ", " + rect.y + ", " + rect.width
                + ", " + rect.height;

        	GrProperty p = new GrProperty(sVal, "X,Y,W,H ( % ) " + valNum);

			props.put("shape" + valNum, p);

            valNum++;
        }

        while (valNum < m_numVals) {
            // Add additional entries if needed...
        	GrProperty p = new GrProperty("0, 0, 0, 0",
                "X,Y,W,H ( % ) " + valNum);

            p.save = false;
			props.put("shape" + valNum, p);

            valNum++;
        }


        return props;
    }

//------------------------------------------------------------------------------
/**
 * Provide the list of properties with the latest values.
 */
    public void setProperties(HashMap props) {

		super.setProperties(props);

    	GrProperty p;

        Iterator iter = props.keySet().iterator();
        while (iter.hasNext()) {
        	String key = (String)iter.next();
            Object obj = props.get(key);
            if (obj instanceof GrProperty) {
				p = (GrProperty)obj;

            	if (key.equals("numVals")) {
            		m_numVals = (Integer)p.value;
                    if (m_numVals < 1 || m_numVals > 50)
                    	m_numVals = 1;
                    while (m_shapeValues.size() > m_numVals)
                        m_shapeValues.remove(m_shapeValues.size() - 1);
                }
            	if (key.startsWith("shape")) {
                    setSizeAtValue(Integer.parseInt(key.substring(5)),
                        (String)p.value);
                }
			}
            if (obj instanceof String) {
            	String value = (String)obj;
            	setProperty(key, value);
            }
        }

        m_bHasChanged = true;
        notifyObservers();
    }

//------------------------------------------------------------------------------
/**
 * Set a single property.
 */
    public void setProperty(String key, String value) {

    	super.setProperty(key, value);

        if (key.equals("numVals")) {
        	m_numVals = (int)(Float.parseFloat(value));
            if (m_numVals < 1 || m_numVals > 50)
            	m_numVals = 1;
            while (m_shapeValues.size() > m_numVals)
                m_shapeValues.remove(m_shapeValues.size() - 1);
        }
        if (key.startsWith("shape")) {
            setSizeAtValue(Integer.parseInt(key.substring(5)), value);
        }
    }

//------------------------------------------------------------------------------
/**
 *
 */
    public void setSizeAtValue(int index, String sVal) {

        boolean bFound = false;

        float[] vals = AsciiIO.parseLineAsFloats(sVal, ", ");
        Rectangle2D.Float rect = new Rectangle2D.Float(vals[0], vals[1],
            vals[2], vals[3]);

        for (int i = 0; i < m_shapeValues.size(); i++) {
            if (i == index) {
            	m_shapeValues.remove(i);
            	m_shapeValues.add(i, rect);
                bFound = true;
                break;
            }
        }

        if (!bFound) {
            // Insert blank values
            while (m_shapeValues.size() < index)
                m_shapeValues.add(new Rectangle2D.Float(0, 0, 0, 0));

        	m_shapeValues.add(index, rect);
        }
    }

//------------------------------------------------------------------------------
/**
 * Applies the format to the page as well as making some one time changes.
 */
    public void initialize() {

    	reformat();
    }

//------------------------------------------------------------------------------
/**
 * Applies the format to the page.
 */
    public void reformat() {

        layoutGraphs();

        m_page.m_bNeedsReformat = false;
    }

//------------------------------------------------------------------------------
/**
 * Layout all showing graphs and reproportion each graph within the panel.
 * Triggers a redraw.
 */
    protected void layoutScreenGraphs() {

        synchronized(m_page) {
            List showList = m_page.getShowList();
            float contentWidth = (float)m_page.getContentRect().width;
            float contentHeight = (float)m_page.getContentRect().height;

            // Find the sum of the frame sizes and use that to find the average.
            Iterator iter = showList.iterator();
            while (iter.hasNext()) {
                Graph graph = (Graph)iter.next();
                graph.layoutFrame();
                Rectangle2D.Double graphFrame = graph.getFrameRect();
            }

            // Reproportion each graph within the panel.
            iter = showList.iterator();
            Iterator shapeIter = m_shapeValues.iterator();
            int i = 0;
            while (iter.hasNext()) {
                Graph graph = (Graph)iter.next();

                Rectangle2D.Float rect = new Rectangle2D.Float(0, 0, 0, 0);
                if (shapeIter.hasNext()) {
                    rect = (Rectangle2D.Float)shapeIter.next();
                }

                // Calculate how much to scale the graph so its frame will be
                // the same size as all the others.
                Rectangle2D.Double graphFrame = graph.getFrameRect();
                Rectangle graphLocal = graph.getLocalRect();

                // Set the new size and position for the graph.
                int newWidth = (int)((rect.width / 100) * contentWidth);
                int newHeight = (int)((rect.height / 100) * contentHeight);
                int newX = (int)((rect.x / 100) * contentWidth);
                int newY = (int)((rect.y / 100) * contentHeight);

                // Only change the size and position if the graph is not
                // being dragged to a new location.
                if (graph != m_page.getMovingGraph()) {
                    graph.setSize(newWidth, newHeight);
                    graph.setXPosition(newX);
                    graph.setYPosition(newY);
                }

                // Set the tick scale so both axes have the same size ticks.
                float xTickScale = 1;
                if (newHeight != 0) xTickScale = m_page.getHeight()
                    / (float)newHeight;
                graph.setXTickScale(xTickScale);
                float yTickScale = 1;
                if (newWidth != 0) yTickScale = m_page.getHeight()
                    / (float)newWidth;
                graph.setYTickScale(yTickScale);

                // Enlarge the frame area if there are no axis labels.
                if (!(graph.getYAxis().getShowTitle()) &&
                    !(graph.getYAxis().getShowText())) {

                    if (showList.size() == 1) {
                        graphFrame.x = 0;
                        graphFrame.width = 1;
                    }
                    else {
                        graphFrame.x = 0.01;
                        graphFrame.width = 1.0 - graphFrame.x - 0.005;
                    }
                }

                if (!(graph.getShowMainTitle())) {
                    if (showList.size() == 1) {
                        graphFrame.y = 0;
                        graphFrame.height = 1;
                    }
                    else {
                        graphFrame.height = 1.0 - graphFrame.y - 0.005;
                    }
                }

                graph.setFrameRect(graphFrame);

                i++;
            }
        }
    }

//------------------------------------------------------------------------------
/**
 * Layout all showing graphs and reproportion each graph within the panel.
 * Triggers a redraw.
 */
    protected void layoutPsGraphs() {

        layoutScreenGraphs();
    }
}
