/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2006-06-06
 * Date started:    1999-11-21
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.sfhydro.gr.formats;

import java.awt.Rectangle;
import java.awt.geom.*;
import java.text.*;
import java.util.*;
import gov.usgs.gr.data.*;
import gov.usgs.sfhydro.data.*;
import gov.usgs.gr.visual.*;
import gov.usgs.gr.visual.ps.*;

//==============================================================================
/**
 * Applies formatting to a PageOfGraphs.
 */
public class WorkingFormatter extends gov.usgs.gr.visual.PageFormatter {

    /** The tree icon to use for this class. */
	public static ArrayList TREE_ICON_HOLDER = new ArrayList(1);
    /** The location of the tree icon. */
    protected String m_iconFile = "GrObject/WorkingFormatter.gif";

//------------------------------------------------------------------------------
/**
 * Create an empty formatter.
 */
    public WorkingFormatter () {

        setDefaults("gov.usgs.sfhydro.gr.formats.WorkingFormatter");
    }

//------------------------------------------------------------------------------
/**
 * Create a page formatter and give it a page to format.
 */
    public WorkingFormatter (PageOfGraphs page) {

        setDefaults("gov.usgs.sfhydro.gr.formats.WorkingFormatter");

    	m_page = page;
    }

//------------------------------------------------------------------------------
/**
 * Create a string representation of this object.
 * @return  The string that describes this object.
 */
    public String toString() {

    	String s = "Working Formatter";

        return s;
 	}

//------------------------------------------------------------------------------
/**
 * Accessor method.
 * @return An ImageIcon to use when representing this object in a tree.
 */
    public javax.swing.ImageIcon getTreeIcon() {

        if (TREE_ICON_HOLDER.size() <= 0)
            setTreeIcon();

    	return (javax.swing.ImageIcon)TREE_ICON_HOLDER.get(0);
    }

//------------------------------------------------------------------------------
/**
 * Loads the tree icon from a file and stores it in the TREE_ICON_HOLDER.
 */
    protected javax.swing.ImageIcon setTreeIcon() {

        javax.swing.ImageIcon icon = loadTreeIcon(m_iconFile);

        if (icon == null)
            icon = super.setTreeIcon();

        TREE_ICON_HOLDER.clear();
        TREE_ICON_HOLDER.add(icon);

        return icon;
    }

//------------------------------------------------------------------------------
/**
 * Makes a copy of this formatter.
 * @return	A copy of this formatter.
 */
    public PageFormatter getCopy() {

    	return new WorkingFormatter();
    }

//------------------------------------------------------------------------------
/**
 * Applies the format to the page as well as making some one time changes.
 */
    public void initialize() {

        synchronized(m_page) {
            List graphList = m_page.getGraphList();
            Iterator iter = graphList.iterator();
            while (iter.hasNext()) {
                Graph graph = (Graph)iter.next();

                graph.deleteAllMarkups();
                graph.getXAxis().setShowGridLines(true);
                graph.getYAxis().setShowGridLines(true);
                graph.getXAxis().setShowMinorTicks(true);
                graph.getYAxis().setShowMinorTicks(true);
                graph.setShowMainTitle(true);
            }

            m_page.deleteAllMarkups();

            if (m_page.m_psOptions != null) {
                m_page.m_psOptions.setPsLandscape(true);
                m_page.m_psOptions.setPsFont("Helvetica-Bold");
                m_page.m_psOptions.setPsFontSize(8);
                m_page.m_psOptions.setPsColor(false);
                m_page.m_psOptions.setPsLineWidth(0.5f);
                m_page.m_psOptions.setPsUniformWidth(false);
                m_page.m_psOptions.setPsSymbolSize(2);
            }

            reformat();
        }
    }

//------------------------------------------------------------------------------
/**
 * Applies the format to the page.
 */
    public void reformat() {

        synchronized(m_page) {
            if (m_page instanceof gov.usgs.gr.visual.ps.PageOfPsGraphs) {
                List showList = m_page.getShowList();
                Iterator iter = showList.iterator();
                while (iter.hasNext()) {
                    Graph graph = (Graph)iter.next();
                    graph.setDrawDeletedOutside(false);
                }
            }

            // Clear the markups from the page, so updated ones can be added.
            m_page.deleteAllMarkups();

            // Look through the page's command list for Open or Overlay
            // commands.  Display the files that were opened in the top right
            // corner of the page.
            List commandList = m_page.getCommandList();
            int numCommands = 0;

            Iterator iter = commandList.iterator();
            while (iter.hasNext()) {
                String s = (String)iter.next();

                // Search for Open and Overlay commands.
                String s2 = "";
                int index = s.toUpperCase().indexOf("OPEN");
                if (index >= 0) {
                    // Get rid of any previous files.
                    m_page.deleteAllMarkups();
                    numCommands = 0;
                    // Skip past the command itself.
                    index += 5;
                }
                else {
                    index = s.toUpperCase().indexOf("OVERLAY");
                    // Skip past the command itself.
                    if (index >= 0) index += 8;
                    // Skip to the next command if this wasn't Open or
                    // Overlay.
                    else continue;
                }

                // Copy the path and filename to a new string, converting
                // any backslashes to double backslashes so they will print
                // in PS.
                for (int i = index; i < s.length(); i++) {
                    s2 += s.charAt(i);
                    if (s.charAt(i) == '\\') s2 += '\\';
                }

                // Stack the filenames one above the other, right justified.
                TextMarkup pathMarkup = new TextMarkup();
                pathMarkup.setText(s2);
                pathMarkup.setPsFont("Helvetica-Oblique");
                pathMarkup.setPsFontSize(12);
                pathMarkup.x = 1;
                pathMarkup.y = 1;
                pathMarkup.xPointOffset = -95;
                pathMarkup.yPointOffset = -75 + (14 * numCommands);
                pathMarkup.xScreenOffset = -500;
                pathMarkup.yScreenOffset = -10 + (30 * numCommands);
                pathMarkup.setCoordinateLevels(1, 1);
                pathMarkup.xJustification = 2;
                pathMarkup.setUserChangeable(false);
                m_page.add(pathMarkup);

                numCommands++;
            }

            // Add the date to the upper left corner of the page.
            TextMarkup dateMarkup = new TextMarkup();
            GregorianCalendar now = new GregorianCalendar();
            DateFormat df = DateFormat.getDateTimeInstance(DateFormat.SHORT,
                DateFormat.SHORT);
            dateMarkup.setText(df.format(now.getTime()));

            dateMarkup.setPsFont("Helvetica-Oblique");
            dateMarkup.setPsFontSize(12);
            dateMarkup.x = 0;
            dateMarkup.y = 1;
            dateMarkup.xPointOffset = 50;
            dateMarkup.yPointOffset = -65;
            dateMarkup.xScreenOffset = 0;
            dateMarkup.yScreenOffset = 5;
            dateMarkup.setCoordinateLevels(1, 1);
            dateMarkup.xJustification = 0;
            dateMarkup.setUserChangeable(false);
            m_page.add(dateMarkup);

            layoutGraphs();

            m_page.m_bNeedsReformat = false;
        }
    }
}
