/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2007-06-26
 * Date started:    2007-01-23
 *
 * Copyright (C) 1998-2007 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.sfhydro.gr.tools;

import gov.usgs.gr.*;
import gov.usgs.gr.data.*;
import gov.usgs.gr.tools.*;
import gov.usgs.gr.visual.*;
import gov.ca.dwr.gr.data.*;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import java.awt.Component;

//==============================================================================
/**
 * An instance of this class can be placed as a child of a PageOfGraphs object
 * to provide an interface to the user for viewing or processing Si3D model
 * output.  Causes a panel to be shown on the toolbar.
 */
public class FilterSet extends gov.usgs.gr.GrObject
    implements gov.usgs.gr.visual.gui.GuiPanelProvider {

    /** An instance of the controller associated with this object. */
    GrController m_grCon = null;
    /** An instance of the page associated with this object. */
    PageOfGraphs m_page = null;
    /** The panel created by this class for displaying on a toolbar. */
    protected JPanel m_panel = new JPanel();
    /** The configuration dialog shown to the user. */
    protected JDialog m_dialog = null;
    /**
     * Whether to show a dialog when giving notification messages to
     * the user.
     */
    protected boolean m_bUseDialogNotification = true;
    /** The lower bounds of the selected area. */
    double m_precedingTime = Double.NEGATIVE_INFINITY;
    /** The upper bounds of the selected area. */
    double m_followingTime = Double.NEGATIVE_INFINITY;
    /** The tree icon to use for this class. */
	public static ArrayList TREE_ICON_HOLDER = new ArrayList(1);
    /** The location of the tree icon. */
    protected String m_iconFile = "GrObject/Si3dPostProcessor.gif";

//------------------------------------------------------------------------------
/**
 * Constructor for creating a default object.
 */
    public FilterSet() {

        // Create the editing panel once so it can be passed out later
        // as needed.
        createPanel();
    }

//------------------------------------------------------------------------------
/**
 * Accessor method.
 * @return An ImageIcon to use when representing this object in a tree.
 */
    public javax.swing.ImageIcon getTreeIcon() {

        if (TREE_ICON_HOLDER.size() <= 0)
            setTreeIcon();

    	return (javax.swing.ImageIcon)TREE_ICON_HOLDER.get(0);
    }

//------------------------------------------------------------------------------
/**
 * Loads the tree icon from a file and stores it in the TREE_ICON_HOLDER.
 */
    protected javax.swing.ImageIcon setTreeIcon() {

        javax.swing.ImageIcon icon = loadTreeIcon(m_iconFile);

        if (icon == null)
            icon = super.setTreeIcon();

        TREE_ICON_HOLDER.clear();
        TREE_ICON_HOLDER.add(icon);

        return icon;
    }

//------------------------------------------------------------------------------
/**
 * Provide the list of properties with the current values for this object.
 * Properties include: <p>
 * workingDir    String  "Working Directory"                           <p>
 * @return  A list of GrProperty objects.
 */
    public GrHashList getProperties() {

        // First add properties from the super class.
        GrHashList props = super.getProperties();

        return props;
    }

//------------------------------------------------------------------------------
/**
 * Set the values of one or more of this object's properties.
 * @see gov.usgs.sfhydro.si3d.Si3dPostProcessor#getProperties getProperties()
 * for a list of accepted properties.
 * @param props A list of GrProperty objects specifying the new values for
 *              certain properties.
 */
    public void setProperties(HashMap props) {

        GrProperty p;

        // For each property with a string value, pass it to the setProperty()
        // method to be handled there.
        Iterator iter = props.keySet().iterator();
        while (iter.hasNext()) {
            String key = (String)iter.next();
            Object obj = props.get(key);
            if (obj instanceof GrProperty) {
                p = (GrProperty)obj;

                // Handle string properties by passing them to the
                // setProperty() method.
                if (p.value instanceof String) {
                    setProperty(key, (String)p.value);
                    continue;
                }
            }
            // Handle string properties by passing them to the
            // setProperty() method.
            if (obj instanceof String) {
                setProperty(key, (String)obj);
            }
        }
    }

//------------------------------------------------------------------------------
/**
 * Set a single property.
 * @see gov.usgs.sfhydro.si3d.Si3dPostProcessor#getProperties getProperties()
 * for a list of accepted properties.
 * @param key   The property name.
 * @param value The new value for the property.
 */
    public void setProperty(String key, String value) {

    }

//------------------------------------------------------------------------------
/**
 * Adds an object as a child, but only if it is an instance of Graph, an
 * IntDataSeries with two or three dimensions, or a DataSeries with three
 * dimensions.  Removes the old child, if a matching one can be found, and
 * replaces it with the new one.
 * @param gro   The object to add.
 * @return  Whether the object was successfully added.
 */
    public boolean add(GrObject gro) {

        // If gro is an object effigy, get the eventual subject and try
        // adding that.
        while (gro instanceof gov.usgs.gr.ObjectEffigy)
            gro = ((ObjectEffigy)gro).getSubject();

        if (m_children.contains(gro)) return false;

        boolean bAdded = false;

        return bAdded;
    }

//------------------------------------------------------------------------------
/**
 * Create the editor panel that will be displayed on the tool bar.
 */
    public void createPanel() {

        // Set up the panel's border and layout.
//        m_panel.setBorder(BorderFactory.createTitledBorder("Filter"));
//        m_panel.setLayout(new BoxLayout(m_panel, BoxLayout.Y_AXIS));
        m_panel.setBorder(BorderFactory.createTitledBorder("Filter"));
        m_panel.setLayout(new GridLayout(2, 1));

        // Define a method to handle the Add button being clicked.
        java.awt.event.ActionListener filterAL =
            new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                filter();
            }
        };

        // Define a method to handle the Add button being clicked.
        java.awt.event.ActionListener rmsAL =
            new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                calculateRms();
            }
        };

        // Add a button to the main panel below the other components.
        JButton filterButton = createButton("Filter", filterAL,
            "filter", "Filter first series in bottom graph");
        byte[] byteArray = {0};
        filterButton.setIcon(new ImageIcon(byteArray));
        filterButton.setMargin(new Insets(0, 10, 0, 10));
        filterButton.setAlignmentX(Component.CENTER_ALIGNMENT);
        m_panel.add(filterButton);
/*
        // Add a button to the main panel below the other components.
        JButton rmsButton = createButton("RMS", rmsAL,
            "rms", "Calculate RMS error");
        filterButton.setIcon(new ImageIcon(byteArray));
        filterButton.setMargin(new Insets(0, 10, 0, 10));
        filterButton.setAlignmentX(Component.CENTER_ALIGNMENT);
        m_panel.add(rmsButton);
*/
        m_panel.setMaximumSize(new java.awt.Dimension(400, 400));
    }

//------------------------------------------------------------------------------
/**
 *
 */
    protected JButton createButton(String text, ActionListener al,
        String actionText, String toolText) {

        JButton button = new JButton(text);
        button.addActionListener(al);
        button.setActionCommand(actionText);
        button.setToolTipText(toolText);
        byte[] byteArray = {0};
        button.setIcon(new ImageIcon(byteArray));
        button.setBorderPainted(true);
        button.setMargin(new Insets(0, 0, 0, 0));

        return button;
    }

//------------------------------------------------------------------------------
/**
 * 
 */
    protected void filter() {

        List<Graph> graphList = m_page.getGraphList();

        while (graphList.size() < 4) {
            m_grCon.newGraph();
            graphList = m_page.getGraphList();
        }
        
        Graph graph1 = graphList.get(0);
        Graph graph2 = graphList.get(1);
        Graph graph3 = graphList.get(2);
        Graph graph4 = graphList.get(3);
        Graph graph5 = null;
//        Graph graph5 = graphList.get(4);
        
        graph2.clear();
        TimeSeries sourceTS = (TimeSeries)graph1.firstChildInstanceOf(
            "gov.usgs.gr.data.DoubleDataSeries", true);
        
        TimeSeries hhSeries =
            (TimeSeries)sourceTS.createCopy(true);
        graph1.add(hhSeries);
        List hhList = new ArrayList();
        hhList.add(hhSeries);
        
        gov.usgs.sfhydro.gr.tools.TidalHighFinder thh =
            new gov.usgs.sfhydro.gr.tools.TidalHighFinder();
        thh.setProperty("span", "" + (30.8d / 24d));
        thh.setProperty("subSpan", "" + (6d / 24d));
        thh.performAction(hhList);
        
        TimeSeries llSeries =
            (TimeSeries)sourceTS.createCopy(true);
        graph1.add(llSeries);
        List llList = new ArrayList();
        llList.add(llSeries);
        
        gov.usgs.sfhydro.gr.tools.TidalLowFinder tll =
            new gov.usgs.sfhydro.gr.tools.TidalLowFinder();
        tll.setProperty("span", "" + (30.8d / 24d));
        tll.setProperty("subSpan", "" + (6d / 24d));
        tll.performAction(llList);
        
        TimeSeries godinSeries =
            (TimeSeries)sourceTS.createCopy(true);
        graph2.add(godinSeries);
        List godinList = new ArrayList();
        godinList.add(godinSeries);
        
        gov.usgs.sfhydro.gr.tools.GodinFilter gf =
            new gov.usgs.sfhydro.gr.tools.GodinFilter();
        gf.performAction(godinList);
        godinSeries.deselectAll();
/*       
        TimeSeries conSeries51 =
            (TimeSeries)sourceTS.createCopy(true);
        graph2.add(conSeries51);
        List conList51 = new ArrayList();
        conList51.add(conSeries51);

        gov.usgs.sfhydro.gr.tools.ConvolutionFilter cf51 =
            new gov.usgs.sfhydro.gr.tools.ConvolutionFilter();
        cf51.setProperty("numK", "51");
        cf51.performAction(conList51);
*/        
        TimeSeries conSeries24 =
            (TimeSeries)sourceTS.createCopy(true);
        graph2.add(conSeries24);
        List conList24 = new ArrayList();
        conList24.add(conSeries24);

        gov.usgs.sfhydro.gr.tools.ConvolutionFilter cf24 =
            new gov.usgs.sfhydro.gr.tools.ConvolutionFilter();
        cf24.setProperty("numK", "24");
        cf24.performAction(conList24);
        conSeries24.deselectAll();

        TimeSeries conSeries120 =
            (TimeSeries)sourceTS.createCopy(true);
        graph2.add(conSeries120);
        List conList120 = new ArrayList();
        conList120.add(conSeries120);

        gov.usgs.sfhydro.gr.tools.ConvolutionFilter cf120 =
            new gov.usgs.sfhydro.gr.tools.ConvolutionFilter();
        cf120.performAction(conList120);
        conSeries120.deselectAll();

//        TimeSeries lowPassSeries =
//            (TimeSeries)sourceTS.createCopy(true);
//        graph2.add(lowPassSeries);
//        List lowPassList = new ArrayList();
//        lowPassList.add(lowPassSeries);

//        gov.usgs.sfhydro.gr.tools.LowPassFilter lpf =
//            new gov.usgs.sfhydro.gr.tools.LowPassFilter();
//        lpf.setProperty("dataInterval", "15m");
//        lpf.setProperty("n", "99");
//        lpf.performAction(lowPassList);

        
        TimeSeries tidalDiffSeries =
            (TimeSeries)hhSeries.createCopy(true);
        graph2.add(tidalDiffSeries);
        tidalDiffSeries.resample(1d / 24d, 0);
	tidalDiffSeries.selectAll();
	tidalDiffSeries.modifySelectedPoints(2, 1, llSeries);
	tidalDiffSeries.deselectAll();
	double[] min = {Double.NEGATIVE_INFINITY, Double.NEGATIVE_INFINITY};
	double[] max = {Double.POSITIVE_INFINITY, 0.001};
	tidalDiffSeries.selectArea(min, max, true);
	tidalDiffSeries.modifySelectedPoints(0, 1, 0.001);
	tidalDiffSeries.deselectAll();
//        tidalDiffSeries.m_bInterpolateSamples = false;
//        tidalDiffSeries.resample(1d, 0);
        
/*        
        TimeSeries meanLlHhSeries =
            (TimeSeries)hhSeries.createCopy(true);
        graph2.add(meanLlHhSeries);
        meanLlHhSeries.resample(1d / 24d, 0);
	meanLlHhSeries.selectAll();
	meanLlHhSeries.modifySelectedPoints(1, 1, llSeries);
	meanLlHhSeries.modifySelectedPoints(3, 1, 0.5);
	meanLlHhSeries.deselectAll();
        meanLlHhSeries.m_bInterpolateSamples = false;
        meanLlHhSeries.resample(1d, 0);
                
        TimeSeries ahSeries =
            (TimeSeries)sourceTS.createCopy(true);
//        graph1.add(ahSeries);
        List ahList = new ArrayList();
        ahList.add(ahSeries);
        
        gov.usgs.sfhydro.gr.tools.TidalHighFinder ah =
            new gov.usgs.sfhydro.gr.tools.TidalHighFinder();
        ah.setProperty("span", "" + (16.3d / 24d));
        ah.setProperty("subSpan", "" + (4.3d / 24d));
        ah.performAction(ahList);
        
        TimeSeries alSeries =
            (TimeSeries)sourceTS.createCopy(true);
//        graph1.add(alSeries);
        List alList = new ArrayList();
        alList.add(alSeries);
        
        gov.usgs.sfhydro.gr.tools.TidalLowFinder al =
            new gov.usgs.sfhydro.gr.tools.TidalLowFinder();
        al.setProperty("span", "" + (16.3d / 24d));
        al.setProperty("subSpan", "" + (4.3d / 24d));
        al.performAction(alList);
        
        TimeSeries allDiffSeries =
            (TimeSeries)ahSeries.createCopy(true);
        graph1.add(allDiffSeries);
        allDiffSeries.resample(1d / 24d, 0);
	allDiffSeries.selectAll();
	allDiffSeries.modifySelectedPoints(2, 1, alSeries);
	allDiffSeries.deselectAll();
	double[] min2 = {Double.NEGATIVE_INFINITY, Double.NEGATIVE_INFINITY};
	double[] max2 = {Double.POSITIVE_INFINITY, 0.001};
	allDiffSeries.selectArea(min2, max2, true);
	allDiffSeries.modifySelectedPoints(0, 1, 0.001);
	allDiffSeries.deselectAll();
        allDiffSeries.m_bInterpolateSamples = false;
        allDiffSeries.resample(1d, 0);
               
        TimeSeries meanAlAhSeries =
            (TimeSeries)ahSeries.createCopy(true);
        graph2.add(meanAlAhSeries);
        meanAlAhSeries.resample(1d / 24d, 0);
	meanAlAhSeries.selectAll();
	meanAlAhSeries.modifySelectedPoints(1, 1, alSeries);
	meanAlAhSeries.modifySelectedPoints(3, 1, 0.5);
	meanAlAhSeries.deselectAll();
        meanAlAhSeries.m_bInterpolateSamples = false;
        meanAlAhSeries.resample(1d, 0);
*/
        
        if (graph3 != null && graph4 != null) {
            TimeSeries rangeGodinRatSeries = calculateRangeToFilterRatio(
                tidalDiffSeries, godinSeries);
            graph3.add(rangeGodinRatSeries);
            
            TimeSeries rangeCon120RatSeries = calculateRangeToFilterRatio(
                tidalDiffSeries, conSeries120);
            graph3.add(rangeCon120RatSeries);
            
            TimeSeries rangeCon24RatSeries = calculateRangeToFilterRatio(
                tidalDiffSeries, conSeries120);
            graph3.add(rangeCon24RatSeries);

            TimeSeries mean24GodinErrorSeries = calculateDailyError(sourceTS,
                godinSeries);
            graph4.add(mean24GodinErrorSeries);

            TimeSeries mean24Con120ErrorSeries = calculateDailyError(sourceTS,
                conSeries120);
            graph4.add(mean24Con120ErrorSeries);

            TimeSeries meanCon24ErrorSeries = calculateDailyError(sourceTS,
                conSeries24);
            graph4.add(meanCon24ErrorSeries);
            
/*            GrHashList dims = new GrHashList();
            DataDimension dim = mean24GodinErrorSeries.getDimension(1);
            dims.put(dim, dim.getAssociatedSeries());
            dim = rangeGodinRatSeries.getDimension(1);
            dims.put(dim, dim.getAssociatedSeries());
            DoubleDataSeries erSeries0 =
                (DoubleDataSeries)DimensionOrganizer.combine(dims, false);
            
            double[] array = erSeries0.getArray();
            for (int i = 0; i < array.length; i++) {
                array[i] = Math.log10(array[i]);
            }
            
            DoubleDataSeries erSeries = new DoubleDataSeries(array, 2);
            graph5.add(erSeries);
            
            Zoom xZoom = graph5.getXZoom();
            Axis xAxis = graph5.getXAxis();
            xAxis.add(xZoom.createCopy(false));*/
        }
        
        double interval1197 = TimeReference.convertToDecimalDay("11.97h");
        double interval12 = TimeReference.convertToDecimalDay("12h");
        double interval1242 = TimeReference.convertToDecimalDay("12.42h");
        double interval2393 = TimeReference.convertToDecimalDay("23.93h");
        double interval2484 = TimeReference.convertToDecimalDay("24.84h");
        double interval24 = TimeReference.convertToDecimalDay("24h");
        double offset = TimeReference.convertToDecimalDay("0");
        double outputInterval = TimeReference.convertToDecimalDay("15m");

        
///        average(convolutionSeries, interval24, offset, outputInterval);
///        average(godinSeries, interval24, offset, outputInterval);

//        TimeSeries resampSeries =
//            (TimeSeries)sourceTS.createCopy(true);
//        graph2.add(resampSeries);
//        double offset = 0;
//        resampSeries.m_bInterpolateSamples = false;
//        resampSeries.resample(interval, offset);
        

        
//        TimeSeries avgSeries1a =
//            (TimeSeries)sourceTS.createCopy(true);
//        graph2.add(avgSeries1a);
//        average(avgSeries1a, interval24, offset, outputInterval);
//        average(avgSeries1a, interval2484, offset, outputInterval);
        
        
 
        
        
//        TimeSeries movAvgSeries =
//            (TimeSeries)sourceTS.createCopy(true);
//        graph2.add(movAvgSeries);
//        double span = interval;
//        movAvgSeries.applyMovingAverage(span, 0);
        
        m_grCon.deselectAll();
        graph1.setSelected(true);
        graph2.setSelected(true);
        if (graph3 != null && graph4 != null) {
            graph3.setSelected(true);
            graph4.setSelected(true);
        }
        if (graph5 != null) {
            graph5.setSelected(true);
        }
        m_page.updateLists();
        m_grCon.zoomAll();
        graph1.setSelected(false);
        graph2.setSelected(false);
        if (graph3 != null && graph4 != null) {
            graph3.setSelected(false);
            graph4.setSelected(false);
        }
        if (graph5 != null) {
            graph5.setSelected(false);
        }
        m_page.updateLists();

    }
    
//------------------------------------------------------------------------------
    /**
     * Replace a data series with its box car average at a given interval and
     * starting at a given offset.  Output can be at a different interval than
     * the averaging interval.
     * @param series            The data series that will be modifoed.
     * @param interval          The span for determining each averaged point.
     * @param offset            The amount to offset the points from zero.
     * @param outputInterval    The spacing of the output.
     */
    public synchronized void average(DoubleDataSeries series, double interval,
        double offset, double outputInterval) {
        
        int xDim = 0;
        
        // Use the absolute value of the supplied interval.
        interval = Math.abs(interval);
        double halfInterval = interval / 2;
        
        // Include a little more area (20% of the interval above and below)
        // just so rounding doesn't keep any points on the boundary from being
        // included in the interpolation.
        double min = series.getMin(xDim) - (interval / 20);
        double max = series.getMax(xDim) + (interval / 20);
        double startOffset = min - offset;
        double startIntervals = (int)(startOffset / interval);
        if (startOffset > 0) startIntervals++;
        double start = (interval * startIntervals) + offset;
        int size = (int)((max - start) / outputInterval) + 1;
        
        int numDims = series.getNumDimensions();
        double[] newArray = new double[size * numDims];
        for (int i = 0; i < newArray.length; i += numDims) {
            // Set the coordinates.
            newArray[i + xDim] = start + ((i / numDims) * outputInterval);
            
            for (int j = 0; j < numDims; j++) {
                double val = 0;
                
                if (j != xDim) {
                    // Find the value for the new point.
                    double spanMin = newArray[i + xDim] - halfInterval;
                    double spanMax = newArray[i + xDim] + halfInterval;
                    val = series.findSpanAverage(spanMin, spanMax, -1, xDim, j);
                    
                    // Interpolate the last dimension's value at this point.
                    newArray[i + j] = val;
                }
            }
        }
        
        // The old list of deleted points was probably invalidated by this
        // operation, so wipe it out.
        series.setArray(newArray);
    }

//------------------------------------------------------------------------------
/**
 * 
 */
    protected void calculateRms() {

        List<Graph> graphList = m_page.getGraphList();

        if (graphList.size() < 2)
            filter();

        while (graphList.size() < 5) {
            m_grCon.newGraph();
            graphList = m_page.getGraphList();
        }
        
        Graph graph1 = graphList.get(0);
        Graph graph2 = graphList.get(1);
        Graph graph5 = graphList.get(4);
        
        graph5.clear();
        
        TimeSeries sourceTS = (TimeSeries)graph1.firstChildInstanceOf(
            "gov.usgs.gr.data.DoubleDataSeries", true);
        List otherSeriesList = graph2.findChildInstancesOf(
            "gov.usgs.gr.data.DoubleDataSeries", true);
        RmsCalculator rmsCalc = new RmsCalculator();
        Iterator iter = otherSeriesList.iterator();
        while (iter.hasNext()) {
            DoubleDataSeries otherTS = (DoubleDataSeries)iter.next();
            DoubleDataSeries rmsSeries =
                rmsCalc.createRmsSeries(sourceTS, otherTS);
            graph5.add(rmsSeries);
        }
    }
    
//------------------------------------------------------------------------------
/**
 * 
 */
    protected TimeSeries calculateRangeToFilterRatio(TimeSeries tidalDiffSeries,
        TimeSeries filterSeries) {
        
        TimeSeries rangeFilterRatSeries =
            (TimeSeries)tidalDiffSeries.createCopy(true);

        // Create a series that is the absolute value of the Godin series.
        TimeSeries filterPositiveSeries =
            (TimeSeries)filterSeries.createCopy(true);
        double[] minNeg = {Double.NEGATIVE_INFINITY, Double.NEGATIVE_INFINITY};
        double[] maxNeg = {Double.POSITIVE_INFINITY, -0.001};
        filterPositiveSeries.selectArea(minNeg, maxNeg, true);
        filterPositiveSeries.modifySelectedPoints(3, 1, -1);
        filterPositiveSeries.deselectAll();
        double[] maxZero = {Double.POSITIVE_INFINITY, 0.001};
        filterPositiveSeries.selectArea(minNeg, maxZero, true);
        filterPositiveSeries.modifySelectedPoints(0, 1, 0.001);

        // Divide the tidal range by the absolute value of Godin.
        rangeFilterRatSeries.selectAll();
        rangeFilterRatSeries.modifySelectedPoints(4, 1, filterPositiveSeries);
        rangeFilterRatSeries.deselectAll();
        rangeFilterRatSeries.m_bInterpolateSamples = false;
        rangeFilterRatSeries.resample(1d, 0);
        
        return rangeFilterRatSeries;
    }
//------------------------------------------------------------------------------
/**
 * 
 */
    protected TimeSeries calculateDailyError(TimeSeries sourceTS,
        TimeSeries filtSeries) {

        double[] minNeg = {Double.NEGATIVE_INFINITY, Double.NEGATIVE_INFINITY};
        double[] maxNeg = {Double.POSITIVE_INFINITY, -0.001};
        double[] maxZero = {Double.POSITIVE_INFINITY, 0.001};

        TimeSeries mean24FiltErrorSeries =
            (TimeSeries)sourceTS.createCopy(true);
        mean24FiltErrorSeries.m_bInterpolateSamples = false;
        mean24FiltErrorSeries.resample(1d, 0);

        // For the Godin, use absolute value of daily Godin.
        TimeSeries filtDailySeries =
            (TimeSeries)filtSeries.createCopy(true);
        filtDailySeries.m_bInterpolateSamples = false;
        filtDailySeries.resample(1d, 0);
        mean24FiltErrorSeries.selectAll();
        mean24FiltErrorSeries.modifySelectedPoints(2, 1, filtDailySeries);
        filtDailySeries.selectArea(minNeg, maxNeg, true);
        filtDailySeries.modifySelectedPoints(3, 1, -1);
        filtDailySeries.deselectAll();
        filtDailySeries.selectArea(minNeg, maxZero, true);
        filtDailySeries.modifySelectedPoints(0, 1, 0.001);

        // Divide by Godin.
        mean24FiltErrorSeries.selectAll();
        mean24FiltErrorSeries.modifySelectedPoints(4, 1, filtDailySeries);
        mean24FiltErrorSeries.deselectAll();

        // Take absolute value
        mean24FiltErrorSeries.selectArea(minNeg, maxNeg, true);
        mean24FiltErrorSeries.modifySelectedPoints(3, 1, -1);
        mean24FiltErrorSeries.deselectAll();
        mean24FiltErrorSeries.selectArea(minNeg, maxZero, true);
        mean24FiltErrorSeries.modifySelectedPoints(0, 1, 0.001);
        mean24FiltErrorSeries.deselectAll();
        
        return mean24FiltErrorSeries;
    }
    
//------------------------------------------------------------------------------
/**
 * Accessor method.
 * @return The editor panel created by this class at initialization time.
 */
    public JPanel getPanel() {

        return m_panel;
    }

//------------------------------------------------------------------------------
/**
 * Tells this interface what the current controller and page are.
 * @param grCon   The GrController to use.
 * @param page    The PageOfGraphs to use.
 */
    public void setControllerAndPage(GrController grCon, PageOfGraphs page) {

        m_grCon = grCon;
        m_page = page;
    }

//------------------------------------------------------------------------------
/**
 * Finds the PageofGraphs that contains m_zoom.
 */
    public void findParentPage() {

        boolean bPageFound = false;
        List observers = getObservers();
        Iterator obIter = observers.iterator();
        while (obIter.hasNext()) {
            // Look for a page parent of the graph.
            GrObject gro = (GrObject)obIter.next();
            if (gro instanceof PageOfGraphs) {
                m_page = (PageOfGraphs)gro;
                bPageFound = true;
                break;
            }
        }
    }

//------------------------------------------------------------------------------
/**
 * Overrides default update() in GrObject.  Called by an observed GrObject's
 * notifyObservers() method to handle changes to the observed object.
 * May use the notifyObservers() method to propagate the event handling up to
 * this object's observers.
 * @param gro   The observed object that changed.
 * @param arg   Additional information that the updated object may choose to
 *              send.
 */
    public void update(GrObject gro, Object arg) {
        m_bHasChanged = true;
        notifyObservers();
    }

//------------------------------------------------------------------------------
/**
 * Create a string representation of this object.
 * @return  The string that describes this object.
 */
    public String toString() {

        String s = "Filter Set";

        return s;
     }
}

