/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2006-04-06
 * Date started:    2001-02-08
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.sfhydro.gr.tools;

import gov.usgs.gr.*;
import gov.usgs.gr.data.*;
import gov.usgs.gr.tools.*;
import gov.usgs.gr.visual.*;
import gov.usgs.gr.visual.gl.*;
import java.io.*;
import java.util.*;
import javax.swing.*;

//==============================================================================
/**
 *
 */

public class ParticleAnimCalculator extends GrObject implements
	java.awt.event.ActionListener {

    /** The tree icon to use for this class. */
	public static ArrayList TREE_ICON_HOLDER = new ArrayList(1);
    /** The location of the tree icon. */
    protected String m_iconFile = "GrObject/ParticleAnimCalculator.gif";

//------------------------------------------------------------------------------
/**
 * Creates an empty object.
 */
    public ParticleAnimCalculator () {
    }

//------------------------------------------------------------------------------
/**
 * Overrides add in GrObject.  Only allows DataDimensions to be added.
 * @param gro   The object to add.
 * @return  Whether the object was successfully added.
 */
    public boolean add(GrObject gro) {

        // If gro is an object effigy, get the eventual subject and try
        // adding that.
        while (gro instanceof gov.usgs.gr.ObjectEffigy)
        	gro = ((ObjectEffigy)gro).getSubject();

        return super.add(gro);
    }

//------------------------------------------------------------------------------
/**
 * Accessor method.
 * @return An ImageIcon to use when representing this object in a tree.
 */
    public javax.swing.ImageIcon getTreeIcon() {

        if (TREE_ICON_HOLDER.size() <= 0)
            setTreeIcon();

    	return (javax.swing.ImageIcon)TREE_ICON_HOLDER.get(0);
    }

//------------------------------------------------------------------------------
/**
 * Loads the tree icon from a file and stores it in the TREE_ICON_HOLDER.
 */
    protected javax.swing.ImageIcon setTreeIcon() {

        javax.swing.ImageIcon icon = loadTreeIcon(m_iconFile);

        if (icon == null)
            icon = super.setTreeIcon();

        TREE_ICON_HOLDER.clear();
        TREE_ICON_HOLDER.add(icon);

        return icon;
    }

//------------------------------------------------------------------------------
/**
 *
 */
    public void recalculate () {

        TimeMarker timeMarker = null;
        ArrayList removeList = new ArrayList();
        ArrayList inputList = new ArrayList();
        Iterator iter = m_children.iterator();
        while (iter.hasNext()) {
            GrObject gro = (GrObject)iter.next();
            if (gro instanceof GlParticleAnimation) removeList.add(gro);
            if (gro instanceof TimeMarker) timeMarker = (TimeMarker)gro;
            if (gro instanceof DataSeries) inputList.add(gro);
        }
        // Remove the items in the remove list.
        iter = removeList.iterator();
        while (iter.hasNext()) {
            remove((GrObject)iter.next());
        }

        if (timeMarker == null) {
            timeMarker = new TimeMarker();
            add(timeMarker);
        }
        timeMarker.clear();

        iter = inputList.iterator();
        while (iter.hasNext()) {
            GrObject gro = (GrObject)iter.next();
            if (gro instanceof DataSeries) {
                DataSeries series = (DataSeries)gro;

//                eliminateRedundantZeros(series);

                GlParticleAnimation glp = new GlParticleAnimation();
                glp.add(series);
                timeMarker.add(series);

                glp.setProperty("title", series.getTitle());
                TextMarkup txt = new TextMarkup();
                txt.setText(series.getTitle());
                txt.setProperty("show", "true");
                txt.setProperty("color", "yellow");
                txt.setProperty("screenFont", "glutBitmap9By15");
                glp.add(txt);
                add(glp);
            }
        }
    }

//------------------------------------------------------------------------------
/**
 *
 */
    public void processFiles(java.awt.Frame frame) {

        // Clear all the children of this object.
        clear();

        // Have the user select the files to be processed.
        File[] files = GrController.showFileDialog("Open File", true,
            new File(System.getProperty("gov.usgs.gr.currentfile")), frame);

        // Read each file and add the main data series as a child.
        for (int i = 0; i < files.length; i++) {
            GrFile grFile = (GrFile)GrController.createObject(
                "gov.usgs.sfhydro.data.GrGsDataFile", "gov.usgs.GrFile", frame);
            if (grFile != null)
                grFile.setProperty("file", files[i].getAbsolutePath());

            try {
                grFile.read();
            }
            catch (IOException e) {
                e.printStackTrace();

                JOptionPane.showMessageDialog(frame, "File "
                    + files[i].getAbsolutePath() + " could not be loaded.",
                    "IO Exception", JOptionPane.ERROR_MESSAGE);
                continue;
            }

            // Add all DataSeries with 3 or more dimensions from the file
            // into this calculator
            Enumeration enumeration = grFile.children();
            while (enumeration.hasMoreElements()) {
        	    GrObject gro = (GrObject)enumeration.nextElement();
                while (gro instanceof ObjectEffigy)
            	    gro = ((ObjectEffigy)gro).getSubject();

                if (gro instanceof DataSeries) {
                    DataSeries series = (DataSeries)gro;
                    if (series.getNumDimensions() >= 3)
                        add(series);
                }
            }
        }

        // Recalculate the series.
        recalculate();
    }

//------------------------------------------------------------------------------
/**
 *
 */
    public void eliminateRedundantZeros(DataSeries series) {

/*
    	float[] array = series.getArray();
        float[] newArray = new float[array.length];
        float total = 0;
        int numDimensions = series.getNumDimensions();

        // Calculate the mean.
        for (int i = 0; i < series.getCount(); i++) {
        	if (array[i * numDimensions + j] != 0) bKeep = true;;
        }

        float mean = total / series.getCount();

        // Create a new array with two points spanning between the endpoints
        // with the value of the last dimension being equal to the mean.
        float[] newArray = new float[numDimensions * 2];
        for (int i = 0; i < (numDimensions - 1); i++) {
        	newArray[i] = array[i];
            newArray[i + numDimensions] = array[array.length - numDimensions];
        }
        newArray[numDimensions - 1] = mean;
        newArray[(numDimensions * 2) - 1] = mean;

        // Replace the old array with the new mean.
        series.setArray(newArray);
*/
    }

//------------------------------------------------------------------------------
/**
 * Overrides default update() in GrObject.  Called by an observed GrObject's
 * notifyObservers() method to handle changes to the observed object.
 * May use the notifyObservers() method to propagate the event handling up to
 * this object's observers.
 * @param gro   The observed object that changed.
 * @param arg   Additional information that the updated object may choose to
 *              send.
 */
    public void update(GrObject gro, Object arg) {

    }

//------------------------------------------------------------------------------
/**
 * Provide the list of menu items that execute commands on this object.
 * @return  A menu item that can be placed on a menu and send an ActionEvent
 *          back to this object when it is chosen.
 */
    public javax.swing.JMenuItem getMenuCommands() {

        javax.swing.JMenu subMenu = new javax.swing.JMenu(
        	"Particle Animation Calculator");
        javax.swing.JMenuItem item = new javax.swing.JMenuItem("Calculate");
        item.addActionListener(this);
        subMenu.add(item);
        item = new javax.swing.JMenuItem("Process Files...");
        item.addActionListener(this);
        subMenu.add(item);

    	return subMenu;
    }

//------------------------------------------------------------------------------
    public void actionPerformed(java.awt.event.ActionEvent e) {

        javax.swing.JMenuItem source =
        	(javax.swing.JMenuItem)(e.getSource());
        String srcText = source.getText();
        if (srcText.equals("Calculate")) {
        	recalculate();
        }
        if (srcText.equals("Process Files...")) {
            java.awt.Frame frame = GrController.getParentFrame(source);
        	processFiles(frame);
        }
    }

//------------------------------------------------------------------------------
/**
 * Create a string representation of this object.
 * @return  The string that describes this object.
 */
    public String toString() {

    	String s = "Particle Animation Calculator";

        return s;
 	}
}

