/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2005-11-04
 * Date started:    2001-09-04
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.sfhydro.gr.tools;

import java.util.*;
import gov.usgs.gr.*;
import gov.usgs.gr.data.*;
import gov.usgs.gr.tools.*;
import gov.usgs.gr.visual.gl.*;

//==============================================================================
/**
 * Calculates the cross sectional path where a vertical surface intersects
 * a surfaces of discontinuous squares.  When a surface data set and a cross
 * secion line XyDataSet are added as children, an output X-Z data set is
 * calculated and added as a third child.  The output series can be viewed
 * if it is copied and the original pasted into a graph.
 */

public class ParticleTracker extends GrObject implements
	java.awt.event.ActionListener {

//------------------------------------------------------------------------------
/**
 * Creates an empty object.
 */
    public ParticleTracker () {
    }


//------------------------------------------------------------------------------
/**
 * Overrides add in GrObject.  Only allows DataDimensions to be added.
 * @param gro   The object to add.
 * @return  Whether the object was successfully added.
 */
    public boolean add(GrObject gro) {

        // If gro is an object effigy, get the eventual subject and try
        // adding that.
        while (gro instanceof gov.usgs.gr.ObjectEffigy)
        	gro = ((ObjectEffigy)gro).getSubject();

        return super.add(gro);
    }

//------------------------------------------------------------------------------
/**
 *
 */
    public void recalculate () {

        // Look for XY data series to use as starting points.  If one
        // exists, assume Z = 0.  If it doesn't, look for 4D txyz series
        // and use the first time as input and also use it for output.
        // Look for a 7D txyzuvw series to use as a vector field.
        // The time steps of the output will match the vector field series.
    }

//------------------------------------------------------------------------------
/**
 * Overrides default update() in GrObject.  Called by an observed GrObject's
 * notifyObservers() method to handle changes to the observed object.
 * May use the notifyObservers() method to propagate the event handling up to
 * this object's observers.
 * @param gro   The observed object that changed.
 * @param arg   Additional information that the updated object may choose to
 *              send.
 */
    public void update(GrObject gro, Object arg) {

    }

//------------------------------------------------------------------------------
/**
 * Provide the list of menu items that execute commands on this object.
 * @return  A menu item that can be placed on a menu and send an ActionEvent
 *          back to this object when it is chosen.
 */
    public javax.swing.JMenuItem getMenuCommands() {

        javax.swing.JMenu subMenu = new javax.swing.JMenu(
        	"Particle Tracker");
        javax.swing.JMenuItem saveItem = new javax.swing.JMenuItem("Calculate");
        saveItem.addActionListener(this);
        subMenu.add(saveItem);
    	return subMenu;
    }

//------------------------------------------------------------------------------
    public void actionPerformed(java.awt.event.ActionEvent e) {

        javax.swing.JMenuItem source =
        	(javax.swing.JMenuItem)(e.getSource());
        String srcText = source.getText();
        if (srcText.equals("Calculate")) {
        	recalculate();
        }
    }

//------------------------------------------------------------------------------
/**
 * Create a string representation of this object.
 * @return  The string that describes this object.
 */
    public String toString() {

    	String s = "Particle Tracker";

        return s;
 	}
}

