/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2007-05-03
 * Date started:    2005-03-18
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.sfhydro.gr.tools;

import gov.usgs.gr.*;
import gov.usgs.gr.data.*;
import gov.usgs.gr.tools.*;
import gov.usgs.gr.visual.*;
import gov.ca.dwr.gr.data.*;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import java.awt.Component;

//==============================================================================
/**
 * An instance of this class can be placed as a child of a PageOfGraphs object
 * to provide an interface to the user for viewing or processing Si3D model
 * output.  Causes a panel to be shown on the toolbar.
 */
public class TsToolSet extends gov.usgs.gr.GrObject
    implements gov.usgs.gr.visual.gui.GuiPanelProvider {

    /** An instance of the controller associated with this object. */
    GrController m_grCon = null;
    /** An instance of the page associated with this object. */
    PageOfGraphs m_page = null;
    /** The panel created by this class for displaying on a toolbar. */
    protected JPanel m_panel = new JPanel();
    /** The configuration dialog shown to the user. */
    protected JDialog m_dialog = null;
    /**
     * Whether to show a dialog when giving notification messages to
     * the user.
     */
    protected boolean m_bUseDialogNotification = true;
    /** The lower bounds of the selected area. */
    double m_precedingTime = Double.NEGATIVE_INFINITY;
    /** The upper bounds of the selected area. */
    double m_followingTime = Double.NEGATIVE_INFINITY;
    /** The tree icon to use for this class. */
	public static ArrayList TREE_ICON_HOLDER = new ArrayList(1);
    /** The location of the tree icon. */
    protected String m_iconFile = "GrObject/Si3dPostProcessor.gif";

//------------------------------------------------------------------------------
/**
 * Constructor for creating a default object.
 */
    public TsToolSet() {

        createPanel();
    }

//------------------------------------------------------------------------------
/**
 * Accessor method.
 * @return An ImageIcon to use when representing this object in a tree.
 */
    public javax.swing.ImageIcon getTreeIcon() {

        if (TREE_ICON_HOLDER.size() <= 0)
            setTreeIcon();

    	return (javax.swing.ImageIcon)TREE_ICON_HOLDER.get(0);
    }

//------------------------------------------------------------------------------
/**
 * Loads the tree icon from a file and stores it in the TREE_ICON_HOLDER.
 */
    protected javax.swing.ImageIcon setTreeIcon() {

        javax.swing.ImageIcon icon = loadTreeIcon(m_iconFile);

        if (icon == null)
            icon = super.setTreeIcon();

        TREE_ICON_HOLDER.clear();
        TREE_ICON_HOLDER.add(icon);

        return icon;
    }

//------------------------------------------------------------------------------
/**
 * Provide the list of properties with the current values for this object.
 * Properties include: <p>
 * workingDir    String  "Working Directory"                           <p>
 * @return  A list of GrProperty objects.
 */
    public GrHashList getProperties() {

        // First add properties from the super class.
        GrHashList props = super.getProperties();

        return props;
    }

//------------------------------------------------------------------------------
/**
 * Set the values of one or more of this object's properties.
 * @see gov.usgs.sfhydro.si3d.Si3dPostProcessor#getProperties getProperties()
 * for a list of accepted properties.
 * @param props A list of GrProperty objects specifying the new values for
 *              certain properties.
 */
    public void setProperties(HashMap props) {

        GrProperty p;

        // For each property with a string value, pass it to the setProperty()
        // method to be handled there.
        Iterator iter = props.keySet().iterator();
        while (iter.hasNext()) {
            String key = (String)iter.next();
            Object obj = props.get(key);
            if (obj instanceof GrProperty) {
                p = (GrProperty)obj;

                // Handle string properties by passing them to the
                // setProperty() method.
                if (p.value instanceof String) {
                    setProperty(key, (String)p.value);
                    continue;
                }
            }
            // Handle string properties by passing them to the
            // setProperty() method.
            if (obj instanceof String) {
                setProperty(key, (String)obj);
            }
        }
    }

//------------------------------------------------------------------------------
/**
 * Set a single property.
 * @see gov.usgs.sfhydro.si3d.Si3dPostProcessor#getProperties getProperties()
 * for a list of accepted properties.
 * @param key   The property name.
 * @param value The new value for the property.
 */
    public void setProperty(String key, String value) {

    }

//------------------------------------------------------------------------------
/**
 * Adds an object as a child, but only if it is an instance of Graph, an
 * IntDataSeries with two or three dimensions, or a DataSeries with three
 * dimensions.  Removes the old child, if a matching one can be found, and
 * replaces it with the new one.
 * @param gro   The object to add.
 * @return  Whether the object was successfully added.
 */
    public boolean add(GrObject gro) {

        // If gro is an object effigy, get the eventual subject and try
        // adding that.
        while (gro instanceof gov.usgs.gr.ObjectEffigy)
            gro = ((ObjectEffigy)gro).getSubject();

        if (m_children.contains(gro)) return false;

        boolean bAdded = false;

        return bAdded;
    }

//------------------------------------------------------------------------------
/**
 * Create the editor panel that will be displayed on the tool bar.
 */
    public void createPanel() {

        // Set up the panel's border and layout.
        m_panel.setBorder(BorderFactory.createTitledBorder("Time Series"));
        m_panel.setLayout(new BoxLayout(m_panel, BoxLayout.Y_AXIS));

        // Define a method to handle the Add button being clicked.
        java.awt.event.ActionListener selectNon15AL =
            new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                eliminateNon15();
            }
        };

        // Create an "Add Points" button.
        // Add it to the main panel below the other components.
        JButton selectNon15Button = createButton("Fix 15", selectNon15AL,
            "fix15", "Eliminate Non-15 Minute Data within the current page");
//        byte[] byteArray = {0};
//        selectNon15Button.setIcon(new ImageIcon(byteArray));
        selectNon15Button.setMargin(new Insets(0, 10, 0, 10));
        selectNon15Button.setAlignmentX(Component.CENTER_ALIGNMENT);
        m_panel.add(selectNon15Button);

        // Create panel for the gap and fill buttons
        JPanel gapFillPanel = new JPanel();
        gapFillPanel.setLayout(new BoxLayout(gapFillPanel, BoxLayout.X_AXIS));

        // Set up the panel's border and layout.
        JPanel gapPanel = new JPanel();
        gapPanel.setBorder(BorderFactory.createTitledBorder("Gap"));
        gapPanel.setLayout(new BoxLayout(gapPanel, BoxLayout.X_AXIS));

        // Define a method to handle the button being clicked.
        java.awt.event.ActionListener prevGapAL =
            new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                selectNextGap(false);
            }
        };

        // Create a "Previous Gap" button.  Add it to the gap panel.
        JButton prevGapButton = createButton("<", prevGapAL, "selectPrevGap",
            "Select Previous Gap");
        gapPanel.add(prevGapButton);

        // Define a method to handle the button being clicked.
        java.awt.event.ActionListener nextGapAL =
            new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                selectNextGap(true);
            }
        };

        // Create a "Next Gap" button.  Add it to the gap panel.
        JButton nextGapButton = createButton(">", nextGapAL, "selectNextGap",
            "Select Next Gap");
        gapPanel.add(nextGapButton);

        gapFillPanel.add(gapPanel);

        // Set up the panel's border and layout.
        JPanel fillPanel = new JPanel();
        fillPanel.setBorder(BorderFactory.createTitledBorder("Fill"));
        fillPanel.setLayout(new BoxLayout(fillPanel, BoxLayout.X_AXIS));

        // Define a method to handle the button being clicked.
        java.awt.event.ActionListener fillParabAL =
            new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                fill(true);
            }
        };

        // Create a "Fill Parab" button.  Add it to the fill panel.
        JButton fillParabButton = createButton("P", fillParabAL, "fillParab",
            "Fill Selected Gap using Parabolic Interpolation");
        fillPanel.add(fillParabButton);

        // Define a method to handle the button being clicked.
        java.awt.event.ActionListener fillLinearAL =
            new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                fill(false);
            }
        };

        // Create a "Fill Linear" button.  Add it to the fill panel.
        JButton fillLinearButton = createButton("L", fillLinearAL, "fillLinear",
            "Fill Selected Gap using Linear Interpolation");
        fillPanel.add(fillLinearButton);

        gapFillPanel.add(fillPanel);
        gapFillPanel.setAlignmentX(Component.CENTER_ALIGNMENT);

        m_panel.add(gapFillPanel);

        m_panel.setMaximumSize(new java.awt.Dimension(400, 400));
    }

//------------------------------------------------------------------------------
/**
 *
 */
    protected JButton createButton(String text, ActionListener al,
        String actionText, String toolText) {

        JButton button = new JButton(text);
        button.addActionListener(al);
        button.setActionCommand(actionText);
        button.setToolTipText(toolText);
//        byte[] byteArray = {0};
//        button.setIcon(new ImageIcon(byteArray));
        button.setBorderPainted(true);
        button.setMargin(new Insets(0, 0, 0, 0));

        return button;
    }

//------------------------------------------------------------------------------
/**
 * Remove all points that aren't on a 15 minute interval.
 */
    protected void eliminateNon15() {

        List dsList = m_page.findChildInstancesOf(
            "gov.usgs.gr.data.DoubleDataSeries", true);

        // Create a data snap shot of the series in this file.
        DataSnapShot snapShot = new DataSnapShot();
        Iterator iter = dsList.iterator();
        while (iter.hasNext()) {
            DoubleDataSeries ds = (DoubleDataSeries)iter.next();
            snapShot.store(ds);
        }

        // Attempt to save a snapshot, so this operation can be undone.
        m_grCon.saveDataSnapShot(snapShot);

        // Create a constant to represent how close to 15 minute spacing
        // is close enough.  Use 30 seconds.
        double sec30 = 0.0003472222222222222;
        double tolerance = sec30;
        double min15 = 0.01041666666666667;
        double span = min15;

        // Find all children that are DataSeries.
        iter = dsList.iterator();
        while (iter.hasNext()) {
            // Make a new series and resample it to 15 minutes with
            // averaging.
            DoubleDataSeries series = (DoubleDataSeries)iter.next();

            // Determine the range of points that are already deleted.
            double rangeMin = Double.POSITIVE_INFINITY;
            double rangeMax = Double.NEGATIVE_INFINITY;
            List<Double> oldDeletedList = series.getDeletedList();
            int delIndex = 0;
            for (double val : oldDeletedList) {
                if ((delIndex % 2) != 0) {
                    delIndex++;
                    continue;
                }

                if (val > rangeMax)
                    rangeMax = val;
                if (val < rangeMin)
                    rangeMin = val;

                delIndex++;
            }

            // Create a list for keeping track of points to be deleted.
            ArrayList<Integer> indexesToDelete = new ArrayList();

            // Create a new deleted list.
            ArrayList deletedList = new ArrayList();

            // Get the arrays from the series.
            double[] oldArray = series.getArray();
            int numOldPoints = series.getNumPoints();
            int numOldDims = series.getNumDimensions();

            if (oldArray[0] < rangeMin)
                rangeMin = oldArray[0];
            int maxOldDelIndex = (int)Math.round(rangeMax / span);

            // Initialize the comparison index to the first value.
            int lastIndex = (int)Math.round(rangeMin / span);
//            if ((oldArray[0] % span) > tolerance)
//                lastIndex--;

            // Loop through the old array looking at each point to see if
            // it needs to be changed, deleted, or kept as is.
            double spanMinusTolerance = span - tolerance;
            for (int i = 0; i < numOldPoints; i++) {
                double oldX = oldArray[i * numOldDims];
                double modX = Math.abs(oldX) % span;

                int index = (int)Math.round(oldX / span);

                if (modX > tolerance && modX < spanMinusTolerance) {
                    indexesToDelete.add(i);
                }

                for (;lastIndex < index; lastIndex++) {
                    deletedList.add(lastIndex * span);
                    deletedList.add(0d);
                }
                lastIndex = index + 1;
            }

            // If the end of the series was deleted points, add deleted points
            // across that range.
            for (;lastIndex <= maxOldDelIndex; lastIndex++) {
                deletedList.add(lastIndex * span);
                deletedList.add(0d);
            }

            series.setDeletedList(deletedList);

            int numToDelete = indexesToDelete.size();
            if (numToDelete <= 0) {
                // Cause redraw.
                series.arrayChanged(DataSeries.DELETED_LIST_CHANGED);
                return;
            }
            else {
                String message = "" + numToDelete
                    + " data points were removed because they were not on"
                    + " a 15 minute inteval.";
                if (m_bUseDialogNotification) {
                    javax.swing.JOptionPane.showMessageDialog(null, message,
                        "Notification",
                        javax.swing.JOptionPane.INFORMATION_MESSAGE);
                }
                else {
                    System.out.println("Time Series Tool Set: " + message);
                }
            }

            // Delete all points marked for deletion.
            int numNewPoints = numOldPoints - numToDelete;
            double[] newArray = new double[numNewPoints * 2];
            int nextIndexToDelete = -1;
            int oldIndex = 0;
            for (int i = 0; i < numNewPoints; i++) {
                while(nextIndexToDelete <= oldIndex) {
                    if (nextIndexToDelete == oldIndex) oldIndex++;
                    if (indexesToDelete.size() <= 0) {
                        nextIndexToDelete = Integer.MAX_VALUE;
                    }
                    else {
                        nextIndexToDelete = indexesToDelete.get(0);
                        indexesToDelete.remove(0);
                    }
                }
                if (nextIndexToDelete > oldIndex) {
                    newArray[i * 2] = oldArray[oldIndex * 2];
                    newArray[i * 2 + 1] = oldArray[oldIndex * 2 + 1];
                }
                oldIndex++;
            }

            series.setArray(newArray);
        }
    }

//------------------------------------------------------------------------------
/**
 *
 */
    protected void selectNextGap (boolean bForward) {

        m_page.deferUpdates();

        // Get a list of all the series on the page.
        List seriesList = m_page.findChildInstancesOf(
            "gov.usgs.gr.data.DoubleDataSeries", true);
        if (seriesList.size() == 0) {
            return;
        }

        List selectedList = m_page.getSelectedSeriesList();
        if (selectedList.size() > 0) {
            DoubleDataSeries lastSelectedSeries = null;
            if (bForward) {
                lastSelectedSeries = (DoubleDataSeries)
                    selectedList.get(selectedList.size() - 1);
            }
            else {
                lastSelectedSeries = (DoubleDataSeries)selectedList.get(0);
            }

            // Cycle the order of the list until the selected series is at the
            // start.
            int index = seriesList.indexOf(lastSelectedSeries);
            for (int i = 0; i < index; i++) {
                DoubleDataSeries series = (DoubleDataSeries)seriesList.get(0);
                seriesList.remove(0);
                seriesList.add(seriesList.size(), series);
            }

            // Order the list from last to first if looking for previous gap.
            if (!bForward) {
                int iterCount = seriesList.size();
                for (int i = 1; i < iterCount; i++) {
                    Object obj = seriesList.remove(i);
                    seriesList.add(i, obj);
                }
            }
        }

        double selectionMin = Double.NEGATIVE_INFINITY;
        double selectionMax = Double.NEGATIVE_INFINITY;

        // Start at the first selected series, or first series if none are
        // selected.  Add a copy of the starting series to the end of the list.
        DoubleDataSeries series = (DoubleDataSeries)seriesList.get(0);
        seriesList.add(seriesList.size(), series);

        // Look at each series in the list.
        Iterator seriesIter = seriesList.iterator();
        while (seriesIter.hasNext()) {
            series = (DoubleDataSeries)seriesIter.next();

            if (!(seriesIter.hasNext()))
                series.deselectAll();

            // Get the array and number of dimensions for this series.
            int numDimensions = series.getNumDimensions();
            double[] array = series.getArray();
////
            // Find the last/first point in the array that is selected.
            int selectedIndex = 0;
            double selectedTime = 0;
            if (bForward) {
                int[] selectedIndexes = series.getSelectedIndexes();
                int numSelected = series.getNumSelected();
                if (numSelected > 0) {
                    selectedIndex = selectedIndexes[numSelected - 1];
                }
            }
            else {
                int[] selectedIndexes = series.getSelectedIndexes();
                int numSelected = series.getNumSelected();
                if (numSelected > 0) {
                    selectedIndex = selectedIndexes[0];
                }
                else {
                    selectedIndex = series.getCount() - 1;
                }
            }
            selectedTime = array[selectedIndex * numDimensions];

            // Get an array of sorted, deleted points.
            double[] deletedTimes = getSortedDeleted(series);

            // Determine which deleted point would follow the last selected
            // point.
            int nextDeletedIndex = 0;
            if (bForward) {
                nextDeletedIndex =
                    Arrays.binarySearch(deletedTimes, selectedTime);
                if (nextDeletedIndex < 0)
                    nextDeletedIndex = -(nextDeletedIndex + 1);
            }
            else {
                nextDeletedIndex =
                    Arrays.binarySearch(deletedTimes, selectedTime);
                if (nextDeletedIndex < 0)
                    nextDeletedIndex = -(nextDeletedIndex + 2);
            }
            // Deselect all points.
            series.deselectAll();

            // If a deleted point is found, select around it and quit searching.
            if (nextDeletedIndex >= 0 &&
                nextDeletedIndex < deletedTimes.length) {

                double nextDeletedTime = deletedTimes[nextDeletedIndex];
                int nextSelectedIndex = series.search(nextDeletedTime, 0);
                m_precedingTime =
                    array[(nextSelectedIndex) * numDimensions];

                series.selectPoint(nextSelectedIndex);
                selectionMin = array[nextSelectedIndex * numDimensions];
                if (nextSelectedIndex < (series.getCount() - 1)) {
                    series.selectPoint(nextSelectedIndex + 1);
                    selectionMax =
                        array[(nextSelectedIndex + 1) * numDimensions];
                    m_followingTime = selectionMax;
                }
                else {
                    selectionMax = array[array.length - numDimensions];
                }
                break;
            }
        }

        // Clean up remaining series
        while (seriesIter.hasNext()) {
            DoubleDataSeries tempSeries = (DoubleDataSeries) seriesIter.next();
            if (seriesIter.hasNext()) {
                tempSeries.deselectAll();
            }
        }

        // Zoom to find the selected gap.
        if (selectionMax != Double.NEGATIVE_INFINITY) {
            double selectionMiddle = (selectionMax + selectionMin) / 2;
            double halfSpan = (selectionMax - selectionMin) * 1.5;
            if (halfSpan < 0.5)
                halfSpan = 0.5;
            selectionMin = selectionMiddle - halfSpan;
            selectionMax = selectionMiddle + halfSpan;

            // Deselect all graphs.
            List graphList = m_page.findChildInstancesOf(
                "gov.usgs.gr.visual.Graph", false);
            Iterator iter = graphList.iterator();
            while (iter.hasNext()) {
                Graph graph = (Graph)iter.next();
                graph.setSelected(false);
            }

            Zoom zoom = findZoomForSeries(series, true);
            zoom.autoScale(selectionMin, selectionMax, 6);
        }

        series.update(null, null);

        m_page.processDeferredUpdates();
        m_page.updateSelectedSeriesList();
    }

//------------------------------------------------------------------------------
/**
 *
 */
    protected void fill (boolean bParabolicInterpolation) {

        List dataList = m_grCon.getPage().getSelectedSeriesList();

        DoubleDataSeries series = null;

        ArrayList addedCodes = new ArrayList();
        boolean bCodesAreChars = true;

        // Set the series to parabolic or linear interpolation.
        Iterator dataIter = dataList.iterator();
        while (dataIter.hasNext()) {
            series = (DoubleDataSeries)dataIter.next();
            series.m_bParabolicInterpolation = bParabolicInterpolation;

            // If the series stores QaQc information, set it to use an added
            // code which reflects the nature of this operation.
            if (series.getClass().getName().equals(
                "gov.ca.dwr.gr.data.QaQcTimeSeries")) {

                QaQcTimeSeries qaqcSeries = (QaQcTimeSeries)series;
                bCodesAreChars = qaqcSeries.getCodesAreChars();

                GrHashList props = qaqcSeries.getProperties();
                GrProperty p = (GrProperty) props.get("addedCode");
                addedCodes.add(p.value);

                if (bParabolicInterpolation)
                    qaqcSeries.setProperty("addedCode", "P");
                else
                    qaqcSeries.setProperty("addedCode", "L");
            }
        }

        if (series == null) {
            selectNextGap(true);
            return;
        }

        DataSnapShot snapShot = new DataSnapShot(dataList);
        m_grCon.saveDataSnapShot(snapShot);

        int oldNumPoints = series.getCount();

        m_page.deferUpdates();

        gov.usgs.gr.tools.SeriesGapFiller filler =
            new gov.usgs.gr.tools.SeriesGapFiller();
        filler.performAction(dataList);

        // Reset the original added code.
        Iterator codesIter = addedCodes.iterator();
        dataIter = dataList.iterator();
        while (codesIter.hasNext()) {
            String value = (String) codesIter.next();
            QaQcTimeSeries qaqcSeries = (QaQcTimeSeries)dataIter.next();
            qaqcSeries.setProperty("addedCode", value);
        }

        // Make sure the same surrounding points are selected after the fill.
        series.selectAll();
        series.deselectAll();
        if (m_precedingTime != Double.NEGATIVE_INFINITY) {
            int minSelectedIndex = series.search(m_precedingTime, 0);
            series.selectPoint(minSelectedIndex);
        }

        if (m_followingTime != Double.NEGATIVE_INFINITY) {
            int maxSelectedIndex = series.search(m_followingTime, 0);
            if ((series.getArray())[maxSelectedIndex
                * series.getNumDimensions()] != m_followingTime &&
                maxSelectedIndex < (series.getCount() - 1)) {

                maxSelectedIndex++;
            }

            series.selectPoint(maxSelectedIndex);
        }

        // If points were filled, show them.  Otherwise, select the next gap.
        if (series.getCount() == oldNumPoints) {
            selectNextGap(true);
            return;
        }
        else {
            series.update(null, null);
            m_page.processDeferredUpdates();
            m_page.updateSelectedSeriesList();
        }
    }

//------------------------------------------------------------------------------
/**
 */
    protected double[] getSortedDeleted(DoubleDataSeries series) {

        // Get the list of deleted points and put the times in a sorted
        // array.
        int numDimensions = series.getNumDimensions();
        List<Double> deletedList = series.getDeletedList();
        double[] deletedTimes =
            new double[deletedList.size() / numDimensions];
        int readIndex = 0;
        int writeIndex = 0;
        for (double time : deletedList) {
            if ((readIndex % numDimensions) != 0) {
                readIndex++;
                continue;
            }

            deletedTimes[writeIndex] = time;

            readIndex++;
            writeIndex++;
        }

        Arrays.sort(deletedTimes);

        return deletedTimes;
    }

//------------------------------------------------------------------------------
/**
 * Accessor method.
 * @return The editor panel created by this class at initialization time.
 */
    public JPanel getPanel() {

        return m_panel;
    }

//------------------------------------------------------------------------------
/**
 * Tells this interface what the current controller and page are.
 * @param grCon   The GrController to use.
 * @param page    The PageOfGraphs to use.
 */
    public void setControllerAndPage(GrController grCon, PageOfGraphs page) {

        m_grCon = grCon;
        m_page = page;
    }

//------------------------------------------------------------------------------
/**
 * Finds the PageofGraphs that contains m_zoom.
 */
    public void findParentPage() {

        boolean bPageFound = false;
        List observers = getObservers();
        Iterator obIter = observers.iterator();
        while (obIter.hasNext()) {
            // Look for a page parent of the graph.
            GrObject gro = (GrObject)obIter.next();
            if (gro instanceof PageOfGraphs) {
                m_page = (PageOfGraphs)gro;
                bPageFound = true;
                break;
            }
        }
    }

//------------------------------------------------------------------------------
/**
 * Finds the PageofGraphs that contains m_zoom.
 */
    public Zoom findZoomForSeries(DataSeries series,
        boolean bSelectParentGraph) {

        List seriesObservers = series.getObservers();
        Iterator seriesObIter = seriesObservers.iterator();
        while (seriesObIter.hasNext()) {
            // Look for a page parent of the graph.
            GrObject gro1 = (GrObject)seriesObIter.next();
            if (gro1 instanceof Curve) {
                Curve curve = (Curve)gro1;
                List curveObservers = curve.getObservers();
                Iterator curveObIter = curveObservers.iterator();
                while (curveObIter.hasNext()) {
                    // Look for a page parent of the graph.
                    GrObject gro2 = (GrObject) curveObIter.next();
                    if (gro2 instanceof Graph) {
                        Graph graph = (Graph)gro2;
                        if (bSelectParentGraph)
                            graph.setSelected(true);
                        return (graph.getXZoom());
                    }
                }
            }
        }

        return null;
    }

//------------------------------------------------------------------------------
/**
 * Overrides default update() in GrObject.  Called by an observed GrObject's
 * notifyObservers() method to handle changes to the observed object.
 * May use the notifyObservers() method to propagate the event handling up to
 * this object's observers.
 * @param gro   The observed object that changed.
 * @param arg   Additional information that the updated object may choose to
 *              send.
 */
    public void update(GrObject gro, Object arg) {
        m_bHasChanged = true;
        notifyObservers();
    }

//------------------------------------------------------------------------------
/**
 * Create a string representation of this object.
 * @return  The string that describes this object.
 */
    public String toString() {

        String s = "Time Series Tool Set";

        return s;
     }
}

