/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2006-06-06
 * Date started:    2003-04-29
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.sfhydro.si3d;

import gov.usgs.gr.data.*;
import gov.usgs.gr.*;
import gov.usgs.gr.visual.*;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import javax.swing.border.*;
import java.text.*;
import java.util.*;

//==============================================================================
/**
 *
 */
public class BathymetryEditorPanel
    extends gov.usgs.gr.visual.gui.DataEditorPanel {

	BorderLayout mainLayout = new BorderLayout();
	JPanel mainLeftPanel = new JPanel();
	BorderLayout mainLeftLayout = new BorderLayout();
	JPanel mainLeftUpperPanel = new JPanel();
	BorderLayout mainLeftUpperLayout = new BorderLayout();
	JPanel operationPanel = new JPanel();
	BorderLayout operationLayout = new BorderLayout();

	JPanel operationSimplePanel = new JPanel();
	GridLayout operationSimpleLayout = new GridLayout(2, 3);
    ButtonGroup operationGroup = new ButtonGroup();
    JRadioButton smoothRadio = new JRadioButton("Smooth");
    JRadioButton landRadio = new JRadioButton("LAND");
    JRadioButton setRadio = new JRadioButton("Set Equal");
    JRadioButton addRadio = new JRadioButton("Add");
    JRadioButton subRadio = new JRadioButton("Subtract");
    JRadioButton mulRadio = new JRadioButton("Multiply");
    JRadioButton divRadio = new JRadioButton("Divide");

	JPanel operandPanel = new JPanel();
	BorderLayout operandLayout = new BorderLayout();
	JPanel operandConstantPanel = new JPanel();
	BorderLayout operandConstantLayout = new BorderLayout();
	JTextField constantValueTextField = new JTextField();

//------------------------------------------------------------------------------
	public BathymetryEditorPanel() {

		try  {
			bathEdInit();
		}
		catch (Exception ex) {
			ex.printStackTrace();
		}
	}

//------------------------------------------------------------------------------
    protected void jbInit() throws Exception {
    }

//------------------------------------------------------------------------------
    protected void bathEdInit() throws Exception {

        // Set up button listeners.
        okButton.setText("OK");
        okButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                okButton_actionPerformed(e);
            }
        });
        applyButton.setText("Apply");
        applyButton.addActionListener(new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                applyButton_actionPerformed(e);
            }
        });
        undoButton.setForeground(Color.blue);
        undoButton.setText("Undo");
        undoButton.addActionListener(new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                undoButton_actionPerformed(e);
            }
        });
        redoButton.setForeground(Color.blue);
        redoButton.setText("Redo");
        redoButton.addActionListener(new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                redoButton_actionPerformed(e);
            }
        });
        cancelButton.setForeground(Color.red);
        cancelButton.setText("Close");
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                cancelButton_actionPerformed(e);
            }
        });
        setRadio.addChangeListener(new javax.swing.event.ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                setRadio_stateChanged(e);
            }
        });
        addRadio.addChangeListener(new javax.swing.event.ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                addRadio_stateChanged(e);
            }
        });
        subRadio.addChangeListener(new javax.swing.event.ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                subRadio_stateChanged(e);
            }
        });
        mulRadio.addChangeListener(new javax.swing.event.ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                mulRadio_stateChanged(e);
            }
        });
        divRadio.addChangeListener(new javax.swing.event.ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                divRadio_stateChanged(e);
            }
        });
        smoothRadio.addChangeListener(new javax.swing.event.ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                smoothRadio_stateChanged(e);
            }
        });
        landRadio.addChangeListener(new javax.swing.event.ChangeListener() {

            public void stateChanged(ChangeEvent e) {
                landRadio_stateChanged(e);
            }
        });
        this.setLayout(mainLayout);
        this.add(mainLeftPanel, BorderLayout.WEST);
        mainLeftPanel.setLayout(mainLeftLayout);
        mainLeftPanel.add(mainLeftUpperPanel, BorderLayout.NORTH);
        mainLeftUpperPanel.setLayout(mainLeftUpperLayout);
        mainLeftUpperPanel.add(operationPanel, BorderLayout.NORTH);
        operationPanel.setBorder(
            BorderFactory.createTitledBorder("Operation"));
        operationPanel.setLayout(operationLayout);
        operationPanel.add(operationSimplePanel, BorderLayout.NORTH);

        // Operation simple panel...
        operationSimplePanel.setBorder(BorderFactory.createEtchedBorder());
        operationSimplePanel.setLayout(operationSimpleLayout);
        operationSimplePanel.add(addRadio);
        operationSimplePanel.add(mulRadio);
        operationSimplePanel.add(setRadio);
        operationSimplePanel.add(subRadio);
        operationSimplePanel.add(divRadio);
        operationSimplePanel.add(landRadio);
//        operationSimplePanel.add(landRadio);
//        operationGroup.add(smoothRadio);
        operationGroup.add(landRadio);
        operationGroup.add(setRadio);
        operationGroup.add(addRadio);
        operationGroup.add(subRadio);
        operationGroup.add(mulRadio);
        operationGroup.add(divRadio);
        setRadio.setSelected(true);

        // Operand constant panel...
        mainLeftUpperPanel.add(operandPanel, BorderLayout.SOUTH);
        operandPanel.setBorder(BorderFactory.createTitledBorder("Operand"));
        operandPanel.setLayout(operandLayout);
        operandPanel.add(operandConstantPanel, BorderLayout.NORTH);
        operandConstantPanel.setLayout(operandConstantLayout);
        operandConstantPanel.add(constantValueTextField, BorderLayout.CENTER);
        constantValueTextField.setText("0          ");

        // OK, Cancel buttons...
        this.add(okCancelPanel, BorderLayout.SOUTH);
        okCancelPanel.add(applyButton);
        okCancelPanel.add(undoButton);
        okCancelPanel.add(redoButton);
        okCancelPanel.add(cancelButton);
    }

//------------------------------------------------------------------------------
/**
 * Read the values from the page and put them in the dialog.
 * <p><b> Pre Conditions </b><br>
 *      The page has been set up.
 */
    public void readFromPage(GrController grCon) {

        m_grCon = grCon;
        m_page = m_grCon.getPage();
        synchronized (m_page) {
            m_selectedSeriesList = m_page.getSelectedSeriesList();
            if (m_selectedSeriesList.size() == 0)
                constantValueTextField.setText("Nothing selected");
        }
    }

//------------------------------------------------------------------------------
/**
 * Write values from the dialog back to the page.
 * <p><b> Pre Conditions </b><br>
 *      The page has been set up.
 */
    protected void writeToPage() {

        if (m_selectedSeriesList.size() == 0) return;

        // Show an hourglass until the job is complete.
        ((Component)getParentFrame()).setCursor(new Cursor(Cursor.WAIT_CURSOR));

        // Get a reference to the second curve if one is selected.
        Graph graph2;
        Curve curve2;
        DoubleDataSeries series2 = null;

        synchronized (m_page) {
            // Get the new value
            double value = Double.parseDouble(constantValueTextField.getText());

            // Determine if changing the x or y coordinates.
            int dimension = 0;

            DataSnapShot snapShot = new DataSnapShot();
            Iterator<DataSeries> iter =
                m_selectedSeriesList.iterator();
            while (iter.hasNext()) {
                // Set the series' data manipulation options.
                DoubleDataSeries series = (DoubleDataSeries)iter.next();
                synchronized (series) {
                    // Save the current state for later undo.
                    snapShot.store(series);

                    if (!landRadio.isSelected()) {
                        // Determine which type of modification is being made.
                        int modificationType = -1;
                        if (setRadio.isSelected()) modificationType = 0;
                        if (addRadio.isSelected()) modificationType = 1;
                        if (subRadio.isSelected()) modificationType = 2;
                        if (mulRadio.isSelected()) modificationType = 3;
                        if (divRadio.isSelected()) modificationType = 4;

                        series.modifySelectedPoints(modificationType,
                            dimension, value);
                    }
                    else {
                        series.modifySelectedPoints(0, dimension, -90);
                    }
                }
            }

            // Store the list of old series for undo.
            m_grCon.saveDataSnapShot(snapShot);
        }
    }

//------------------------------------------------------------------------------
    protected void smoothRadio_stateChanged(ChangeEvent e) {

        enableComponents();
    }

//------------------------------------------------------------------------------
    protected void landRadio_stateChanged(ChangeEvent e) {

        enableComponents();
    }

//------------------------------------------------------------------------------
    protected void enableComponents() {

        if (m_enabling) return;
        else m_enabling = true;
        try {
            // Gray out input parameter options if the "None" operation is
            // selected.
            if (smoothRadio.isSelected()) {
                constantValueTextField.setEnabled(false);
            }
            else {
                // Dimension options should always be
                // available unless the operation is "None".

                // Enable the appropriate constant or curve options.
                constantValueTextField.setEnabled(true);
            }
        }
        catch (Exception ex) { m_enabling = false; }
        catch (Error er) { m_enabling = false; }

        m_enabling = false;
    }
}

