/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2005-11-04
 * Date started:    2005-05-13
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.sfhydro.si3d;

import gov.usgs.gr.*;
import gov.usgs.gr.data.*;
import java.io.*;
import java.util.*;
import javax.swing.*;

//==============================================================================
/**
 * Calculates the path that a particle would take through a time-variable
 * velocity field.
 */

public class ParticleProbabilityCalculator extends GrObject implements
	java.awt.event.ActionListener {

    /**
     * Holds any error descriptions that should be shown to the user in
     * the properties dialog.
     */
    protected String m_errorString = null;
    /** The name of the data set's file. */
    protected File m_file = new File("/");

    protected int[][] m_rgbColumns = new int[4][];
    protected int[] m_columnAssignments = null;

    //==========================================================================
    /**
     * An object that can be sorted and tell how an interleaved array can be
     * sorted.
     */
    public class SortableObject  {
        int index;
        double[] array;

        //----------------------------------------------------------------------
        /**
         * Compares an object to this one for sorting.
         * @param o The object to compare to this one.
         */
        public int compareTo(SortableObject o) {
            if (Math.round(array[index]) > Math.round(array[o.index]))
                return 1;

            if (Math.round(array[index]) < Math.round(array[o.index]))
                return -1;

            if (Math.round(array[index + 1]) > Math.round(array[o.index + 1]))
                return 1;

            if (Math.round(array[index + 1]) < Math.round(array[o.index + 1]))
                return -1;

            if (Math.round(array[index + 2]) > Math.round(array[o.index + 2]))
                return 1;

            if (Math.round(array[index + 2]) < Math.round(array[o.index + 2]))
                return -1;

            return 0;

        }
    }

//------------------------------------------------------------------------------
/**
 * Creates an empty object.
 */
    public ParticleProbabilityCalculator () {

    }

//------------------------------------------------------------------------------
/**
 * Adds an object as a child, but only if it is an instance of ParticleTracker.
 * @param gro   The object to add.
 * @return  Whether the object was successfully added.
 */
    public boolean add(GrObject gro) {

        // If gro is an object effigy, get the eventual subject and try
        // adding that.
        while (gro instanceof gov.usgs.gr.ObjectEffigy)
        	gro = ((ObjectEffigy)gro).getSubject();

        // Add allowable objects...
        if (gro instanceof ParticleTracker) {

            super.add(gro);
            return true;
        }


        return false;
    }

//------------------------------------------------------------------------------
/**
 * Provide the list of properties with the latest values.
 */
    public GrHashList getProperties() {

    	GrHashList props = super.getProperties();

        props.put("file", new GrProperty(
        	m_file.getAbsolutePath(), "File"));

        String str = "";
        for (int i = 0; i < 4; i++) {
            if (i > 0)
                str += ",";

            for (int j = 0; j < m_rgbColumns[i].length; j++) {
                if (j > 0)
                    str += "+";

                str += m_rgbColumns[i][j];
            }
        }
        props.put("columnAssignments", new GrProperty(
            str, "Column Assignments"));

        return props;
    }

//------------------------------------------------------------------------------
/**
 * Provide the list of properties with the latest values.
 */
    public void setProperties(HashMap props) {

		super.setProperties(props);

    	GrProperty p;

        Iterator iter = props.keySet().iterator();
        while (iter.hasNext()) {
        	String key = (String)iter.next();
            Object obj = props.get(key);
            if (obj instanceof GrProperty) {
				p = (GrProperty)obj;

            	if (key.equals("file")) {
            		m_file = new File((String)p.value);
                }
            }
            if (obj instanceof String) {
				String value = (String)obj;
                setProperty(key, value);
            }
        }
    }

//------------------------------------------------------------------------------
/**
 * Set a single property.
 */
    public void setProperty(String key, String value) {

    	super.setProperty(key, value);

        if (key.equals("file")) {
            m_file = new File(value);
        }
        if (key.equals("columnAssignments")) {
            int maxColumn = 0;
            String[] columnTokens = AsciiIO.parseLine(value, ",");
            for (int i = 0; i < 4; i++) {
                if (i > columnTokens.length) {
                    m_rgbColumns[i] = new int[0];
                    continue;
                }

                m_rgbColumns[i] =
                    AsciiIO.parseLineAsInts(columnTokens[i], "+");

                for (int j = 0; j < m_rgbColumns.length; j++) {
                    if (m_rgbColumns[i][j] > maxColumn)
                        maxColumn = m_rgbColumns[i][j];
                }
            }

            m_columnAssignments = new int[maxColumn + 1];
            for (int i = 0; i < 4; i++) {
                for (int j = 0; j < m_rgbColumns[i].length; j++) {
                    m_columnAssignments[m_rgbColumns[i][j]] = i;
                }
            }
        }

    }

//------------------------------------------------------------------------------
/**
 */
    public void createAnimation () {
/*
        int[][] totals = new int[numWetPoints][];
        for (int i = 0; i < numWetPoints; i++) {
            totals[i] = new int[4];
        }
        int[] count = new int[numWetPoints];
        float[] outArray = new float[numWetPoints * 3];

        RafDataSeries outRafSeries = new gov.usgs.gr.data.RafDataSeries(3);
        String newFilename = m_file.getAbsolutePath();
        outRafSeries.setProperty("file", newFilename);
        outRafSeries.openRAF("rwd");

        int lastTime = -1;
        // while not end of series
            // Read time, index, and finish line into variables
            if (time == lastTime || lastTime == -1) {
                // totals[wetIndex][m_columnAssignments[finishLineIndex]]++
                // count[wetIndex]++
	        }
            else {
                int outIndex = 0;
                for (int i = 0; i < numWetPoints; i++) {
                    for (int j = 0; j < 3; j++) {
                        if (count[wetIndex] > 0) {
                            outArray[outIndex] = (float) ((double)
                                totals[wetIndex][j] / (double) count[wetIndex]);
                        }
                        else {
                            outArray[outIndex] = 0;
                        }
                        outIndex++;
                        if (outIndex > outArray.length) {
                            outRafSeries.writeValuesFrom(outArray, 0, 0,
                                outIndex);
                            outIndex = 0;
                        }
                    }
                }

                if (outIndex > 0)
                    outRafSeries.writeValuesFrom(outArray, 0, 0, outIndex);
            }
            lastTime = time;

        outRafSeries.closeRAF();
        // Create XML file with times and particle graphics.
 */
    }

//------------------------------------------------------------------------------
/**
 * Provide the list of menu items that execute commands on this object.
 * @return  A submenu of commands.
 */
    public javax.swing.JMenuItem getMenuCommands() {

        javax.swing.JMenu subMenu = new javax.swing.JMenu(
        	"Particle Tracker");
        javax.swing.JMenuItem item =
            new javax.swing.JMenuItem("Create Animation");
        item.addActionListener(this);
        subMenu.add(item);

    	return subMenu;
    }

//------------------------------------------------------------------------------
/**
 * Handles action events, such as a recalculate command.
 * @param e The action event.
 */
    public void actionPerformed(java.awt.event.ActionEvent e) {

        // Check the source to see if it is a command.
        javax.swing.JMenuItem source =
        	(javax.swing.JMenuItem)(e.getSource());
        String srcText = source.getText();

        // Calculate command.
        if (srcText.equals("Create Animation")) {
            // Note the start time.
            GregorianCalendar gc1 = new GregorianCalendar();
            long time1 = gc1.getTimeInMillis();

        	createAnimation();

            // Write out the elapsed time if it was greater than 10 seconds.
            GregorianCalendar gc2 = new GregorianCalendar();
            long time2 = gc2.getTimeInMillis();
            if ((time2 - time1) > 10000) {
                java.text.SimpleDateFormat formatter = new
                    java.text.SimpleDateFormat(
                    "MMM'.' dd',' yyyy 'at' hh:mm:ss");
                System.out.println("PTM Start: "
                    + formatter.format(gc1.getTime()));
                System.out.println("PTM End:   "
                    + formatter.format(gc2.getTime()));
            }
        }

    }

//------------------------------------------------------------------------------
/**
 * Create a string representation of this object.
 * If an error was detected during an operation, the class's error string
 * is appended.
 * @return  The string that describes this object.
 */
    public String toString() {

    	String s = "Particle Probability Calculator";

        if (m_errorString != null)
            s += m_errorString;

        return s;
 	}
}

