/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2006-06-06
 * Date started:    2003-06-02
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.sfhydro.si3d;

import gov.usgs.gr.*;
import gov.usgs.gr.data.*;
import gov.usgs.sfhydro.si3d.*;
import java.awt.Point;
import java.util.*;

//==============================================================================
/**
 *
 */
public class RawBathymetryCalculator extends GrObject
    implements java.awt.event.ActionListener {

    protected double m_xMin = 0;
    protected double m_xMax = -1;
    protected double m_yMin = 0;
    protected double m_yMax = -1;
    protected double m_gridWidth = 0;
    protected double m_dryLand = -90;
    protected double m_lowestLand = 0;
    protected double m_landSubstitute = 0;
    protected double m_highestWater = 0;
    protected double m_highestWetDepth = 0;
    protected double m_extendedHighestWater = Double.POSITIVE_INFINITY;
    protected double m_singleCellDryPercentage = 0.8;

    protected double m_oldXspacing = 0;
    protected int m_oldNumI = 0;
    protected int m_oldNumJ = 0;
    protected int m_newNumI = 0;
    protected int m_newNumJ = 0;
    protected int m_numOldGridsPerNew = 0;
    protected double[] m_oldArray = null;
    protected IntDataSeries m_oldDryCellSeries = null;

//------------------------------------------------------------------------------
/**
 * Constructor for creating a blank data series.
 */
    public RawBathymetryCalculator() {

        setDefaults("gov.usgs.sfhydro.si3d.RawBathymetryCalculator");
    }

//------------------------------------------------------------------------------
/**
 * Add a child object.  Accepts DoubleDataSeries, TimeMarker.  Only holds one DS
 * at a time.
 */
    public boolean add(GrObject gro) {

        // If gro is an object effigy, get the eventual subject and try
        // adding that.
        while (gro instanceof gov.usgs.gr.ObjectEffigy)
        	gro = ((ObjectEffigy)gro).getSubject();

    	if (m_children.contains(gro)) return false;

        if (gro instanceof RawBathymetry) {
            super.add(gro);

            return true;
        }
        if (gro instanceof GrObjectContainer) {
            super.add(gro);

            return true;
        }

        return false;
    }

//------------------------------------------------------------------------------
/**
 * Provide the list of properties with the latest values.
 */
    public GrHashList getProperties() {

        GrHashList props = super.getProperties();

        props.put("xMin", new GrProperty(m_xMin, "X Min"));
        props.put("xMax", new GrProperty(m_xMax, "X Max"));
        props.put("yMin", new GrProperty(m_yMin, "Y Min"));
        props.put("yMax", new GrProperty(m_yMax, "Y Max"));
        props.put("gridWidth", new GrProperty(m_gridWidth, "Grid Width"));
        props.put("dryLand", new GrProperty(m_dryLand, "Dry Land"));
        props.put("lowestLand", new GrProperty(m_lowestLand, "Lowest Land"));
        props.put("landSubstitute", new GrProperty(
            m_landSubstitute, "Land Substitute"));
        props.put("highestWater", new GrProperty(
            m_highestWater, "Highest Water"));
        props.put("highestWetDepth", new GrProperty(
            m_highestWetDepth, "Highest Wet Depth"));
        props.put("extendedHighestWater", new GrProperty(
            m_extendedHighestWater, "Extended Highest Water"));
        props.put("singleCellDryPercentage", new GrProperty(
            m_singleCellDryPercentage, "Single Cell Dry Percentage"));

        return props;
    }

//------------------------------------------------------------------------------
/**
 * Provide the list of properties with the latest values.
 */
    public void setProperties(HashMap props) {

        super.setProperties(props);

        GrProperty p;

        Iterator iter = props.keySet().iterator();
        while (iter.hasNext()) {
            String key = (String)iter.next();
            Object obj = props.get(key);
            if (obj instanceof GrProperty) {
                p = (GrProperty)obj;

                if (p.value instanceof String) {
                    setProperty(key, (String)p.value);
                    continue;
                }

                if (key.equals("xMin"))
                    m_xMin = (Double)p.value;
                if (key.equals("xMax"))
                    m_xMax = (Double)p.value;
                if (key.equals("yMin"))
                    m_yMin = (Double)p.value;
                if (key.equals("yMax"))
                    m_yMax = (Double)p.value;
                if (key.equals("gridWidth"))
                    m_gridWidth = (Double)p.value;
                if (key.equals("dryLand"))
                    m_dryLand = (Double)p.value;
                if (key.equals("lowestLand"))
                    m_lowestLand = (Double)p.value;
                if (key.equals("landSubstitute"))
                    m_landSubstitute = (Double)p.value;
                if (key.equals("highestWater"))
                    m_highestWater = (Double)p.value;
                if (key.equals("highestWetDepth"))
                    m_highestWetDepth = (Double)p.value;
                if (key.equals("extendedHighestWater"))
                    m_extendedHighestWater = (Double)p.value;
                if (key.equals("singleCellDryPercentage"))
                    m_singleCellDryPercentage = (Double)p.value;
            }
            if (obj instanceof String) {
                String value = (String)obj;
                setProperty(key, value);
            }
        }
    }

//------------------------------------------------------------------------------
/**
 * Set a single property.
 */
    public void setProperty(String key, String value) {

        super.setProperty(key, value);

        if (key.equals("xMin"))
            m_xMin = Double.parseDouble(value);
        if (key.equals("xMax"))
            m_xMax = Double.parseDouble(value);
        if (key.equals("yMin"))
            m_yMin = Double.parseDouble(value);
        if (key.equals("yMax"))
            m_yMax = Double.parseDouble(value);
        if (key.equals("gridWidth"))
            m_gridWidth = Double.parseDouble(value);
        if (key.equals("dryLand"))
            m_dryLand = Double.parseDouble(value);
        if (key.equals("lowestLand"))
            m_lowestLand = Double.parseDouble(value);
        if (key.equals("landSubstitute"))
            m_landSubstitute = Double.parseDouble(value);
        if (key.equals("highestWater"))
            m_highestWater = Double.parseDouble(value);
        if (key.equals("highestWetDepth"))
            m_highestWetDepth = Double.parseDouble(value);
        if (key.equals("extendedHighestWater"))
            m_extendedHighestWater = Double.parseDouble(value);
        if (key.equals("singleCellDryPercentage"))
            m_singleCellDryPercentage = Double.parseDouble(value);
    }

//------------------------------------------------------------------------------
/**
 *
 */
    public boolean readRawBathVariables() {

        RawBathymetry rawBath = null;

        Iterator iter = m_children.iterator();
        while (iter.hasNext()) {
            GrObject gro = (GrObject)iter.next();
            if (gro instanceof RawBathymetry) {
                rawBath = (RawBathymetry)(gro);
                break;
            }
        }

        // Check that the member variables have been initialized.
        if (rawBath == null) {
            return false;
        }

        DoubleDataSeries oldSeries = rawBath.getBathymetrySeries();
        m_oldArray = oldSeries.getArray();
        m_oldDryCellSeries = rawBath.getDryCellSeries();

        // Get grid spacing information from the Bathymetry object.
        GrHashList bathProps = rawBath.getProperties();
        GrProperty p = (GrProperty)bathProps.get("xSpacing");
        m_oldXspacing = (Double)p.value;
        p = (GrProperty)bathProps.get("numI");
        m_oldNumI = (Integer)p.value;
        p = (GrProperty)bathProps.get("numJ");
        m_oldNumJ = (Integer)p.value;

        return true;
    }

//------------------------------------------------------------------------------
/**
 *
 */
    public void createGrid() {

        if (!readRawBathVariables())
            return;

        if ((m_xMax - m_xMin) < 0) {
            m_xMin = 0;
            m_xMax = (m_oldNumI - 1) * m_oldXspacing;
        }
        if ((m_yMax - m_yMin) < 0) {
            m_yMin = 0;
            m_yMax = (m_oldNumJ - 1) * m_oldXspacing;
        }

        m_newNumI = (int)Math.round((m_xMax - m_xMin) / m_gridWidth);
        m_newNumJ = (int)Math.round((m_yMax - m_yMin) / m_gridWidth);

        double[] newArray = null;

        if (m_gridWidth < m_oldXspacing) {
            if (m_oldXspacing % m_gridWidth > 0.01 || m_gridWidth <= 0) {

                System.err.println("Error generating new grid.");
                System.err.println("Old grid spacing must be evenly divisible"
                    + " into new spacing");
                return;
            }

            newArray = createFinerGridArray();
        }
        else {
            m_numOldGridsPerNew = (int)Math.round(m_gridWidth / m_oldXspacing);

            if (((double)m_numOldGridsPerNew -
                (m_gridWidth / m_oldXspacing)) > 0.01) {

                System.err.println("Error generating new grid.");
                System.err.println("Old grid spacing must be evenly divisible"
                    + " into new spacing");
                return;
            }

            newArray = createCourserGridArray();
        }

        DoubleDataSeries cornerDepthSeries =
            new DoubleDataSeries(newArray, 1);

        RawBathymetry newRawBath = new RawBathymetry();
        newRawBath.setProperty("xSpacing", "" + m_gridWidth);
        newRawBath.setProperty("ySpacing", "" + m_gridWidth);
        newRawBath.setProperty("zSpacing", "2");
        newRawBath.setProperty("numI", "" + m_newNumI);
        newRawBath.setProperty("numJ", "" + m_newNumJ);
        newRawBath.add(cornerDepthSeries);

        GrObjectContainer groc = new GrObjectContainer();
        groc.setProperty("title", "Generated " + Math.round(m_gridWidth));
        groc.add(newRawBath);
        add(groc);

        if (m_numOldGridsPerNew > 1) {
            GrObject dryCellGro = generateDryCells(newRawBath);
            newRawBath.add(dryCellGro);
            recalculateDepths(newRawBath, true, true);
            double[] min = new double[1];
            double[] max = new double[1];
            min[0] = m_lowestLand;
            max[0] = m_highestWetDepth;
            cornerDepthSeries.deselectAll();
            cornerDepthSeries.selectArea(min, max, true);
            cornerDepthSeries.modifySelectedPoints(0, 0, m_highestWetDepth);
            cornerDepthSeries.deselectAll();
            GrObject barrierGro = generateThinWallBarriers(newRawBath);
            newRawBath.add(barrierGro);
        }
    }

//------------------------------------------------------------------------------
/**
 *
 */
    public double[] createCourserGridArray() {

        double[] newArray = new double[m_newNumI * m_newNumJ];

        int xOffset = (int)Math.round(m_xMin / m_gridWidth);
        int yOffset = (int)Math.round(m_yMin / m_gridWidth);

        int centerRadius = m_numOldGridsPerNew / 2;

        for (int i = 0; i < m_newNumI; i++) {
            for (int j = 0; j < m_newNumJ; j++) {
                double volumeTotal = 0;
                double areaTotal = 0;
                int oldCenterI = i * m_numOldGridsPerNew;
                int oldCenterJ = j * m_numOldGridsPerNew;

                // Calculate the volume of the inner area;
                for (int oldI = (oldCenterI - centerRadius);
                    oldI < (oldCenterI + centerRadius); oldI++) {

                    for (int oldJ = (oldCenterJ - centerRadius);
                        oldJ < (oldCenterJ + centerRadius); oldJ++) {

                        double[] corners = getCornerDepths(oldI, oldJ,
                            m_oldArray, m_oldNumI, m_oldNumJ);

                        if (corners == null)
                            continue;

                        double avgDepth = (corners[0] + corners[1] + corners[2]
                            + corners[3]) / 4;
                        volumeTotal += avgDepth;
                        areaTotal++;
                    }
                }

                // Calculate the volume of the extended four-cell area;
                double extendedVolume = 0;
                double extendedArea = 0;
                for (int oldI = (oldCenterI - m_numOldGridsPerNew);
                    oldI < (oldCenterI + m_numOldGridsPerNew); oldI++) {

                    for (int oldJ = (oldCenterJ - m_numOldGridsPerNew);
                        oldJ < (oldCenterJ + m_numOldGridsPerNew); oldJ++) {

                        double[] corners = getCornerDepths(oldI, oldJ,
                            m_oldArray, m_oldNumI, m_oldNumJ);

                        if (corners == null)
                            continue;

                        double avgDepth = (corners[0] + corners[1] + corners[2]
                            + corners[3]) / 4;
                        extendedVolume += avgDepth;
                        extendedArea++;
                    }
                }

                // For odd number of grids, add the volume of the outer rim
                // that is 1/2 the width of the old grid.
                if ((m_numOldGridsPerNew % 2) != 0) {
                    double edgeArea = 0.5;
                    double cornerArea = 0.25;

                    // Add the bottom edge.
                    int oldJ = oldCenterJ - centerRadius - 1;
                    for (int oldI = (oldCenterI - centerRadius);
                        oldI < (oldCenterI + centerRadius); oldI++) {

                        double[] corners = getCornerDepths(oldI, oldJ,
                            m_oldArray, m_oldNumI, m_oldNumJ);

                        if (corners == null)
                            continue;

                        double avgDepth = ((corners[0] + corners[1]) * 3
                            + corners[2] + corners[3]) / 8;
                        volumeTotal += (avgDepth * edgeArea);
                        areaTotal += edgeArea;
                    }

                    // Add the top edge.
                    oldJ = oldCenterJ + centerRadius;
                    for (int oldI = (oldCenterI - centerRadius);
                        oldI < (oldCenterI + centerRadius); oldI++) {

                        double[] corners = getCornerDepths(oldI, oldJ,
                            m_oldArray, m_oldNumI, m_oldNumJ);

                        if (corners == null)
                            continue;

                        double avgDepth = ((corners[0] + corners[1]) * 3
                            + corners[2] + corners[3]) / 8;
                        volumeTotal += (avgDepth * edgeArea);
                        areaTotal += edgeArea;
                    }

                    // Add the left edge.
                    int oldI = oldCenterI - centerRadius - 1;
                    for (oldJ = (oldCenterJ - centerRadius);
                        oldJ < (oldCenterJ + centerRadius); oldJ++) {

                        double[] corners = getCornerDepths(oldI, oldJ,
                            m_oldArray, m_oldNumI, m_oldNumJ);

                        if (corners == null)
                            continue;

                        double avgDepth = ((corners[0] + corners[3]) * 3
                            + corners[1] + corners[2]) / 8;
                        volumeTotal += (avgDepth * edgeArea);
                        areaTotal += edgeArea;
                    }

                    // Add the right edge.
                    oldI = oldCenterI + centerRadius;
                    for (oldJ = (oldCenterJ - centerRadius);
                        oldJ < (oldCenterJ + centerRadius); oldJ++) {

                        double[] corners = getCornerDepths(oldI, oldJ,
                            m_oldArray, m_oldNumI, m_oldNumJ);

                        if (corners == null)
                            continue;

                        double avgDepth = ((corners[0] + corners[3]) * 3
                            + corners[1] + corners[2]) / 8;
                        volumeTotal += (avgDepth * edgeArea);
                        areaTotal += edgeArea;
                    }

                    // Add the bottom left corner.
                    oldI = oldCenterI - centerRadius - 1;
                    oldJ = oldCenterJ - centerRadius - 1;
                    double[] corners = getCornerDepths(oldI, oldJ,
                        m_oldArray, m_oldNumI, m_oldNumJ);

                    if (corners != null) {
                        double avgDepth = (corners[2] * 9 + corners[0]
                            + (corners[1] + corners[3]) * 3) / 16;
                        volumeTotal += (avgDepth * cornerArea);
                        areaTotal += cornerArea;
                    }

                    // Add the bottom right corner.
                    oldI = oldCenterI + centerRadius;
                    oldJ = oldCenterJ - centerRadius - 1;
                    corners = getCornerDepths(oldI, oldJ, m_oldArray,
                        m_oldNumI, m_oldNumJ);

                    if (corners != null) {
                        double avgDepth = (corners[3] * 9 + corners[1]
                            + (corners[0] + corners[2]) * 3) / 16;
                        volumeTotal += (avgDepth * cornerArea);
                        areaTotal += cornerArea;
                    }

                    // Add the top right corner.
                    oldI = oldCenterI + centerRadius;
                    oldJ = oldCenterJ + centerRadius;
                    corners = getCornerDepths(oldI, oldJ, m_oldArray,
                        m_oldNumI, m_oldNumJ);

                    if (corners != null) {
                        double avgDepth = (corners[0] * 9 + corners[2]
                            + (corners[1] + corners[3]) * 3) / 16;
                        volumeTotal += (avgDepth * cornerArea);
                        areaTotal += cornerArea;
                    }

                    // Add the top left corner.
                    oldI = oldCenterI - centerRadius - 1;
                    oldJ = oldCenterJ + centerRadius;
                    corners = getCornerDepths(oldI, oldJ, m_oldArray,
                        m_oldNumI, m_oldNumJ);

                    if (corners != null) {
                        double avgDepth = (corners[1] * 9 + corners[3]
                            + (corners[1] + corners[2]) * 3) / 16;
                        volumeTotal += (avgDepth * cornerArea);
                        areaTotal += cornerArea;
                    }
                }

                double newDepth = m_dryLand;
                if (areaTotal > 0)
                    newDepth = volumeTotal / areaTotal;
                if (newDepth < m_highestWater) {
                    double extendedDepth = m_dryLand;
                    if (extendedArea > 0)
                        extendedDepth = extendedVolume / extendedArea;
                    if (extendedDepth < m_extendedHighestWater) {
                        newDepth = m_dryLand;
                    }
                    else {
                        newDepth = m_highestWetDepth;
                    }
                }
                else {
                    if (newDepth < m_highestWetDepth) {
                        newDepth = m_highestWetDepth;
                    }
                }
                newArray[j * m_newNumI + i] = newDepth;
            }
        }

        return newArray;
    }

//------------------------------------------------------------------------------
/**
 *
 */
    public double[] createFinerGridArray() {

        double[] newArray = new double[m_newNumI * m_newNumJ];

        int numDivs = (int)Math.round(m_oldXspacing / m_gridWidth);

        for (int oldI = 0; oldI <= m_oldNumI; oldI++) {
            for (int oldJ = 0; oldJ <= m_oldNumJ; oldJ++) {
                double[] corners = getCornerDepths(oldI, oldJ,
                    m_oldArray, m_oldNumI, m_oldNumJ);

                if (corners == null)
                    continue;

                newArray[oldJ * numDivs * m_newNumI + oldI * numDivs] =
                    corners[0];

                if (oldI == m_oldNumI || oldJ == m_oldNumJ)
                    continue;

                // Render each sub area as a separate square.
                for (int m = 0; m < numDivs; m++) {
                    for (int n = 0; n < numDivs; n++) {
                        // Calculate the Z value of the intermediate point.
                        double posX0 = (double)m / (double)numDivs;
                        double posY0 = (double)n / (double)numDivs;
                        double posX1 = (double)(m + 1) / (double)numDivs;
                        double posY1 = (double)(n + 1) / (double)numDivs;

                        double bottomZ = corners[0] * (1 - posX0)
                            + corners[1] * posX0;
                        double topZ = corners[3] * (1 - posX0)
                            + corners[2] * posX0;
                        double avgZ = bottomZ * (1 - posY0) + topZ * posY0;

                        if (avgZ < m_highestWater)
                            avgZ = m_dryLand;

                        newArray[(oldJ * numDivs + n) * m_newNumI
                            + (oldI * numDivs + m)] = avgZ;
                    }
                }
            }
        }

        return newArray;
    }

//------------------------------------------------------------------------------
/**
 * Look for raw bathymetry objects within object containers within this object.
 * For each one found, recalculate its depths.
 */
    public void recalculateDepths(boolean bIncludeAllWet, boolean bIncludeDry) {

        if (!readRawBathVariables())
            return;

        Iterator iter = m_children.iterator();
        while (iter.hasNext()) {
            GrObject gro = (GrObject)iter.next();
            if (gro instanceof GrObjectContainer) {
                GrObjectContainer groc = (GrObjectContainer)(gro);
                Enumeration grocEnum =groc.children();
                RawBathymetry newRawBath = null;
                while (grocEnum.hasMoreElements()) {
                    GrObject grocChild = (GrObject)grocEnum.nextElement();
                    if (grocChild instanceof RawBathymetry) {
                        newRawBath = (RawBathymetry)grocChild;
                    }
                }

                // Get grid spacing information from the Bathymetry object.
                GrHashList bathProps = newRawBath.getProperties();
                GrProperty p = (GrProperty)bathProps.get("xSpacing");
                m_gridWidth = (Double)p.value;
                p = (GrProperty)bathProps.get("numI");
                m_newNumI = (Integer)p.value;
                p = (GrProperty)bathProps.get("numJ");
                m_newNumJ = (Integer)p.value;

                m_numOldGridsPerNew = (int)
                    Math.round(m_gridWidth / m_oldXspacing);
                if (((double)m_numOldGridsPerNew -
                    (m_gridWidth / m_oldXspacing)) > 0.01) {

                    // Old grid spacing must be evenly divisible into
                    // new spacing.
                    continue;
                }

                recalculateDepths(newRawBath, bIncludeAllWet, bIncludeDry);
            }
        }
    }

//------------------------------------------------------------------------------
/**
 *
 */
    public void recalculateDepths(RawBathymetry newRawBath,
        boolean bIncludeAllWet, boolean bIncludeDry) {

        DoubleDataSeries newSeries = newRawBath.getBathymetrySeries();
        double[] newArray = newSeries.getArray();
        IntDataSeries newDryCellSeries = newRawBath.getDryCellSeries();
        int numSelected = newSeries.getNumSelected();
        int[] tempArray = newSeries.getSelectedIndexes();
        int[] selectedIndexes = new int[numSelected];
        for (int i = 0; i < numSelected; i++) {
            selectedIndexes[i] = tempArray[i];
        }
        Arrays.sort(selectedIndexes);
        int selectedOffset = 0;

        if ((m_xMax - m_xMin) < 0) {
            m_xMin = 0;
            m_xMax = (m_oldNumI - 1) * m_oldXspacing;
        }
        if ((m_yMax - m_yMin) < 0) {
            m_yMin = 0;
            m_yMax = (m_oldNumJ - 1) * m_oldXspacing;
        }

        int xOffset = (int)Math.round(m_xMin / m_gridWidth);
        int yOffset = (int)Math.round(m_yMin / m_gridWidth);

        int centerRadius = m_numOldGridsPerNew / 2;

        for (int j = 0; j < m_newNumJ; j++) {
            for (int i = 0; i < m_newNumI; i++) {
                int newArrayOffset = j * m_newNumI + i;
                if (numSelected > 0) {
                    if (numSelected <= selectedOffset) {
                        continue;
                    }
                    if (newArrayOffset == selectedIndexes[selectedOffset]) {
//                        System.out.println("Recalculating depth for "
//                            + "selected point at (" + i + ", " + j + ")");

                        selectedOffset++;
                    }
                    else {
                        continue;
                    }
                }

                double volumeTotal = 0;
                double areaTotal = 0;
                int oldCenterI = i * m_numOldGridsPerNew;
                int oldCenterJ = j * m_numOldGridsPerNew;

                double newDepth = newArray[newArrayOffset];

                // Only change the value to land if all values or values that
                // are all-wet are included.
                if (bIncludeAllWet) {
                    if (newDepth < m_lowestLand) {
                        newDepth = m_dryLand;
                        newArray[j * m_newNumI + i] = newDepth;
                        continue;
                    }
                }

                boolean bDry = true;
                boolean bAllWet = true;

                // If the new top-right cell is not dry and the old cells
                // within it calculate to wet, add it to the total area and
                // volume.
                if (isWet(i, j, newArray, m_newNumI, m_newNumJ,
                    newDryCellSeries)) {

//System.out.println("oldCenterI = " + oldCenterI + ", oldCenterJ = "
//    + oldCenterJ + ", centerRadius = " + centerRadius);
                    // Calculate the volume of the inner area;
                    for (int oldI = oldCenterI;
                        oldI < (oldCenterI + centerRadius); oldI++) {

                        for (int oldJ = oldCenterJ;
                            oldJ < (oldCenterJ + centerRadius); oldJ++) {

//System.out.println("(" + oldI + ", " + oldJ + ")");
                            double[] corners = getCornerDepths(oldI, oldJ,
                                m_oldArray, m_oldNumI, m_oldNumJ);

                            if (corners == null)
                                continue;

                            double avgDepth = (corners[0] + corners[1]
                                + corners[2] + corners[3]) / 4;
                            volumeTotal += avgDepth;
                            areaTotal++;
//System.out.println("avgDepth = " + avgDepth
//    + ", m_landSubstitute = " + m_landSubstitute);
                            if (avgDepth > m_landSubstitute)
                                bDry = false;
                            else
                                bAllWet = false;
                        }
                    }

                    // For odd number of grids, add the volume of the outer rim
                    // that is 1/2 the width of the old grid.
                    if ((m_numOldGridsPerNew % 2) != 0) {
                        double edgeArea = 0.5;
                        double cornerArea = 0.25;

                        // Add the top edge.
                        int oldJ = oldCenterJ + centerRadius;
                        for (int oldI = oldCenterI;
                            oldI < (oldCenterI + centerRadius); oldI++) {

                            double[] corners = getCornerDepths(oldI, oldJ,
                                m_oldArray, m_oldNumI, m_oldNumJ);

                            if (corners == null)
                                continue;

                            double avgDepth = ((corners[0] + corners[1]) * 3
                                + corners[2] + corners[3]) / 8;
                            volumeTotal += (avgDepth * edgeArea);
                            areaTotal += edgeArea;
                            if (avgDepth > m_landSubstitute)
                                bDry = false;
                            else
                                bAllWet = false;
                        }

                        // Add the right edge.
                        int oldI = oldCenterI + centerRadius;
                        for (oldJ = oldCenterJ;
                            oldJ < (oldCenterJ + centerRadius); oldJ++) {

                            double[] corners = getCornerDepths(oldI, oldJ,
                                m_oldArray, m_oldNumI, m_oldNumJ);

                            if (corners == null)
                                continue;

                            double avgDepth = ((corners[0] + corners[3]) * 3
                                + corners[1] + corners[2]) / 8;
                            volumeTotal += (avgDepth * edgeArea);
                            areaTotal += edgeArea;
                            if (avgDepth > m_landSubstitute)
                                bDry = false;
                            else
                                bAllWet = false;

                        }

                        // Add the top right corner.
                        oldI = oldCenterI + centerRadius;
                        oldJ = oldCenterJ + centerRadius;
                        double[] corners = getCornerDepths(oldI, oldJ,
                            m_oldArray, m_oldNumI, m_oldNumJ);

                        if (corners != null) {
                            double avgDepth = (corners[0] * 9 + corners[2]
                                + (corners[1] + corners[3]) * 3) / 16;
                            volumeTotal += (avgDepth * cornerArea);
                            areaTotal += cornerArea;
                            if (avgDepth > m_landSubstitute)
                                bDry = false;
                            else
                                bAllWet = false;
                        }
                    }
                }
                else {
                    bAllWet = false;
                }

                // If the new top-left cell is not dry and the old cells
                // within it calculate to wet, add it to the total area and
                // volume.
                if (isWet(i - 1, j, newArray, m_newNumI, m_newNumJ,
                    newDryCellSeries)) {

                    // Calculate the volume of the inner area;
                    for (int oldI = (oldCenterI - centerRadius);
                        oldI < oldCenterI; oldI++) {

                        for (int oldJ = oldCenterJ;
                            oldJ < (oldCenterJ + centerRadius); oldJ++) {

                            double[] corners = getCornerDepths(oldI, oldJ,
                                m_oldArray, m_oldNumI, m_oldNumJ);

                            if (corners == null)
                                continue;

                            double avgDepth = (corners[0] + corners[1]
                                + corners[2] + corners[3]) / 4;
                            volumeTotal += avgDepth;
                            areaTotal++;
                            if (avgDepth > m_landSubstitute)
                                bDry = false;
                            else
                                bAllWet = false;
                        }
                    }

                    // For odd number of grids, add the volume of the outer rim
                    // that is 1/2 the width of the old grid.
                    if ((m_numOldGridsPerNew % 2) != 0) {
                        double edgeArea = 0.5;
                        double cornerArea = 0.25;

                        // Add the top edge.
                        int oldJ = oldCenterJ + centerRadius;
                        for (int oldI = (oldCenterI - centerRadius);
                            oldI < oldCenterI; oldI++) {

                            double[] corners = getCornerDepths(oldI, oldJ,
                                m_oldArray, m_oldNumI, m_oldNumJ);

                            if (corners == null)
                                continue;

                            double avgDepth = ((corners[0] + corners[1]) * 3
                                + corners[2] + corners[3]) / 8;
                            volumeTotal += (avgDepth * edgeArea);
                            areaTotal += edgeArea;
                            if (avgDepth > m_landSubstitute)
                                bDry = false;
                            else
                                bAllWet = false;
                        }

                        // Add the left edge.
                        int oldI = oldCenterI - centerRadius - 1;
                        for (oldJ = oldCenterJ;
                            oldJ < (oldCenterJ + centerRadius); oldJ++) {

                            double[] corners = getCornerDepths(oldI, oldJ,
                                m_oldArray, m_oldNumI, m_oldNumJ);

                            if (corners == null)
                                continue;

                            double avgDepth = ((corners[0] + corners[3]) * 3
                                + corners[1] + corners[2]) / 8;
                            volumeTotal += (avgDepth * edgeArea);
                            areaTotal += edgeArea;
                            if (avgDepth > m_landSubstitute)
                                bDry = false;
                            else
                                bAllWet = false;
                        }

                        // Add the top left corner.
                        oldI = oldCenterI - centerRadius - 1;
                        oldJ = oldCenterJ + centerRadius;
                        double[] corners = getCornerDepths(oldI, oldJ,
                            m_oldArray, m_oldNumI, m_oldNumJ);

                        if (corners != null) {
                            double avgDepth = (corners[1] * 9 + corners[3]
                                + (corners[1] + corners[2]) * 3) / 16;
                            volumeTotal += (avgDepth * cornerArea);
                            areaTotal += cornerArea;
                            if (avgDepth > m_landSubstitute)
                                bDry = false;
                            else
                                bAllWet = false;
                        }
                    }
                }
                else {
                    bAllWet = false;
                }

                // If the new bottom-left cell is not dry and the old cells
                // within it calculate to wet, add it to the total area and
                // volume.
                if (isWet(i - 1, j - 1, newArray, m_newNumI, m_newNumJ,
                    newDryCellSeries)) {

                    // Calculate the volume of the inner area;
                    for (int oldI = (oldCenterI - centerRadius);
                        oldI < oldCenterI; oldI++) {

                        for (int oldJ = (oldCenterJ - centerRadius);
                            oldJ < oldCenterJ; oldJ++) {

                            double[] corners = getCornerDepths(oldI, oldJ,
                                m_oldArray, m_oldNumI, m_oldNumJ);

                            if (corners == null)
                                continue;

                            double avgDepth = (corners[0] + corners[1]
                                + corners[2] + corners[3]) / 4;
                            volumeTotal += avgDepth;
                            areaTotal++;
                            if (avgDepth > m_landSubstitute)
                                bDry = false;
                            else
                                bAllWet = false;
                        }
                    }

                    // For odd number of grids, add the volume of the outer rim
                    // that is 1/2 the width of the old grid.
                    if ((m_numOldGridsPerNew % 2) != 0) {
                        double edgeArea = 0.5;
                        double cornerArea = 0.25;

                        // Add the bottom edge.
                        int oldJ = oldCenterJ - centerRadius - 1;
                        for (int oldI = (oldCenterI - centerRadius);
                            oldI < oldCenterI; oldI++) {

                            double[] corners = getCornerDepths(oldI, oldJ,
                                m_oldArray, m_oldNumI, m_oldNumJ);

                            if (corners == null)
                                continue;

                            double avgDepth = ((corners[0] + corners[1]) * 3
                                + corners[2] + corners[3]) / 8;
                            volumeTotal += (avgDepth * edgeArea);
                            areaTotal += edgeArea;
                            if (avgDepth > m_landSubstitute)
                                bDry = false;
                            else
                                bAllWet = false;
                        }

                        // Add the left edge.
                        int oldI = oldCenterI - centerRadius - 1;
                        for (oldJ = (oldCenterJ - centerRadius);
                            oldJ < oldCenterJ; oldJ++) {

                            double[] corners = getCornerDepths(oldI, oldJ,
                                m_oldArray, m_oldNumI, m_oldNumJ);

                            if (corners == null)
                                continue;

                            double avgDepth = ((corners[0] + corners[3]) * 3
                                + corners[1] + corners[2]) / 8;
                            volumeTotal += (avgDepth * edgeArea);
                            areaTotal += edgeArea;
                            if (avgDepth > m_landSubstitute)
                                bDry = false;
                            else
                                bAllWet = false;
                        }

                        // Add the bottom left corner.
                        oldI = oldCenterI - centerRadius - 1;
                        oldJ = oldCenterJ - centerRadius - 1;
                        double[] corners = getCornerDepths(oldI, oldJ,
                            m_oldArray, m_oldNumI, m_oldNumJ);

                        if (corners != null) {
                            double avgDepth = (corners[2] * 9 + corners[0]
                                + (corners[1] + corners[3]) * 3) / 16;
                            volumeTotal += (avgDepth * cornerArea);
                            areaTotal += cornerArea;
                            if (avgDepth > m_landSubstitute)
                                bDry = false;
                            else
                                bAllWet = false;
                        }
                    }
                }
                else {
                    bAllWet = false;
                }

                // If the new bottom-right cell is not dry and the old cells
                // within it calculate to wet, add it to the total area and
                // volume.
                if (isWet(i, j - 1, newArray, m_newNumI, m_newNumJ,
                    newDryCellSeries)) {

                    // Calculate the volume of the inner area;
                    for (int oldI = oldCenterI;
                        oldI < (oldCenterI + centerRadius); oldI++) {

                        for (int oldJ = (oldCenterJ - centerRadius);
                            oldJ < oldCenterJ; oldJ++) {

                            double[] corners = getCornerDepths(oldI, oldJ,
                                m_oldArray, m_oldNumI, m_oldNumJ);

                            if (corners == null)
                                continue;

                            double avgDepth = (corners[0] + corners[1]
                                + corners[2] + corners[3]) / 4;
                            volumeTotal += avgDepth;
                            areaTotal++;
                            if (avgDepth > m_landSubstitute)
                                bDry = false;
                            else
                                bAllWet = false;
                        }
                    }

                    // For odd number of grids, add the volume of the outer rim
                    // that is 1/2 the width of the old grid.
                    if ((m_numOldGridsPerNew % 2) != 0) {
                        double edgeArea = 0.5;
                        double cornerArea = 0.25;

                        // Add the bottom edge.
                        int oldJ = oldCenterJ - centerRadius - 1;
                        for (int oldI = oldCenterI;
                            oldI < (oldCenterI + centerRadius); oldI++) {

                            double[] corners = getCornerDepths(oldI, oldJ,
                                m_oldArray, m_oldNumI, m_oldNumJ);

                            if (corners == null)
                                continue;

                            double avgDepth = ((corners[0] + corners[1]) * 3
                                + corners[2] + corners[3]) / 8;
                            volumeTotal += (avgDepth * edgeArea);
                            areaTotal += edgeArea;
                            if (avgDepth > m_landSubstitute)
                                bDry = false;
                            else
                                bAllWet = false;
                        }

                        // Add the right edge.
                        int oldI = oldCenterI + centerRadius;
                        for (oldJ = (oldCenterJ - centerRadius);
                            oldJ < oldCenterJ; oldJ++) {

                            double[] corners = getCornerDepths(oldI, oldJ,
                                m_oldArray, m_oldNumI, m_oldNumJ);

                            if (corners == null)
                                continue;

                            double avgDepth = ((corners[0] + corners[3]) * 3
                                + corners[1] + corners[2]) / 8;
                            volumeTotal += (avgDepth * edgeArea);
                            areaTotal += edgeArea;
                            if (avgDepth > m_landSubstitute)
                                bDry = false;
                            else
                                bAllWet = false;
                        }

                        // Add the bottom right corner.
                        oldI = oldCenterI + centerRadius;
                        oldJ = oldCenterJ - centerRadius - 1;
                        double[] corners = getCornerDepths(oldI, oldJ,
                            m_oldArray, m_oldNumI, m_oldNumJ);

                        if (corners != null) {
                            double avgDepth = (corners[3] * 9 + corners[1]
                                + (corners[0] + corners[2]) * 3) / 16;
                            volumeTotal += (avgDepth * cornerArea);
                            areaTotal += cornerArea;
                            if (avgDepth > m_landSubstitute)
                                bDry = false;
                            else
                                bAllWet = false;
                        }
                    }
                }
                else {
                    bAllWet = false;
                }

                newDepth = m_dryLand;
                if (!bDry) {
                    if (areaTotal > 0)
                        newDepth = volumeTotal / areaTotal;
                    if (newDepth > m_highestWater) {
                        if (newDepth < m_highestWetDepth) {
                            newDepth = m_highestWetDepth;
                        }
                    }
                    else {
                        newDepth = m_dryLand;
                    }
                }
//System.out.println("incallwet = " + bIncludeAllWet + ", incdry = " + bIncludeDry
//    + ", allwet = " + bAllWet + ", dry = " + bDry + ", newDepth = " + newDepth);
                // Only change the value if both all wet and dry are to be
                // included, or if it matches the values.
                if ((bIncludeAllWet && bAllWet) || (bIncludeDry && bDry)
                    || (!bIncludeAllWet && !bIncludeDry && !bAllWet && !bDry)
                    || (bIncludeAllWet && bIncludeDry)) {

                    newArray[j * m_newNumI + i] = newDepth;
                }
            }
        }

        newSeries.arrayChanged();
        checkDryCellSeries(newRawBath);
    }

//------------------------------------------------------------------------------
/**
 *
 */
    public double[] getCornerDepths(int i, int j, double[] depthGrid,
        int numI, int numJ) {

        double[] corners = null;

        // Check that the cell coordinates are not off the grid.
        if (i < 0 || i >= (numI - 1) ||
            j < 0 || j >= (numJ - 1)) {

            return corners;
        }

        // Look up the depths for the corners from the depth grid array.
        corners = new double[4];
        corners[0] = depthGrid[j * numI + i];
        corners[1] = depthGrid[j * numI + i + 1];
        corners[2] = depthGrid[(j + 1) * numI + i + 1];
        corners[3] = depthGrid[(j + 1) * numI + i];

        // Clamp land points to the lowest land value.
        for (int n = 0; n < 4; n++) {
            if (corners[n] < m_lowestLand)
                corners[n] = m_landSubstitute;
        }

        return corners;
    }

//------------------------------------------------------------------------------
/**
 *
 */
    public GrObject generateThinWallBarriers(RawBathymetry newRawBath) {

        DoubleDataSeries newSeries = newRawBath.getBathymetrySeries();
        double[] newArray = newSeries.getArray();
        IntDataSeries newDryCellSeries = newRawBath.getDryCellSeries();

        ArrayList barrierList = new ArrayList();
        int[] wetIndexes = newRawBath.getWetIndexSeries().getArray();
        for (int n = 0; n < wetIndexes.length; n++) {
            int i = wetIndexes[n] / 10000;
            int j = wetIndexes[n] - (i * 10000);

            if (i > 0 && i < (m_newNumI - 1)) {
                // If the east neighbor cell is wet, check for a TWB
                // between them.

                if (isWet(i - 1, j, newArray, m_newNumI, m_newNumJ,
                        newDryCellSeries)) {

                    boolean bEastMustBeClear = false;
                    boolean bWestMustBeClear = false;
                    if (i < (m_newNumI - 2)) {
                        // If the east neighbor's, east neighbor is wet, then
                        // there must be a clear path past the min twb area
                        // of that cell.
                        if (isWet(i + 1, j, newArray, m_newNumI, m_newNumJ,
                                    newDryCellSeries)) {

                            bEastMustBeClear = true;
                        }
                    }
                    if (i > 1) {
                        // If the east neighbor's, east neighbor is wet, then
                        // there must be a clear path past the min twb area
                        // of that cell.
                        if (isWet(i - 2, j, newArray, m_newNumI, m_newNumJ,
                                newDryCellSeries)) {

                            bWestMustBeClear = true;
                        }
                    }
                    if (getNsTwbRecommendation(i, j, bEastMustBeClear,
                        bWestMustBeClear)) {

                        int comboIJ = i * 10000 + j;
                        barrierList.add(comboIJ);
                        comboIJ = i * 10000 + (j + 1);
                        barrierList.add(comboIJ);
                    }
                }

                if (j < (m_newNumJ - 2)) {
                    // If the east neighbor cell is wet, check for a TWB
                    // between them.
                    if (isWet(i, j + 1, newArray, m_newNumI, m_newNumJ,
                            newDryCellSeries)) {

                        boolean bNorthMustBeClear = false;
                        boolean bSouthMustBeClear = false;
                        if (j < (m_newNumJ - 3)) {
                            // If the north neighbor's, north neighbor is wet,
                            // then there must be a clear path past the min twb
                            // area of that cell.
                            if (isWet(i, j + 2, newArray, m_newNumI, m_newNumJ,
                                    newDryCellSeries)) {

                                bNorthMustBeClear = true;
                            }
                        }
                        if (j > 0) {
                            // If the north neighbor's, north neighbor is wet,
                            // then there must be a clear path past the min twb
                            // area of that cell.
                            if (isWet(i, j - 1, newArray, m_newNumI, m_newNumJ,
                                    newDryCellSeries)) {

                                bSouthMustBeClear = true;
                            }
                        }
                        if (getEwTwbRecommendation(i, j + 1, bNorthMustBeClear,
                            bSouthMustBeClear)) {

                            int comboIJ = i * 10000 + (j + 1);
                            barrierList.add(comboIJ);
                            comboIJ = (i + 1) * 10000 + (j + 1);
                            barrierList.add(comboIJ);
                        }
                    }
                }
            }
        }

        IntDataSeries series = new IntDataSeries(2, "Thin Wall Barriers", 2);
        series.setArray(barrierList);

        System.out.println("Generated " + (barrierList.size() / 2)
            + " barrier segments.");

        return series;
    }

//------------------------------------------------------------------------------
/**
 *
 */
    public boolean getNsTwbRecommendation(int i, int j,
        boolean bEastMustBeClear, boolean bWestMustBeClear) {

        ArrayList toBeCheckedList = new ArrayList();
        ArrayList deadEndList = new ArrayList();

        int max = m_numOldGridsPerNew / 2;
        int min = -max;
        max--;
        if ((m_numOldGridsPerNew % 2) != 0) {
            min--;
        }

        int oldCenterI = i * m_numOldGridsPerNew;
        int oldCenterJ = j * m_numOldGridsPerNew;
        min += oldCenterI;
        max += oldCenterI;

        for (int n = oldCenterJ; n < (oldCenterJ + m_numOldGridsPerNew); n++) {
            if (isWet(min, n, m_oldArray, m_oldNumI, m_oldNumJ,
                m_oldDryCellSeries)) {

                toBeCheckedList.add(new Point(min, n));
            }
        }

        // Add two additional points along the edges and closer to the end.
        toBeCheckedList.add(new Point(min + 1, oldCenterJ));
        toBeCheckedList.add(
            new Point(min + 1, (oldCenterJ + m_numOldGridsPerNew - 1)));

        if (!bWestMustBeClear && toBeCheckedList.size() == 0) {
            return false;
        }

        int furthestI = min;

        while (toBeCheckedList.size() > 0) {
            Point currentOldSquare =
                (Point)toBeCheckedList.get(toBeCheckedList.size() - 1);

            int currentI = currentOldSquare.x;
            int currentJ = currentOldSquare.y;

            if (currentI > furthestI)
                furthestI = currentI;

            if (!bEastMustBeClear && currentI >= max) {
                return false;
            }

            deadEndList.add(currentOldSquare);
            toBeCheckedList.remove(toBeCheckedList.size() - 1);

            // Check if west neighbor is wet and isn't in either list.
            Point neighbor = new Point(currentI - 1, currentJ);
            if (neighbor.x > min && isWet(neighbor.x, neighbor.y, m_oldArray,
                m_oldNumI, m_oldNumJ, m_oldDryCellSeries)
                && !isPointInList(neighbor, toBeCheckedList)
                && !isPointInList(neighbor, deadEndList)) {

                toBeCheckedList.add(neighbor);
            }
            // Check if north neighbor is wet and isn't in either list.
            neighbor = new Point(currentI, currentJ + 1);
            if (neighbor.y < (oldCenterJ + m_numOldGridsPerNew) &&
                isWet(neighbor.x, neighbor.y, m_oldArray, m_oldNumI, m_oldNumJ,
                m_oldDryCellSeries)
                && !isPointInList(neighbor, toBeCheckedList)
                && !isPointInList(neighbor, deadEndList)) {

                toBeCheckedList.add(neighbor);
            }
            // Check if south neighbor is wet and isn't in either list.
            neighbor = new Point(currentI, currentJ - 1);
            if (neighbor.y >= oldCenterJ &&
                isWet(neighbor.x, neighbor.y, m_oldArray, m_oldNumI, m_oldNumJ,
                m_oldDryCellSeries)
                && !isPointInList(neighbor, toBeCheckedList)
                && !isPointInList(neighbor, deadEndList)) {

                toBeCheckedList.add(neighbor);
            }
            // Check if east neighbor is wet and isn't in either list.
            neighbor = new Point(currentI + 1, currentJ);
            if (isWet(neighbor.x, neighbor.y, m_oldArray, m_oldNumI, m_oldNumJ,
                m_oldDryCellSeries)
                && !isPointInList(neighbor, toBeCheckedList)
                && !isPointInList(neighbor, deadEndList)) {

                if (currentI >= max) {
                    return false;
                }

                if (neighbor.x <= max) {
                    toBeCheckedList.add(neighbor);
                }
            }
            else {
                // East must be clear, max I was reached but old east was not
                // clear.  If all the old cells at i = max + 1 are dry, don't
                // add the barrier.  If a barrier needs to be added, it will be
                // added one cell east.
                if (currentI >= max) {
                    boolean bEastBlocked = true;
                    for (int jVal = oldCenterJ;
                        jVal < (oldCenterJ + m_numOldGridsPerNew); jVal++) {

                        if (isWet(max + 1, jVal, m_oldArray, m_oldNumI,
                            m_oldNumJ, m_oldDryCellSeries)) {

                            bEastBlocked = false;
                        }
                    }
                    if (bEastBlocked)
                        return false;
                }
            }
        }

        if (bEastMustBeClear)
            return true;

        for (int iVal = furthestI + 1; iVal <= max; iVal++) {
            for (int jVal = oldCenterJ;
                jVal < (oldCenterJ + m_numOldGridsPerNew); jVal++) {

                if (isWet(iVal, jVal, m_oldArray, m_oldNumI, m_oldNumJ,
                    m_oldDryCellSeries)) {

                    return true;
                }
            }
        }

        return false;
    }

//------------------------------------------------------------------------------
/**
 *
 */
    public boolean getEwTwbRecommendation(int i, int j,
        boolean bNorthMustBeClear, boolean bSouthMustBeClear) {

        ArrayList toBeCheckedList = new ArrayList();
        ArrayList deadEndList = new ArrayList();

        int max = m_numOldGridsPerNew / 2;
        int min = -max;
        max--;
        if ((m_numOldGridsPerNew % 2) != 0) {
            min--;
        }

        int oldCenterI = i * m_numOldGridsPerNew;
        int oldCenterJ = j * m_numOldGridsPerNew;
        min += oldCenterJ;
        max += oldCenterJ;

        for (int n = oldCenterI; n < (oldCenterI + m_numOldGridsPerNew); n++) {
            if (isWet(n, min, m_oldArray, m_oldNumI, m_oldNumJ,
                m_oldDryCellSeries)) {

                toBeCheckedList.add(new Point(n, min));
            }
        }

        // Add two additional points along the edges and closer to the end.
        toBeCheckedList.add(new Point(oldCenterI, min + 1));
        toBeCheckedList.add(
            new Point((oldCenterI + m_numOldGridsPerNew - 1), min + 1));

        if (!bSouthMustBeClear && toBeCheckedList.size() == 0) {
            return false;
        }

        int furthestJ = min;

        while (toBeCheckedList.size() > 0) {
            Point currentOldSquare =
                (Point)toBeCheckedList.get(toBeCheckedList.size() - 1);

            int currentI = currentOldSquare.x;
            int currentJ = currentOldSquare.y;

            if (currentJ > furthestJ)
                furthestJ = currentJ;

            if (!bNorthMustBeClear && currentJ >= max) {
                return false;
            }

            deadEndList.add(currentOldSquare);
            toBeCheckedList.remove(toBeCheckedList.size() - 1);

            // Check if south neighbor is wet and isn't in either list.
            Point neighbor = new Point(currentI, currentJ - 1);
            if (neighbor.y > min &&
                isWet(neighbor.x, neighbor.y, m_oldArray, m_oldNumI, m_oldNumJ,
                m_oldDryCellSeries)
                && !isPointInList(neighbor, toBeCheckedList)
                && !isPointInList(neighbor, deadEndList)) {

                toBeCheckedList.add(neighbor);
            }
            // Check if west neighbor is wet and isn't in either list.
            neighbor = new Point(currentI - 1, currentJ);
            if (neighbor.x >= oldCenterI &&
                isWet(neighbor.x, neighbor.y, m_oldArray, m_oldNumI, m_oldNumJ,
                m_oldDryCellSeries)
                && !isPointInList(neighbor, toBeCheckedList)
                && !isPointInList(neighbor, deadEndList)) {

                toBeCheckedList.add(neighbor);
            }
            // Check if east neighbor is wet and isn't in either list.
            neighbor = new Point(currentI + 1, currentJ);
            if (neighbor.x < (oldCenterI + m_numOldGridsPerNew) &&
                isWet(neighbor.x, neighbor.y, m_oldArray, m_oldNumI, m_oldNumJ,
                m_oldDryCellSeries)
                && !isPointInList(neighbor, toBeCheckedList)
                && !isPointInList(neighbor, deadEndList)) {

                if (neighbor.x <= max) {
                    toBeCheckedList.add(neighbor);
                }
            }
            // Check if north neighbor is wet and isn't in either list.
            neighbor = new Point(currentI, currentJ + 1);
            if (isWet(neighbor.x, neighbor.y, m_oldArray, m_oldNumI, m_oldNumJ,
                m_oldDryCellSeries)
                && !isPointInList(neighbor, toBeCheckedList)
                && !isPointInList(neighbor, deadEndList)) {

                if (currentJ >= max) {
                    return false;
                }

                toBeCheckedList.add(neighbor);
            }
            else {
                // North must be clear, max J was reached but old north was not
                // clear.  If all the old cells at j = max + 1 are dry, don't
                // add the barrier because it is up against land.  If a barrier
                // needs to be added, it will be added one cell north.
                if (currentJ >= max) {
                    boolean bNorthBlocked = true;
                    for (int iVal = oldCenterI;
                        iVal < (oldCenterI + m_numOldGridsPerNew); iVal++) {

                        if (isWet(iVal, max + 1, m_oldArray, m_oldNumI,
                            m_oldNumJ, m_oldDryCellSeries)) {

                            bNorthBlocked = false;
                        }
                    }
                    if (bNorthBlocked)
                        return false;
                }
            }
        }

        if (bNorthMustBeClear)
            return true;

        for (int iVal = oldCenterI;
            iVal < (oldCenterI + m_numOldGridsPerNew); iVal++) {
            for (int jVal = furthestJ + 1; jVal <= max; jVal++) {

                if (isWet(iVal, jVal, m_oldArray, m_oldNumI, m_oldNumJ,
                    m_oldDryCellSeries)) {

                    return true;
                }
            }
        }

        return false;

    }

//------------------------------------------------------------------------------
/**
 *
 */
    public boolean isPointInList(Point point, List list) {

        Iterator iter = list.iterator();
        while (iter.hasNext()) {
            Point listPoint = (Point)iter.next();
            if (listPoint.x == point.x && listPoint.y == point.y)
                return true;
        }

        return false;
    }

//------------------------------------------------------------------------------
/**
 *
 */
    public boolean isWet(int i, int j, double[] depthArray,
        int numI, int numJ, IntDataSeries dryCellSeries) {

        // Check that the cell coordinates are not off the grid.
        if (i < 0 || i >= (numI - 1) ||
            j < 0 || j >= (numJ - 1)) {

            return false;
        }

        int[] dryCellArray = null;
        if (dryCellSeries != null) {
            dryCellArray = dryCellSeries.getArray();
        }

        double[] corners = getCornerDepths(i, j, depthArray, numI, numJ);

        if (corners == null)
            return false;

        boolean bAllWet = (corners[0] > m_highestWater &&
            corners[1] > m_highestWater && corners[2] > m_highestWater &&
            corners[3] > m_highestWater);

        if (bAllWet) {
            if (dryCellSeries != null) {
                int comboIJ = i * 10000 + j;
                int index = dryCellSeries.search(comboIJ, 0);
                if (index >= 0 && dryCellArray[index] == comboIJ)
                    return false;
            }

            return true;
        }
        else {
            return false;
        }
    }

//------------------------------------------------------------------------------
/**
 * Look for raw bathymetry objects within object containers within this object.
 * For each one found that is alone in its container, generate thin wall
 * barriers and add them to the container.
 */
    public void generateThinWallBarriers() {

        if (!readRawBathVariables())
            return;

        Iterator iter = m_children.iterator();
        while (iter.hasNext()) {
            GrObject gro = (GrObject)iter.next();
            if (gro instanceof GrObjectContainer) {
                GrObjectContainer groc = (GrObjectContainer)(gro);
                Enumeration grocEnum =groc.children();
                RawBathymetry newRawBath = null;
                boolean bRawBathIsAlone = true;
                while (grocEnum.hasMoreElements()) {
                    GrObject grocChild = (GrObject)grocEnum.nextElement();
                    if (grocChild instanceof RawBathymetry) {
                        if (newRawBath == null)
                            newRawBath = (RawBathymetry)grocChild;
                        else
                            newRawBath = null;
                    }
                    else {
                        bRawBathIsAlone = false;
                    }
                }

                if (!bRawBathIsAlone)
                    continue;

                // Get grid spacing information from the Bathymetry object.
                GrHashList bathProps = newRawBath.getProperties();
                GrProperty p = (GrProperty)bathProps.get("xSpacing");
                m_gridWidth = (Double)p.value;
                p = (GrProperty)bathProps.get("numI");
                m_newNumI = (Integer)p.value;
                p = (GrProperty)bathProps.get("numJ");
                m_newNumJ = (Integer)p.value;

                m_numOldGridsPerNew = (int)
                    Math.round(m_gridWidth / m_oldXspacing);
                if (((double)m_numOldGridsPerNew -
                    (m_gridWidth / m_oldXspacing)) > 0.01) {

                    // Old grid spacing must be evenly divisible into
                    // new spacing.
                    continue;
                }

                GrObject barrierGro = generateThinWallBarriers(newRawBath);
                newRawBath.add(barrierGro);
            }
        }
    }

//------------------------------------------------------------------------------
/**
 *
 */
    public GrObject generateDryCells(RawBathymetry newRawBath) {

        DoubleDataSeries newSeries = newRawBath.getBathymetrySeries();
        double[] newArray = newSeries.getArray();

        ArrayList dryCellList = new ArrayList();
        int[] wetIndexes = newRawBath.getWetIndexSeries().getArray();
        for (int n = 0; n < wetIndexes.length; n++) {
            int i = wetIndexes[n] / 10000;
            int j = wetIndexes[n] - (i * 10000);

            int oldCenterI = i * m_numOldGridsPerNew;
            int oldCenterJ = j * m_numOldGridsPerNew;

            double oldTotalArea = m_numOldGridsPerNew * m_numOldGridsPerNew;
            double oldNumDry = 0;
            for (int iVal = oldCenterI;
                iVal < (oldCenterI + m_numOldGridsPerNew); iVal++) {
                for (int jVal = oldCenterJ;
                    jVal < (oldCenterJ + m_numOldGridsPerNew); jVal++) {

                    if (!isWet(iVal, jVal, m_oldArray, m_oldNumI, m_oldNumJ,
                        m_oldDryCellSeries)) {

                        oldNumDry++;
                    }
                }
            }

            if ((oldNumDry / oldTotalArea) >= m_singleCellDryPercentage) {
                int comboIJ = i * 10000 + j;
                dryCellList.add(comboIJ);
            }
        }

        IntDataSeries series = new IntDataSeries(1, "Dry Cells", 1);
        series.setArray(dryCellList);
        series.getDimension(0).setDimensionLocked(true);

        System.out.println("Generated " + (dryCellList.size() / 2)
            + " dry cells.");

        return series;
    }

//------------------------------------------------------------------------------
/**
 *
 */
    public void generateDryCells() {

        if (!readRawBathVariables())
            return;

        Iterator iter = m_children.iterator();
        while (iter.hasNext()) {
            GrObject gro = (GrObject)iter.next();
            if (gro instanceof GrObjectContainer) {
                GrObjectContainer groc = (GrObjectContainer)(gro);
                Enumeration grocEnum =groc.children();
                RawBathymetry newRawBath = null;
                boolean bRawBathIsAlone = true;
                while (grocEnum.hasMoreElements()) {
                    GrObject grocChild = (GrObject)grocEnum.nextElement();
                    if (grocChild instanceof RawBathymetry) {
                        if (newRawBath == null)
                            newRawBath = (RawBathymetry)grocChild;
                        else
                            newRawBath = null;
                    }
                    else {
                        bRawBathIsAlone = false;
                    }
                }

                if (!bRawBathIsAlone)
                    continue;

                // Get grid spacing information from the Bathymetry object.
                GrHashList bathProps = newRawBath.getProperties();
                GrProperty p = (GrProperty)bathProps.get("xSpacing");
                m_gridWidth = (Double)p.value;
                p = (GrProperty)bathProps.get("numI");
                m_newNumI = (Integer)p.value;
                p = (GrProperty)bathProps.get("numJ");
                m_newNumJ = (Integer)p.value;

                m_numOldGridsPerNew = (int)
                    Math.round(m_gridWidth / m_oldXspacing);
                if (((double)m_numOldGridsPerNew -
                    (m_gridWidth / m_oldXspacing)) > 0.01) {

                    // Old grid spacing must be evenly divisible into
                    // new spacing.
                    continue;
                }

                GrObject dryCellGro = generateDryCells(newRawBath);
                newRawBath.add(dryCellGro);
            }
        }
    }

//------------------------------------------------------------------------------
/**
 * Check the raw bathymetry object and remove any dry cells that are on
 * dry land.
 */
    public void checkDryCellSeries(RawBathymetry rawBath) {

        // Retrieve the depth array dry cell array from the raw bathymetry.
        DoubleDataSeries depthSeries = rawBath.getBathymetrySeries();
        double[] depthArray = depthSeries.getArray();
        IntDataSeries dryCellSeries = rawBath.getDryCellSeries();
        if (dryCellSeries == null)
            return;

        int[] dryCellArray = dryCellSeries.getArray();
        dryCellSeries.deselectAll();

        // Get grid size information from the Bathymetry object.
        GrHashList bathProps = rawBath.getProperties();
        GrProperty p = (GrProperty)bathProps.get("numI");
        int numI = (Integer)p.value;
        p = (GrProperty)bathProps.get("numJ");
        int numJ = (Integer)p.value;

        // Look at each dry cell in the list.
        int numDryCells = dryCellArray.length;
        for (int n = 0; n < numDryCells; n++) {
            int i = dryCellArray[n] / 10000;
            int j = dryCellArray[n] - (i * 10000);

            // Check if all the corners of the specified cell are wet
            // in the depth array.
            double[] corners = getCornerDepths(i, j, depthArray, numI, numJ);

            if (corners == null)
                return;

            boolean bAllWet = (corners[0] > m_lowestLand &&
                corners[1] > m_lowestLand && corners[2] > m_lowestLand &&
                corners[3] > m_lowestLand);

            if (!bAllWet) {
                // Select the dry cell to be deleted.
                dryCellSeries.selectPoint(n);
            }
        }

        // Delete all selected dry cells.
        dryCellSeries.deleteSelectedPoints();
    }

//------------------------------------------------------------------------------
/**
 * Overrides default update() in GrObject.  Called by an observed GrObject's
 * notifyObservers() method to handle changes to the observed object.
 * May use the notifyObservers() method to propagate the event handling up to
 * this object's observers.
 * @param gro   The observed object that changed.
 * @param arg   Additional information that the updated object may choose to
 *              send.
 */
    public void update(GrObject gro, Object arg) {

/*
    if (gro instanceof GrObjectContainer) {
            GrObjectContainer groc = (GrObjectContainer)(gro);
            Enumeration grocEnum =groc.children();
            while (grocEnum.hasMoreElements()) {
                GrObject grocChild = (GrObject)grocEnum.nextElement();
                if (grocChild instanceof RawBathymetry) {
                    checkDryCellSeries((RawBathymetry) grocChild);
                }
            }
        }
*/

        m_bHasChanged = true;
        notifyObservers();
    }

//------------------------------------------------------------------------------
/**
 * Provide the list of menu items that execute commands on this object.
 * @return  A menu item that can be placed on a menu and send an ActionEvent
 *          back to this object when it is chosen.
 */
    public javax.swing.JMenuItem getMenuCommands() {

        javax.swing.JMenu subMenu = new javax.swing.JMenu(
        	"Raw Bathymetry Calculator");
        javax.swing.JMenuItem item = new javax.swing.JMenuItem(
            "Create New Grid");
        item.addActionListener(this);
        subMenu.add(item);
        javax.swing.JMenu calcSubMenu = new javax.swing.JMenu(
            "Recalculate Depths");
        item = new javax.swing.JMenuItem(
            "All");
        item.addActionListener(this);
        calcSubMenu.add(item);
        item = new javax.swing.JMenuItem(
            "Wet");
        item.addActionListener(this);
        calcSubMenu.add(item);
        item = new javax.swing.JMenuItem(
            "Dry");
        item.addActionListener(this);
        calcSubMenu.add(item);
        item = new javax.swing.JMenuItem(
            "Mixed");
        item.addActionListener(this);
        calcSubMenu.add(item);
        subMenu.add(calcSubMenu);
        item = new javax.swing.JMenuItem(
            "Generate Dry Cells");
        item.addActionListener(this);
        subMenu.add(item);
        item = new javax.swing.JMenuItem(
            "Generate Thin Wall Barriers");
        item.addActionListener(this);
        subMenu.add(item);

    	return subMenu;
    }

//------------------------------------------------------------------------------
    public void actionPerformed(java.awt.event.ActionEvent e) {

        javax.swing.JMenuItem source =
        	(javax.swing.JMenuItem)(e.getSource());
        String srcText = source.getText();
        if (srcText.equals("Create New Grid")) {
        	createGrid();
        }
        if (srcText.equals("All")) {
            recalculateDepths(true, true);
        }
        if (srcText.equals("Wet")) {
            recalculateDepths(true, false);
        }
        if (srcText.equals("Dry")) {
            recalculateDepths(false, true);
        }
        if (srcText.equals("Mixed")) {
            recalculateDepths(false, false);
        }
        if (srcText.equals("Generate Dry Cells")) {
            generateDryCells();
        }
        if (srcText.equals("Generate Thin Wall Barriers")) {
            generateThinWallBarriers();
        }
    }

//------------------------------------------------------------------------------
/**
 * Create a string representation of this object.
 * @return  The string that describes this object.
 */
    public String toString(){

        String s =  "Raw Bathymetry Calculator";

        return s;
    }
}






