/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2007-02-15
 * Date started:    2002-06-06
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.sfhydro.si3d.gl;

import com.sun.opengl.util.*;
import gov.usgs.gr.*;
import gov.usgs.gr.data.*;
import gov.usgs.gr.visual.*;
import gov.usgs.gr.visual.gl.*;
import gov.usgs.sfhydro.si3d.*;
import gov.usgs.sfhydro.si3d.ps.*;
import java.awt.Rectangle;
import java.awt.geom.*;
import java.text.*;
import java.util.*;
import javax.media.opengl.*;
import javax.media.opengl.glu.*;

//==============================================================================
/**
 * A series of discontinuous triangle fan surfaces.
 */
public class GlBathymetryGrid extends gov.usgs.gr.visual.GraphicalElement
    implements GlElement, SelectableGE, java.awt.event.ActionListener {
    
    /** The OpenGL GLAutoDrawable component. */
    protected GLAutoDrawable gld = null;
    /**
     * The main OpenGL object with all the gl drawing methods.  For
     * readability, the m_ prefix wasn't used.
     */
    protected GL gl = null;
    /** The OpenGL utility class with all the glu drawing methods. */
    protected GLU glu = null;
    /** The OpenGL utility toolkit class with all the glut methods. */
    protected GLUT glut = null;
    
///    private int m_numRedraws = 0;
    
    protected Bathymetry m_bath = null;
    protected RawBathymetry m_rawBath = null;
    protected Grid2PsOptions m_grid2psOptions = null;
    protected double m_xSpacing = 0;
    protected double m_ySpacing = 0;
    protected double m_zSpacing = 0;
    protected DoubleDataSeries m_bathSeries = null;
    protected DoubleDataSeries m_rawBathSeries = null;
    protected int[] m_drawIndexes = null;
    protected boolean m_bDrawIndexesValid = false;
    protected int m_numWetPoints = 0;
    protected int m_numI = 0;
    protected int m_numJ = 0;
    protected int m_numK = 0;
    protected double m_zScale = 1;
    protected double m_xOffset = 0;
    protected double m_yOffset = 0;
    protected boolean m_bDrawVerticalCenterLines = true;
    protected boolean m_bDrawCells = false;
    protected boolean m_bDrawLayers = false;
    protected int m_iViewMin = 0;
    protected int m_iViewMax = 0;
    protected int m_jViewMin = 0;
    protected int m_jViewMax = 0;
    protected int m_levelOfDetail = 1;
    protected boolean m_bDrawSolidEdges = true;
    protected boolean m_bForceSolidEdges = false;
    protected int m_drawPartiallyWetThreshold = 100000;
    protected int m_drawWetSquaresThreshold = 10000;
    protected int m_drawGridThreshold = 150;
    protected int m_drawDepthsThreshold = 90;
    protected int m_drawUvwThreshold = 50;
    protected double m_highestWaterLevel = 0;
    protected double m_lowestLandLevel = 0;
    protected double m_deepestColorLevel = 0;
    protected double m_csCoefficient = 0;
    protected boolean m_bDrawVelLocations = false;
    protected ColorScheme m_colorScheme = new InterpolatingColorScheme();
    protected double[] m_orphanCorners = null;
    protected double[] m_wetDiagonals = null;
    /** The color to use when drawing partially wet cells. */
    protected float[] m_partiallyWetColor = {0, 0.3f, 0};
    /** Whether to draw partially wet cells. */
    protected boolean m_bDrawPartiallyWet = true;
    /** Whether to draw markers for depths that aren't on a fully wet cell. */
    protected boolean m_bDrawOrphanCorners = false;
    /** Whether to draw markers for two wet diagonal neighbors. */
    protected boolean m_bDrawWetDiagonals = false;
    /** The color to use when drawing land cells. */
    protected float[] m_landColor = {0.9f, 0.85f, 0.7f};
    /** The color to use when drawing dry cells. */
    protected float[] m_dryCellColor = {0.4f, 0.37f, 0.1f};
    /** The color to use when drawing thin wall barriers. */
    protected float[] m_barrierColor = {1, 1, 0};
    /** Whether to draw partially wet cells. */
    protected boolean m_bDrawLand = true;
    /** Whether to draw partially the surface of each cell. */
    protected boolean m_bDrawSurface = true;
    
    /** The color of the lines. */
    protected float[] m_color = {0, 1, 0};
    
    /** The tree icon to use for this class. */
//	public static javax.swing.ImageIcon TREE_ICON =
//    	new javax.swing.ImageIcon(System.getProperty("gov.usgs.gr.grhome")
//        + "/res/GrObject/GlDiscontinuousTriFanSurface.gif");
    
//------------------------------------------------------------------------------
    /**
     * Constructor for creating a blank data series.
     */
    public GlBathymetryGrid(){
        
        setDefaults("gov.usgs.sfhydro.si3d.gl.GlBathymetryGrid");
        initializeColorScheme();
        add(new Grid2PsOptions());
    }
    
//------------------------------------------------------------------------------
    /**
     * Constructor for creating a curve from a data series.
     */
    public GlBathymetryGrid(Bathymetry bath){
        
        setDefaults("gov.usgs.sfhydro.si3d.gl.GlBathymetryGrid");
        
        setBathymetry(bath);
        initializeColorScheme();
        add(new Grid2PsOptions());
    }
    
//------------------------------------------------------------------------------
    /**
     * Provide the list of properties with the latest values.
     */
    public GrHashList getProperties() {
        
        GrHashList props = super.getProperties();
        
        props.put("zScale", new GrProperty(m_zScale, "Z Scale"));
        props.put("drawVerticalCenterLines", new GrProperty(
            m_bDrawVerticalCenterLines, "Draw Vertical Center Lines"));
        props.put("drawCells", new GrProperty(
            m_bDrawCells, "Draw Cell Outlines"));
        props.put("drawLayers", new GrProperty(
            m_bDrawLayers, "Draw Layers"));
        props.put("drawSurface", new GrProperty(
            m_bDrawSurface, "Draw Surface"));
        
        ColorProperty cp = new ColorProperty();
        cp.description = "Color";
        cp.setColor(m_color);
        props.put("color", cp);
        
        props.put("drawOrphanCorners", new GrProperty(
            m_bDrawOrphanCorners, "Mark Orphan Corners"));
        props.put("drawWetDiagonals", new GrProperty(
            m_bDrawWetDiagonals, "Mark 2 Wet Diagonals"));
        props.put("drawPartiallyWet", new GrProperty(
            m_bDrawPartiallyWet, "Draw Partially Wet Cells"));
        
        cp = new ColorProperty();
        cp.description = "Partially Wet Color";
        cp.setColor(m_partiallyWetColor);
        props.put("partiallyWetColor", cp);
        
        props.put("drawLand", new GrProperty(m_bDrawLand, "Draw Land"));
        
        cp = new ColorProperty();
        cp.description = "Land Color";
        cp.setColor(m_landColor);
        props.put("landColor", cp);
        
        cp = new ColorProperty();
        cp.description = "Dry Cell Color";
        cp.setColor(m_dryCellColor);
        props.put("dryCellColor", cp);
        
        cp = new ColorProperty();
        cp.description = "Barrier Color";
        cp.setColor(m_barrierColor);
        props.put("barrierColor", cp);
        
//        props.put("drawSoftEdge", new GrProperty(
//        	m_bDrawSoftEdge, "Draw Soft Edge"));
        props.put("levelOfDetail", new GrProperty(
            m_levelOfDetail, "Level of Detail"));
        props.put("drawWetSquaresThreshold", new GrProperty(
            m_drawWetSquaresThreshold, "Draw Wet Squares Threshold"));
        props.put("drawGridThreshold", new GrProperty(
            m_drawGridThreshold, "Draw Grid Threshold"));
        props.put("drawDepthsThreshold", new GrProperty(
            m_drawDepthsThreshold, "Draw Depths Threshold"));
        props.put("drawUvwThreshold", new GrProperty(
            m_drawUvwThreshold, "Draw Uvw Threshold"));
        props.put("highestWater", new GrProperty(
            m_highestWaterLevel, "Highest Water Level"));
        props.put("lowestLand", new GrProperty(
            m_lowestLandLevel, "Lowest Land Level"));
        props.put("deepestLevel", new GrProperty(
            m_deepestColorLevel, "Deepest Color Level"));
        props.put("csCoefficient", new GrProperty(
            m_csCoefficient, "Color Scheme Coefficient (-1 to 1)"));
        
        return props;
    }
    
//------------------------------------------------------------------------------
    /**
     * Provide the list of properties with the latest values.
     */
    public void setProperties(HashMap props) {
        
        super.setProperties(props);
        
        GrProperty p;
        
        Iterator iter = props.keySet().iterator();
        while (iter.hasNext()) {
            String key = (String)iter.next();
            Object obj = props.get(key);
            if (obj instanceof GrProperty) {
                p = (GrProperty)obj;
                
                if (p.value instanceof String) {
                    setProperty(key, (String)p.value);
                    continue;
                }
                
                if (key.equals("zScale"))
                    m_zScale = (Double)p.value;
                if (key.equals("drawVerticalCenterLines"))
                    m_bDrawVerticalCenterLines = (Boolean)p.value;
                if (key.equals("drawCells"))
                    m_bDrawCells = (Boolean)p.value;
                if (key.equals("drawLayers"))
                    m_bDrawLayers = (Boolean)p.value;
                if (key.equals("drawSurface"))
                    m_bDrawSurface = (Boolean)p.value;
                if (key.equals("color")) {
                    ColorProperty cp = (ColorProperty)p;
                    m_color = cp.getColorArray();
                }
                if (key.equals("drawOrphanCorners"))
                    m_bDrawOrphanCorners = (Boolean)p.value;
                if (key.equals("drawWetDiagonals"))
                    m_bDrawWetDiagonals = (Boolean)p.value;
                if (key.equals("drawPartiallyWet"))
                    m_bDrawPartiallyWet = (Boolean)p.value;
                if (key.equals("partiallyWetColor")) {
                    ColorProperty cp = (ColorProperty)p;
                    m_partiallyWetColor = cp.getColorArray();
                }
                if (key.equals("drawLand"))
                    m_bDrawLand = (Boolean)p.value;
                if (key.equals("landColor")) {
                    ColorProperty cp = (ColorProperty)p;
                    m_landColor = cp.getColorArray();
                }
                if (key.equals("dryCellColor")) {
                    ColorProperty cp = (ColorProperty)p;
                    m_dryCellColor = cp.getColorArray();
                }
                if (key.equals("barrierColor")) {
                    ColorProperty cp = (ColorProperty)p;
                    m_barrierColor = cp.getColorArray();
                }
                if (key.equals("levelOfDetail")) {
                    m_levelOfDetail = ((Integer)p.value);
                    if (m_levelOfDetail < 1)
                        m_levelOfDetail = 1;
                    if (m_levelOfDetail > 30)
                        m_levelOfDetail = 30;
                }
                if (key.equals("drawWetSquaresThreshold"))
                    m_drawWetSquaresThreshold = ((Integer)p.value);
                if (key.equals("drawGridThreshold"))
                    m_drawGridThreshold = ((Integer)p.value);
                if (key.equals("drawDepthsThreshold"))
                    m_drawDepthsThreshold = ((Integer)p.value);
                if (key.equals("drawUvwThreshold"))
                    m_drawUvwThreshold = ((Integer)p.value);
                if (key.equals("highestWater"))
                    m_highestWaterLevel = (Double)p.value;
                if (key.equals("lowestLand"))
                    m_lowestLandLevel = (Double)p.value;
                if (key.equals("deepestLevel"))
                    m_deepestColorLevel = (Double)p.value;
                if (key.equals("csCoefficient")) {
                    m_csCoefficient = (Double)p.value;
                    if (m_csCoefficient < -1)
                        m_csCoefficient = -1;
                    if (m_csCoefficient > 1)
                        m_csCoefficient = 1;
                }
            }
        }
        
        m_bNeedsRedraw = true;
        m_bHasChanged = true;
        notifyObservers();
    }
    
//------------------------------------------------------------------------------
    /**
     * Set a single property.
     */
    public void setProperty(String key, String value) {
        
        super.setProperty(key, value);
        
        if (key.equals("zScale"))
            m_zScale = Double.parseDouble(value);
        if (key.equals("drawVerticalCenterLines"))
            m_bDrawVerticalCenterLines = Boolean.parseBoolean(value);
        if (key.equals("drawCells"))
            m_bDrawCells = Boolean.parseBoolean(value);
        if (key.equals("drawLayers"))
            m_bDrawLayers = Boolean.parseBoolean(value);
        if (key.equals("drawSurface"))
            m_bDrawSurface = Boolean.parseBoolean(value);
        if (key.equals("color")) {
            m_color = ColorProperty.getColorArray(value);
        }
        if (key.equals("drawOrphanCorners"))
            m_bDrawOrphanCorners = Boolean.parseBoolean(value);
        if (key.equals("drawWetDiagonals"))
            m_bDrawWetDiagonals = Boolean.parseBoolean(value);
        if (key.equals("drawPartiallyWet"))
            m_bDrawPartiallyWet = Boolean.parseBoolean(value);
        if (key.equals("drawSolidEdges"))
            m_bDrawSolidEdges = Boolean.parseBoolean(value);
        if (key.equals("partiallyWetColor")) {
            m_partiallyWetColor = ColorProperty.getColorArray(value);
        }
        if (key.equals("drawLand"))
            m_bDrawLand = Boolean.parseBoolean(value);
        if (key.equals("landColor"))
            m_landColor = ColorProperty.getColorArray(value);
        if (key.equals("dryCellColor"))
            m_dryCellColor = ColorProperty.getColorArray(value);
        if (key.equals("barrierColor"))
            m_barrierColor = ColorProperty.getColorArray(value);
        if (key.equals("levelOfDetail")) {
            m_levelOfDetail = Integer.parseInt(value);
            if (m_levelOfDetail < 1)
                m_levelOfDetail = 1;
            if (m_levelOfDetail > 30)
                m_levelOfDetail = 30;
        }
        if (key.equals("drawPartiallyWetThreshold"))
            m_drawPartiallyWetThreshold = Integer.parseInt(value);
        if (key.equals("drawWetSquaresThreshold"))
            m_drawWetSquaresThreshold = Integer.parseInt(value);
        if (key.equals("drawGridThreshold"))
            m_drawGridThreshold = Integer.parseInt(value);
        if (key.equals("drawDepthsThreshold"))
            m_drawDepthsThreshold = Integer.parseInt(value);
        if (key.equals("drawUvwThreshold"))
            m_drawUvwThreshold = Integer.parseInt(value);
        if (key.equals("highestWater"))
            m_highestWaterLevel = Double.parseDouble(value);
        if (key.equals("lowestLand"))
            m_lowestLandLevel = Double.parseDouble(value);
        if (key.equals("deepestLevel"))
            m_deepestColorLevel = Double.parseDouble(value);
        if (key.equals("csCoefficient")) {
            m_csCoefficient = Double.parseDouble(value);
            if (m_csCoefficient < -1)
                m_csCoefficient = -1;
            if (m_csCoefficient > 1)
                m_csCoefficient = 1;
        }
        
        m_bNeedsRedraw = true;
    }
    
//------------------------------------------------------------------------------
    /**
     * Add an object to the curve.  Only accepts DoubleDataSeries.  Only holds one at
     * a time.
     */
    public boolean add(GrObject gro) {
        
        // If gro is an object effigy, get the eventual subject and try
        // adding that.
        while (gro instanceof gov.usgs.gr.ObjectEffigy)
            gro = ((ObjectEffigy)gro).getSubject();
        
        if (m_children.contains(gro)) return false;
        
        if (gro instanceof Bathymetry) {
            if (m_bath != null)
                remove(m_bath);
            setBathymetry((Bathymetry)gro);
            
            return super.add(gro);
        }
        
        if (gro instanceof RawBathymetry) {
            if (m_rawBath != null)
                remove(m_rawBath);
            setRawBathymetry((RawBathymetry)gro);
            
            return super.add(gro);
        }
        if (gro instanceof ColorScheme) {
            if (m_colorScheme != null)
                remove((GrObject)m_colorScheme);
            super.add(gro);
            m_colorScheme = (ColorScheme)gro;
            
            return true;
        }
        if (gro instanceof Grid2PsOptions) {
            if (m_grid2psOptions != null)
                remove((GrObject)m_grid2psOptions);
            super.add(gro);
            m_grid2psOptions = (Grid2PsOptions)gro;
            
            return true;
        }
        
        return false;
    }
    
//------------------------------------------------------------------------------
    /**
     * Sets the Bathymetry object.
     */
    public void setBathymetry(Bathymetry bath) {
        
        m_bath = bath;
        GrHashList bathProps = m_bath.getProperties();
        GrProperty p = (GrProperty)bathProps.get("xSpacing");
        m_xSpacing = (Double)p.value;
        p = (GrProperty)bathProps.get("ySpacing");
        m_ySpacing = (Double)p.value;
        p = (GrProperty)bathProps.get("zSpacing");
        m_zSpacing = (Double)p.value;
        m_bathSeries = m_bath.getBathymetrySeries();
        m_numI = (int)Math.round(m_bathSeries.getMax(0)) + 1;
        m_numJ = (int)Math.round(m_bathSeries.getMax(1)) + 1;
        m_numK = (int)Math.round(m_bathSeries.getMax(2));
    }
    
//------------------------------------------------------------------------------
    /**
     * Sets the Bathymetry object.
     */
    public void setRawBathymetry(RawBathymetry rawBath) {
        
        m_rawBath = rawBath;
        GrHashList props = m_rawBath.getProperties();
        GrProperty p = (GrProperty)props.get("xSpacing");
        m_xSpacing = (Double)p.value;
        p = (GrProperty)props.get("ySpacing");
        m_ySpacing = (Double)p.value;
        p = (GrProperty)props.get("zSpacing");
        m_zSpacing = (Double)p.value;
        p = (GrProperty)props.get("numI");
        m_numI = ((Integer)p.value);
        p = (GrProperty)props.get("numJ");
        m_numJ = ((Integer)p.value);
        m_rawBathSeries = m_rawBath.getBathymetrySeries();
        
        m_highestWaterLevel = -m_zSpacing;
        m_deepestColorLevel = m_rawBathSeries.getMax(0);
        
        m_bDrawIndexesValid = false;
        
        checkGrid();
    }
    
//------------------------------------------------------------------------------
    /**
     * Sets the Bathymetry object.
     */
    public void initializeColorScheme() {
        
        InterpolatingColorScheme cs = new InterpolatingColorScheme();
        cs.setProperty("colorValue0", "(139,106,52) 0.0");
        cs.setProperty("colorValue1", "(0,204,153) 1.5");
        cs.setProperty("colorValue2", "(0,210,210) 4.0");
        cs.setProperty("colorValue3", "(0,153,153) 10.0");
        cs.setProperty("colorValue4", "(100,150,255) 20.0");
        cs.setProperty("colorValue5", "(70,70,255) 30.0");
        cs.setProperty("colorValue6", "(135,105,143) 50.0");
        cs.setProperty("colorValue7", "(146,60,111) 80.0");
        add(cs);
    }
    
//------------------------------------------------------------------------------
    /**
     * Accessor method.
     * @return An ImageIcon to use when representing this object in a tree.
     */
//    public javax.swing.ImageIcon getTreeIcon() {
//    	return (javax.swing.ImageIcon)TREE_ICON_HOLDER.get(0);
//    }
    
//------------------------------------------------------------------------------
    /**
     * Sets the OpenGL objects to use for accessing drawing commands.
     * @param newgld   The OpenGL GLAutoDrawable component.
     * @param newgl    The OpenGL object containing the drawing commands.
     * @param newglu   The OpenGL utilites object.
     */
    public void setOpenGL(GLAutoDrawable newgld, GL newgl, GLU newglu) {
        
        if (glut == null || newgl != gl || newglu != glu)
            glut = new GLUT();
        
        gld = newgld;
        gl = newgl;
        glu = newglu;
    }
    
//------------------------------------------------------------------------------
    /**
     * Issue the OpenGL commands to draw this series. Assumes the origin has been
     * moved to (0, 0), scaling has occurred, and line type is solid.
     */
    public void drawGL() {
        
        // Before drawing anything, check if the curve is supposed to be shown.
        if (!m_bShow || (m_bath == null && m_rawBath == null)) {
            m_bNeedsRedraw = false;
            
            return;
        }
        
///        System.out.println("Frame " + m_numRedraws);
///        m_numRedraws++;
///        if (m_numRedraws > 1000)
///            m_numRedraws = 0;
        
        double xMin = m_contentTrans.x;
        double xMax = m_contentTrans.x + m_contentTrans.width;
        double yMin = m_contentTrans.y;
        double yMax = m_contentTrans.y + m_contentTrans.height;
        
        m_iViewMin = (int)Math.round(xMin / m_xSpacing) - 1;
        m_iViewMax = (int)Math.round(xMax / m_xSpacing) + 2;
        m_jViewMin = (int)Math.round(yMin / m_ySpacing) - 1;
        m_jViewMax = (int)Math.round(yMax / m_ySpacing) + 2;
        
        if (m_iViewMin < 0) m_iViewMin = 0;
        if (m_iViewMax > m_numI) m_iViewMax = m_numI;
        if (m_iViewMax < m_iViewMin) m_iViewMax = m_iViewMin;
        if (m_jViewMin < 0) m_jViewMin = 0;
        if (m_jViewMax > m_numJ) m_jViewMax = m_numJ;
        if (m_jViewMax < m_jViewMin) m_jViewMax = m_jViewMin;
        
        if (m_bath != null)
            drawProcessed();
        else
            if (m_rawBath != null)
                drawRaw();
        
        
/*
        for (int i = 0; i < m_numI; i++) {
            for (int j = 0; j < m_numJ; j++) {
                gl.glVertex3(((float)i + 0.5) * m_xSpacing,
                    ((float)j + 0.5) * m_ySpacing, 0);
                gl.glVertex3(((float)i + 0.5) * m_xSpacing,
                    ((float)j + 0.5) * m_ySpacing,
                    m_numK * m_zSpacing * m_zScale);
            }
        }
 */
        
        m_bNeedsRedraw = false;
    }
    
//------------------------------------------------------------------------------
    /**
     *
     */
    public void drawProcessed(){
        int[] indexArray = m_bath.get2DIndexArray();
        int[] lookupArray = m_bath.get2DLookupArray(indexArray);
        double[] depthArray = m_bath.getAverageDepthArray();
        
        DoubleDataSeries bathSeries = m_bath.getBathymetrySeries();
        int numI = (int)Math.round(bathSeries.getMax(0)) + 1;
        
        // Save the previous line color and pattern.
        gl.glPushAttrib(gl.GL_LINE_BIT | gl.GL_CURRENT_BIT);
        
        gl.glColor3fv(m_color, 0);
        if (m_bDrawVerticalCenterLines) {
            gl.glBegin(gl.GL_LINES);
            for (int i = 0; i < depthArray.length; i++) {
                double x = ((double)indexArray[i * 2] - 1) * m_xSpacing;
                double y = ((double)indexArray[i * 2 + 1] - 1) * m_ySpacing;
                double z = depthArray[i];
                gl.glVertex3d(x, y, 0);
                gl.glVertex3d(x, y, z * m_zScale);
            }
            gl.glEnd();
        }
        
        if (m_bDrawCells) {
            double halfXinc = m_xSpacing / 2;
            double halfYinc = m_ySpacing / 2;
            
            gl.glBegin(gl.GL_LINES);
            for (int i = 0; i < depthArray.length; i++) {
                int currentI = indexArray[i * 2];
                int currentJ = indexArray[i * 2 + 1];
                int blOffset = (currentI - 1) + (currentJ - 1) * numI;
                int brOffset = (currentI + 1) + (currentJ - 1) * numI;
                int trOffset = (currentI + 1) + (currentJ + 1) * numI;
                int tlOffset = (currentI - 1) + (currentJ + 1) * numI;
                int blLookup = -1;
                if (blOffset >= 0 && blOffset < lookupArray.length)
                    blLookup = lookupArray[blOffset];
                int brLookup = -1;
                if (brOffset >= 0 && brOffset < lookupArray.length)
                    brLookup = lookupArray[brOffset];
                int trLookup = -1;
                if (trOffset >= 0 && trOffset < lookupArray.length)
                    trLookup = lookupArray[trOffset];
                int tlLookup = -1;
                if (tlOffset >= 0 && tlOffset < lookupArray.length)
                    tlLookup = lookupArray[tlOffset];
                double blZ = -m_zSpacing;
                if (blLookup >= 0)
                    blZ = depthArray[blLookup];
                double brZ = -m_zSpacing;
                if (brLookup >= 0)
                    brZ = depthArray[brLookup];
                double trZ = -m_zSpacing;
                if (trLookup >= 0)
                    trZ = depthArray[trLookup];
                double tlZ = -m_zSpacing;
                if (tlLookup >= 0)
                    tlZ = depthArray[tlLookup];
                double x = ((double)indexArray[i * 2] - 1) * m_xSpacing;
                double y = ((double)indexArray[i * 2 + 1] - 1) * m_ySpacing;
                double z = depthArray[i];
                if (z > blZ) {
                    gl.glVertex3d(x - halfXinc, y - halfYinc, blZ * m_zScale);
                    gl.glVertex3d(x - halfXinc, y - halfYinc, z * m_zScale);
                }
                if (z > brZ) {
                    gl.glVertex3d(x + halfXinc, y - halfYinc, brZ * m_zScale);
                    gl.glVertex3d(x + halfXinc, y - halfYinc, z * m_zScale);
                }
                if (z > trZ) {
                    gl.glVertex3d(x + halfXinc, y + halfYinc, trZ * m_zScale);
                    gl.glVertex3d(x + halfXinc, y + halfYinc, z * m_zScale);
                }
                if (z > tlZ) {
                    gl.glVertex3d(x - halfXinc, y + halfYinc, tlZ * m_zScale);
                    gl.glVertex3d(x - halfXinc, y + halfYinc, z * m_zScale);
                }
            }
            gl.glEnd();
            
            for (int i = 0; i < depthArray.length; i++) {
                double x = ((double)indexArray[i * 2] - 1) * m_xSpacing;
                double y = ((double)indexArray[i * 2 + 1] - 1) * m_ySpacing;
                double z = depthArray[i];
                
                gl.glBegin(gl.GL_LINE_STRIP);
                gl.glVertex3d(x - halfXinc, y - halfYinc, z * m_zScale);
                gl.glVertex3d(x + halfXinc, y - halfYinc, z * m_zScale);
                gl.glVertex3d(x + halfXinc, y + halfYinc, z * m_zScale);
                gl.glVertex3d(x - halfXinc, y + halfYinc, z * m_zScale);
                gl.glVertex3d(x - halfXinc, y - halfYinc, z * m_zScale);
                gl.glEnd();
            }
        }
        
        
        if (m_bDrawLayers) {
            double halfXinc = m_xSpacing / 2;
            double halfYinc = m_ySpacing / 2;
            
            gl.glPushAttrib(gl.GL_LINE_BIT | gl.GL_CURRENT_BIT);
            gl.glEnable(gl.GL_LINE_STIPPLE);
            gl.glLineStipple(1, Gr.LOOSE_DOTTED_LINE_PATTERN );
            for (int i = 0; i < depthArray.length; i++) {
                double x = ((double)indexArray[i * 2] - 1) * m_xSpacing;
                double y = ((double)indexArray[i * 2 + 1] - 1) * m_ySpacing;
                double z = depthArray[i];
                
                double zInc = z % (double)m_zSpacing;
                if (zInc <= 0)
                    zInc = (double)m_zSpacing;
                z -= zInc;
                zInc = (double)m_zSpacing;
                
                while (z >= 0) {
                    gl.glBegin(gl.GL_LINE_STRIP);
                    gl.glVertex3d(x - halfXinc, y - halfYinc, z * m_zScale);
                    gl.glVertex3d(x + halfXinc, y - halfYinc, z * m_zScale);
                    gl.glVertex3d(x + halfXinc, y + halfYinc, z * m_zScale);
                    gl.glVertex3d(x - halfXinc, y + halfYinc, z * m_zScale);
                    gl.glVertex3d(x - halfXinc, y - halfYinc, z * m_zScale);
                    gl.glEnd();
                    
                    z -= zInc;
                }
            }
            gl.glPopAttrib();
        }
        gl.glPopAttrib();
    }
    
//------------------------------------------------------------------------------
    /**
     *
     */
    public void drawRaw() {
        
        if (m_rawBathSeries == null)
            return;
        
        // Draw grid lines.
        int iSpan = m_iViewMax - m_iViewMin;
        int jSpan = m_jViewMax - m_jViewMin;
        if (iSpan < m_drawGridThreshold && jSpan < m_drawGridThreshold &&
            iSpan > 0 && jSpan > 0) {
            
            if (iSpan < m_drawUvwThreshold && jSpan < m_drawUvwThreshold) {
                m_bDrawVelLocations = true;
                drawGrid();
                drawDepthSquares();
                drawDepthNumbers();
                drawSelectedPoints(true);
            } else {
                m_bDrawVelLocations = false;
                if (iSpan < m_drawDepthsThreshold &&
                    jSpan < m_drawDepthsThreshold) {
                    
                    drawGrid();
                    drawDepthSquares();
                    drawDepthNumbers();
                    drawSelectedPoints(true);
                } else {
                    drawDepthSquares();
                    drawGrid();
                    drawSelectedPoints(false);
                }
            }
        } else {
            // Draw color squares representing depth.
            drawDepthSquares();
            drawSelectedPoints(false);
        }
        if (m_bDrawOrphanCorners)
            drawOrphanCorners();
        if (m_bDrawWetDiagonals)
            drawWetDiagonals();
    }
    
//------------------------------------------------------------------------------
    /**
     *
     */
    public void drawDepthSquares() {
        
        double[] array = m_rawBathSeries.getArray();
        if (m_drawIndexes == null ||
            (m_drawIndexes.length != (array.length * 2))) {
            
            m_drawIndexes = new int[array.length * 2];
            m_bDrawIndexesValid = false;
        }
        
        double minZ = m_rawBathSeries.getMin(0);
        double maxZ = m_rawBathSeries.getMax(0);
        double spanZ = maxZ - minZ;
        
        // translate (m_xOffset, m_yOffset)
        
        // If drawing in 2D, draw the land as a big rectangle under the wet
        // cells.
        if (m_bDrawLand && m_zScale == 0) {
            gl.glBlendFunc(gl.GL_ONE, gl.GL_ZERO);
            gl.glBegin(gl.GL_QUADS);
//            gl.glColor3fv(m_landColor, 0);
            gl.glColor4f(m_landColor[0], m_landColor[1], m_landColor[2], 0.1f);
            gl.glVertex2d(0, 0);
            gl.glVertex2d((m_numI - 1) * m_xSpacing, 0);
            gl.glVertex2d((m_numI - 1) * m_xSpacing, (m_numJ - 1) * m_ySpacing);
            gl.glVertex2d(0, (m_numJ - 1) * m_ySpacing);
            gl.glEnd();
        }
        
        int iSpan = m_iViewMax - m_iViewMin;
        int jSpan = m_jViewMax - m_jViewMin;
        
        if ((iSpan > m_drawWetSquaresThreshold &&
            iSpan > m_drawPartiallyWetThreshold)
            || (jSpan > m_drawWetSquaresThreshold &&
            jSpan > m_drawPartiallyWetThreshold) || iSpan <= 0 || jSpan <= 0) {
            
            return;
        }
        
        if (iSpan > m_drawWetSquaresThreshold ||
            jSpan > m_drawWetSquaresThreshold) {
            
            if (!m_bDrawSolidEdges && !m_bForceSolidEdges)
                m_bDrawIndexesValid = false;
            m_bForceSolidEdges = true;
            
            gl.glEnable(gl.GL_BLEND);
            gl.glEnable(gl.GL_POLYGON_SMOOTH);
            gl.glBlendFunc(gl.GL_SRC_ALPHA, gl.GL_DST_ALPHA);
        } else {
            if (!m_bDrawSolidEdges && m_bForceSolidEdges)
                m_bDrawIndexesValid = false;
            
            m_bForceSolidEdges = false;
            
            gl.glDisable(gl.GL_BLEND);
            gl.glDisable(gl.GL_POLYGON_SMOOTH);
        }
        
        gl.glBlendFunc(gl.GL_SRC_ALPHA, gl.GL_ONE_MINUS_SRC_ALPHA);
        
        // If the draw indexes are valid and drawing a fairly large area...
        if (m_bDrawIndexesValid && (iSpan * jSpan) < (m_numWetPoints * 2)) {
            int iViewMax = m_iViewMax;
            int jViewMax = m_jViewMax;
            if (iViewMax > (m_numI - 1)) iViewMax = m_numI - 1;
            if (jViewMax > (m_numJ - 1)) jViewMax = m_numJ - 1;
            
///int squareCount = 0;
            gl.glBegin(gl.GL_QUADS);
            for (int i = m_iViewMin; i < iViewMax; i++) {
                for (int j = m_jViewMin; j < jViewMax; j++) {
                    drawDepthSquare(i, j, array);
///squareCount++;
                }
            }
            gl.glEnd();
///System.out.println("" + squareCount + " squares processed (< numWetPoints*2)");
        }
        // Draw indexes are not valid or drawing a fairly small area...
        else {
            if (m_bDrawIndexesValid) {
///int squareCount = 0;
                gl.glBegin(gl.GL_QUADS);
                for (int n = 0; n < m_numWetPoints; n++) {
                    int i = m_drawIndexes[n * 2];
                    int j = m_drawIndexes[n * 2 + 1];
                    drawDepthSquare(i, j, array);
///squareCount++;
                }
                gl.glEnd();
///System.out.println("" + squareCount + " squares processed (existing indexes)");
                //System.out.println("" + m_numWetPoints + " of " + array.length
                //    + " points are wet.");
            }
            // Draw indexes are not valid, so create them...
            else {
///int squareCount = 0;
                gl.glBegin(gl.GL_QUADS);
                m_numWetPoints = 0;
                for (int i = 0; i < (m_numI - 1); i++) {
                    for (int j = 0; j < (m_numJ - 1); j++) {
                        int numWetCorners = drawDepthSquare(i, j, array);
///squareCount++;
                        if (numWetCorners >= 1 && ((numWetCorners < 4) ||
///                            (!m_bDrawSolidEdges && !m_bForceSolidEdges))) {
                            (!m_bForceSolidEdges))) {
                            
                            m_drawIndexes[m_numWetPoints * 2] = i;
                            m_drawIndexes[m_numWetPoints * 2 + 1] = j;
                            m_numWetPoints++;
                        }
                    }
                }
                m_bDrawIndexesValid = true;
                gl.glEnd();
///System.out.println("" + squareCount + " squares processed (indexes generated)");
            }
        }
        
        // Draw dry cells.
        IntDataSeries dryCellSeries = m_rawBath.getDryCellSeries();
        if (dryCellSeries != null) {
            int[] dryCellArray = dryCellSeries.getArray();
            int numDryCells = dryCellArray.length;
            gl.glColor3fv(m_dryCellColor, 0);
            gl.glBegin(gl.GL_QUADS);
            for (int n = 0; n < numDryCells; n++) {
                double i = dryCellArray[n] / 10000;
                double j = dryCellArray[n] - (i * 10000);
                gl.glVertex2d(i * m_xSpacing, j * m_ySpacing);
                gl.glVertex2d((i + 1) * m_xSpacing, j * m_ySpacing);
                gl.glVertex2d((i + 1) * m_xSpacing, (j+1) * m_ySpacing);
                gl.glVertex2d(i * m_xSpacing, (j + 1) * m_ySpacing);
            }
            gl.glEnd();
            
            drawSelectedDryCells();
        }
        
        if (!m_bDrawWetDiagonals && m_bDrawSolidEdges)
            drawWetDiagonals();
        
        // Draw thin-wall barriers.
        drawBarriers();
        
        gl.glDisable(gl.GL_BLEND);
        gl.glDisable(gl.GL_POLYGON_SMOOTH);
        
    }
    
//------------------------------------------------------------------------------
    /**
     *
     */
    public int drawDepthSquare(int i, int j, double[] array) {
        
        double blZ = array[j * m_numI + i];
        double brZ = array[j * m_numI + i + 1];
        double trZ = array[(j + 1) * m_numI + i + 1];
        double tlZ = array[(j + 1) * m_numI + i];
        
        int numWetCorners = 0;
        boolean bWetBL = false;
        boolean bWetBR = false;
        boolean bWetTR = false;
        boolean bWetTL = false;
        if (blZ >= m_highestWaterLevel) {
            numWetCorners++;
            bWetBL = true;
        }
        if (brZ >= m_highestWaterLevel) {
            numWetCorners++;
            bWetBR = true;
        }
        if (trZ >= m_highestWaterLevel) {
            numWetCorners++;
            bWetTR = true;
        }
        if (tlZ >= m_highestWaterLevel) {
            numWetCorners++;
            bWetTL = true;
        }
        
        if (numWetCorners > 0 && numWetCorners < 4 &&
            (m_bDrawSolidEdges || m_bForceSolidEdges)) {
            
            // Draw a solid-colored square over the entire cell.
            gl.glColor3fv(m_partiallyWetColor, 0);
            gl.glVertex2d(i * m_xSpacing, j * m_ySpacing);
            gl.glVertex2d((i + 1) * m_xSpacing, j * m_ySpacing);
            gl.glVertex2d((i + 1) * m_xSpacing, (j+1) * m_ySpacing);
            gl.glVertex2d(i * m_xSpacing, (j + 1) * m_ySpacing);
            
            return numWetCorners;
        }
        if (numWetCorners > 0) {
            // Store variables.
            double x1 = i * m_xSpacing;
            double x2 = (i + 1) * m_xSpacing;
            double y1 = j * m_ySpacing;
            double y2 = (j + 1) * m_ySpacing;
            double lod = (double)m_levelOfDetail;
            
            // Render each sub area as a separate square.
            for (int m = 0; m < m_levelOfDetail; m++) {
                for (int n = 0; n < m_levelOfDetail; n++) {
                    // Draw a portion of the cell as wet.
                    // Define the intersection of the water surface and land.
                    double lX = m * m_xSpacing / lod + x1;
                    double rX = (m + 1) * m_xSpacing / lod + x1;
                    double bY = n * m_ySpacing / lod + y1;
                    double tY = (n + 1) * m_ySpacing / lod + y1;
                    
                    
                    // Calculate the Z values of the corners of the
                    // sub area from the corners of the outer cell.
                    double posX0 = (double)m / lod;
                    double posY0 = (double)n / lod;
                    double posX1 = (double)(m + 1) / lod;
                    double posY1 = (double)(n + 1) / lod;
                    
                    double bottomZ = blZ * (1 - posX0) + brZ * posX0;
                    double topZ = tlZ * (1 - posX0) + trZ * posX0;
                    double blZ0 = bottomZ * (1 - posY0) + topZ * posY0;
                    
                    bottomZ = blZ * (1 - posX1) + brZ * posX1;
                    topZ = tlZ * (1 - posX1) + trZ * posX1;
                    double brZ0 = bottomZ * (1 - posY0) + topZ * posY0;
                    
                    bottomZ = blZ * (1 - posX1) + brZ * posX1;
                    topZ = tlZ * (1 - posX1) + trZ * posX1;
                    double trZ0 = bottomZ * (1 - posY1) + topZ * posY1;
                    
                    bottomZ = blZ * (1 - posX0) + brZ * posX0;
                    topZ = tlZ * (1 - posX0) + trZ * posX0;
                    double tlZ0 = bottomZ * (1 - posY1) + topZ * posY1;
                    
                    double numWetCorners0 = 0;
                    boolean bWetBL0 = false;
                    boolean bWetBR0 = false;
                    boolean bWetTR0 = false;
                    boolean bWetTL0 = false;
                    if (blZ0 >= m_highestWaterLevel) {
                        numWetCorners0++;
                        bWetBL0 = true;
                    }
                    if (brZ0 >= m_highestWaterLevel) {
                        numWetCorners0++;
                        bWetBR0 = true;
                    }
                    if (trZ0 >= m_highestWaterLevel) {
                        numWetCorners0++;
                        bWetTR0 = true;
                    }
                    if (tlZ0 >= m_highestWaterLevel) {
                        numWetCorners0++;
                        bWetTL0 = true;
                    }
                    
                    double bMidX = 1;
                    if ((!bWetBL0 && bWetBR0) || (bWetBL0 && !bWetBR0)) {
                        bMidX =
                            (blZ0 - m_highestWaterLevel) / (blZ0 - brZ0);
                    }
                    bMidX *= (m_xSpacing / lod);
                    bMidX += lX;
                    double rMidY = 1;
                    if ((!bWetBR0 && bWetTR0) || (bWetBR0 && !bWetTR0)) {
                        rMidY =
                            (brZ0 - m_highestWaterLevel) / (brZ0 - trZ0);
                    }
                    rMidY *= (m_ySpacing / lod);
                    rMidY += bY;
                    double tMidX = 1;
                    if ((!bWetTR0 && bWetTL0) || (bWetTR0 && !bWetTL0)) {
                        tMidX = (trZ0 - m_highestWaterLevel) /
                            (trZ0 - tlZ0);
                    }
                    tMidX *= (m_xSpacing / lod);
                    tMidX = rX - tMidX;
                    double lMidY = 1;
                    if ((!bWetTL0 && bWetBL0) || (bWetTL0 && !bWetBL0)) {
                        lMidY = (tlZ0 - m_highestWaterLevel) /
                            (tlZ0 - blZ0);
                    }
                    lMidY *= (m_ySpacing / lod);
                    lMidY = tY - lMidY;
                    float[] edgeColor =
                        m_colorScheme.getColor(m_highestWaterLevel);
                    
                    if (bWetBL0 && numWetCorners0 == 1) {
                        gl.glColor3fv(edgeColor, 0);
                        gl.glVertex2d(bMidX, bY);
                        gl.glVertex2d(lX, lMidY);
                        gl.glVertex2d(lX, lMidY);
                        gl.glColor3fv(m_colorScheme.getColor(blZ0), 0);
                        gl.glVertex2d(lX, bY);
                    }
                    if (bWetBR0 && numWetCorners0 == 1) {
                        gl.glColor3fv(edgeColor, 0);
                        gl.glVertex2d(rX, rMidY);
                        gl.glVertex2d(bMidX, bY);
                        gl.glVertex2d(bMidX, bY);
                        gl.glColor3fv(m_colorScheme.getColor(brZ0), 0);
                        gl.glVertex2d(rX, bY);
                    }
                    if (bWetTR0 && numWetCorners0 == 1) {
                        gl.glColor3fv(edgeColor, 0);
                        gl.glVertex2d(tMidX, tY);
                        gl.glVertex2d(rX, rMidY);
                        gl.glVertex2d(rX, rMidY);
                        gl.glColor3fv(m_colorScheme.getColor(trZ0), 0);
                        gl.glVertex2d(rX, tY);
                    }
                    if (bWetTL0 && numWetCorners0 == 1) {
                        gl.glColor3fv(edgeColor, 0);
                        gl.glVertex2d(lX, lMidY);
                        gl.glVertex2d(tMidX, tY);
                        gl.glVertex2d(tMidX, tY);
                        gl.glColor3fv(m_colorScheme.getColor(tlZ0), 0);
                        gl.glVertex2d(lX, tY);
                    }
                    if (bWetBL0 && bWetBR0 && numWetCorners0 == 2) {
                        gl.glColor3fv(m_colorScheme.getColor(blZ0), 0);
                        gl.glVertex2d(lX, bY);
                        gl.glColor3fv(m_colorScheme.getColor(brZ0), 0);
                        gl.glVertex2d(rX, bY);
                        gl.glColor3fv(edgeColor, 0);
                        gl.glVertex2d(rX, rMidY);
                        gl.glVertex2d(lX, lMidY);
                    }
                    if (bWetBR0 && bWetTR0 && numWetCorners0 == 2) {
                        gl.glColor3fv(m_colorScheme.getColor(brZ0), 0);
                        gl.glVertex2d(rX, bY);
                        gl.glColor3fv(m_colorScheme.getColor(trZ0), 0);
                        gl.glVertex2d(rX, tY);
                        gl.glColor3fv(edgeColor, 0);
                        gl.glVertex2d(tMidX, tY);
                        gl.glVertex2d(bMidX, bY);
                    }
                    if (bWetTR0 && bWetTL0 && numWetCorners0 == 2) {
                        gl.glColor3fv(m_colorScheme.getColor(trZ0), 0);
                        gl.glVertex2d(rX, tY);
                        gl.glColor3fv(m_colorScheme.getColor(tlZ0), 0);
                        gl.glVertex2d(lX, tY);
                        gl.glColor3fv(edgeColor, 0);
                        gl.glVertex2d(lX, lMidY);
                        gl.glVertex2d(rX, rMidY);
                    }
                    if (bWetTL0 && bWetBL0 && numWetCorners0 == 2) {
                        gl.glColor3fv(m_colorScheme.getColor(tlZ0), 0);
                        gl.glVertex2d(lX, tY);
                        gl.glColor3fv(m_colorScheme.getColor(blZ0), 0);
                        gl.glVertex2d(lX, bY);
                        gl.glColor3fv(edgeColor, 0);
                        gl.glVertex2d(bMidX, bY);
                        gl.glVertex2d(tMidX, tY);
                    }
                    if (bWetBL0 && bWetTR0 && numWetCorners0 == 2) {
                        gl.glColor3fv(m_colorScheme.getColor(blZ0), 0);
                        gl.glVertex2d(lX, bY);
                        gl.glColor3fv(edgeColor, 0);
                        gl.glVertex2d(bMidX, bY);
                        double avgZ = (blZ + trZ) / 2;
                        gl.glColor3fv(m_colorScheme.getColor(avgZ), 0);
                        gl.glVertex2d((lX + rX) / 2, (bY + tY) / 2);
                        gl.glColor3fv(edgeColor, 0);
                        gl.glVertex2d(lX, lMidY);
                        gl.glColor3fv(m_colorScheme.getColor(trZ0), 0);
                        gl.glVertex2d(rX, tY);
                        gl.glColor3fv(edgeColor, 0);
                        gl.glVertex2d(tMidX, tY);
                        gl.glColor3fv(m_colorScheme.getColor(avgZ), 0);
                        gl.glVertex2d((lX + rX) / 2, (bY + tY) / 2);
                        gl.glColor3fv(edgeColor, 0);
                        gl.glVertex2d(rX, rMidY);
                    }
                    if (bWetBR0 && bWetTL0 && numWetCorners0 == 2) {
                        gl.glColor3fv(m_colorScheme.getColor(brZ0), 0);
                        gl.glVertex2d(rX, bY);
                        gl.glColor3fv(edgeColor, 0);
                        gl.glVertex2d(rX, rMidY);
                        double avgZ = (brZ + tlZ) / 2;
                        gl.glColor3fv(m_colorScheme.getColor(avgZ), 0);
                        gl.glVertex2d((lX + rX) / 2, (bY + tY) / 2);
                        gl.glColor3fv(edgeColor, 0);
                        gl.glVertex2d(bMidX, bY);
                        gl.glColor3fv(m_colorScheme.getColor(tlZ0), 0);
                        gl.glVertex2d(lX, tY);
                        gl.glColor3fv(edgeColor, 0);
                        gl.glVertex2d(lX, lMidY);
                        gl.glColor3fv(m_colorScheme.getColor(avgZ), 0);
                        gl.glVertex2d((lX + rX) / 2, (bY + tY) / 2);
                        gl.glColor3fv(edgeColor, 0);
                        gl.glVertex2d(tMidX, tY);
                    }
                    if (!bWetBL0 && numWetCorners0 == 3) {
                        gl.glColor3fv(edgeColor, 0);
                        gl.glVertex2d(lX, lMidY);
                        gl.glVertex2d(lX, lMidY);
                        gl.glVertex2d(bMidX, bY);
                        gl.glColor3fv(m_colorScheme.getColor(tlZ0), 0);
                        gl.glVertex2d(lX, tY);
                        gl.glVertex2d(lX, tY);
                        gl.glColor3fv(edgeColor, 0);
                        gl.glVertex2d(bMidX, bY);
                        gl.glColor3fv(m_colorScheme.getColor(brZ0), 0);
                        gl.glVertex2d(rX, bY);
                        gl.glColor3fv(m_colorScheme.getColor(trZ0), 0);
                        gl.glVertex2d(rX, tY);
                    }
                    if (!bWetBR0 && numWetCorners0 == 3) {
                        gl.glColor3fv(edgeColor, 0);
                        gl.glVertex2d(bMidX, bY);
                        gl.glVertex2d(bMidX, bY);
                        gl.glVertex2d(rX, rMidY);
                        gl.glColor3fv(m_colorScheme.getColor(trZ0), 0);
                        gl.glVertex2d(rX, tY);
                        gl.glVertex2d(rX, tY);
                        gl.glColor3fv(m_colorScheme.getColor(tlZ0), 0);
                        gl.glVertex2d(lX, tY);
                        gl.glColor3fv(m_colorScheme.getColor(blZ0), 0);
                        gl.glVertex2d(lX, bY);
                        gl.glColor3fv(edgeColor, 0);
                        gl.glVertex2d(bMidX, bY);
                    }
                    if (!bWetTR0 && numWetCorners0 == 3) {
                        gl.glColor3fv(edgeColor, 0);
                        gl.glVertex2d(rX, rMidY);
                        gl.glVertex2d(rX, rMidY);
                        gl.glVertex2d(tMidX, tY);
                        gl.glColor3fv(m_colorScheme.getColor(tlZ0), 0);
                        gl.glVertex2d(lX, tY);
                        gl.glVertex2d(lX, tY);
                        gl.glColor3fv(m_colorScheme.getColor(blZ0), 0);
                        gl.glVertex2d(lX, bY);
                        gl.glColor3fv(m_colorScheme.getColor(brZ0), 0);
                        gl.glVertex2d(rX, bY);
                        gl.glColor3fv(edgeColor, 0);
                        gl.glVertex2d(rX, rMidY);
                    }
                    if (!bWetTL0 && numWetCorners0 == 3) {
                        gl.glColor3fv(edgeColor, 0);
                        gl.glVertex2d(tMidX, tY);
                        gl.glVertex2d(tMidX, tY);
                        gl.glVertex2d(lX, lMidY);
                        gl.glColor3fv(m_colorScheme.getColor(blZ0), 0);
                        gl.glVertex2d(lX, bY);
                        gl.glVertex2d(lX, bY);
                        gl.glColor3fv(m_colorScheme.getColor(brZ0), 0);
                        gl.glVertex2d(rX, bY);
                        gl.glColor3fv(m_colorScheme.getColor(trZ0), 0);
                        gl.glVertex2d(rX, tY);
                        gl.glColor3fv(edgeColor, 0);
                        gl.glVertex2d(tMidX, tY);
                    }
                    if (numWetCorners0 == 4 && m_bDrawSurface &&
                        !m_bForceSolidEdges) {
                        
                        gl.glColor3fv(m_colorScheme.getColor(blZ0), 0);
                        gl.glVertex2d(lX, bY);
                        gl.glColor3fv(m_colorScheme.getColor(brZ0), 0);
                        gl.glVertex2d(rX, bY);
                        gl.glColor3fv(m_colorScheme.getColor(trZ0), 0);
                        gl.glVertex2d(rX, tY);
                        gl.glColor3fv(m_colorScheme.getColor(tlZ0), 0);
                        gl.glVertex2d(lX, tY);
                    }
                }
            }
        }
        
        return numWetCorners;
    }
    
//------------------------------------------------------------------------------
    /**
     *
     */
    public void drawBarriers() {
        
        IntDataSeries barrierSeries = m_rawBath.getBarrierSeries();
        if (barrierSeries != null) {
            int[] barrierArray = barrierSeries.getArray();
            int numBarriers = barrierArray.length / 2;
            
            // Save the calling function's GL settings.
            gl.glPushAttrib(gl.GL_LINE_BIT | gl.GL_CURRENT_BIT);
            gl.glColor3fv(m_barrierColor, 0);
            gl.glLineWidth(4.0f);
            gl.glBegin(gl.GL_LINES);
            for (int n = 0; n < numBarriers; n++) {
                double i1 = barrierArray[n * 2] / 10000;
                double j1 = barrierArray[n * 2 ] - (i1 * 10000);
                double i2 = barrierArray[n * 2 + 1] / 10000;
                double j2 = barrierArray[n * 2 + 1] - (i2 * 10000);
                
                // Slightly lengthen the line so it overlaps a little.
                if (i1 > i2) {
                    i1 += 0.01;
                    i2 -= 0.01;
                }
                if (i2 > i1) {
                    i1 -= 0.01;
                    i2 += 0.01;
                }
                if (j1 > j2) {
                    j1 += 0.01;
                    j2 -= 0.01;
                }
                if (j2 > j1) {
                    j1 -= 0.01;
                    j2 += 0.01;
                }
                
                gl.glVertex2d(i1 * m_xSpacing, j1 * m_ySpacing);
                gl.glVertex2d(i2 * m_xSpacing, j2 * m_ySpacing);
            }
            gl.glEnd();
            gl.glPopAttrib();
        }
        
        // drawSelectedBarriers();
    }
    
//------------------------------------------------------------------------------
    /**
     *
     */
    public void drawSelectedPoints(boolean bTextShowing) {
        
        int[] selectedIndexes = m_rawBathSeries.getSelectedIndexes();
        int numSelected = m_rawBathSeries.getNumSelected();
        if (numSelected <= 0)
            return;
        
        // Save the calling function's GL settings.
        gl.glPushAttrib(gl.GL_LINE_BIT | gl.GL_CURRENT_BIT);
        gl.glLineWidth(2.0f);
        
        if (bTextShowing) {
            gl.glColor3f(1, 1, 0);
            for (int m = 0; m < numSelected; m++) {
                double i = selectedIndexes[m] % m_numI;
                double j = selectedIndexes[m] / m_numI;
                
                gl.glBegin(gl.GL_LINE_STRIP);
                gl.glVertex2d((i - 0.4) * m_xSpacing, (j - 0.25) *m_ySpacing);
                gl.glVertex2d((i - 0.4) * m_xSpacing, (j - 0.5) *m_ySpacing);
                gl.glVertex2d((i - 0.15) * m_xSpacing, (j - 0.5) *m_ySpacing);
                gl.glVertex2d((i + 0.25) * m_xSpacing, (j - 0.25)*m_ySpacing);
                gl.glVertex2d((i + 0.5) * m_xSpacing, (j + 0.15) *m_ySpacing);
                gl.glVertex2d((i + 0.5) * m_xSpacing, (j + 0.4) *m_ySpacing);
                gl.glVertex2d((i + 0.25) * m_xSpacing, (j + 0.4) *m_ySpacing);
                gl.glEnd();
            }
        } else {
            gl.glColor3f(1, 1, 0);
            for (int m = 0; m < numSelected; m++) {
                gl.glBegin(gl.GL_LINE_LOOP);
                double i = selectedIndexes[m] % m_numI;
                double j = selectedIndexes[m] / m_numI;
                
                gl.glVertex2d((i - 0.4) * m_xSpacing, (j - 0.4) * m_ySpacing);
                gl.glVertex2d((i + 0.4) * m_xSpacing, (j - 0.4) * m_ySpacing);
                gl.glVertex2d((i + 0.4) * m_xSpacing, (j + 0.4) * m_ySpacing);
                gl.glVertex2d((i - 0.4) * m_xSpacing, (j + 0.4) * m_ySpacing);
                gl.glEnd();
            }
        }
        
        gl.glPopAttrib();
    }
    
//------------------------------------------------------------------------------
    /**
     *
     */
    public void drawSelectedDryCells() {
        
        IntDataSeries dryCellSeries = m_rawBath.getDryCellSeries();
        if (dryCellSeries == null)
            return;
        
        int numSelected = dryCellSeries.getNumSelected();
        if (numSelected <= 0)
            return;
        
        int[] dryCellArray = dryCellSeries.getArray();
        int numDryCells = dryCellArray.length;
        int[] selectedIndexes = dryCellSeries.getSelectedIndexes();
        
        // Save the calling function's GL settings.
        gl.glPushAttrib(gl.GL_LINE_BIT | gl.GL_CURRENT_BIT);
        gl.glLineWidth(2.0f);
        
        gl.glColor3f(1, 1, 0);
        for (int m = 0; m < numSelected; m++) {
            int selectedIndex = selectedIndexes[m];
            if (selectedIndex >= numDryCells)
                continue;
            
            gl.glBegin(gl.GL_LINE_LOOP);
            double i = dryCellArray[selectedIndex] / 10000;
            double j = dryCellArray[selectedIndex] - (i * 10000);
            gl.glVertex2d(i * m_xSpacing, j * m_ySpacing);
            gl.glVertex2d((i + 1) * m_xSpacing, j * m_ySpacing);
            gl.glVertex2d((i + 1) * m_xSpacing, (j+1) * m_ySpacing);
            gl.glVertex2d(i * m_xSpacing, (j + 1) * m_ySpacing);
            gl.glEnd();
        }
        
        gl.glPopAttrib();
    }
    
//------------------------------------------------------------------------------
    /**
     *
     */
    public void drawSelectedBarriers() {
        
        IntDataSeries barrierSeries = m_rawBath.getBarrierSeries();
        if (barrierSeries == null)
            return;
        
        int numSelected = barrierSeries.getNumSelected();
        if (numSelected <= 0)
            return;
        
        int[] barrierArray = barrierSeries.getArray();
        int numDryCells = barrierArray.length;
        int[] selectedIndexes = barrierSeries.getSelectedIndexes();
        
        // Save the calling function's GL settings.
        gl.glPushAttrib(gl.GL_LINE_BIT | gl.GL_CURRENT_BIT);
        gl.glLineWidth(6.0f);
        gl.glLineStipple(1, Gr.DASHED_LINE_PATTERN);
        
        gl.glColor3f(0, 1, 0);
        for (int m = 0; m < numSelected; m++) {
            int selectedIndex = selectedIndexes[m];
            if (selectedIndex >= numDryCells)
                continue;
            
            gl.glBegin(gl.GL_LINES);
            double i1 = barrierArray[selectedIndex] / 10000;
            double j1 = barrierArray[selectedIndex] - (i1 * 10000);
            double i2 = barrierArray[selectedIndex + 1] / 10000;
            double j2 = barrierArray[selectedIndex + 1] - (i2 * 10000);
            gl.glVertex2d(i1 * m_xSpacing, j1 * m_ySpacing);
            gl.glVertex2d(i2 * m_xSpacing, j2 * m_ySpacing);
            gl.glEnd();
        }
        
        gl.glPopAttrib();
    }
//------------------------------------------------------------------------------
    /**
     *
     */
    public void drawOrphanCorners() {
        
        gl.glPushAttrib(gl.GL_LINE_BIT | gl.GL_CURRENT_BIT);
        gl.glLineWidth(4.0f);
        gl.glColor3f(1, 0, 0);
        
        for (int n = 0; n < m_orphanCorners.length; n += 2) {
            double i = m_orphanCorners[n];
            if ((n + 1) >= m_orphanCorners.length)
                break;
            double j = m_orphanCorners[n + 1];
            
            if (i < m_iViewMin || i > m_iViewMax
                || j < m_jViewMin || j > m_jViewMax) {
                
                continue;
            }
            
            gl.glBegin(gl.GL_LINE_LOOP);
            gl.glVertex2d((i - 0.45) * m_xSpacing, (j - 0.45) * m_ySpacing);
            gl.glVertex2d((i + 0.45) * m_xSpacing, (j - 0.45) * m_ySpacing);
            gl.glVertex2d((i + 0.45) * m_xSpacing, (j + 0.45) * m_ySpacing);
            gl.glVertex2d((i - 0.45) * m_xSpacing, (j + 0.45) * m_ySpacing);
            gl.glEnd();
        }
        
        gl.glPopAttrib();
    }
    
//------------------------------------------------------------------------------
    /**
     *
     */
    public void drawWetDiagonals() {
        
        gl.glPushAttrib(gl.GL_LINE_BIT | gl.GL_CURRENT_BIT);
        gl.glLineWidth(4.0f);
        if (m_bDrawWetDiagonals)
            gl.glColor3f(1, 0, 0);
        else
            gl.glColor3fv(m_dryCellColor, 0);
        
        for (int n = 0; n < m_wetDiagonals.length; n += 2) {
            double i = m_wetDiagonals[n];
            if ((n + 1) >= m_wetDiagonals.length)
                break;
            double j = m_wetDiagonals[n + 1];
            
            if (i < m_iViewMin || i > m_iViewMax
                || j < m_jViewMin || j > m_jViewMax) {
                
                continue;
            }
            
            if (m_bDrawWetDiagonals)
                gl.glBegin(gl.GL_LINE_LOOP);
            else
                gl.glBegin(gl.GL_QUADS);
            gl.glVertex2d((i - 0.55) * m_xSpacing, j * m_ySpacing);
            gl.glVertex2d(i * m_xSpacing, (j - 0.55) * m_ySpacing);
            gl.glVertex2d((i + 0.55) * m_xSpacing, j * m_ySpacing);
            gl.glVertex2d(i * m_xSpacing, (j + 0.55) * m_ySpacing);
            gl.glEnd();
        }
        
        gl.glPopAttrib();
        
    }
    
//------------------------------------------------------------------------------
    /**
     *
     */
    public void drawGrid(){
        
        // Save the calling function's GL settings.
        gl.glPushAttrib(gl.GL_LINE_BIT | gl.GL_CURRENT_BIT);
        
        // Draw a dashed cyan box around the zoom area.
        gl.glColor3f(1, 1, 1);
        gl.glLineWidth(1.0f);
        gl.glEnable(gl.GL_LINE_STIPPLE);
        gl.glLineStipple(1, Gr.DOTTED_LINE_PATTERN);
        
        gl.glBegin(gl.GL_LINES);
        for (int i = m_iViewMin; i < m_iViewMax; i++) {
            gl.glVertex2d(i * m_xSpacing, 0);
            gl.glVertex2d(i * m_xSpacing, (m_numJ - 1) * m_ySpacing);
        }
        for (int j = m_jViewMin; j < m_jViewMax; j++) {
            gl.glVertex2d(0, j * m_ySpacing);
            gl.glVertex2d((m_numI - 1) * m_xSpacing, j * m_ySpacing);
        }
        gl.glEnd();
        
        gl.glPopAttrib();
    }
    
//------------------------------------------------------------------------------
    /**
     *
     */
    public void drawDepthNumbers() {
        
        double[] array = m_rawBathSeries.getArray();
        
        int uWidth = glut.glutStrokeWidth(glut.STROKE_ROMAN, 'U');
        double halfU = uWidth * 0.5;
        int vWidth = glut.glutStrokeWidth(glut.STROKE_ROMAN, 'V');
        double halfV = vWidth * 0.5;
        int wWidth = glut.glutStrokeWidth(glut.STROKE_ROMAN, 'W');
        double halfW = wWidth * 0.5;
        NumberFormat form = new DecimalFormat("0.0");
        
        // Save the calling function's GL settings.
        gl.glPushAttrib(gl.GL_LINE_BIT | gl.GL_CURRENT_BIT);
        
        gl.glColor3f(1, 1, 1);
        gl.glLineWidth(2.0f);
        
        for (int i = m_iViewMin; i < m_iViewMax; i++) {
            for (int j = m_jViewMin; j < m_jViewMax; j++) {
                double z = array[j * m_numI + i];
                int depth10 = (int)Math.round(z * 10);
                if (Double.isNaN(z) || z < m_highestWaterLevel) continue;
                
                String depthString = null;
                if (m_bDrawVelLocations)
                    depthString = form.format(((double)depth10) / 10);
                else
                    depthString = "" + depth10;
                int depthWidth = glut.glutStrokeLength(glut.STROKE_ROMAN,
                    depthString);
                double x = i * m_xSpacing;
                double y = j * m_ySpacing;
                gl.glPushMatrix();
                gl.glTranslated(x, y, 0);
                if (m_bDrawVelLocations)
                    gl.glScaled(0.003 * m_xSpacing, 0.003 * m_ySpacing, 1);
                else
                    gl.glScaled(0.0045 * m_xSpacing, 0.0045 * m_ySpacing, 1);
                gl.glRotatef(45, 0, 0, 1);
                gl.glTranslated(-depthWidth * 0.5, -50, 0);
                glut.glutStrokeString(glut.STROKE_ROMAN, "" + depthString);
                gl.glPopMatrix();
                
                if (i > 0 && j > 0 && m_bDrawVelLocations) {
                    double blZ = array[(j - 1) * m_numI + i - 1] * 10;
                    double brZ = array[(j - 1) * m_numI + i] * 10;
                    double tlZ = array[j * m_numI + i - 1] * 10;
                    gl.glPushMatrix();
                    gl.glTranslated(x, y - (m_ySpacing * 0.5), 0);
                    gl.glScaled(0.001 * m_xSpacing, 0.001 * m_ySpacing, 1);
                    gl.glTranslated(0, -50, 0);
                    if (z >= 0 && brZ >= 0) {
                        gl.glTranslated(-halfU, 0, 0);
                        glut.glutStrokeString(glut.STROKE_ROMAN, "U");
                        gl.glTranslated(-halfU, 0, 0);
                    }
                    
                    gl.glTranslated(-500, 0, 0);
                    if (blZ >= 0 && brZ >= 0 && z >= 0 && tlZ >= 0) {
                        gl.glTranslated(-halfW, 0, 0);
                        gl.glBegin(gl.GL_LINE_STRIP);
                        gl.glVertex2i(15, 80);
                        gl.glVertex2i(45, 72);
                        gl.glVertex2i(75, 70);
                        gl.glVertex2i(95, 85);
                        gl.glVertex2i(85, 100);
                        gl.glVertex2i(55, 85);
                        gl.glVertex2i(15, 40);
                        gl.glVertex2i(80, 40);
                        gl.glVertex2i(10, 0);
                        gl.glEnd();
//                        glut.glutStrokeString(glut.GLUT_STROKE_ROMAN, "W");
                        gl.glTranslated(halfW, 0, 0);
                    }
                    
                    if (z >= 0 && tlZ >= 0) {
                        gl.glTranslated(-halfV, 500, 0);
                        glut.glutStrokeString(glut.STROKE_ROMAN, "V");
                    }
                    gl.glPopMatrix();
                }
                
            }
        }
        
        gl.glPopAttrib();
    }
    
//------------------------------------------------------------------------------
    /**
     * Find the minimum and maximum boundary values for this object as it is shown
     * for the specified dimension.
     * @param dimension The dimension to report on.  X=0, Y=1.
     * @return The minimum (stored as point.x) and maximum (stored as point.y)
     *         values for the graph contents.   null if hidden.
     */
    public Point2D.Double getBounds(int dimension){
        
        if (m_bath == null && m_rawBath == null) return null;
        
        if (dimension == 0) {
            Point2D.Double p = new Point2D.Double(m_xOffset,
                m_xOffset + ((double)(m_numI - 1) * m_xSpacing));
            return p;
        }
        if (dimension == 1) {
            Point2D.Double p = new Point2D.Double(m_yOffset,
                m_yOffset + ((double)(m_numJ - 1) * m_ySpacing));
            return p;
        }
        
        return null;
    }
    
//------------------------------------------------------------------------------
    /**
     * Overrides default update() in GrObject.  Called by an observed GrObject's
     * notifyObservers() method to handle changes to the observed object.
     * May use the notifyObservers() method to propagate the event handling up to
     * this object's observers.
     * @param gro   The observed object that changed.
     * @param arg   Additional information that the updated object may choose to
     *              send.
     */
    public void update(GrObject gro, Object arg) {
        
        if (gro == m_rawBath) {
            boolean bNeedReinit = true;
            
            // Check the reason for the update to see if reinitialization is
            // required.
            if (arg != null && arg instanceof List) {
                List list = (List)arg;
                if (list.get(0) instanceof Integer &&
                    list.get(1) instanceof DataSeries) {
                    
                    int code = (Integer)list.get(0);
                    if (code != DataSeries.NEW_ARRAY &&
                        code != DataSeries.DATA_MODIFIED) {
                        
                        bNeedReinit = false;
                    }
                }
            }
            
            if (bNeedReinit)
                setRawBathymetry(m_rawBath);
        }
        super.update(gro, arg);
    }
    
//------------------------------------------------------------------------------
    /**
     * Selects or deselects all points within the zoom window.
     * @param selectRectMin	The lower left corner of the rectangle describing the
     *                      selected area.
     * @param selectRectMax	The upper right corner of the rectangle describing the
     *                      selected area.
     * @param bAdding	True if selecting points, false if deselecting.
     * @param bExclusive	True if only one curve should be selected or,
     *						if multiple curves are already selected, only
     *						the selected curves can have more points selected.
     * @param bDeselectFirst	True if old selection is deselected before selecting
     *							new points.
     * @return	An unmodifiable list of selected data series.
     */
    public List selectWindow(Point2D.Double selectRectMin,
        Point2D.Double selectRectMax, boolean bAdding,
        boolean bExclusive, boolean bDeselectFirst) {
        
        if (m_rawBathSeries == null)
            return null;
        
        if (bDeselectFirst) m_rawBathSeries.deselectAll();
        if (!m_bShow)
            return null;
        
        if (m_rawBathSeries.getDimension(0).getDimensionLocked())
            return null;
        
        int iMin = (int)((selectRectMin.x / m_xSpacing) + 0.6);
        int iMax = (int)((selectRectMax.x / m_xSpacing) - 0.1);
        int jMin = (int)((selectRectMin.y / m_ySpacing) + 0.6);
        int jMax = (int)((selectRectMax.y / m_ySpacing) - 0.1);
        if (iMax > iMin || jMax > jMin) {
            iMin = (int)((selectRectMin.x / m_xSpacing) + 0.75);
            iMax = (int)((selectRectMax.x / m_xSpacing) + 0.25);
            jMin = (int)((selectRectMin.y / m_ySpacing) + 0.75);
            jMax = (int)((selectRectMax.y / m_ySpacing) + 0.25);
        }
        
        if (iMin < 0) iMin = 0;
        if (iMax >= m_numI) iMax = m_numI - 1;
        if (iMax < iMin) iMax = iMin;
        if (jMin < 0) jMin = 0;
        if (jMax >= m_numJ) jMax = m_numJ - 1;
        if (jMax < jMin) jMax = jMin;
        
        // Check the thin wall barriers
/*
        int numDepthsSelected = 0;
        if (m_rawBathSeries != null)
            numDepthsSelected = m_rawBathSeries.getNumSelected();
        IntDataSeries barrierSeries = m_rawBath.getBarrierSeries();
        if (barrierSeries != null && numDepthsSelected == 0) {
            if (bDeselectFirst) barrierSeries.deselectAll();
 
            int comboIJ = iMin * 10000 + jMin;
            int minIndex = barrierSeries.search(comboIJ);
            comboIJ = iMax * 10000 + jMax;
            int maxIndex = barrierSeries.search(comboIJ);
            int[] barrierArray = barrierSeries.getArray();
            int numSelected = barrierSeries.getNumSelected();
            int[] selectedIndexes = barrierSeries.getSelectedIndexes();
 
            for (int i = minIndex; i <= maxIndex; i++) {
                float twbI1 = barrierArray[i * 2] / 10000;
                float twbJ1 = barrierArray[i * 2] - (twbI1 * 10000);
                float twbI2 = barrierArray[i * 2 + 1] / 10000;
                float twbJ2 = barrierArray[i * 2 + 1] - (twbI2 * 10000);
                if ((twbJ1 >= jMin && twbJ1 <= jMax) ||
                    (twbJ2 >= jMin && twbJ2 <= jMax)) {
 
                    // Check if the point is already in the selected list.
                    int selectedIndex = -1;
                    for (int n = 0; n < numSelected; n++) {
                        if (selectedIndexes[n] == i) {
                            // Selected point was found to be already
                            // stored here.
                            selectedIndex = n;
                            break;
                        }
                    }
 
                    // If adding points to the selection list...
                    if (bAdding) {
                        // Add the point if it wasn't found in the search above.
                        if (selectedIndex < 0)
                            barrierSeries.selectPoint(i);
                    }
                    else {
                        // Deselect the point if it was already selected.
                        if (selectedIndex >= 0)
                            barrierSeries.deselectPoint(selectedIndex);
                    }
                }
            }
 
            if (barrierSeries.getNumSelected() > 0) {
                List list = new ArrayList();
 
                return Collections.unmodifiableList(list);
            }
        }
 
        // Check the dry cells
        IntDataSeries dryCellSeries = m_rawBath.getDryCellSeries();
        if (dryCellSeries != null && numDepthsSelected == 0) {
            if (bDeselectFirst) dryCellSeries.deselectAll();
 
            int comboIJ = iMin * 10000 + jMin;
            int minIndex = dryCellSeries.search(comboIJ);
            comboIJ = iMax * 10000 + jMax;
            int maxIndex = dryCellSeries.search(comboIJ);
            int[] dryCellArray = dryCellSeries.getArray();
            int numSelected = dryCellSeries.getNumSelected();
            int[] selectedIndexes = dryCellSeries.getSelectedIndexes();
 
            for (int i = minIndex; i <= maxIndex; i++) {
                float dryI = dryCellArray[i] / 10000;
                float dryJ = dryCellArray[i] - (dryI * 10000);
                if (dryJ >= jMin && dryJ <= jMax) {
                    // Check if the point is already in the selected list.
                    int selectedIndex = -1;
                    for (int n = 0; n < numSelected; n++) {
                        if (selectedIndexes[n] == i) {
                            // Selected point was found to be already
                            // stored here.
                            selectedIndex = n;
                            break;
                        }
                    }
 
                    // If adding points to the selection list...
                    if (bAdding) {
                        // Add the point if it wasn't found in the search above.
                        if (selectedIndex < 0)
                            dryCellSeries.selectPoint(i);
                    }
                    else {
                        // Deselect the point if it was already selected.
                        if (selectedIndex >= 0)
                            dryCellSeries.deselectPoint(selectedIndex);
                    }
                }
            }
 
            if (dryCellSeries.getNumSelected() > 0) {
                List list = new ArrayList();
 
                return Collections.unmodifiableList(list);
            }
        }
 */
        
        // Check the corner depths.
        int numRawPoints = m_rawBathSeries.getNumPoints();
        int numSelected = m_rawBathSeries.getNumSelected();
        int[] selectedIndexes = m_rawBathSeries.getSelectedIndexes();
        
        for (int i = iMin; i <= iMax; i++) {
            for (int j = jMin; j <= jMax; j++) {
                int index = j * m_numI + i;
                if (index >= 0 && index < numRawPoints) {
                    // Check if the point is already in the selected list.
                    int selectedIndex = -1;
                    for (int n = 0; n < numSelected; n++) {
                        if (selectedIndexes[n] == index) {
                            // Selected point was found to be already
                            // stored here.
                            selectedIndex = n;
                            break;
                        }
                    }
                    
                    // If adding points to the selection list...
                    if (bAdding) {
                        // Add the point if it wasn't found in the search above.
                        if (selectedIndex < 0)
                            m_rawBathSeries.selectPoint(index);
                    } else {
                        // Deselect the point if it was already selected.
                        if (selectedIndex >= 0)
                            m_rawBathSeries.deselectPoint(selectedIndex);
                    }
                } else
                    System.err.println("Could not select i = " + i + ", j = "
                        + j);
            }
        }
        
        List list = new ArrayList();
        if (m_rawBathSeries.getNumSelected() > 0)
            list.add(m_rawBathSeries);
        
        return Collections.unmodifiableList(list);
    }
    
//------------------------------------------------------------------------------
    /**
     * Looks for problems with the grid.
     */
    public void checkGrid() {
        
        ArrayList orphanCornerList = new ArrayList();
        ArrayList wetDiagonalList = new ArrayList();
        double[] array = m_rawBathSeries.getArray();
        IntDataSeries dryCellSeries = m_rawBath.getDryCellSeries();
        boolean[] bWet = new boolean[(m_numI - 1) * (m_numJ - 1)];
        RawBathymetryCalculator calc = new RawBathymetryCalculator();
        
        // Look at each cell in the grid.
        for (int i = 0; i < (m_numI - 1); i++) {
            for (int j = 0; j < (m_numJ - 1); j++) {
                bWet[j * (m_numI - 1) + i] =
                    calc.isWet(i, j, array, m_numI, m_numJ, dryCellSeries);
            }
        }
        
        for (int i = 0; i < m_numI; i++) {
            for (int j = 0; j < m_numJ; j++) {
                double depth = array[j * m_numI + i];
                if (depth < m_highestWaterLevel)
                    continue;
                
                boolean bBLwet = false;
                boolean bBRwet = false;
                boolean bTRwet = false;
                boolean bTLwet = false;
                boolean bEdge = false;
                
                if (i > 0 && j > 0) {
                    if (bWet[(j - 1) * (m_numI - 1) + (i - 1)]) {
                        bBLwet = true;
                    }
                } else {
                    bEdge = true;
                }
                if (i < (m_numI - 1) && j > 0) {
                    if (bWet[(j - 1) * (m_numI - 1) + i]) {
                        bBRwet = true;
                    }
                } else {
                    bEdge = true;
                }
                if (i < (m_numI - 1) && j < (m_numJ - 1)) {
                    if (bWet[j * (m_numI - 1) + i]) {
                        bTRwet = true;
                    }
                } else {
                    bEdge = true;
                }
                if (i > 0 && j < (m_numJ - 1)) {
                    if (bWet[j * (m_numI - 1) + (i - 1)]) {
                        bTLwet = true;
                    }
                } else {
                    bEdge = true;
                }
                
                if (!bBLwet && !bBRwet && !bTRwet && !bTLwet) {
                    orphanCornerList.add(i);
                    orphanCornerList.add(j);
                }
                if (!bEdge) {
                    if ((bBLwet && !bBRwet && bTRwet && !bTLwet)
                    || (!bBLwet && bBRwet && !bTRwet && bTLwet)) {
                        
                        wetDiagonalList.add(i);
                        wetDiagonalList.add(j);
                    }
                }
            }
        }
        
        m_orphanCorners = new double[orphanCornerList.size()];
        int index = 0;
        Iterator iter = orphanCornerList.iterator();
        while (iter.hasNext()) {
            m_orphanCorners[index] = (double)(((Integer)iter.next()));
            index++;
        }
        
        m_wetDiagonals = new double[wetDiagonalList.size()];
        index = 0;
        iter = wetDiagonalList.iterator();
        while (iter.hasNext()) {
            m_wetDiagonals[index] = (double)(((Integer)iter.next()));
            index++;
        }
    }
    
//------------------------------------------------------------------------------
    /**
     * Provide the list of menu items that execute commands on this object.
     * @return  A menu item that can be placed on a menu and send an ActionEvent
     *          back to this object when it is chosen.
     */
    public javax.swing.JMenuItem getMenuCommands() {
        
        javax.swing.JMenu subMenu = new javax.swing.JMenu(
            "Bathymetry Grid");
        javax.swing.JMenuItem item = new javax.swing.JMenuItem(
            "Regenerate Color Scheme");
        item.addActionListener(this);
        subMenu.add(item);
        
        return subMenu;
    }
    
//------------------------------------------------------------------------------
    public void actionPerformed(java.awt.event.ActionEvent e) {
        
        javax.swing.JMenuItem source =
            (javax.swing.JMenuItem)(e.getSource());
        String srcText = source.getText();
        if (srcText.equals("Regenerate Color Scheme")) {
            regenerateColorScheme();
        }
    }
    
//------------------------------------------------------------------------------
    public void regenerateColorScheme() {
        
        if (m_colorScheme == null)
            add(new InterpolatingColorScheme());
        if (!(m_colorScheme instanceof InterpolatingColorScheme))
            add(new InterpolatingColorScheme());
        
        // Use the parabola to interpolate the y value.
        double[] coeffs = DoubleDataSeries.fitParabola(
            0, 0, 0.5, m_csCoefficient, 1, 1);
        double exponent = 1;
        if (m_csCoefficient > 0 && m_csCoefficient < 0.99999)
            exponent = 1 / (1 - m_csCoefficient);
        if (m_csCoefficient < 0 && m_csCoefficient > -0.99999)
            exponent = 1 + m_csCoefficient;
        
        
        double range = m_deepestColorLevel - m_lowestLandLevel;
        
        float colors[][] = {{139,106,52}, {0,204,153}, {0,100,100},
        {0,186,210}, {95,37,181}, {154,139,232}, {135,88,113},
        {205,128,163}, {197,92,61}, {241,205,120}};
        
        InterpolatingColorScheme cs =
            (InterpolatingColorScheme)m_colorScheme;
        cs.setColorAtValue("CLEAR");
//        cs.setColorAtValue("(" + (m_landColor[0] * 255) + ","
//	    + (m_landColor[1] * 255) + "," + (m_landColor[2] * 255) + ")"
//	    + m_highestWaterLevel);
///        cs.setColorAtValue("(255,255,0)" + m_highestWaterLevel);
        cs.setColorAtValue("(85,65,35)" + m_highestWaterLevel);
//        cs.setColorAtValue("(139,106,52)" + 0);
        
        
        // Format the number for output.
        java.text.NumberFormat form = null;
        java.text.NumberFormat form0 = new java.text.DecimalFormat("0");
        java.text.NumberFormat form1 = new java.text.DecimalFormat("0.#");
        java.text.NumberFormat form2 = new java.text.DecimalFormat("0.##");
        java.text.NumberFormat form3 = new java.text.DecimalFormat("0.###");
        java.text.NumberFormat form4 = new java.text.DecimalFormat("0.####");
        java.text.NumberFormat form5 = new java.text.DecimalFormat("0.#####");
        java.text.NumberFormat form6 = new java.text.DecimalFormat("0.######");
        
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < 10; i++) {
            double val = Math.pow(0.1 * (double)i, exponent);
            val *= range;
            val += m_lowestLandLevel;
            
            // Choose the approriate format depending on the size of the
            // number.
            if (val > 1000000)  form = form0;
            else {
                if (val > 100000) form = form0;
                else {
                    if (val > 10000)  form = form1;
                    else {
                        if (val > 1000) form = form2;
                        else {
                            if (val > 100) form = form3;
                            else {
                                if (val > 10) form = form4;
                                else form = form5;
                            }
                        }
                    }
                }
            }
            
            cs.setColorAtValue("(" + colors[i][0] + "," + colors[i][1] + ","
                + colors[i][2] + ")" + val);
        }
    }
    
//------------------------------------------------------------------------------
    /**
     * Create a string representation of this object.
     * @return  The string that describes this object.
     */
    public String toString(){
        
        String s = null;
        if (m_bShow)
            s = "GL Bathymetry Grid: " + m_title;
        else
            s = "(GL Bathymetry Grid): " + m_title;
        
        return s;
    }
}






