/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2006-10-19
 * Date started:    2003-07-02
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.sfhydro.si3d.gui;

import gov.usgs.gr.*;
import gov.usgs.gr.data.*;
import gov.usgs.gr.visual.*;
import gov.usgs.sfhydro.si3d.*;
import java.awt.BorderLayout;
import java.awt.Rectangle;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;

//==============================================================================
/**
 *
 */
public class BarrierEditorProvider extends gov.usgs.gr.GrObject
    implements gov.usgs.gr.visual.gui.GuiPanelProvider,
    gov.usgs.gr.visual.gui.GrMouseHandler {


    /** An instance of the controller associated with this object. */
    GrController m_grCon = null;
    /** An instance of the page associated with this object. */
    PageOfGraphs m_page = null;
    protected JPanel m_panel = new JPanel();
    protected RawBathymetry m_rawBath = null;
    protected IntDataSeries m_dryCellSeries = null;
    protected IntDataSeries m_barrierSeries = null;
    protected ButtonGroup m_addDelGroup = new ButtonGroup();
    protected ButtonGroup m_typeGroup = new ButtonGroup();
    JRadioButton m_noneButton = new JRadioButton();
    JRadioButton m_addButton = new JRadioButton("Add");
    JRadioButton m_deleteButton = new JRadioButton("Delete");
    JRadioButton m_dryCellButton = new JRadioButton("Dry Cell");
    JRadioButton m_thinWallButton = new JRadioButton("Thin Wall");
    protected int m_addDelMode = 0;
    protected int m_typeMode = 0;
    protected int m_mouseX = 0;
    protected int m_mouseY = 0;
    protected boolean m_bDepthsLocked = false;

//------------------------------------------------------------------------------
	public BarrierEditorProvider() {

        createPanel();
	}

//------------------------------------------------------------------------------
/**
 * Overrides add in GrObject.
 */
    public boolean add(GrObject gro) {

        // If gro is an object effigy, get the eventual subject and try
        // adding that.
        while (gro instanceof gov.usgs.gr.ObjectEffigy)
            gro = ((ObjectEffigy)gro).getSubject();

        if (m_children.contains(gro)) return false;

        boolean bAdded = false;

        if (gro instanceof RawBathymetry) {
            m_rawBath = (RawBathymetry)gro;
            m_dryCellSeries = m_rawBath.getDryCellSeries();
            m_barrierSeries = m_rawBath.getBarrierSeries();
            m_bDepthsLocked =
                m_rawBath.getBathymetrySeries().getDimensionLocked(0);

            return super.add(gro);
        }

        return bAdded;
    }

//------------------------------------------------------------------------------
    public void createPanel() {


        java.awt.event.ActionListener addDelAL =
            new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                    if (e.getActionCommand().equals("Add")) {
                        if (m_addDelMode == 1) {
                            m_addDelMode = 0;
                            m_noneButton.setSelected(true);
                        }
                        else {
                            m_addDelMode = 1;
                        }
                    }
                if (e.getActionCommand().equals("Delete")) {
                    if (m_addDelMode == 2) {
                        m_addDelMode = 0;
                        m_noneButton.setSelected(true);
                    }
                    else {
                        m_addDelMode = 2;
                    }
                }
            }
        };

/*
    java.awt.event.ActionListener typeAL =
            new java.awt.event.ActionListener() {

            public void actionPerformed(ActionEvent e) {
                    if (e.getActionCommand().equals("Dry Cell")) {
                        if (m_mode == 1) {
                            m_mode = 0;
                            m_noneButton.setSelected(true);
                        }
                        else {
                            m_mode = 1;
                        }
                    }
                if (e.getActionCommand().equals("Vertical")) {
                    if (m_mode == 2) {
                        m_mode = 0;
                        m_noneButton.setSelected(true);
                    }
                    else {
                        m_mode = 2;
                    }
                }
            }
        };
*/

        m_panel.setBorder(BorderFactory.createTitledBorder("Barrier"));
        BorderLayout mainLayout = new BorderLayout();
        m_panel.setLayout(new BoxLayout(m_panel, BoxLayout.Y_AXIS));

        m_addDelGroup.add(m_noneButton);
        m_addButton.addActionListener(addDelAL);
        m_addButton.setActionCommand("Add");
        m_addButton.setToolTipText("Add");
        byte[] byteArray = {0};
        m_addButton.setIcon(new ImageIcon(byteArray));
        m_addButton.setBorderPainted(true);
        m_addDelGroup.add(m_addButton);
        m_panel.add(m_addButton);
        m_deleteButton.addActionListener(addDelAL);
        m_deleteButton.setActionCommand("Delete");
        m_deleteButton.setToolTipText("Delete");
        m_deleteButton.setIcon(new ImageIcon(byteArray));
        m_deleteButton.setBorderPainted(true);
        m_addDelGroup.add(m_deleteButton);
        m_panel.add(m_deleteButton);

        JPanel radioPanel = new JPanel();
        radioPanel.setBorder(BorderFactory.createTitledBorder("Type"));
        radioPanel.setLayout(new BoxLayout(radioPanel, BoxLayout.Y_AXIS));
//        m_dryCellButton.setActionCommand("Dry Cell");
        m_dryCellButton.setToolTipText("Dry Cell");
        m_dryCellButton.setSelected(true);
//        m_addDryCellButton.addActionListener(al);
        m_typeGroup.add(m_dryCellButton);
        radioPanel.add(m_dryCellButton);
        m_thinWallButton.setActionCommand("Thin Wall");
        m_thinWallButton.setToolTipText("Thin Wall Barrier");
//        m_thinWallButton.addActionListener(al);
//        m_addVertButton.setIcon(new ImageIcon(byteArray));
//        m_addVertButton.setBorderPainted(true);
        m_typeGroup.add(m_thinWallButton);
        radioPanel.add(m_thinWallButton);
        m_panel.add(radioPanel);
    }

//------------------------------------------------------------------------------
    public JPanel getPanel() {

        return m_panel;
    }

//------------------------------------------------------------------------------
// Mouse methods...
    public void mouseEntered(MouseEvent e) {}
    public void mouseClicked(MouseEvent e) {}
    public void mouseMoved(MouseEvent e) {}

//------------------------------------------------------------------------------
/**
 *
 */
    public void mouseDragged(MouseEvent e) {

        if (m_grCon == null || m_page == null)
            return;

        boolean bButton3 = ((e.getModifiers() & e.BUTTON3_MASK) != 0);
        if (bButton3)
            return;

//        m_page.setZoomWindow(m_mouseX, m_mouseY, e.getX(), e.getY());
    }

//------------------------------------------------------------------------------
/**
 *
 * @param e     The mouse event object containing the mouse position.
 */
    public void mousePressed(MouseEvent e) {

        if (m_grCon == null || m_page == null)
            return;

        boolean bButton3 = ((e.getModifiers() & e.BUTTON3_MASK) != 0);
        if (bButton3)
            return;

        m_mouseX = e.getX();
        m_mouseY = e.getY();
    }

//------------------------------------------------------------------------------
/**
 *
 * @param e     The mouse event object containing the mouse position.
 */
    public void mouseReleased(MouseEvent e) {

        if (m_grCon == null || m_page == null)
            return;

        boolean bButton3 = ((e.getModifiers() & e.BUTTON3_MASK) != 0);
        if (bButton3)
            return;

        if (m_addDelMode == 0)
            return;

        // Find the graph nearest to the mouse.
        Graph startGraph = m_page.getNearestChild(m_mouseX, m_mouseY);
        int currentX = e.getX();
        int currentY = e.getY();
        Graph endGraph = m_page.getNearestChild(currentX, currentY);
        if (startGraph != endGraph)
            return;
        int xMin = (currentX < m_mouseX) ? currentX : m_mouseX;
        int xMax = (currentX > m_mouseX) ? currentX : m_mouseX;
        int yMin = (currentY < m_mouseY) ? currentY : m_mouseY;
        int yMax = (currentY > m_mouseY) ? currentY : m_mouseY;
        int width = xMax - xMin;
        int height = yMax - yMin;
        Rectangle screenRect = new Rectangle(xMin, yMin, width, height);
        Rectangle2D.Double userRect =
            m_page.screenToUser(screenRect, startGraph);
        if (m_dryCellButton.isSelected()) {
            addDryCells(userRect);
        }
        else {
//            if (m_addDelMode == 1)
//                addThinWallBarriers(userRect);
//            else
                deleteThinWallBarriers(userRect);
        }
    }

//------------------------------------------------------------------------------
/**
 *
 * @param e     The mouse event object containing the mouse position.
 */
    public void mouseExited(MouseEvent e) {}


//------------------------------------------------------------------------------
/**
 * Tells this interface what the current controller and page are.
 */
    public void setControllerAndPage(GrController grCon, PageOfGraphs page) {

        m_grCon = grCon;
        m_page = page;
    }

//------------------------------------------------------------------------------
/**
 *
 */
    public void addDryCells(Rectangle2D.Double rect) {

        if (m_rawBath == null)
            return;

        if (m_dryCellSeries == null) {
            m_dryCellSeries = new IntDataSeries(1);
            m_dryCellSeries.setTitle("Dry Cells");
            m_rawBath.add(m_dryCellSeries);
        }

        GrHashList bathProps = m_rawBath.getProperties();
        GrProperty p = (GrProperty)bathProps.get("xSpacing");
        double gridSpacing = (Double)p.value;
        p = (GrProperty)bathProps.get("numI");
        int numI = ((Integer)p.value);
        p = (GrProperty)bathProps.get("numJ");
        int numJ = ((Integer)p.value);

        int iMin = (int)(rect.x / gridSpacing);
        int iMax = (int)((rect.x + rect.width) / gridSpacing);
        int jMin = (int)(rect.y / gridSpacing);
        int jMax = (int)((rect.y + rect.height) / gridSpacing);

        int[] dryCellArray = m_dryCellSeries.getArray();
        m_dryCellSeries.deselectAll();

        for (int i = iMin; i <= iMax; i++) {
            for (int j = jMin; j <= jMax; j++) {
                if (m_addDelMode == 1) {
                    // Adding...
                    if (!m_rawBath.isWet(i, j)) {
                        continue;
                    }
                    if (i < 0 || i >= (numI - 1) || j < 0 || j >= (numJ - 1))
                        continue;

                    addDryCell(i, j);
                }
                else {
                    // Deleting...
                    int comboIJ = i * 10000 + j;
                    int index = m_dryCellSeries.search(comboIJ);
                    if (dryCellArray[index] == comboIJ)
                        m_dryCellSeries.selectPoint(index);
                }
            }
        }

        if (m_addDelMode == 2) {
            m_dryCellSeries.deleteSelectedPoints();
        }
    }

//------------------------------------------------------------------------------
/**
 *
 */
    public void addDryCell(int i, int j) {

        if (m_rawBath == null)
            return;

        if (m_dryCellSeries == null) {
            m_dryCellSeries = new IntDataSeries(1);
            m_dryCellSeries.setTitle("Dry Cells");
            m_rawBath.add(m_dryCellSeries);
        }

        int[] oldArray = m_dryCellSeries.getArray();

        int comboIJ = i * 10000 + j;
        int index = m_dryCellSeries.search(comboIJ);
        if (index >= 0) {
            if (oldArray[index] == comboIJ)
                return;

            if (oldArray[index] > comboIJ)
                index--;
        }

        int[] newArray = new int[oldArray.length + 1];
        for (int n = 0; n <= index; n++) {
            newArray[n] = oldArray[n];
        }
        newArray[index + 1] = comboIJ;
        for (int n = index + 1; n < oldArray.length; n++) {
            newArray[n + 1] = oldArray[n];
        }

        m_dryCellSeries.setArray(newArray);
    }

//------------------------------------------------------------------------------
/**
 *
 */
    public void addThinWallBarriers(Rectangle2D.Double rect) {

        if (m_rawBath == null)
            return;

        if (m_barrierSeries == null) {
            m_barrierSeries = new IntDataSeries(2);
            m_barrierSeries.setTitle("Thin Wall Barriers");
            m_rawBath.add(m_barrierSeries);

        }

        GrHashList bathProps = m_rawBath.getProperties();
        GrProperty p = (GrProperty)bathProps.get("xSpacing");
        double gridSpacing = (Double)p.value;
        p = (GrProperty)bathProps.get("numI");
        int numI = ((Integer)p.value);
        p = (GrProperty)bathProps.get("numJ");
        int numJ = ((Integer)p.value);

        int iMin = (int)Math.round(rect.x / gridSpacing);
        int iMax = (int)Math.round((rect.x + rect.width) / gridSpacing);
        int jMin = (int)(rect.y / gridSpacing);
        int jMax = (int)((rect.y + rect.height) / gridSpacing);
/*
        if (m_mode == 3) {
            iMin = (int)(rect.x / gridSpacing);
            iMax = (int)((rect.x + rect.width) / gridSpacing);
            jMin = Math.round(rect.y / gridSpacing);
            jMax = Math.round((rect.y + rect.height) / gridSpacing);
        }


        for (int i = iMin; i <= iMax; i++) {
            for (int j = jMin; j <= jMax; j++) {
                if (!m_rawBath.isWet(i, j)) {
                    continue;
                }
                if (m_mode == 3) {
                    if (!m_rawBath.isWet(i, j - 1)) {
                        continue;
                    }
                }
                else {
                    if (!m_rawBath.isWet(i - 1, j)) {
                        continue;
                    }
                }
                if (i < 0 || i > (numI - 1) || j < 0 || j > (numJ - 1))
                    continue;

                addThinWallBarrier(i, j);
            }
        }
*/
    }

//------------------------------------------------------------------------------
/**
 *
 */
    public void addThinWallBarrier(int combo1IJ, int combo2IJ) {

        if (m_rawBath == null)
            return;

        if (m_barrierSeries == null) {
            m_barrierSeries = new IntDataSeries(2);
            m_barrierSeries.setTitle("Thin Wall Barriers");
            m_rawBath.add(m_barrierSeries);

        }

        int[] oldArray = m_barrierSeries.getArray();

//        int combo1IJ = i1 * 10000 + j1;
//        int combo2IJ = i1 * 10000 + j1;

        // Check if the barrier is already in the series.
        int index = m_barrierSeries.search(combo1IJ);
        if (index >= 0 && oldArray[index * 2] == combo1IJ) {
            if (oldArray[index * 2 + 1] == combo2IJ) {
                return;
            }
        }
        else {
            // Check the index below
            index--;
            if (index >= 0 && oldArray[index * 2] == combo1IJ) {
                if (oldArray[index * 2 + 1] == combo2IJ) {
                    return;
                }
            }
            else {
                // Check the index above
                index += 2;
                if ((index * 2) < oldArray.length &&
                    oldArray[index * 2] == combo1IJ) {

                    if (oldArray[index * 2 + 1] == combo2IJ) {
                        return;
                    }
                }
            }
        }

        // Insert the barrier.
        index = m_barrierSeries.search(combo1IJ);
        if (index >= 0 && oldArray[index] > combo1IJ)
            index--;

        int[] newArray = new int[oldArray.length + 2];
        for (int n = 0; n <= (index * 2 + 1); n++) {
            newArray[n] = oldArray[n];
        }
        newArray[(index + 1) * 2] = combo1IJ;
        newArray[(index + 1) * 2 + 1] = combo2IJ;
        for (int n = (index + 1) * 2; n < oldArray.length; n++) {
            newArray[n + 2] = oldArray[n];
        }

        m_barrierSeries.setArray(newArray);
    }

//------------------------------------------------------------------------------
/**
 *
 */
/*
    public void addThinWallBarrier(int i, int j) {


        if (m_rawBath == null || m_barrierSeries == null)
            return;

        int[] oldArray = m_barrierSeries.getArray();

        int comboIJ = i * 10000 + j;
        int index = m_dryCellSeries.search(comboIJ);
        if (oldArray[index] == comboIJ)
            return;

        if (oldArray[index] > comboIJ)
            index--;

        int[] newArray = new int[oldArray.length + 1];
        for (int n = 0; n <= index; n++) {
            newArray[n] = oldArray[n];
        }
        newArray[index + 1] = comboIJ;
        for (int n = index + 1; n < oldArray.length; n++) {
            newArray[n + 1] = oldArray[n];
        }

        m_dryCellSeries.setArray(newArray);
*/

/*
if (m_rawBath == null || m_barrierSeries == null)
            return;

        int[] oldArray = m_barrierSeries.getArray();
        int oldCount = m_barrierSeries.getCount();

        int comboIJ = i * 10000 + j;
        int index = m_barrierSeries.search(comboIJ);
        if (oldArray[index] == comboIJ)
            return;

        if (oldArray[index] > comboIJ)
            index--;


        int[] newArray = new int[oldArray.length + 2];
        for (int n = 0; n <= index; n++) {
            newArray[n * 2] = oldArray[n * 2];
            newArray[n * 2 + 1] = oldArray[n * 2 + 1];
        }
        newArray[(index + 1) * 2] = comboIJ;
        int endIJ = i * 10000 + j + 1;
///        if (m_mode == 3)
///            endIJ = (i + 1) * 10000 + j;
        newArray[(index + 1) * 2 + 1] = endIJ;
        for (int n = index + 1; n < oldCount; n++) {
            newArray[(n + 1) * 2] = oldArray[n * 2];
            newArray[(n + 1) * 2 + 1] = oldArray[n * 2 + 1];
        }

        m_barrierSeries.setArray(newArray);
*/
//    }

//------------------------------------------------------------------------------
/**
 *
 */
    public void deleteThinWallBarriers(Rectangle2D.Double rect) {

        if (m_rawBath == null)
            return;

        if (m_barrierSeries == null) {
            m_barrierSeries = new IntDataSeries(2);
            m_barrierSeries.setTitle("Thin Wall Barriers");
            m_rawBath.add(m_barrierSeries);

        }

        GrHashList bathProps = m_rawBath.getProperties();
        GrProperty p = (GrProperty)bathProps.get("xSpacing");
        double gridSpacing = (Double)p.value;
        p = (GrProperty)bathProps.get("numI");
        int numI = ((Integer)p.value);
        p = (GrProperty)bathProps.get("numJ");
        int numJ = ((Integer)p.value);

        int nearestIMin = (int)Math.round(rect.x / gridSpacing);
        int nearestIMax = (int)Math.round((rect.x + rect.width) / gridSpacing);
        int nearestJMin = (int)Math.round(rect.y / gridSpacing);
        int nearestJMax = (int)Math.round((rect.y + rect.height) / gridSpacing);
        int[] min = new int[2];
        int[] max = new int[2];
        int comboIJ = nearestIMin * 10000 + nearestJMin;
        min[0] = comboIJ;
        min[1] = comboIJ;
        comboIJ = nearestIMax * 10000 + nearestJMax;
        max[0] = comboIJ;
        max[1] = comboIJ;

        m_barrierSeries.deselectAll();
        int[] barrierArray = m_barrierSeries.getArray();

        if (min[0] == max[0]) {
            // Select one barrier...
            double xOffset = (rect.x / gridSpacing) - (double)nearestIMin;
            double yOffset = (rect.y / gridSpacing) - (double)nearestJMin;
            if (Math.abs(xOffset) < Math.abs(yOffset)) {
                if (yOffset < 0) {
                    comboIJ = nearestIMin * 10000 + nearestJMin - 1;
                    min[0] = comboIJ;
                    max[0] = comboIJ;
                }
                else {
                    comboIJ = nearestIMin * 10000 + nearestJMin + 1;
                    min[1] = comboIJ;
                    max[1] = comboIJ;
                }
            }
            else {
                if (xOffset < 0) {
                    comboIJ = (nearestIMin - 1) * 10000 + nearestJMin;
                    min[0] = comboIJ;
                    max[0] = comboIJ;
                }
                else {
                    comboIJ = (nearestIMin + 1) * 10000 + nearestJMin;
                    min[1] = comboIJ;
                    max[1] = comboIJ;
                }
            }
            if (m_addDelMode == 1) {
                addThinWallBarrier(min[0], max[1]);
            }
            else {
                m_barrierSeries.selectArea(min, max, true);
                m_barrierSeries.deleteSelectedPoints();
            }
        }
        else {
            int iMin0 = min[0] / 10000;
            int jMin0 = min[0] - (iMin0 * 10000);
            int iMax0 = max[0] / 10000;
            int jMax0 = max[0] - (iMax0 * 10000);


            for (int i = iMin0; i <= iMax0; i++) {
                for (int j = jMin0; j <= jMax0; j++) {
                    comboIJ = i * 10000 + j;
                    int horzEndIJ = (i + 1) * 10000 + j;
                    int vertEndIJ = i * 10000 + j + 1;
                    int index = m_barrierSeries.search(comboIJ);
                    if (barrierArray[index * 2] == comboIJ) {
                        // Check if it is a sideways barrier.
                        if (barrierArray[index * 2 + 1] == horzEndIJ) {
                            m_barrierSeries.selectPoint(index);
                        }
                        else {
                            // Check if it is an up and down barrier.
                            if (barrierArray[index * 2 + 1] == vertEndIJ) {
                                m_barrierSeries.selectPoint(index);
                            }
                        }
                        // Check the index above and below, because there can be
                        // two barriers with the same start point.
                        index--;
                        if ((index * 2) > 0 && barrierArray[index * 2] ==
                            comboIJ) {

                            // Check if it is a sideways barrier.
                            if (barrierArray[index * 2 + 1] == horzEndIJ) {
                                m_barrierSeries.selectPoint(index);
                            }
                            else {
                                // Check if it is a up and down barrier.
                                if (barrierArray[index * 2 + 1] == vertEndIJ) {
                                    m_barrierSeries.selectPoint(index);
                                }
                            }
                        }
                        index += 2;
                        if ((index * 2) < barrierArray.length &&
                            barrierArray[index * 2] == comboIJ) {

                                // Check if it is a sideways barrier.
                                if (barrierArray[index * 2 + 1] == horzEndIJ) {
                                    m_barrierSeries.selectPoint(index);
                                }
                                else {
                                    // Check if it is a up and down barrier.
                                    if (barrierArray[index * 2 + 1] ==
                                        vertEndIJ) {

                                        m_barrierSeries.selectPoint(index);
                                    }
                                }
                        }
                    }
                }
            }
            m_barrierSeries.deleteSelectedPoints();
        }
    }

//------------------------------------------------------------------------------
/**
 * Overrides default update() in GrObject.  Called by an observed GrObject's
 * notifyObservers() method to handle changes to the observed object.
 * May use the notifyObservers() method to propagate the event handling up to
 * this object's observers.
 * @param gro   The observed object that changed.
 * @param arg   Additional information that the updated object may choose to
 *              send.
 */
    public void update(GrObject gro, Object arg) {

        if (gro == m_rawBath) {
            m_dryCellSeries = m_rawBath.getDryCellSeries();
            m_barrierSeries = m_rawBath.getBarrierSeries();
            m_bDepthsLocked =
                m_rawBath.getBathymetrySeries().getDimensionLocked(0);
        }

        m_bHasChanged = true;
        notifyObservers();
    }
//------------------------------------------------------------------------------
/**
 * Create a string representation of this object.
 * @return  The string that describes this object.
 */
    public String toString() {

        String s = "Barrier Editor";

        return s;
     }
}

