/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2006-11-20
 * Date started:    2006-10-20
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.sfhydro.si3d.gui;

import java.awt.Canvas;
import java.awt.Component;

/**
 *
 * @author  jmd
 */
public class GrBathCalcFrame extends javax.swing.JFrame {
    
    GrBathCalcController m_grCon = null;
    Canvas m_canvas = null;
    double[] m_areaUnitScales = {1, 1e-4, 1e-6, 10.76391, 2.47104369e-4,
        3.86102158e-7};
    double[] m_volUnitScales = {1, 1e-4, 1e-9, 35.3147, 8.10713087e-4,
        2.39912758e-10};
    
    protected double m_totalArea = 0;
    protected double m_reachableArea = 0;
    protected double m_totalVolume = 0;
    protected double m_reachableVolume = 0;

    
    /** Creates new form GrBathCalcFrame */
    public GrBathCalcFrame() {
        initComponents();
    }
    /** Creates new form GrBathCalcFrame */
    public GrBathCalcFrame(GrBathCalcController grCon) {
        this();
	m_grCon = grCon;
    }
    
//------------------------------------------------------------------------------
    public void setComponent(Component component) {

        m_canvas = (Canvas)component;
        getContentPane().remove(drawingPanel);
        getContentPane().add(m_canvas, java.awt.BorderLayout.CENTER);
    }
    
//------------------------------------------------------------------------------
    public void setGridSpacing(int spacing) {

        gridSpacingLabel.setText("" + spacing + " m Spacing");
    }
    
//------------------------------------------------------------------------------
    public void setGridSize(int numI, int numJ) {

        gridSizeLabel.setText("" + numI + " X " + numJ + " Points");
    }
    
//------------------------------------------------------------------------------
    public void setUnitIndexes(int areaIndex, int volIndex) {

        areaUnitsCombo.setSelectedIndex(areaIndex);
        volUnitsCombo.setSelectedIndex(volIndex);
    }
    
//------------------------------------------------------------------------------
    public void setCoords(String ijString, String xyString, String depthString) {

        ijLabel.setText(ijString);
        xyLabel.setText(xyString);
        depthLabel.setText(depthString);
    }

//------------------------------------------------------------------------------
    public void setArea(double totalArea, double reachableArea) {
	
	m_totalArea = totalArea;
	m_reachableArea = reachableArea;
	writeArea();
    }

//------------------------------------------------------------------------------
    public void setVolume(double totalVolume, double reachableVolume) {

	m_totalVolume = totalVolume;
	m_reachableVolume = reachableVolume;
	writeVolume();
    }
//------------------------------------------------------------------------------
    public void writeArea() {
	
        int areaUnitIndex = areaUnitsCombo.getSelectedIndex();
	double scale = m_areaUnitScales[areaUnitIndex];
        double totalArea = m_totalArea * scale;
        areaTotalValueLabel.setText(String.format("%g", totalArea));
        double reachableArea = m_reachableArea * scale;
        areaReachableTextField.setText(String.format("%g", reachableArea));
    }

//------------------------------------------------------------------------------
    public void writeVolume() {

        int volUnitIndex = volUnitsCombo.getSelectedIndex();
	double scale = m_volUnitScales[volUnitIndex];
        double totalVolume = m_totalVolume * scale;
        volTotalValueLabel.setText(String.format("%g", totalVolume));
        double reachableVolume = m_reachableVolume * scale;
        volumeReachableTextField.setText(String.format("%g", reachableVolume));
    }
     
//------------------------------------------------------------------------------
    public double getWaterLevel() {

	double waterLevel = 0;
	try {
	    String text = waterLevelTextField.getText();
            waterLevel = Double.parseDouble(text);
	    if (waterLevel <= -8 || waterLevel >= 1000)
		waterLevel = 0;
        }
        catch (NumberFormatException e) {
	    waterLevelTextField.setText("0");
        }
	
	return waterLevel;
    }
     
//------------------------------------------------------------------------------
    public double getCutoffDepth(double waterLevel) {

	double cutoffDepth = Double.POSITIVE_INFINITY;
	try {
	    String text = cutoffDepthTextField.getText();
            cutoffDepth = Double.parseDouble(text);
	    if (cutoffDepth <= -waterLevel || cutoffDepth >= 1000)
		cutoffDepth = Double.POSITIVE_INFINITY;
        }
        catch (NumberFormatException e) {
	    cutoffDepthTextField.setText("NONE");
        }
	
	return cutoffDepth;
    }


    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        actionButtonGroup = new javax.swing.ButtonGroup();
        areaTotalLabel = new javax.swing.JLabel();
        areaReachableLabel = new javax.swing.JLabel();
        areaTotalValueLabel = new javax.swing.JLabel();
        volTotalLabel = new javax.swing.JLabel();
        volReachableLabel = new javax.swing.JLabel();
        volTotalValueLabel = new javax.swing.JLabel();
        bottomPanel = new javax.swing.JPanel();
        volumePanel = new javax.swing.JPanel();
        volUnitsLabel = new javax.swing.JLabel();
        volUnitsCombo = new javax.swing.JComboBox();
        volumeReachableTextField = new javax.swing.JTextField();
        areaPanel = new javax.swing.JPanel();
        areaUnitsLabel = new javax.swing.JLabel();
        areaUnitsCombo = new javax.swing.JComboBox();
        areaReachableTextField = new javax.swing.JTextField();
        inputPanel = new javax.swing.JPanel();
        waterLevelLabel = new javax.swing.JLabel();
        cutoffDepthLabel = new javax.swing.JLabel();
        waterLevelTextField = new javax.swing.JTextField();
        cutoffDepthTextField = new javax.swing.JTextField();
        westPanel = new javax.swing.JPanel();
        viewPanel = new javax.swing.JPanel();
        zoomOutButton = new javax.swing.JButton();
        zoomInButton = new javax.swing.JButton();
        showAllButton = new javax.swing.JButton();
        actionPanel = new javax.swing.JPanel();
        panModeRadio = new javax.swing.JRadioButton();
        chooseModeRadio = new javax.swing.JRadioButton();
        defineModeRadio = new javax.swing.JRadioButton();
        coordPanel = new javax.swing.JPanel();
        ijLabel = new javax.swing.JLabel();
        xyLabel = new javax.swing.JLabel();
        depthLabel = new javax.swing.JLabel();
        ijDescriptionLabel = new javax.swing.JLabel();
        xyDescriptionLabel = new javax.swing.JLabel();
        depthDescriptionLabel = new javax.swing.JLabel();
        gridPanel = new javax.swing.JPanel();
        gridSizeLabel = new javax.swing.JLabel();
        gridSpacingLabel = new javax.swing.JLabel();
        boundariesButton = new javax.swing.JButton();
        drawingPanel = new javax.swing.JPanel();

        areaTotalLabel.setFont(new java.awt.Font("SansSerif", 1, 14));
        areaTotalLabel.setText("Total");
        areaReachableLabel.setFont(new java.awt.Font("SansSerif", 1, 14));
        areaReachableLabel.setText("Reachable");
        areaTotalValueLabel.setFont(new java.awt.Font("SansSerif", 1, 14));
        areaTotalValueLabel.setText("0");
        volTotalLabel.setFont(new java.awt.Font("SansSerif", 1, 14));
        volTotalLabel.setText("Total");
        volReachableLabel.setFont(new java.awt.Font("SansSerif", 1, 14));
        volReachableLabel.setText("Reachable");
        volTotalValueLabel.setFont(new java.awt.Font("SansSerif", 1, 14));
        volTotalValueLabel.setText("0");

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosed(java.awt.event.WindowEvent evt) {
                formWindowClosed(evt);
            }
        });

        volumePanel.setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)), "Volume", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("SansSerif", 1, 18)));
        volumePanel.setMaximumSize(new java.awt.Dimension(500, 500));
        volUnitsLabel.setFont(new java.awt.Font("SansSerif", 1, 14));
        volUnitsLabel.setText("Units");

        volUnitsCombo.setFont(new java.awt.Font("SansSerif", 1, 14));
        volUnitsCombo.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "cubic meters", "hectare meters", "cubic kilometers", "cubic feet", "acre feet", "cubic miles" }));
        volUnitsCombo.setLightWeightPopupEnabled(false);
        volUnitsCombo.setMaximumSize(new java.awt.Dimension(200, 200));
        volUnitsCombo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                volUnitsComboActionPerformed(evt);
            }
        });

        volumeReachableTextField.setEditable(false);
        volumeReachableTextField.setFont(new java.awt.Font("Serif", 1, 24));
        volumeReachableTextField.setText("0");

        javax.swing.GroupLayout volumePanelLayout = new javax.swing.GroupLayout(volumePanel);
        volumePanel.setLayout(volumePanelLayout);
        volumePanelLayout.setHorizontalGroup(
            volumePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(volumePanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(volumePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(volumePanelLayout.createSequentialGroup()
                        .addComponent(volUnitsLabel)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(volUnitsCombo, 0, 222, Short.MAX_VALUE))
                    .addComponent(volumeReachableTextField, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 264, Short.MAX_VALUE))
                .addContainerGap())
        );
        volumePanelLayout.setVerticalGroup(
            volumePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(volumePanelLayout.createSequentialGroup()
                .addComponent(volumeReachableTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(volumePanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(volUnitsLabel)
                    .addComponent(volUnitsCombo, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        areaPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)), "Area", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("SansSerif", 1, 18)));
        areaPanel.setMaximumSize(new java.awt.Dimension(500, 500));
        areaUnitsLabel.setFont(new java.awt.Font("SansSerif", 1, 14));
        areaUnitsLabel.setText("Units");

        areaUnitsCombo.setFont(new java.awt.Font("SansSerif", 1, 14));
        areaUnitsCombo.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "square meters", "hectares", "square kilometers", "square feet", "acres", "square miles" }));
        areaUnitsCombo.setLightWeightPopupEnabled(false);
        areaUnitsCombo.setMaximumSize(new java.awt.Dimension(200, 200));
        areaUnitsCombo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                areaUnitsComboActionPerformed(evt);
            }
        });

        areaReachableTextField.setEditable(false);
        areaReachableTextField.setFont(new java.awt.Font("Serif", 1, 24));
        areaReachableTextField.setText("0");

        javax.swing.GroupLayout areaPanelLayout = new javax.swing.GroupLayout(areaPanel);
        areaPanel.setLayout(areaPanelLayout);
        areaPanelLayout.setHorizontalGroup(
            areaPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(areaPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(areaPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(areaPanelLayout.createSequentialGroup()
                        .addComponent(areaUnitsLabel)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(areaUnitsCombo, 0, 223, Short.MAX_VALUE))
                    .addComponent(areaReachableTextField, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 265, Short.MAX_VALUE))
                .addContainerGap())
        );
        areaPanelLayout.setVerticalGroup(
            areaPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(areaPanelLayout.createSequentialGroup()
                .addComponent(areaReachableTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(areaPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(areaUnitsLabel)
                    .addComponent(areaUnitsCombo, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        inputPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)), "Input", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("SansSerif", 1, 18)));
        inputPanel.setMaximumSize(new java.awt.Dimension(500, 500));
        waterLevelLabel.setFont(new java.awt.Font("SansSerif", 1, 14));
        waterLevelLabel.setText("Water Level (m)");
        waterLevelLabel.setToolTipText("Water Level (in meters, positive upward)");

        cutoffDepthLabel.setFont(new java.awt.Font("SansSerif", 1, 14));
        cutoffDepthLabel.setText("Cutoff Depth (m)");
        cutoffDepthLabel.setToolTipText("Cutoff Depth (in meters, positive downward) for Volume Calculations");

        waterLevelTextField.setFont(new java.awt.Font("SansSerif", 1, 14));
        waterLevelTextField.setText("0");
        waterLevelTextField.setToolTipText("Water Level (in meters, positive upward)");

        cutoffDepthTextField.setFont(new java.awt.Font("SansSerif", 1, 14));
        cutoffDepthTextField.setText("NONE");
        cutoffDepthTextField.setToolTipText("Cutoff Depth (in meters, positive downward) for Volume Calculations");
        cutoffDepthTextField.setMaximumSize(new java.awt.Dimension(200, 200));

        javax.swing.GroupLayout inputPanelLayout = new javax.swing.GroupLayout(inputPanel);
        inputPanel.setLayout(inputPanelLayout);
        inputPanelLayout.setHorizontalGroup(
            inputPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(inputPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(inputPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(waterLevelLabel, javax.swing.GroupLayout.DEFAULT_SIZE, 165, Short.MAX_VALUE)
                    .addComponent(cutoffDepthLabel))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(inputPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                    .addComponent(waterLevelTextField)
                    .addComponent(cutoffDepthTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 94, Short.MAX_VALUE))
                .addContainerGap())
        );
        inputPanelLayout.setVerticalGroup(
            inputPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(inputPanelLayout.createSequentialGroup()
                .addGroup(inputPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(waterLevelLabel)
                    .addComponent(waterLevelTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(inputPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(cutoffDepthLabel)
                    .addComponent(cutoffDepthTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(31, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout bottomPanelLayout = new javax.swing.GroupLayout(bottomPanel);
        bottomPanel.setLayout(bottomPanelLayout);
        bottomPanelLayout.setHorizontalGroup(
            bottomPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(bottomPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(inputPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(areaPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(volumePanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        bottomPanelLayout.setVerticalGroup(
            bottomPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(bottomPanelLayout.createSequentialGroup()
                .addGroup(bottomPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(inputPanel, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(volumePanel, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(areaPanel, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        getContentPane().add(bottomPanel, java.awt.BorderLayout.SOUTH);

        westPanel.setMaximumSize(new java.awt.Dimension(200, 2000));
        viewPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)), "View", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("SansSerif", 1, 18)));
        viewPanel.setFont(new java.awt.Font("SansSerif", 1, 14));
        viewPanel.setMaximumSize(new java.awt.Dimension(500, 500));
        viewPanel.setName("View");
        zoomOutButton.setFont(new java.awt.Font("SansSerif", 1, 14));
        zoomOutButton.setText("Zoom Out <<");
        zoomOutButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                zoomOutButtonActionPerformed(evt);
            }
        });

        zoomInButton.setFont(new java.awt.Font("SansSerif", 1, 14));
        zoomInButton.setText(">> Zoom In");
        zoomInButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                zoomInButtonActionPerformed(evt);
            }
        });

        showAllButton.setFont(new java.awt.Font("SansSerif", 1, 14));
        showAllButton.setLabel("<< Show All >>");
        showAllButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                showAllButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout viewPanelLayout = new javax.swing.GroupLayout(viewPanel);
        viewPanel.setLayout(viewPanelLayout);
        viewPanelLayout.setHorizontalGroup(
            viewPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(viewPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(viewPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(zoomInButton, javax.swing.GroupLayout.DEFAULT_SIZE, 205, Short.MAX_VALUE)
                    .addComponent(zoomOutButton, javax.swing.GroupLayout.DEFAULT_SIZE, 205, Short.MAX_VALUE)
                    .addComponent(showAllButton, javax.swing.GroupLayout.DEFAULT_SIZE, 205, Short.MAX_VALUE))
                .addContainerGap())
        );
        viewPanelLayout.setVerticalGroup(
            viewPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(viewPanelLayout.createSequentialGroup()
                .addComponent(zoomInButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(zoomOutButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(showAllButton)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        actionPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)), "Mouse Action", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Microsoft Sans Serif", 1, 18)));
        actionButtonGroup.add(panModeRadio);
        panModeRadio.setFont(new java.awt.Font("SansSerif", 1, 14));
        panModeRadio.setSelected(true);
        panModeRadio.setText("Pan");
        panModeRadio.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        panModeRadio.setMargin(new java.awt.Insets(0, 0, 0, 0));
        panModeRadio.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                panModeRadioActionPerformed(evt);
            }
        });

        actionButtonGroup.add(chooseModeRadio);
        chooseModeRadio.setFont(new java.awt.Font("SansSerif", 1, 14));
        chooseModeRadio.setText("Choose Calc Region");
        chooseModeRadio.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        chooseModeRadio.setMargin(new java.awt.Insets(0, 0, 0, 0));
        chooseModeRadio.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                chooseModeRadioActionPerformed(evt);
            }
        });

        actionButtonGroup.add(defineModeRadio);
        defineModeRadio.setFont(new java.awt.Font("SansSerif", 1, 14));
        defineModeRadio.setText("Define Boundaries");
        defineModeRadio.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        defineModeRadio.setMargin(new java.awt.Insets(0, 0, 0, 0));
        defineModeRadio.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                defineModeRadioActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout actionPanelLayout = new javax.swing.GroupLayout(actionPanel);
        actionPanel.setLayout(actionPanelLayout);
        actionPanelLayout.setHorizontalGroup(
            actionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(actionPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(actionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(panModeRadio)
                    .addComponent(chooseModeRadio)
                    .addComponent(defineModeRadio))
                .addContainerGap(61, Short.MAX_VALUE))
        );
        actionPanelLayout.setVerticalGroup(
            actionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(actionPanelLayout.createSequentialGroup()
                .addComponent(panModeRadio)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(chooseModeRadio)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(defineModeRadio)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        coordPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)), "Coordinates", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("SansSerif", 1, 18)));
        coordPanel.setMaximumSize(new java.awt.Dimension(500, 500));
        ijLabel.setFont(new java.awt.Font("SansSerif", 1, 14));
        ijLabel.setText("    0");
        ijLabel.setToolTipText("(I, J) Coordinate of Mouse");

        xyLabel.setFont(new java.awt.Font("SansSerif", 1, 14));
        xyLabel.setText("    0");
        xyLabel.setToolTipText("(X, Y) Coordinate of Mouse");

        depthLabel.setFont(new java.awt.Font("SansSerif", 1, 14));
        depthLabel.setText("    0");
        depthLabel.setToolTipText("Depth in meters of point nearest the mouse");

        ijDescriptionLabel.setFont(new java.awt.Font("SansSerif", 1, 14));
        ijDescriptionLabel.setText("(I, J)");

        xyDescriptionLabel.setFont(new java.awt.Font("SansSerif", 1, 14));
        xyDescriptionLabel.setText("(X, Y)");

        depthDescriptionLabel.setFont(new java.awt.Font("SansSerif", 1, 14));
        depthDescriptionLabel.setText("Depth (m)");

        javax.swing.GroupLayout coordPanelLayout = new javax.swing.GroupLayout(coordPanel);
        coordPanel.setLayout(coordPanelLayout);
        coordPanelLayout.setHorizontalGroup(
            coordPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(coordPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(coordPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(depthDescriptionLabel)
                    .addComponent(xyDescriptionLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(ijDescriptionLabel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(coordPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(xyLabel, javax.swing.GroupLayout.DEFAULT_SIZE, 128, Short.MAX_VALUE)
                    .addComponent(depthLabel, javax.swing.GroupLayout.DEFAULT_SIZE, 128, Short.MAX_VALUE)
                    .addComponent(ijLabel, javax.swing.GroupLayout.DEFAULT_SIZE, 128, Short.MAX_VALUE))
                .addContainerGap())
        );
        coordPanelLayout.setVerticalGroup(
            coordPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(coordPanelLayout.createSequentialGroup()
                .addGroup(coordPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(ijDescriptionLabel)
                    .addComponent(ijLabel))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(coordPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(xyDescriptionLabel)
                    .addComponent(xyLabel))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(coordPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(depthDescriptionLabel)
                    .addComponent(depthLabel))
                .addContainerGap())
        );

        gridPanel.setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)), "Grid", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("SansSerif", 1, 18)));
        gridPanel.setMaximumSize(new java.awt.Dimension(500, 500));
        gridSizeLabel.setFont(new java.awt.Font("SansSerif", 1, 14));
        gridSizeLabel.setText("???? X ???? points");

        gridSpacingLabel.setFont(new java.awt.Font("SansSerif", 1, 14));
        gridSpacingLabel.setText("???? m Spacing");

        javax.swing.GroupLayout gridPanelLayout = new javax.swing.GroupLayout(gridPanel);
        gridPanel.setLayout(gridPanelLayout);
        gridPanelLayout.setHorizontalGroup(
            gridPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(gridPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(gridPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(gridSizeLabel, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.PREFERRED_SIZE, 174, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(gridSpacingLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 174, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(46, Short.MAX_VALUE))
        );
        gridPanelLayout.setVerticalGroup(
            gridPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(gridPanelLayout.createSequentialGroup()
                .addComponent(gridSizeLabel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(gridSpacingLabel)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        boundariesButton.setFont(new java.awt.Font("SansSerif", 1, 14));
        boundariesButton.setText("Boundaries...");
        boundariesButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                boundariesButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout westPanelLayout = new javax.swing.GroupLayout(westPanel);
        westPanel.setLayout(westPanelLayout);
        westPanelLayout.setHorizontalGroup(
            westPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(westPanelLayout.createSequentialGroup()
                .addGroup(westPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(westPanelLayout.createSequentialGroup()
                        .addGap(15, 15, 15)
                        .addComponent(actionPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                    .addGroup(westPanelLayout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(gridPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                    .addGroup(westPanelLayout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(coordPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                    .addGroup(westPanelLayout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(viewPanel, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                    .addGroup(westPanelLayout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(boundariesButton, javax.swing.GroupLayout.DEFAULT_SIZE, 245, Short.MAX_VALUE)))
                .addContainerGap())
        );
        westPanelLayout.setVerticalGroup(
            westPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(westPanelLayout.createSequentialGroup()
                .addGap(16, 16, 16)
                .addComponent(actionPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(boundariesButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(viewPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(coordPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(gridPanel, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(73, Short.MAX_VALUE))
        );
        getContentPane().add(westPanel, java.awt.BorderLayout.WEST);

        javax.swing.GroupLayout drawingPanelLayout = new javax.swing.GroupLayout(drawingPanel);
        drawingPanel.setLayout(drawingPanelLayout);
        drawingPanelLayout.setHorizontalGroup(
            drawingPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 687, Short.MAX_VALUE)
        );
        drawingPanelLayout.setVerticalGroup(
            drawingPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 650, Short.MAX_VALUE)
        );
        getContentPane().add(drawingPanel, java.awt.BorderLayout.CENTER);

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void boundariesButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_boundariesButtonActionPerformed
        m_grCon.manageBoundarySet();
    }//GEN-LAST:event_boundariesButtonActionPerformed

    private void defineModeRadioActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_defineModeRadioActionPerformed
        
        if (defineModeRadio.isSelected())
            m_grCon.setMode(2);
    }//GEN-LAST:event_defineModeRadioActionPerformed

    private void chooseModeRadioActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_chooseModeRadioActionPerformed
        
        if (chooseModeRadio.isSelected())
            m_grCon.setMode(1);
    }//GEN-LAST:event_chooseModeRadioActionPerformed

    private void panModeRadioActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_panModeRadioActionPerformed

        if (panModeRadio.isSelected())
            m_grCon.setMode(0);
    }//GEN-LAST:event_panModeRadioActionPerformed

    private void formWindowClosed(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosed
	m_grCon.writeGuiOptions();
	System.exit(0);
    }//GEN-LAST:event_formWindowClosed

    private void showAllButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_showAllButtonActionPerformed
	
	m_grCon.zoomAll();
    }//GEN-LAST:event_showAllButtonActionPerformed

    private void zoomOutButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_zoomOutButtonActionPerformed

	m_grCon.zoomOut();
    }//GEN-LAST:event_zoomOutButtonActionPerformed

    private void zoomInButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_zoomInButtonActionPerformed
	
	m_grCon.zoomIn();
    }//GEN-LAST:event_zoomInButtonActionPerformed

    private void areaUnitsComboActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_areaUnitsComboActionPerformed

	m_grCon.m_options.m_areaUnitsIndex = areaUnitsCombo.getSelectedIndex();
	writeArea();
    }//GEN-LAST:event_areaUnitsComboActionPerformed

    private void volUnitsComboActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_volUnitsComboActionPerformed

	m_grCon.m_options.m_volUnitsIndex = volUnitsCombo.getSelectedIndex();
	writeVolume();
    }//GEN-LAST:event_volUnitsComboActionPerformed
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                new GrBathCalcFrame().setVisible(true);
            }
        });
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup actionButtonGroup;
    private javax.swing.JPanel actionPanel;
    private javax.swing.JPanel areaPanel;
    private javax.swing.JLabel areaReachableLabel;
    private javax.swing.JTextField areaReachableTextField;
    private javax.swing.JLabel areaTotalLabel;
    private javax.swing.JLabel areaTotalValueLabel;
    private javax.swing.JComboBox areaUnitsCombo;
    private javax.swing.JLabel areaUnitsLabel;
    private javax.swing.JPanel bottomPanel;
    private javax.swing.JButton boundariesButton;
    private javax.swing.JRadioButton chooseModeRadio;
    private javax.swing.JPanel coordPanel;
    private javax.swing.JLabel cutoffDepthLabel;
    private javax.swing.JTextField cutoffDepthTextField;
    private javax.swing.JRadioButton defineModeRadio;
    private javax.swing.JLabel depthDescriptionLabel;
    private javax.swing.JLabel depthLabel;
    private javax.swing.JPanel drawingPanel;
    private javax.swing.JPanel gridPanel;
    private javax.swing.JLabel gridSizeLabel;
    private javax.swing.JLabel gridSpacingLabel;
    private javax.swing.JLabel ijDescriptionLabel;
    private javax.swing.JLabel ijLabel;
    private javax.swing.JPanel inputPanel;
    private javax.swing.JRadioButton panModeRadio;
    private javax.swing.JButton showAllButton;
    private javax.swing.JPanel viewPanel;
    private javax.swing.JLabel volReachableLabel;
    private javax.swing.JLabel volTotalLabel;
    private javax.swing.JLabel volTotalValueLabel;
    private javax.swing.JComboBox volUnitsCombo;
    private javax.swing.JLabel volUnitsLabel;
    private javax.swing.JPanel volumePanel;
    private javax.swing.JTextField volumeReachableTextField;
    private javax.swing.JLabel waterLevelLabel;
    private javax.swing.JTextField waterLevelTextField;
    private javax.swing.JPanel westPanel;
    private javax.swing.JLabel xyDescriptionLabel;
    private javax.swing.JLabel xyLabel;
    private javax.swing.JButton zoomInButton;
    private javax.swing.JButton zoomOutButton;
    // End of variables declaration//GEN-END:variables

    private java.awt.Canvas createGLCanvas()
    {
	return new javax.media.opengl.GLCanvas();
    }
}
