/**
 * @(#)             Gr graphing application
 * @author          John Donovan
 *                  U.S. Geological Survey, Water Resources Division
 * @version         2006-11-01
 * Date started:    2006-10-29
 *
 * Copyright (C) 1998-2008 John M. Donovan
 * See the file Gr.java in the package gov.usgs.gr for licensing and
 * warranty information.  Also see license.txt in the package gov.usgs.
 */

package gov.usgs.sfhydro.si3d.gui;

import gov.usgs.gr.*;
import java.util.*;

//==============================================================================
/**
 * An object that has properties and contains other objects in a hierarchy.
 * It can be displayed in a tree or written as XML.
 */
public class GrBathCalcGuiOptions extends gov.usgs.gr.GrObject {
    
    public int m_gridSpacing = 10;
    public int m_areaUnitsIndex = 0;
    public int m_volUnitsIndex = 0;
    
//------------------------------------------------------------------------------
    /**
     * Construct an empty object.
     */
    public GrBathCalcGuiOptions() {
	
	setDefaults("gov.usgs.sfhydro.si3d.gui.GrBathCalcGuiOptions");
    }
    
//------------------------------------------------------------------------------
    /**
     * Provide the list of properties with the latest values.
     */
    public GrHashList getProperties() {
	
	GrHashList props = super.getProperties();
	
	props.put("gridSpacing", new GrProperty(
	    new Integer(m_gridSpacing), "Grid Spacing"));
	props.put("areaUnitsIndex", new GrProperty(
	    new Integer(m_areaUnitsIndex), "Area Units Index"));
	props.put("volUnitsIndex", new GrProperty(
	    new Integer(m_volUnitsIndex), "Volume Units Index"));
	
	return props;
    }
    
//------------------------------------------------------------------------------
    /**
     * Provide the list of properties with the latest values.
     */
    public void setProperties(HashMap props) {
	
	super.setProperties(props);
	
	GrProperty p;
	
	Iterator iter = props.keySet().iterator();
	while (iter.hasNext()) {
	    String key = (String)iter.next();
	    Object obj = props.get(key);
	    if (obj instanceof GrProperty) {
		p = (GrProperty)obj;
		
                if (key.equals("gridSpacing"))
                    m_gridSpacing = (Integer)p.value;
                if (key.equals("areaUnitsIndex"))
                    m_areaUnitsIndex = (Integer)p.value;
                if (key.equals("volUnitsIndex"))
                    m_volUnitsIndex = (Integer)p.value;
	    }
	    if (obj instanceof String) {
		String value = (String)obj;
		setProperty(key, value);
	    }
	}
    }
    
//------------------------------------------------------------------------------
    /**
     * Set a single property.
     */
    public void setProperty(String key, String value) {
	
	super.setProperty(key, value);
	
        if (key.equals("gridSpacing"))
            m_gridSpacing = (int)Math.round(Double.parseDouble(value));
        if (key.equals("areaUnitsIndex"))
            m_areaUnitsIndex = (int)Math.round(Double.parseDouble(value));
        if (key.equals("volUnitsIndex"))
            m_volUnitsIndex = (int)Math.round(Double.parseDouble(value));
    }
    
//------------------------------------------------------------------------------
    /**
     * Create a string representation of this object.
     * @return  The string that describes this object.
     */
    public String toString() {
	
	String s = "GrBathCalc GUI Options";
	return s;
    }
}




